@@ -1,7 +1,7 @@
 /** @file app_getvcp.c
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -28,6 +28,8 @@
 #include "base/sleep.h"
 #include "base/vcp_version.h"
 
+#include "cmdline/parsed_cmd.h"
+
 #include "vcp/vcp_feature_codes.h"
 
 #include "dynvcp/dyn_feature_codes.h"
@@ -66,7 +68,7 @@ app_show_single_vcp_value_by_feature_table_entry(
    DBGTRC(debug, TRACE_GROUP,
          "Starting. Getting feature 0x%02x for %s", entry->code, dh_repr(dh) );
 
-   DDCA_MCCS_Version_Spec vspec      = get_vcp_version_by_display_handle(dh);
+   DDCA_MCCS_Version_Spec vspec      = get_vcp_version_by_dh(dh);
    Public_Status_Code     psc        = 0;
    DDCA_Vcp_Feature_Code  feature_id = entry->code;
 
@@ -102,16 +104,13 @@ app_show_single_vcp_value_by_feature_table_entry(
 #endif
 
 
-/* Shows a single VCP value specified by its #Display_Feature_Metadata
+/**  Shows a single VCP value specified by its #Display_Feature_Metadata
  *
- * Arguments:
- *    dh          handle of open display
- *    meta        feature metadata
- *
- * Returns:
- *    status code 0 = normal
- *                DDCRC_INVALID_OPERATION - feature is deprecated or write-only
- *                from get_formatted_value_for_feature_table_entry()
+ *   \param  dh           handle of open display
+ *   \param  meta         feature metadata
+ *   \return status code  0 = normal
+ *                        DDCRC_INVALID_OPERATION - feature is deprecated or write-only
+ *                        from get_formatted_value_for_feature_table_entry()
  */
 DDCA_Status
 app_show_single_vcp_value_by_dfm(
@@ -119,12 +118,12 @@ app_show_single_vcp_value_by_dfm(
       Display_Feature_Metadata *  dfm)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. Getting feature 0x%02x for %s",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "Getting feature 0x%02x for %s",
                                dfm->feature_code, dh_repr(dh) );
 
    // DDCA_Feature_Metadata * extmeta = dfm_to_ddca_feature_metadata(meta);
 
-   DDCA_MCCS_Version_Spec vspec      = get_vcp_version_by_display_handle(dh);
+   DDCA_MCCS_Version_Spec vspec      = get_vcp_version_by_dh(dh);
    DDCA_Status            ddcrc      = 0;
    DDCA_Vcp_Feature_Code  feature_id = dfm->feature_code;
 
@@ -143,7 +142,7 @@ app_show_single_vcp_value_by_dfm(
 
    if (ddcrc == 0) {
       char * formatted_value = NULL;
-      ddcrc = ddc_get_formatted_value_for_display_feature_metadata(
+      ddcrc = ddc_get_formatted_value_for_dfm(
                dh,
                dfm,
                false,      /* suppress_unsupported */
@@ -156,33 +155,46 @@ app_show_single_vcp_value_by_dfm(
       }
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", psc_desc(ddcrc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, ddcrc, "");
    return ddcrc;
 }
 
 
+/**  Shows a single VCP value specified by its feature code
+ *
+ *   \param  dh           handle of open display
+ *   \param  feature_id   feature code
+ *   \param  force        generate default metadata if unknown feature id
+ *   \return 0 - success
+ *           DDCRC_UNKNOWN_FEATURE unrecognized feature id and **force** not specified
+ *           from #app_show_single_vcp_value_by_dfm()
+ *
+ *   Looks up the #Display_Feature_Metadata record for the feature id and calls
+ *   #app_show_single_vcp_value_by_dfm() to display the value.
+ *   Generates a dummy #Display_Feature_Metadata record for features in the
+ *   reserved manufacturer range (xE0..xFF).
+ *   if #force is specified, also generates a dummy metadata record for
+ *   unrecognized features.
+ */
 Public_Status_Code
-app_show_single_vcp_value_by_feature_id_new_dfm(
+app_show_single_vcp_value_by_feature_id(
       Display_Handle *      dh,
       DDCA_Vcp_Feature_Code feature_id,
       bool                  force)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. Getting feature 0x%02x for %s, force=%s",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "Getting feature 0x%02x for %s, force=%s",
                               feature_id, dh_repr(dh), sbool(force) );
 
    Public_Status_Code         psc = 0;
 
    Display_Feature_Metadata * dfm =
-   dyn_get_feature_metadata_by_dh_dfm(
+   dyn_get_feature_metadata_by_dh(
          feature_id,
          dh,
          force || feature_id >= 0xe0    // with_default
          );
 
-   // VCP_Feature_Table_Entry *  entry = NULL;
-
-
    if (!dfm) {
       printf("Unrecognized VCP feature code: 0x%02x\n", feature_id);
       psc = DDCRC_UNKNOWN_FEATURE;
@@ -192,7 +204,7 @@ app_show_single_vcp_value_by_feature_id_new_dfm(
       dfm_free(dfm);
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", psc_desc(psc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc, "");
    return psc;
 }
 
@@ -209,15 +221,15 @@ app_show_single_vcp_value_by_feature_id_new_dfm(
  *    status code       from show_vcp_values()
  */
 Public_Status_Code
-app_show_vcp_subset_values_by_display_handle(
+app_show_vcp_subset_values_by_dh(
         Display_Handle *    dh,
         VCP_Feature_Subset  subset_id,
         Feature_Set_Flags   flags,
         Byte_Bit_Flags      features_seen)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP,
-          "Starting. dh=%s, subset_id=%s, flags=%s, features_seen=%p",
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+          "dh=%s, subset_id=%s, flags=%s, features_seen=%p",
           dh_repr(dh), feature_subset_name(subset_id), feature_set_flag_names_t(flags), features_seen );
 
    GPtrArray * collector = NULL;
@@ -233,6 +245,7 @@ app_show_vcp_subset_values_by_display_handle(
          DBGMSG("Returning: %s", psc_desc(psc));
       }
    }
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc, "");
    return psc;
 }
 
@@ -249,7 +262,7 @@ app_show_vcp_subset_values_by_display_handle(
  * Returns:
  *    nothing
  */
-void app_show_vcp_subset_values_by_display_ref(
+void app_show_vcp_subset_values_by_dref(
         Display_Ref *       dref,
         VCP_Feature_Subset  subset_id,
         bool                show_unsupported)
@@ -279,44 +292,64 @@ void app_show_vcp_subset_values_by_display_ref(
 #endif
 
 
-/* Shows the VCP values for all features indicated by a Feature_Set_Ref
+/**  Shows the VCP values for all features indicated by a #Feature_Set_Ref
  *
- * Arguments:
- *    dh                display handle
- *    fsref             feature set reference
- *    flags             option flags
- *
- * Returns:
- *    status code       from app_show_single_vcp_value_by_feature_id() or
- *                           app_show_subset_values_by_display_handle()
+ *   \param  dh      display handle
+ *   \param  fsref   feature set reference
+ *   \param  flags   option flags
+ *   \return status code from #app_show_single_vcp_value_by_feature_id_new_dfm() or
+ *                            #app_show_subset_values_by_dh()
  */
 Public_Status_Code
-app_show_feature_set_values_by_display_handle(
+app_show_feature_set_values_by_dh(
       Display_Handle *     dh,
-      Feature_Set_Ref *    fsref,
-      Feature_Set_Flags    flags)
+      Parsed_Cmd *         parsed_cmd)
 {
    bool debug = false;
    if (debug || IS_TRACING()) {
-      char * s0 = feature_set_flag_names_t(flags);
-      DBGMSG("Starting. dh: %s. fsref: %s, flags: %s", dh_repr(dh), fsref_repr_t(fsref), s0);
-      // dbgrpt_feature_set_ref(fsref,1);
+      char * s0 = feature_set_flag_names_t(parsed_cmd->flags);
+      DBGTRC_STARTING(debug, TRACE_GROUP, "dh: %s. fsref: %s, flags: %s", dh_repr(dh), fsref_repr_t(parsed_cmd->fref), s0);
+      // dbgrpt_feature_set_ref(parsed_cmd->fref,1);
    }
 
+   Feature_Set_Ref *    fsref = parsed_cmd->fref;
+
+   // DBGMSG("parsed_cmd->flags: 0x%04x", parsed_cmd->flags);
+   Feature_Set_Flags flags = 0x00;
+   if (parsed_cmd->flags & CMD_FLAG_SHOW_UNSUPPORTED)
+      flags |= FSF_SHOW_UNSUPPORTED;
+   if (parsed_cmd->flags & CMD_FLAG_FORCE)
+      flags |= FSF_FORCE;
+   if (parsed_cmd->flags & CMD_FLAG_NOTABLE)
+      flags |= FSF_NOTABLE;
+   if (parsed_cmd->flags & CMD_FLAG_RW_ONLY)
+      flags |= FSF_RW_ONLY;
+   if (parsed_cmd->flags & CMD_FLAG_RO_ONLY)
+      flags |= FSF_RO_ONLY;
+   // this is nonsense, getvcp on a WO feature should be caught by parser
+   if (parsed_cmd->flags & CMD_FLAG_WO_ONLY) {
+      // flags |= FSF_WO_ONLY;
+      DBGMSG("Invalid: GETVCP for WO features");
+      assert(false);
+   }
+   // char * s0 = feature_set_flag_names(flags);
+   // DBGMSG("flags: 0x%04x - %s", flags, s0);
+   // free(s0);
+
    Public_Status_Code psc = 0;
    if (fsref->subset == VCP_SUBSET_SINGLE_FEATURE) {
-      psc = app_show_single_vcp_value_by_feature_id_new_dfm(
+      psc = app_show_single_vcp_value_by_feature_id(
             dh, fsref->specific_feature, true);
    }
    else {
-      psc = app_show_vcp_subset_values_by_display_handle(
+      psc = app_show_vcp_subset_values_by_dh(
             dh,
             fsref->subset,
             flags,
             NULL);
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Returning: %s", psc_desc(psc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc, "");
    return psc;
 }
 
@@ -325,77 +358,71 @@ app_show_feature_set_values_by_display_handle(
 // Watch for changed VCP values
 //
 
-void reset_vcp_x02(Display_Handle * dh) {
+// Depending on monitor, writing 1 to feature x02 may throw the user
+// out of the on-stream display.  Use carefully.
+
+static void
+reset_vcp_x02(Display_Handle * dh) {
+   bool debug = false;
    Error_Info * ddc_excp = ddc_set_nontable_vcp_value(dh, 0x02, 0x01);
    if (ddc_excp) {
-      DBGMSG("set_nontable_vcp_value_by_display_handle() returned %s", errinfo_summary(ddc_excp) );
+      DBGMSG("set_nontable_vcp_value_by_dh() returned %s", errinfo_summary(ddc_excp) );
       errinfo_free(ddc_excp);
    }
    else
-      DBGMSG("reset feature x02 (new control value) successful");
-}
-
-bool new_control_values_exist(Display_Handle * dh) {
-   bool debug = false;
-   Parsed_Nontable_Vcp_Response * p_nontable_response = NULL;
-   // DBGMSF(debug, "VCP version: %d.%d", vspec.major, vspec.minor);
-   bool result = false;
-    Error_Info * ddc_excp = ddc_get_nontable_vcp_value(
-             dh,
-             0x02,
-             &p_nontable_response);
-    if (ddc_excp) {
-       DBGMSG("get_nontable_vcp_value() returned %s", errinfo_summary(ddc_excp) );
-       errinfo_free(ddc_excp);
-    }
-
-    else if (p_nontable_response->sl == 0x01) {
-       DBGMSF(debug, "No new control values found");
-       free(p_nontable_response);
-    }
-    else {
-       DBGMSG("New control values exist. x02 value: 0x%02x", p_nontable_response->sl);
-       free(p_nontable_response);
-       p_nontable_response = NULL;
-       result = true;
-    }
-    return result;
+      DBGMSF(debug, "reset feature x02 (new control value) successful");
 }
 
 
-/** Gets the ID of the next changed feature from VCP feature x52,
- *  then reads and displays the value of that feature.
+/** Gets the ID of the next changed feature from VCP feature x52. If the feature
+ *  code is other than x00, reads and displays the value of that feature.
  *
  *  \param   dh  #Display_Handle
- *  \return  id of changed feature, 0x00 if none
+ *  \param   p_changed_feature   return feature id read from feature x52
+ *  \return  error reading feature x52
+ *
+ *  \remark
+ *  The return value reflects only x52 errors, not any errors reading
+ *  the feature id that is displayed
  */
+static Error_Info *
+show_changed_feature(Display_Handle * dh, Byte * p_changed_feature) {
+   bool debug = false;
+   Parsed_Nontable_Vcp_Response * nontable_response_loc = NULL;
+   Error_Info * result = NULL;
+   Error_Info * x52_error = ddc_get_nontable_vcp_value(dh, 0x52, &nontable_response_loc);
+   DBGMSF(debug, "ddc_get_nontable_vcp_value( x52 ) returned %s", errinfo_summary(x52_error));
+   if (x52_error) {
+      if (x52_error->status_code == DDCRC_REPORTED_UNSUPPORTED ||
+          x52_error->status_code == DDCRC_DETERMINED_UNSUPPORTED)
+      {
+         // printf("Feature x02 (New Control Value) reports new control values exist, but feature x52 (Active Control) unsupported\n");
+         result = errinfo_new2(x52_error->status_code, __func__,
+               "Feature x02 (New Control Value) reports that changed VCP feature values exist, but feature x52 (Active Control) is unsupported");
+         errinfo_free(x52_error);
+      }
+      else {
+         // DBGMSG("get_nontable_vcp_value() for VCP feature x52 returned %s", errinfo_summary(x52_error) );
+         result = errinfo_new_with_cause2(
+                  x52_error->status_code, x52_error, __func__, "Error reading feature x02");
+      }
+   }
 
-Byte show_changed_feature(Display_Handle * dh) {
-   Parsed_Nontable_Vcp_Response * p_nontable_response = NULL;
-   Byte changed_feature = 0x00;
-   Error_Info * ddc_excp = ddc_get_nontable_vcp_value(
-              dh,
-              0x52,
-              &p_nontable_response);
-  // psc = (ddc_excp) ? ddc_excp->psc : 0;
-  if (ddc_excp) {
-     DBGMSG("get_nontable_vcp_value() for VCP feature x52 returned %s", errinfo_summary(ddc_excp) );
-     errinfo_free(ddc_excp);
+  else {  // getvcp x52 succeeded
+     *p_changed_feature = nontable_response_loc->sl;
+     free(nontable_response_loc);
+     DBGMSF(debug, "getvcp(x52) returned value 0x%02x", *p_changed_feature);
+     if (*p_changed_feature)
+        app_show_single_vcp_value_by_feature_id(dh, *p_changed_feature, false);
   }
-  else {
-     changed_feature = p_nontable_response->sl;
-     free(p_nontable_response);
-     if (changed_feature)
-        app_show_single_vcp_value_by_feature_id_new_dfm(dh, changed_feature, false);
-  }
-  return changed_feature;
+  return result;
 }
 
 
 /* Checks for VCP feature changes by:
  *   - reading feature x02 to check if changes exist,
  *   - querying feature x52 for the id of a changed feature
- *   - reading the value of the changed feature.
+ *   - reading and showing the value of the changed feature.
  *
  * If the VCP version is 2.1 or less a single feature is
  * read from x52.  For VCP version 3.0 and 2.2, x52 is a
@@ -403,19 +430,17 @@ Byte show_changed_feature(Display_Handle * dh) {
  *
  * Finally, 1 is written to feature x02 as a reset.
  *
- * Arguments:
- *    dh      display handle
+ * \param  dh    #Display_Handle
+ * \param  force_no_fifo never treat feature x52 as a FIFO
+ * \param  changes_reported set true if any changes were detected
+ * \return error report, NULL if none
  */
-void
-app_read_changes(Display_Handle * dh) {
+static Error_Info *
+app_read_changes(Display_Handle * dh, bool force_no_fifo, bool* changes_reported) {
    bool debug = false;
    DBGMSF(debug, "Starting");
    int MAX_CHANGES = 20;
-
-   // read 02h
-   // xff: no user controls
-   // x01: no new control values
-   // x02: new control values exist
+   *changes_reported = false;
 
    /* Per the 3.0 and 2.2 specs, feature x52 is a FIFO to be read until value x00 indicates empty
     * What apparently happens on 2.1 (U3011) is that each time feature x02 is reset with value x01
@@ -423,34 +448,97 @@ app_read_changes(Display_Handle * dh) {
     * of changes is flushed
     */
 
-   DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
+   DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_dh(dh);
+   // DBGMSF(debug, "VCP version: %d.%d", vspec.major, vspec.minor);
 
-   if (new_control_values_exist(dh)) {
-      if ( vcp_version_le(vspec, DDCA_VSPEC_V21) ) {
-         show_changed_feature(dh);
+   // Read feature x02 to determine if any features have changed
+   //   xff: no user controls
+   //   x01: no new control values
+   //   x02: new control values exist
+   Parsed_Nontable_Vcp_Response * p_nontable_response = NULL;
+
+   Error_Info * result = NULL;
+   Error_Info * x02_error = ddc_get_nontable_vcp_value(dh,0x02,&p_nontable_response);
+   if (x02_error) {
+      DBGMSG("get_nontable_vcp_value() for feature 0x02 returned error %s", errinfo_summary(x02_error) );
+      // errinfo_free(ddc_excp);
+      result = errinfo_new_with_cause2(x02_error->status_code, x02_error, __func__,
+                                       "Error reading feature x02");
+   }
+   else {
+      Byte x02_value = p_nontable_response->sl;
+      DBGMSF(debug, "get_nontable_vcp_value() for feature 0x02 returned value 0x%02x", x02_value );
+      free(p_nontable_response);
+
+      if (x02_value == 0xff) {
+         DBGMSF(debug, "No user controls exist");
+         result = errinfo_new2(DDCRC_DETERMINED_UNSUPPORTED, __func__,
+                        "Feature x02 (New Control Value) reports No User Controls");
       }
-      else {  // x52 is a FIFO
-         int ctr = 0;
-         for (;ctr < MAX_CHANGES; ctr++) {
-            Byte cur_feature = show_changed_feature(dh);
-            if (cur_feature == 0x00) {
-               DBGMSG("No more changed features found");
-               break;
+
+      else if (x02_value == 0x01) {
+         DBGMSF(debug, "No new control values found");
+         result = NULL;
+      }
+
+      else if (x02_value != 0x02){
+         DBGMSF(debug, "x02 value = 0x%02x", x02_value);
+         result = errinfo_new2(DDCRC_DETERMINED_UNSUPPORTED, __func__,
+               "Feature x02 (New Control Value) reports unexpected value 0x%02", x02_value);
+      }
+
+      else {
+         DBGMSF(debug, "New control values exist. x02 value: 0x%02x", x02_value);
+         Byte changed_feature_id;
+
+         if ( vcp_version_le(vspec, DDCA_VSPEC_V21)  || force_no_fifo) {
+            Error_Info * x52_error = show_changed_feature(dh, &changed_feature_id);
+            // MCCS spec requires that feature x02 be reset, otherwise it remains at x02
+            // and the same value is read again
+            // But on some displays it also turns off the OSD: HPZ22i
+            // For other displays it does not turn off the OSD, so the user can make
+            // additional changes:  Dell U3011
+            reset_vcp_x02(dh);
+            result = x52_error;
+            if (!x52_error)
+               *changes_reported = true;
+         }
+
+         else {  // x52 is a FIFO
+            int ctr = 0;
+            for (;ctr < MAX_CHANGES; ctr++) {
+               Byte changed_feature_id = 0x00;
+               Error_Info * x52_error = show_changed_feature(dh, &changed_feature_id);
+               if (x52_error) {
+                  result = x52_error;
+                  goto bye;
+               }
+               *changes_reported = true;
+               if (changed_feature_id == 0x00) {
+                  DBGMSG("No more changed features found");
+                  reset_vcp_x02(dh);
+                  result =  NULL;
+                  break;
+               }
+            }
+            if (ctr == MAX_CHANGES) {
+               DBGMSG("Reached loop guard value MAX_CHANGES (%d)", MAX_CHANGES);
+               reset_vcp_x02(dh);
+               result = NULL;
             }
          }
-         if (ctr == MAX_CHANGES) {
-            DBGMSG("Reached loop guard value MAX_CHANGES (%d)", MAX_CHANGES);
-         }
       }
-      reset_vcp_x02(dh);
    }
+
+ bye:
+   return result;
 }
 
 
 #ifdef USE_USB
-void
+static void
 app_read_changes_usb(Display_Handle * dh) {
-   bool debug = true;
+   bool debug = false;
    DBGMSF(debug, "Starting");
    // bool new_values_found = false;
 
@@ -487,32 +575,51 @@ app_read_changes_usb(Display_Handle * dh) {
 #endif
 
 
-
 /* Infinite loop watching for VCP feature changes reported by the display.
  *
- * Arguments:
- *    dh        display handle
+ * \param  dh  #Display_Handle
+ * \param  force_no_fifo if true, do not regard feature x52 aa a FIFO queue,
+ *                       even if VCP code is >= 2.2
  *
- * Returns:
- *    does not return - halts with program termination
+ * Returns only if an error occurs, otherwise runs forever
  */
 void
-app_read_changes_forever(Display_Handle * dh) {
+app_read_changes_forever(Display_Handle * dh, bool force_no_fifo) {
    bool debug = false;
 
    printf("Watching for VCP feature changes on display %s\n", dh_repr(dh));
    printf("Type ^C to exit...\n");
    // show version here instead of in called function to declutter debug output:
-   DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
+   DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_dh(dh);
    DBGMSF(debug, "VCP version: %d.%d", vspec.major, vspec.minor);
+   reset_vcp_x02(dh);
    while(true) {
+      bool changes_reported = false;
 #ifdef USE_USB
       if (dh->dref->io_path.io_mode == DDCA_IO_USB)
          app_read_changes_usb(dh);
       else
 #endif
-         app_read_changes(dh);
+      {
+         Error_Info * erec = app_read_changes(dh, force_no_fifo, &changes_reported);
+         if (erec) {
+            if (debug)
+               DBGMSG("Fatal error reading changes: %s", errinfo_summary(erec));
 
-      sleep_millis( 2500);
+            printf("%s\n", erec->detail);
+            DDCA_Status rc = erec->status_code;
+            errinfo_free(erec);
+            if (rc == DDCRC_NULL_RESPONSE) {
+               printf("Continuing WATCH execution\n");
+            }
+            else {
+               printf("Terminating WATCH\n");
+               return;
+            }
+         }
+      }
+
+      if (!changes_reported)
+         sleep_millis( 2500);
    }
 }
