@@ -3,42 +3,53 @@
  *  Primary file for the ENVIRONMENT command
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
+// #define SYSENV_QUICK_TEST_RUN 1
 
 /** \cond */
 #include <config.h>
 
 // #define _GNU_SOURCE 1       // for function group_member
 #include <assert.h>
+#include <dirent.h>
 #include <errno.h>
 #include <string.h>
 #include <sys/utsname.h>
 
 #include "util/data_structures.h"
 #include "util/edid.h"
+#include "util/linux_util.h"
 #include "util/report_util.h"
 #include "util/string_util.h"
 #include "util/subprocess_util.h"
+#include "util/sysfs_i2c_util.h"
 #include "util/sysfs_util.h"
+#include "util/xdg_util.h"
 #ifdef PROBE_USING_SYSTEMD
 #include "util/systemd_util.h"
 #endif
 #ifdef USE_X11
 #include "util/x11_util.h"
 #endif
+#ifdef ENABLE_UDEV
 #include "util/udev_i2c_util.h"
+#ifdef ENABLE_USB
 #include "util/udev_usb_util.h"
+#endif
+#endif
 /** \endcond */
 
 #include "base/build_info.h"
 #include "base/core.h"
 #include "base/linux_errno.h"
+#include "base/rtti.h"
 
-#ifdef HAVE_ADL
-#include "adl/adl_shim.h"
-#endif
+#include "i2c/i2c_sysfs.h"
+
+#include "ddc/ddc_displays.h"     // for ddc_ensure_displays_detected()
+#include "ddc/ddc_watch_displays.h"
 
 #include "query_sysenv_access.h"
 #include "query_sysenv_base.h"
@@ -53,6 +64,8 @@
 
 #include "query_sysenv.h"
 
+// Default trace class for this file
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_ENV;
 
 /** Compile time and runtime checks of endianness.
  *
@@ -105,7 +118,7 @@ static void query_base_env(Env_Accumulator * accum) {
    int d1 = d0+1;
    int d2 = d0+2;
 
-   rpt_vstring(d0, "ddcutil version: %s", BUILD_VERSION);
+   rpt_vstring(d0, "ddcutil version: %s", get_full_ddcutil_version());
    rpt_nl();
 
    sysenv_rpt_file_first_line("/proc/version", NULL, 0);
@@ -191,7 +204,6 @@ static void query_base_env(Env_Accumulator * accum) {
 }
 
 
-
 #ifdef OLD
 
 /* Checks for installed packages i2c-tools and libi2c-dev
@@ -257,16 +269,7 @@ static void driver_specific_tests(struct driver_name_node * driver_list) {
       found_driver_specific_checks = true;
       rpt_nl();
       rpt_vstring(0,"Performing ADL specific checks...");
-#ifdef HAVE_ADL
-     if (!adlshim_is_available()) {
-        set_output_level(DDCA_OL_VERBOSE);  // force error msg that names missing dll
-        bool ok = adlshim_initialize();
-        if (!ok)
-           printf("WARNING: Using AMD proprietary video driver fglrx but unable to load ADL library\n");
-     }
-#else
-     rpt_vstring(0,"WARNING: Using AMD proprietary video driver fglrx but ddcutil built without ADL support");
-#endif
+      rpt_vstring(0,"WARNING: Using AMD proprietary video driver fglrx but ddcutil built without ADL support");
    }
 
    if (!found_driver_specific_checks)
@@ -288,7 +291,7 @@ static void driver_specific_tests(struct driver_name_node * driver_list) {
 void query_x11() {
    GPtrArray* edid_recs = get_x11_edids();
    rpt_nl();
-   rpt_vstring(0,"EDIDs reported by X11 for connected xrandr outputs:");
+   rpt_vstring(0,"*** EDIDs reported by X11 for connected xrandr outputs ***");
    // DBGMSG("Got %d X11_Edid_Recs\n", edid_recs->len);
 
    for (int ndx=0; ndx < edid_recs->len; ndx++) {
@@ -346,27 +349,16 @@ void query_x11() {
 #endif
 
 
-//
-// i2cdetect
-//
-
-/** Examines /dev/i2c devices using command i2cdetect, if it exists.
- *
- *  \param  i2c_device_numbers  I2C bus numbers to check
- *
- */
-static void query_using_i2cdetect(Byte_Value_Array i2c_device_numbers) {
+static void query_using_shell_command(Byte_Value_Array i2c_device_numbers,
+                const char * pattern,
+                const char * command_name)
+{
    assert(i2c_device_numbers);
 
-#ifdef SYSENV_QUICK_TEST_RUN
-   DBGMSG("!!! Skipping i2cdetect checks to speed up testing !!!");
-   return;
-#endif
-
    int d0 = 0;
    int d1 = 1;
 
-   rpt_vstring(d0,"Examining I2C buses using i2cdetect... ");
+   rpt_vstring(d0,"Examining I2C buses using %s... ", command_name);
    sysenv_rpt_current_time(NULL, d1);
 
    if (bva_length(i2c_device_numbers) == 0) {
@@ -376,19 +368,20 @@ static void query_using_i2cdetect(Byte_Value_Array i2c_device_numbers) {
       for (int ndx=0; ndx< bva_length(i2c_device_numbers); ndx++) {
          int busno = bva_get(i2c_device_numbers, ndx);
          if (sysfs_is_ignorable_i2c_device(busno)) {
-            // calling i2cdetect for an SMBUs device fills dmesg with error messages
             rpt_nl();
-            rpt_vstring(d1, "Device /dev/i2c-%d is a SMBus or other ignorable device.  Skipping i2cdetect.", busno);
+            rpt_vstring(d1, "Device /dev/i2c-%d is a SMBus or other ignorable device."
+                            "  Skipping %s.", busno, command_name);
          }
          else {
-            char cmd[80];
-            snprintf(cmd, 80, "i2cdetect -y %d", busno);
+            char cmd[200];
+            snprintf(cmd, 200, pattern, busno);
+           //    "udevadm info --attribute-walk --path=$(udevadm info --query=path --name=i2c-%d)", busno);
             rpt_nl();
             rpt_vstring(d1,"Probing bus /dev/i2c-%d using command \"%s\"", busno, cmd);
             int rc = execute_shell_cmd_rpt(cmd, 2 /* depth */);
             // DBGMSG("execute_shell_cmd(\"%s\") returned %d", cmd, rc);
             if (rc != 1) {
-               rpt_vstring(d1,"i2cdetect command unavailable");
+               rpt_vstring(d1,"%s command unavailable", command_name);
                break;
             }
          }
@@ -396,20 +389,22 @@ static void query_using_i2cdetect(Byte_Value_Array i2c_device_numbers) {
    }
 }
 
-
+#ifdef ENABLE_UDEV
 /** Queries UDEV for devices in subsystem "i2c-dev".
  *  Also looks for devices with name attribute "DPMST"
  */
 static void probe_i2c_devices_using_udev() {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
    char * subsys_name = "i2c-dev";
-   rpt_vstring(0,"Probing I2C devices using udev, susbsystem %s...", subsys_name);
+   rpt_vstring(0,"*** Probe I2C devices using udev, susbsystem %s ***", subsys_name);
    sysenv_rpt_current_time(NULL, 1);
    // probe_udev_subsystem() is in udev_util.c, which is only linked in if USE_USB
 
    // Detailed scan of I2C device information.   TMI
    // probe_udev_subsystem(subsys_name, /*show_usb_parent=*/ false, 1);
-   // rpt_nl();
 
+   rpt_nl();
    GPtrArray * summaries = get_i2c_devices_using_udev();
    report_i2c_udev_device_summaries(summaries, "Summary of udev I2C devices",1);
 
@@ -435,6 +430,141 @@ static void probe_i2c_devices_using_udev() {
    summaries = find_devices_by_sysattr_name(nameattr);
    report_i2c_udev_device_summaries(summaries, "Summary of udev DPMST devices...",1);
    free_udev_device_summaries(summaries);   // ok if summaries == NULL
+   rpt_nl();
+
+   DBGTRC_DONE(debug, TRACE_GROUP, "");
+}
+#endif
+
+
+void rpt_module_status(int depth, const char * module_name) {
+   int module_status =  module_status_using_libkmod(module_name);
+   switch( module_status) {
+   case KERNEL_MODULE_NOT_FOUND:    // 0
+      rpt_vstring(depth,"Kernel module %-16s not found", module_name);
+      break;
+   case KERNEL_MODULE_BUILTIN:      // 1
+      rpt_vstring(depth,"Kernel module %-16s is builtin", module_name);
+      break;
+   case KERNEL_MODULE_LOADABLE_FILE: // 2
+      if (is_module_loaded_using_libkmod(module_name))
+         rpt_vstring(depth, "Kernel module %-16s is loaded", module_name);
+      else
+         rpt_vstring(depth, "Kernel module %-16s found but not loaded", module_name);
+      break;
+   default:
+      rpt_vstring(depth, "Error %s from module_status_using_libkmod() for %s",
+                  psc_desc(module_status), module_name);
+      break;
+   }
+}
+
+
+void query_loaded_modules_using_libkmod() {
+   rpt_vstring(0,"*** Checking if modules are loaded or builtin... ***");
+
+   char ** pmodule_names = get_known_video_driver_module_names();
+   char * curmodule;
+   int ndx;
+   for (ndx=0; (curmodule=pmodule_names[ndx]) != NULL; ndx++) {
+      rpt_module_status(1, curmodule);
+   }
+   pmodule_names = get_other_driver_module_names();
+   for (ndx=0; (curmodule=pmodule_names[ndx]) != NULL; ndx++) {
+      rpt_module_status(1, curmodule);
+   }
+}
+
+
+static GPtrArray * get_path_application_files(const char * path, const char * application) {
+   GPtrArray * fqfns = g_ptr_array_new_with_free_func(free);
+   Null_Terminated_String_Array dirs = strsplit(path, ":");
+   char * dirname;
+   for (int ndx = 0; (dirname=dirs[ndx]); ndx++) {
+      char * appdir = (dirname[strlen(dirname)-1] == '/')
+              ? g_strdup_printf("%s%s",  dirname, application)
+              : g_strdup_printf("%s/%s", dirname, application);
+      DIR * d = opendir(appdir);
+      if (d) {
+         struct dirent *directory_entry;
+         while ((directory_entry = readdir(d)) != NULL) {
+            // printf("%s\n", directory_entry->d_name);
+            if (directory_entry->d_type == DT_REG) {
+               char * fq_name = g_strdup_printf("%s/%s", appdir, directory_entry->d_name);
+               g_ptr_array_add(fqfns, fq_name);
+            }
+         }
+         closedir(d);
+      }
+      free(appdir);
+   }
+   ntsa_free(dirs, true);
+   return fqfns;
+}
+
+
+static void query_xdg_files(int depth) {
+   int d1 = depth+1;
+   int d2 = depth+2;
+   int d3 = depth+3;
+
+   rpt_label(depth, "*** XDG Directory Settings ***");
+
+   rpt_label(d1, "XDG Base Directory Environment Variables:");
+   rpt_vstring(d2, "$%-15s: %s", "XDG_DATA_HOME",   getenv("XDG_DATA_HOME"));
+   rpt_vstring(d2, "$%-15s: %s", "XDG_CONFIG_HOME", getenv("XDG_CONFIG_HOME"));
+   rpt_vstring(d2, "$%-15s: %s", "XDG_STATE_HOME",  getenv("XDG_STATE_HOME"));
+   rpt_vstring(d2, "$%-15s: %s", "XDG_CACHE_HOME",  getenv("XDG_CACHE_HOME"));
+   rpt_vstring(d2, "$%-15s: %s", "XDG_DATA_DIRS",   getenv("XDG_DATA_DIRS"));
+   rpt_vstring(d2, "$%-15s: %s", "XDG_CONFIG_DIRS", getenv("XDG_CONFIG_DIRS"));
+   rpt_nl();
+
+   rpt_label(depth, "XDG Utility Functions:");
+   char * s = NULL;
+   s = xdg_data_home_dir();   rpt_vstring(d2, "xdg_data_home_dir():      %s", s);  free(s);
+   s = xdg_config_home_dir(); rpt_vstring(d2, "xdg_config_home_dir():    %s", s);  free(s);
+   s = xdg_cache_home_dir();  rpt_vstring(d2, "xdg_cache_home_dir():     %s", s);  free(s);
+   s = xdg_state_home_dir();  rpt_vstring(d2, "xdg_state_home_dir():     %s", s);  free(s);
+   s = xdg_data_path();       rpt_vstring(d2, "xdg_data_path():          %s", s);  free(s);
+   s = xdg_config_path();     rpt_vstring(d2, "xdg_config_path():        %s", s);  free(s);
+   rpt_nl();
+
+   rpt_label(depth, "*** ddcutil Configuration, Cache, and Data files ***");
+   char * config_fn = find_xdg_config_file("ddcutil", "ddcutilrc");
+   rpt_nl();
+   if (config_fn) {
+      rpt_vstring(d1, "Found configuration file: %s", config_fn);
+      rpt_file_contents(config_fn, /*verbose=*/ true, d2);
+      free(config_fn);
+   }
+   else
+      rpt_label(d1, "Configuration file ddcutilrc not found");
+   rpt_nl();
+
+   char * cache_fn = find_xdg_cache_file("ddcutil", "capabilities");
+   if (cache_fn) {
+      rpt_vstring(d1, "Found capabilities cache file: %s", cache_fn);
+      rpt_file_contents(cache_fn, /*verbose=*/ true, d2);
+      free(cache_fn);
+   }
+   else
+      rpt_label(d1, "Capabilities cache file not found");
+   rpt_nl();
+
+   rpt_label(d1, "Files on data path:");
+   char * data_path = xdg_data_path();
+   GPtrArray * data_files = get_path_application_files(data_path, "ddcutil");
+   for (int ndx = 0; ndx < data_files->len; ndx++) {
+      char * fn = g_ptr_array_index(data_files, ndx);
+      rpt_label(d2, fn);
+      if (str_ends_with(fn, ".mccs")) {
+         rpt_file_contents(fn, /*verbose=*/ true, d3);
+         rpt_nl();
+      }
+   }
+   free(data_path);
+   g_ptr_array_free(data_files, true);
+   rpt_nl();
 }
 
 
@@ -470,6 +600,7 @@ void final_analysis(Env_Accumulator * accum, int depth) {
    }
 
    // TODO: Also compare dev_i2c_devices vs sys_bus_i2c_devices ?
+   assert(accum->dev_i2c_device_numbers);    // already set
    if (bva_length(accum->dev_i2c_device_numbers) == 0 &&
        accum->module_i2c_dev_needed &&
        !accum->i2c_dev_loaded_or_builtin)
@@ -588,25 +719,47 @@ void final_analysis(Env_Accumulator * accum, int depth) {
  * Returns:      nothing
  */
 void query_sysenv() {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
+   rpt_label(0,
+       "The following tests probe the runtime environment using multiple overlapping methods.");
+
+   char * s = getenv("SYSENV_QUICK_TEST");
+   if (s && strlen(s) > 0) {
+      sysfs_quick_test = true;
+      rpt_label(0, "Environment variable SYSFS_QUICK_TEST is set.  Skipping some tests.");
+   }
+
+   i2c_force_slave_addr_flag = true;    // be a bully
+
+   ddc_ensure_displays_detected();
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "display detection complete");
    device_xref_init();
 
    Env_Accumulator * accumulator = env_accumulator_new();
    DDCA_Output_Level output_level = get_output_level();
-   if (output_level >= DDCA_OL_VERBOSE)
+   if (output_level >= DDCA_OL_VERBOSE) {
       sysenv_rpt_current_time(NULL, 1);
+      rpt_nl();
+   }
+
+   if (output_level >= DDCA_OL_VV) {
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "--VV only output: report_build_options()");
+      report_build_options(0);
+      rpt_nl();
+   }
 
-   rpt_nl();
    rpt_vstring(0,"*** Basic System Information ***");
    rpt_nl();
-   query_base_env(accumulator);
 
+   query_base_env(accumulator);
    rpt_nl();
+
    rpt_vstring(0,"*** Primary Check 1: Identify video card and driver ***");
    rpt_nl();
    query_card_and_driver_using_sysfs(accumulator);
-
-
    rpt_nl();
+
    rpt_vstring(0,"*** Primary Check 2: Check that /dev/i2c-* exist and writable ***");
    rpt_nl();
    accumulator->dev_i2c_device_numbers = identify_i2c_devices();
@@ -615,13 +768,13 @@ void query_sysenv() {
    // rpt_vstring(0, "Identified %d I2C devices", bva_length(accumulator->dev_i2c_device_numbers));
    // rpt_nl();
    check_i2c_devices(accumulator);
-
    rpt_nl();
+
    rpt_vstring(0,"*** Primary Check 3: Check that module i2c_dev is loaded ***");
    rpt_nl();
    check_i2c_dev_module(accumulator, 0);
-
    rpt_nl();
+
    rpt_vstring(0,"*** Primary Check 4: Driver specific checks ***");
    rpt_nl();
    driver_specific_tests(accumulator->driver_list);
@@ -640,29 +793,64 @@ void query_sysenv() {
 #endif
 
    rpt_nl();
+
    rpt_vstring(0,"*** Additional probes ***");
-   // printf("Gathering card and driver information...\n");
-   rpt_nl();
-   query_proc_modules_for_video();
-   if (!accumulator->is_arm) {
-      // rpt_nl();
-      // query_card_and_driver_using_lspci();
-      //rpt_nl();
-      //query_card_and_driver_using_lspci_alt();
-   }
-   rpt_nl();
-   query_loaded_modules_using_sysfs();
    rpt_nl();
    query_sys_bus_i2c(accumulator);
+   rpt_nl();
+
+   env_accumulator_report(accumulator, 0);
+   rpt_nl();
+
+   final_analysis(accumulator, 0);
 
    if (output_level >= DDCA_OL_VERBOSE) {
       rpt_nl();
-      query_proc_driver_nvidia();
-
       rpt_nl();
+      rpt_label(0, "*** Additional checks for remote diagnosis ***");
+      rpt_nl();
+
+      rpt_vstring(0, "*** Displays as reported by DETECT Command ***");
+      /* int display_ct =  */ ddc_report_displays(     // function used by DETECT command
+                                 true,   // include_invalid_displays
+                                 1);     // logical depth
+      // printf("Detected: %d displays\n", display_ct);   // not needed
+
+      query_loaded_modules_using_libkmod();
+      rpt_nl();
+
+      // printf("Gathering card and driver information...\n");
+      query_proc_modules_for_video();
+      if (!accumulator->is_arm) {
+         // rpt_nl();
+         // query_card_and_driver_using_lspci();
+         //rpt_nl();
+         //query_card_and_driver_using_lspci_alt();
+      }
+      rpt_nl();
+
+      if ( driver_name_list_find_exact(accumulator->driver_list, "nvidia")) {
+         query_proc_driver_nvidia();
+         rpt_nl();
+      }
+      if (driver_name_list_find_exact(accumulator->driver_list, "amdgpu")) {
+         rpt_vstring(0, "amdgpu configuration parameters:");
+         query_sys_amdgpu_parameters(1);
+         rpt_nl();
+      }
+
+      rpt_vstring(0, "Checking display manager environment variables...");
+      char * s = getenv("DISPLAY");
+      rpt_vstring(1, "DISPLAY=%s", (s) ? s : "(not set)");
+      s = getenv("WAYLAND_DISPLAY");
+      rpt_vstring(1, "WAYLAND_DISPLAY=%s", (s) ? s : "(not set)");
+      s = getenv("XDG_SESSION_TYPE");
+      rpt_vstring(1, "XDG_SESSION_TYPE=%s", (s) ? s : "(not set)");
+      rpt_nl();
+
       query_i2c_buses();
-
       rpt_nl();
+
       rpt_vstring(0,"xrandr connection report:");
       execute_shell_cmd_rpt("xrandr|grep connected", 1 /* depth */);
       rpt_nl();
@@ -670,36 +858,44 @@ void query_sysenv() {
       rpt_vstring(0,"Checking for possibly conflicting programs...");
       execute_shell_cmd_rpt("ps aux | grep ddccontrol | grep -v grep", 1);
       rpt_nl();
+      execute_shell_cmd_rpt("lsmod | grep ddcci | grep -v grep", 1);
+      rpt_nl();
 
-      query_using_i2cdetect(accumulator->dev_i2c_device_numbers);
+      if (sysfs_quick_test)
+         DBGMSG("!!! Skipping i2cdetect and get-edid|parse-edid to speed up testing !!!");
+      else {
+         query_using_shell_command(accumulator->dev_i2c_device_numbers,
+                                   "i2cdetect -y %d",   // command to issue
+                                   "i2cdetect");        // command name for error message
+         rpt_nl();
+         query_using_shell_command(accumulator->dev_i2c_device_numbers,
+                                   "get-edid -b %d -i | parse-edid",   // command to issue
+                                   "get-edid | parse-edid");        // command name for error message
 
+         if (get_output_level() >= DDCA_OL_VV) {
+            DBGTRC_NOPREFIX(debug, TRACE_GROUP, "--VV only output: test_read_variants()");
+            test_edid_read_variants(accumulator);
+         }
+      }
       raw_scan_i2c_devices(accumulator);
 
 #ifdef USE_X11
       query_x11();
 #endif
 
+#ifdef ENABLE_UDEV
       probe_i2c_devices_using_udev();
+#endif
 
       // temp
       // get_i2c_smbus_devices_using_udev();
 
-      Driver_Name_Node * driver_list = accumulator->driver_list;
-      rpt_nl();
-      rpt_vstring(0, "amdgpu configuration parameters:");
-      if (driver_name_list_find_exact(driver_list, "amdgpu")) {
-         query_sys_amdgpu_parameters(1);
+      if (sysfs_quick_test)
+         DBGMSG("!!! Skipping config file and log checking to speed up testing !!!");
+      else {
+         probe_config_files(accumulator);
+         probe_logs(accumulator);
       }
-      // else
-      //    rpt_vstring(0, "!!! not amdgpu");
-      rpt_nl();
-
-#ifdef SYSENV_QUICK_TEST_RUN
-      DBGMSG("!!! Skipping config file and log checking to speed up testing !!!");
-#else
-      probe_config_files(accumulator);
-      probe_logs(accumulator);
-#endif
 
 #ifdef USE_LIBDRM
       probe_using_libdrm();
@@ -708,18 +904,56 @@ void query_sysenv() {
 #endif
 
       query_drm_using_sysfs();
+      rpt_nl();
+
+      rpt_title("Query file system for i2c nodes under /sys/class/drm/card*...", 0);
+      execute_shell_cmd_rpt("ls -ld /sys/class/drm/card*/card*/i2c*", 1);
+      rpt_title("Query file system for i2c nodes under /sys/class/drm/card*/ddc/i2c-dev/...", 0);
+      execute_shell_cmd_rpt("ls -ld /sys/class/drm/card*/card*/ddc/i2c-dev/i2c*", 1);
 
       device_xref_report(0);
 
       probe_modules_d(0);
+
+      dump_sysfs_i2c();
+      rpt_nl();
+
+#ifdef OLD
+      if (get_output_level() >= DDCA_OL_VV) {
+         rpt_nl();
+         rpt_label(0, "*** Calling get_sysfs_drm_card_numbers(), get_sysfs_drm_displays() from ddc_watch.c... ***");
+         Byte_Bit_Flags drm_card_numbers = get_sysfs_drm_card_numbers();
+         if (bbf_count_set(drm_card_numbers) > 0) {
+            query_drm_using_sysfs();
+         }
+      }
+#endif
+
+#ifdef TMI
+#ifdef ENABLE_UDEV
+      if (get_output_level() >= DDCA_OL_VV) {
+         rpt_nl();
+         query_using_shell_command(accumulator->dev_i2c_device_numbers,
+        //   "udevadm info --attribute-walk --path=$(udevadm info --query=path --name=i2c-%d)",
+           "udevadm info --attribute-walk /dev/i2c-%d",
+                                   "udevadm");
+      }
+#endif
+#endif
+
+      query_xdg_files(0);
    }
 
-   rpt_nl();
-   env_accumulator_report(accumulator, 0);
-
-   rpt_nl();
-   final_analysis(accumulator, 0);
-
    env_accumulator_free(accumulator);     // make Coverity happy
+   DBGTRC_DONE(debug, TRACE_GROUP, "");
+}
+
+
+void init_sysenv() {
+   RTTI_ADD_FUNC(query_sysenv);
+#ifdef ENABLE_UDEV
+   RTTI_ADD_FUNC(probe_i2c_devices_using_udev);
+#endif
+   init_query_sysfs();
 }
 
