@@ -3,7 +3,7 @@
  *  drm reporting for the environment command
  */
 
-// Copyright (C) 2017-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2017-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -12,7 +12,7 @@
 /** \cond */
 #include <assert.h>
 #include <errno.h>
-#include <glib.h>
+#include <glib-2.0/glib.h>
 #include <fcntl.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -145,7 +145,7 @@ static void probe_open_device_using_libdrm(int fd, int depth) {
 
    rpt_nl();
 
-   DBGTRC(debug, TRACE_GROUP, "Starting. fd=%d", fd);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "fd=%d", fd);
 
    // succeeds if run as root, fails w errno=EACCES(13) if not
    // but no effect on subsequent failures for nvidia
@@ -496,7 +496,7 @@ static void probe_open_device_using_libdrm(int fd, int depth) {
    }
 
 bye:
-   DBGTRC0(debug, TRACE_GROUP, "Done");
+   DBGTRC_DONE(debug, TRACE_GROUP, "");
    rpt_nl();
    return;
 }
@@ -670,22 +670,25 @@ GPtrArray * get_dri_device_names_using_filesys() {
  * the functions, set errno=22 (EINVAL).
  */
 void probe_using_libdrm() {
-   rpt_title("Probing connected monitors using libdrm...",0);
+   int d0 = 0;
+   int d1 = d0+1;
+   int d2 = d0+2;
+   rpt_label(d0, "*** Probing connected monitors using libdrm ***");
 
    if ( directory_exists("/proc/driver/nvidia/") ) {
       rpt_nl();
       rpt_vstring(1,"Checking Nvidia options to see if experimental kernel modesetting enabled:");
       char * cmd = "modprobe -c | grep \"^options nvidia\"";
-      rpt_vstring(1, "Executing command: %s", cmd);
-      execute_shell_cmd_rpt(cmd, 2 /* depth */);
+      rpt_vstring(d1, "Executing command: %s", cmd);
+      execute_shell_cmd_rpt(cmd, d2);
    }
 
    // Check libdrm version, since there seems to be some sensitivity
    rpt_nl();
    if (is_command_in_path("pkg-config")) {
-      rpt_vstring(1, "Checking libdrm version using pkg-config...");
+      rpt_vstring(d1, "Checking libdrm version using pkg-config...");
       char * cmd = "pkg-config --modversion libdrm";
-      execute_shell_cmd_rpt(cmd, 2);
+      execute_shell_cmd_rpt(cmd, d2);
    }
 
    else {
@@ -693,15 +696,15 @@ void probe_using_libdrm() {
 
       if (is_command_in_path("dpkg-query")) {
          char * cmd = "dpkg-query -l libdrm2 | grep ii";
-         rpt_vstring(1, "Checking libdrm version using dpkg-query...");
-         execute_shell_cmd_rpt(cmd, 2);
+         rpt_vstring(d1, "Checking libdrm version using dpkg-query...");
+         execute_shell_cmd_rpt(cmd, d2);
       }
 
       rpt_nl();
       if (is_command_in_path("rpm")) {
          char * cmd = "rpm -qa | grep libdrm";
-         rpt_vstring(1, "Checking libdrm version using rpm...");
-         execute_shell_cmd_rpt(cmd, 2);
+         rpt_vstring(d1, "Checking libdrm version using rpm...");
+         execute_shell_cmd_rpt(cmd, d2);
       }
    }
 
@@ -713,8 +716,8 @@ void probe_using_libdrm() {
    rpt_nl();
    // returns 1 if the DRM driver is loaded, 0 otherwise
    int drm_available = drmAvailable();
-   rpt_vstring(1, "Has a DRM kernel driver been loaded? (drmAvailable()): %s",
-                  sbool(drm_available));
+   rpt_vstring(d1, "Has a DRM kernel driver been loaded? (drmAvailable()): %s",
+                   sbool(drm_available));
 
 #ifdef DOESNT_WORK
    // function drmOpenMinor() is static
