@@ -1,8 +1,8 @@
 /** @file base_init.c
- * Master base services initialization
+ *  Initialize and release base services.
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "util/error_info.h"
@@ -21,7 +21,9 @@
 /** Master initialization function for files in subdirectory base
  */
 void init_base_services() {
-   init_msg_control();
+   bool debug = false;
+   if (debug)
+      printf("(%s) Starting.\n", __func__);
    errinfo_init(psc_name, psc_desc);
    init_sleep_stats();
    init_execution_stats();
@@ -29,4 +31,11 @@ void init_base_services() {
    // init_linux_errno();
    init_thread_data_module();
    init_displays();
+   init_ddc_packets();
+   if (debug)
+      printf("(%s) Done\n", __func__);
+}
+
+void release_base_services() {
+   release_thread_data_module();
 }
