@@ -1,13 +1,14 @@
 /** \file base_init.h"
- *  Master base services initialization
+ *  Initialize and release base services.
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef BASE_SERVICES_H_
 #define BASE_SERVICES_H_
 
 void init_base_services();
+void release_base_services();
 
 #endif /* BASE_SERVICES_H_ */
