@@ -13,18 +13,24 @@
  * - debug and trace messages
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef BASE_CORE_H_
 #define BASE_CORE_H_
 
+#include "config.h"
+
 /** \cond */
+#ifdef TARGET_BSD
+#else
 #include <linux/limits.h>
+#endif
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <syslog.h>
 /** \endcond */
 
 #include "public/ddcutil_types.h"
@@ -32,19 +38,7 @@
 #include "util/coredefs.h"
 #include "util/error_info.h"
 
-
-// temp location
-//! I2C retry limit types
-typedef enum{
-   WRITE_ONLY_TRIES_OP,       /**< Maximum write-only operation tries */
-   WRITE_READ_TRIES_OP,       /**< Maximum read-write operation tries */
-   MULTI_PART_READ_OP,        /**< Maximum multi-part read operation tries */
-   MULTI_PART_WRITE_OP        /**< Maximum multi-part write operation tries */
-} Retry_Operation;
-#define RETRY_OP_COUNT 4
-typedef uint16_t Retry_Op_Value;
-
-
+#include "base/core_per_thread_settings.h"
 
 //
 // Common macros
@@ -53,15 +47,6 @@ typedef uint16_t Retry_Op_Value;
 #define ASSERT_MARKER(_struct_ptr, _marker_value) \
    assert(_struct_ptr && memcmp(_struct_ptr->marker, _marker_value, 4) == 0)
 
-#define ASSERT_IFF(_cond1, _cond2) \
-   assert( ( (_cond1) && (_cond2) ) || ( !(_cond1) && !(_cond2) ) )
-
-
-//
-// Initialization
-//
-void init_msg_control();
-
 
 //
 // Standard function call arguments and return values
@@ -76,6 +61,7 @@ typedef Byte Call_Options;
 #define CALLOPT_WARN_FINDEX  0x10    ///< issue warning msg re hiddev_field_info.field_index change
 #define CALLOPT_FORCE        0x08    ///< ignore various validity checks
 #define CALLOPT_WAIT         0x04    ///< wait on locked resources, if false then fail
+#define CALLOPT_FORCE_SLAVE_ADDR 0x02 ///< use op I2C_SLAVE_FORCE (not currently used)
 
 char * interpret_call_options_t(Call_Options calloptions);
 
@@ -92,38 +78,15 @@ typedef struct {
 #endif
 
 
-//
-// Global redirection for messages that normally go to stdout and stderr,
-// used within functions that are part of the shared library.
-//
-// extern FILE * FOUT;
-// extern FILE * FERR;
-
-void set_fout(FILE * fout);
-void set_ferr(FILE * ferr);
-void set_fout_to_default();
-void set_ferr_to_default();
-
-FILE * fout();
-FILE * ferr();
-
-
-//
-// Message level control
-//
-
-DDCA_Output_Level get_output_level();
-DDCA_Output_Level set_output_level(DDCA_Output_Level newval);
-char *            output_level_name(DDCA_Output_Level val);
-
-
 //
 // Trace message control
 //
 
 extern bool dbgtrc_show_time;       // prefix debug/trace messages with elapsed time
+extern bool dbgtrc_show_wall_time;  // prefix debug/trace messages with wall time
 extern bool dbgtrc_show_thread_id;  // prefix debug/trace messages with thread id
 
+void set_libddcutil_output_destination(const char * filename, const char * traced_unit);
 void add_traced_function(const char * funcname);
 bool is_traced_function( const char * funcname);
 void show_traced_functions();
@@ -133,7 +96,8 @@ bool is_traced_file( const char * filename);
 void show_traced_files();
 
 DDCA_Trace_Group trace_class_name_to_value(char * name);
-void set_trace_levels(DDCA_Trace_Group trace_flags);
+void set_trace_groups(DDCA_Trace_Group trace_flags);
+void add_trace_groups(DDCA_Trace_Group trace_flags);
 // char * get_active_trace_group_names();  // unimplemented
 void show_trace_groups();
 
@@ -151,6 +115,13 @@ bool is_tracing(DDCA_Trace_Group trace_group, const char * filename, const char
 
 #define IS_TRACING_BY_FUNC_OR_FILE() is_tracing(DDCA_TRC_NONE, __FILE__, __func__)
 
+#define IS_DBGTRC(debug_flag, group) \
+    ( (debug_flag)  || is_tracing((group), __FILE__, __func__) )
+
+typedef uint16_t Dbgtrc_Options;
+#define DBGTRC_OPTIONS_NONE   0
+#define DBGTRC_OPTIONS_SYSLOG 1
+
 
 //
 //  Error_Info reporting
@@ -174,8 +145,11 @@ bool is_report_ddc_errors_enabled();
 bool is_reporting_ddc(DDCA_Trace_Group trace_group, const char * filename, const char * funcname);
 #define IS_REPORTING_DDC() is_reporting_ddc(TRACE_GROUP, __FILE__, __func__)
 
-bool ddcmsg(DDCA_Trace_Group trace_group, const char* funcname, const int lineno, const char* fn, char* format, ...);
-#define DDCMSG0(format, ...) ddcmsg(TRACE_GROUP, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
+bool ddcmsg(
+      DDCA_Trace_Group trace_group,
+      const char* funcname, const int lineno, const char* fn,
+      char* format, ...);
+// #define DDCMSG0(format, ...) ddcmsg(TRACE_GROUP, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
 /** Variant of **DDCMSG** that takes an explicit trace group as an argument.
  *
@@ -202,6 +176,9 @@ bool ddcmsg(DDCA_Trace_Group trace_group, const char* funcname, const int lineno
 // Show report levels for all types
 void show_reporting();
 
+// report ddcutil version
+void show_ddcutil_version();
+
 
 //
 // Issue messages of various types
@@ -214,63 +191,168 @@ void severemsg(
         char *       format,
         ...);
 
+extern bool trace_to_syslog;
+
 bool dbgtrc(
         DDCA_Trace_Group  trace_group,
+        Dbgtrc_Options    options,
         const char * funcname,
         const int    lineno,
         const char * fn,
         char *       format,
         ...);
 
+bool dbgtrc_returning(
+        DDCA_Trace_Group  trace_group,
+        Dbgtrc_Options    options,
+        const char * funcname,
+        const int    lineno,
+        const char * fn,
+        int          rc,
+        char *       format,
+        ...);
+
+bool dbgtrc_returning_errinfo(
+        DDCA_Trace_Group  trace_group,
+        Dbgtrc_Options    options,
+        const char * funcname,
+        const int    lineno,
+        const char * fn,
+        Error_Info * errs,
+        char *       format,
+        ...);
+
+bool dbgtrc_returning_expression(
+        DDCA_Trace_Group  trace_group,
+        Dbgtrc_Options    options,
+        const char * funcname,
+        const int    lineno,
+        const char * fn,
+        const char * retval_expression,
+        char *       format,
+        ...);
+
+
+/* __assert_fail() is not part of the C spec, it is part of the Linux
+ * implementation of assert(), etc., which are macros.
+ * It is reported to not exist on Termux.
+ * However, if  "assert(#_assertion);" is used instead of "__assert_fail(...);",
+ * the program does not terminate.
+ */
+// n. using ___LINE__ instead of line in __assert_fail() causes compilation error
+#ifdef NDEBUG
+#define TRACED_ASSERT(_assertion) \
+   do { \
+   } while (0)
+#else
+#define TRACED_ASSERT(_assertion) \
+   do { \
+      if (_assertion) { \
+         ;              \
+      }                 \
+      else {           \
+         /* int line = __LINE__; */  \
+         dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__,   \
+                      "Assertion failed: \"%s\" in file %s at line %d",  \
+                      #_assertion, __FILE__,  __LINE__);   \
+         syslog(LOG_ERR, "Assertion failed: \"%s\" in file %s at line %d",  \
+                         #_assertion, __FILE__,  __LINE__);   \
+         /* assert(#_assertion); */ \
+         /*  __assert_fail(#_assertion, __FILE__, line, __func__); */  \
+         /* don't need assertion info, dbgtrc() and dbgtrc() have been called */ \
+         exit(1); \
+      } \
+   } while (0)
+#endif
+
+
+#ifndef TRACED_ASSERT_IFF
+#define TRACED_ASSERT_IFF(_cond1, _cond2) \
+   TRACED_ASSERT( ( (_cond1) && (_cond2) ) || ( !(_cond1) && !(_cond2) ) )
+#endif
+
+#ifdef UNUSED
+#define TRACED_ABORT(_assertion) \
+   do { \
+      int line = __LINE__;  \
+      dbgtrc(true, __func__, __LINE__, __FILE__,   \
+                   "Assertion failed: \"%s\" in file %s at line %d",  \
+                   #_assertion, __FILE__,  __LINE__);   \
+      syslog(LOG_ERR, "Assertion failed: \"%s\" in file %s at line %d",  \
+                      #_assertion, __FILE__,  __LINE__);   \
+      __assert_fail(#_assertion, __FILE__, line, __func__); \
+   } while (0)
+#endif
+
 
 #define SEVEREMSG(          format, ...) \
    severemsg(          __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
-
 // cannot map to dbgtrc, writes to stderr, not stdout
-// #define SEVEREMSG(format, ...) dbgtrc(0xff,       __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
+// #define SEVEREMSG(format, ...) dbgtrc(DDCA_TRC_ALL,       DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
-#define DBGMSG(            format, ...) dbgtrc(0xff, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
-// workaround: if -Wpedantic, ISO C does not allow variadic macro calls that have no variable arguments
-#define DBGMSG0(           text) dbgtrc(0xff, __func__, __LINE__, __FILE__, text)
+#define DBGMSG(            format, ...) dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
 #define DBGMSF(debug_flag, format, ...) \
-   do { if (debug_flag) dbgtrc( 0xff, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__); }  while(0)
-#define DBGMSF0(debug_flag, text) \
-   do { if (debug_flag) dbgtrc( 0xff, __func__, __LINE__, __FILE__, text); }  while(0)
-
-
-#define TRCMSG(            format, ...) \
-   dbgtrc(TRACE_GROUP, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
-
-// which of these are really useful?
-// not currently used: TRCALWAYS, TRCMSGTG, TRCMSGTF
-#define TRCALWAYS(            format, ...) \
-   dbgtrc(0xff,        __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
-#define TRCMSGTG(trace_group, format, ...) \
-   dbgtrc(trace_group, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
-#define TRCMSGTF(trace_flag, format, ...) \
-    do { if (trace_flag) dbgtrc(0xff, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__); }  while(0)
-// alt: dbgtrc( ( (trace_flag) ? (0xff) : TRACE_GROUP ), __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
+   do { if (debug_flag) dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__); }  while(0)
 
 // For messages that are issued either if tracing is enabled for the appropriate trace group or
 // if a debug flag is set.
 #define DBGTRC(debug_flag, trace_group, format, ...) \
-    dbgtrc( ( (debug_flag) ) ? 0xff : (trace_group), __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
+    dbgtrc( ( (debug_flag) ) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
-#define DBGTRC0(debug_flag, trace_group, format) \
-    dbgtrc( ( (debug_flag) ) ? 0xff : (trace_group), __func__, __LINE__, __FILE__, format)
+#define DBGTRC_SYSLOG(debug_flag, trace_group, format, ...) \
+    dbgtrc( ( (debug_flag) ) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_SYSLOG, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
+
+#define DBGTRC_STARTING(debug_flag, trace_group, format, ...) \
+    dbgtrc( ( (debug_flag) ) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__, "Starting  "format, ##__VA_ARGS__)
+
+#define DBGTRC_DONE(debug_flag, trace_group, format, ...) \
+    dbgtrc( ( (debug_flag) ) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__, "Done      "format, ##__VA_ARGS__)
+
+#define DBGTRC_NOPREFIX(debug_flag, trace_group, format, ...) \
+    dbgtrc( ( (debug_flag) ) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__, "          "format, ##__VA_ARGS__)
+
+#define DBGTRC_RETURNING(debug_flag, trace_group, rc, format, ...) \
+    dbgtrc_returning( \
+          ( (debug_flag) ) ? DDCA_TRC_ALL : (trace_group), \
+          DBGTRC_OPTIONS_NONE, \
+          __func__, __LINE__, __FILE__, \
+          rc, format, ##__VA_ARGS__)
+
+#define DBGTRC_RET_BOOL(debug_flag, trace_group, bool_result, format, ...) \
+    dbgtrc_returning_expression( \
+          ( (debug_flag) ) ? DDCA_TRC_ALL : (trace_group), \
+          DBGTRC_OPTIONS_NONE, \
+          __func__, __LINE__, __FILE__, SBOOL(bool_result), format, ##__VA_ARGS__)
+
+#define DBGTRC_RET_ERRINFO(debug_flag, trace_group, errinfo_result, format, ...) \
+    dbgtrc_returning_errinfo( \
+          ( (debug_flag) ) ? DDCA_TRC_ALL : (trace_group), \
+          DBGTRC_OPTIONS_NONE, \
+          __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__)
 
 
 // typedef (*dbg_struct_func)(void * structptr, int depth);
 #define DBG_RET_STRUCT(_flag, _structname, _dbgfunc, _structptr) \
 if (_flag) { \
-   dbgtrc(0xff, __func__, __LINE__, __FILE__, "Returning %s at %p", #_structname, _structptr); \
+   dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, \
+         __func__, __LINE__, __FILE__, \
+         "Returning %s at %p", #_structname, _structptr); \
    if (_structptr) { \
       _dbgfunc(_structptr, 1); \
    } \
 }
 
+#define DBGTRC_RET_STRUCT(_flag, _trace_group, _structname, _dbgfunc, _structptr) \
+if ( (_flag) || (is_tracing(_trace_group, __FILE__, __func__)) ) { \
+   dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, \
+          __func__, __LINE__, __FILE__, \
+          "Returning %s at %p", #_structname, _structptr); \
+   if (_structptr) { \
+      _dbgfunc(_structptr, 1); \
+   } \
+}
 
 //
 // Error handling
@@ -303,13 +385,7 @@ void program_logic_error(
 #define PROGRAM_LOGIC_ERROR(format, ...) \
    program_logic_error(__func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
-DDCA_Error_Detail * error_info_to_ddca_detail(Error_Info * erec);
-DDCA_Error_Detail * dup_error_detail(DDCA_Error_Detail * old);
-void free_error_detail(DDCA_Error_Detail * ddca_erec);
-void report_error_detail(DDCA_Error_Detail * ddca_erec, int depth);
-
-void free_thread_error_detail();
-DDCA_Error_Detail * get_thread_error_detail();
-void save_thread_error_detail(DDCA_Error_Detail * error_detail);
+void set_default_thread_output_settings(FILE * fout, FILE * ferr);
+void init_core();
 
 #endif /* BASE_CORE_H_ */
