@@ -2,7 +2,7 @@
  * Functions for creating DDC packets and interpreting DDC response packets.
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -18,8 +18,10 @@
 #include "util/utilrpt.h"
 /** \endcond */
 
+#include "base/core.h"
 #include "base/ddc_errno.h"
 #include "base/execution_stats.h"
+#include "base/rtti.h"
 
 #include "base/ddc_packets.h"
 
@@ -235,7 +237,8 @@ create_empty_ddc_packet(int max_size, const char * tag) {
    DDC_Packet * packet = malloc(sizeof(DDC_Packet));
    packet->raw_bytes = buffer_new(max_size, "empty DDC packet");
    if (tag) {
-      g_strlcpy(packet->tag, tag, MAX_DDC_TAG);
+      strncpy(packet->tag, tag, sizeof(packet->tag));  // no need to check if packet->tag truncated
+      packet->tag[sizeof(packet->tag)-1] = '\0';
    }
    else
       packet->tag[0] = '\0';
@@ -245,7 +248,7 @@ create_empty_ddc_packet(int max_size, const char * tag) {
 
    DBGMSF(debug, "Done. Returning %p, packet->tag=%p", packet, packet->tag);
    if (debug)
-      dbgrpt_packet(packet, 1);
+      dbgrpt_packet(packet, 2);
 
    return packet;
 }
@@ -506,8 +509,10 @@ create_ddc_base_response_packet(
    DDC_Packet ** packet_ptr_addr)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP,
-          "Starting. i2c_response_bytes=%s", hexstring_t(i2c_response_bytes, 20) );
+   DBGTRC_STARTING(debug, TRACE_GROUP, "i2c_response_bytes=%p, response_bytes_buffer_size=%d",
+                              i2c_response_bytes, response_bytes_buffer_size);
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "i2c_response_bytes -> |%s|",
+                              hexstring_t(i2c_response_bytes, response_bytes_buffer_size) );
 
    int result = DDCRC_OK;
    DDC_Packet * packet = NULL;
@@ -559,8 +564,7 @@ create_ddc_base_response_packet(
    else
       *packet_ptr_addr = NULL;
 
-   DBGTRC(debug, TRACE_GROUP,
-          "Returning %s, *packet_ptr_addr=%p", ddcrc_desc_t(result), *packet_ptr_addr);
+   DBGTRC_RETURNING(debug, TRACE_GROUP, result, "*packet_ptr_addr=%p", *packet_ptr_addr);
    assert( (result==DDCRC_OK && *packet_ptr_addr) || (result != DDCRC_OK && !*packet_ptr_addr));
    return result;
 }
@@ -591,15 +595,16 @@ create_ddc_response_packet(
        DDC_Packet **   packet_ptr_addr)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP,
-          "Starting. i2c_response_bytes=%s", hexstring_t(i2c_response_bytes, 20));
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+          "response_bytes_buffer_size=%d, i2c_response_bytes=|%s|",
+          response_bytes_buffer_size, hexstring_t(i2c_response_bytes, response_bytes_buffer_size));
 
    Status_DDC result = create_ddc_base_response_packet(
                           i2c_response_bytes,
                           response_bytes_buffer_size,
                           tag,
                           packet_ptr_addr);
-   // DBGMSG("create_ddc_base_response_packet() returned %d, *packet_ptr_addr=%p", result, *packet_ptr_addr);
+   DBGMSF(debug, "create_ddc_base_response_packet() returned %d, *packet_ptr_addr=%p", result, *packet_ptr_addr);
    if (result == 0) {
       if (isNullPacket(*packet_ptr_addr)) {
          result = DDCRC_NULL_RESPONSE;
@@ -612,7 +617,7 @@ create_ddc_response_packet(
    if (result != DDCRC_OK && *packet_ptr_addr) {
       // if (debug)
       //    DBGMSG("failure, freeing response packet at %p", *packet_ptr_addr);
-      TRCMSG("failure, freeing response packet at %p", *packet_ptr_addr);
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "failure, freeing response packet at %p", *packet_ptr_addr);
       // does this cause the free(readbuf) failure in try_read?
       free_ddc_packet(*packet_ptr_addr);
       *packet_ptr_addr = 0;
@@ -622,8 +627,7 @@ create_ddc_response_packet(
       log_status_code(result, __func__);
    }
 
-   DBGTRC(debug, TRACE_GROUP,
-          "Returning %s, *packet_ptr_addr=%p", ddcrc_desc_t(result), *packet_ptr_addr);
+   DBGTRC_RETURNING(debug, TRACE_GROUP, result, "*packet_ptr_addr=%p", *packet_ptr_addr);
    assert( (result==DDCRC_OK && *packet_ptr_addr) || (result != DDCRC_OK && !*packet_ptr_addr));
    return result;
 }
@@ -749,8 +753,8 @@ interpret_vcp_feature_response_std(
        Parsed_Nontable_Vcp_Response* aux_data)   // record in which interpreted feature response will be stored
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP,
-          "Starting. requested_vcp_code: 0x%02x, vcp_data_bytes: %s",
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+          "requested_vcp_code: 0x%02x, vcp_data_bytes: %s",
           requested_vcp_code, hexstring3_t(vcp_data_bytes, bytect, " ", 4, false));
 
    int result = DDCRC_OK;
@@ -785,10 +789,11 @@ interpret_vcp_feature_response_std(
          if (vcpresp->result_code == 0x01) {
             // Do not report as DDC error if VCP code is 0x00, since that value is used
             // for probing.
-            bool msg_emitted = DBGTRC(debug, TRACE_GROUP,
+            // bool msg_emitted =
+                  DBGTRC_NOPREFIX(debug, TRACE_GROUP,
                                       "Unsupported VCP Code: 0x%02x", vcpresp->vcp_opcode);
-            if (requested_vcp_code != 0x00 && !msg_emitted)
-               DDCMSG(debug, "Unsupported VCP Code: 0x%02x", vcpresp->vcp_opcode);
+            // if (requested_vcp_code != 0x00 && !msg_emitted)
+            //    DDCMSG(debug, "Unsupported VCP Code: 0x%02x", vcpresp->vcp_opcode);
             aux_data->valid_response = true;
          }
          else {
@@ -803,10 +808,10 @@ interpret_vcp_feature_response_std(
          int max_val = (vcpresp->mh << 8) | vcpresp->ml;
          int cur_val = (vcpresp->sh << 8) | vcpresp->sl;
 
-         DBGTRC(debug, TRACE_GROUP,
+         DBGTRC_NOPREFIX(debug, TRACE_GROUP,
                 "vcp_opcode = 0x%02x, vcp_type_code=0x%02x, max_val=%d (0x%04x), cur_val=%d (0x%04x)",
                 vcpresp->vcp_opcode, vcpresp->vcp_typecode, max_val, max_val, cur_val, cur_val);
-         DBGTRC(debug, TRACE_GROUP, "valid_response=%s", sbool(valid_response));
+         DBGTRC_NOPREFIX(debug, TRACE_GROUP, "valid_response=%s", sbool(valid_response));
 
          aux_data->valid_response   = true;
          aux_data->supported_opcode = true;
@@ -821,7 +826,7 @@ interpret_vcp_feature_response_std(
    }
    
    // result = DDCRC_DDC_DATA; // force error for testing
-   DBGTRC(debug, TRACE_GROUP, "Returning %s", psc_desc(result));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, result, "");
    return result;
 }
 
@@ -890,7 +895,8 @@ create_ddc_typed_response_packet(
       DDC_Packet**    packet_ptr_addr)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. i2c_response_bytes=%s", hexstring_t(i2c_response_bytes, 20) );
+   DBGTRC_STARTING(debug, TRACE_GROUP, "response_bytes_buffer_size=%d, response_bytes=|%s|",
+                              response_bytes_buffer_size, hexstring_t(i2c_response_bytes, response_bytes_buffer_size) );
 
    // DBGMSG("before create_ddc_response_packet(), *packet_ptr_addr=%p", *packet_ptr_addr);
    // n. may return DDC_NULL_RESPONSE??   (old note)
@@ -900,7 +906,7 @@ create_ddc_typed_response_packet(
                expected_type,
                tag,
                packet_ptr_addr);
-   DBGTRC(debug, TRACE_GROUP, "Create_ddc_response_packet() returned %s, *packet_ptr_addr=%p",
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Create_ddc_response_packet() returned %s, *packet_ptr_addr=%p",
                                psc_desc(rc), *packet_ptr_addr);
    if (rc == 0) {
       DDC_Packet * packet = *packet_ptr_addr;
@@ -943,15 +949,16 @@ create_ddc_typed_response_packet(
       *packet_ptr_addr = NULL;
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Returning %s, *packet_ptr=%p", ddcrc_desc_t(rc), *packet_ptr_addr);
+   DBGTRC_RETURNING(debug, TRACE_GROUP, rc, "*packet_ptr=%p", *packet_ptr_addr);
    if ( (debug || IS_TRACING()) && rc >= 0)
-      dbgrpt_packet(*packet_ptr_addr, 1);
+      dbgrpt_packet(*packet_ptr_addr, 2);
 
    assert( (rc == 0 && *packet_ptr_addr) || (rc != 0 && !*packet_ptr_addr));
    return rc;
 }
 
 
+#ifdef UNUSED
 Status_DDC
 create_ddc_multi_part_read_response_packet(
       Byte           response_type,
@@ -961,6 +968,7 @@ create_ddc_multi_part_read_response_packet(
       DDC_Packet **  packet_ptr)
 {
    bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "response_type=0x%02x, tag=%s, packet_ptr=%p");
 
    DDC_Packet * packet = NULL;
    Status_DDC rc = create_ddc_response_packet(i2c_response_bytes,
@@ -970,7 +978,8 @@ create_ddc_multi_part_read_response_packet(
                                               &packet);
    if (rc != 0) {
       // DBGMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_description(rc), packet);
-      TRCMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_desc_t(rc), packet);
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "create_ddc_response_packet() returned %s, packet=%p",
+                                 psc_name_code(rc), packet);
    }
    if (rc == 0) {
       // dump_packet(packet);
@@ -980,7 +989,7 @@ create_ddc_multi_part_read_response_packet(
       if (data_len < min_data_len || data_len > max_data_len) {
          DDCMSG(debug, "Invalid data fragment_length_wo_null: %d", data_len);
          if (IS_REPORTING_DDC())
-            dbgrpt_packet(packet, 1);
+            dbgrpt_packet(packet, 2);
          rc = COUNT_STATUS_CODE(DDCRC_DDC_DATA);    // was DDCRC_INVALID_DATA
       }
       else {
@@ -999,9 +1008,10 @@ create_ddc_multi_part_read_response_packet(
    }
    if (rc == 0)
       *packet_ptr = packet;
+   DBGTRC_RETURNING(debug, TRACE_GROUP, rc, "*packet_ptr=%p", *packet_ptr);
    return rc;
 }
-
+#endif
 
 // VCP Feature response
 
@@ -1015,6 +1025,8 @@ create_ddc_getvcp_response_packet(
        DDC_Packet **  packet_ptr)
 {
    bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "expected_vcp_opcode=0x%02x, packet_ptr=%p",
+                              expected_vcp_opcode, packet_ptr);
 
    DDC_Packet * packet = NULL;
    Status_DDC rc = create_ddc_response_packet(
@@ -1025,7 +1037,8 @@ create_ddc_getvcp_response_packet(
                &packet);
    if (rc != 0) {
       // DBGMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_description(rc), packet);
-      TRCMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_desc_t(rc), packet);
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "create_ddc_response_packet() returned %s, packet=%p",
+                                 ddcrc_desc_t(rc), packet);
    }
    if (rc == 0) {
       // dump_packet(packet);
@@ -1035,7 +1048,7 @@ create_ddc_getvcp_response_packet(
          // dump_packet(packet);
          DDCMSG(debug, "Invalid data length: %d, should be 8", data_len);
          if ( IS_REPORTING_DDC() )
-            dbgrpt_packet(packet, 1);
+            dbgrpt_packet(packet, 2);
          rc = COUNT_STATUS_CODE(DDCRC_DDC_DATA);     // was DDCRC_INVALID_DATA
       }
       else {
@@ -1054,6 +1067,7 @@ create_ddc_getvcp_response_packet(
    }
    if (rc == 0)
       *packet_ptr = packet;
+   DBGTRC_RETURNING(debug, TRACE_GROUP, rc, "*packet_ptr=%p", *packet_ptr);
    return rc;
 }
 
@@ -1118,3 +1132,14 @@ Status_DDC get_vcp_cur_value(DDC_Packet * packet, int * value_ptr) {
    }
    return rc;
 }
+
+
+void init_ddc_packets() {
+   RTTI_ADD_FUNC(create_ddc_base_response_packet);
+   RTTI_ADD_FUNC(create_ddc_getvcp_response_packet);
+// RTTI_ADD_FUNC(create_ddc_multi_part_read_response_packet);
+   RTTI_ADD_FUNC(create_ddc_response_packet);
+   RTTI_ADD_FUNC(create_ddc_typed_response_packet);
+   RTTI_ADD_FUNC(interpret_vcp_feature_response_std);
+}
+
