@@ -2,7 +2,7 @@
  * Monitor identifier, reference, handle
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -15,11 +15,14 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "util/data_structures.h"
 #include "util/glib_util.h"
 #include "util/string_util.h"
 #include "util/report_util.h"
+#ifdef ENABLE_UDEV
 #include "util/udev_util.h"
 #include "util/udev_usb_util.h"
+#endif
 /** \endcond */
 
 #include "public/ddcutil_types.h"
@@ -27,6 +30,7 @@
 
 #include "core.h"
 #include "monitor_model_key.h"
+#include "rtti.h"
 #include "vcp_version.h"
 
 #include "displays.h"
@@ -460,47 +464,50 @@ void dbgrpt_display_identifier(Display_Identifier * pdid, int depth) {
  *  The returned pointer is valid until the #Display_Identifier is freed.
  */
 char * did_repr(Display_Identifier * pdid) {
-   if (!pdid->repr) {
-      char * did_type_name = display_id_type_name(pdid->id_type);
-      switch (pdid->id_type) {
-      case(DISP_ID_BUSNO):
-            pdid->repr = g_strdup_printf(
-                     "Display Id[type=%s, bus=/dev/i2c-%d]", did_type_name, pdid->busno);
-            break;
-      case(DISP_ID_ADL):
-            pdid->repr = g_strdup_printf(
-                     "Display Id[type=%s, adlno=%d.%d]", did_type_name, pdid->iAdapterIndex, pdid->iDisplayIndex);
-            break;
-      case(DISP_ID_MONSER):
-            pdid->repr = g_strdup_printf(
-                     "Display Id[type=%s, mfg=%s, model=%s, sn=%s]",
-                     did_type_name, pdid->mfg_id, pdid->model_name, pdid->serial_ascii);
-            break;
-      case(DISP_ID_EDID):
-      {
-            char * hs = hexstring(pdid->edidbytes, 128);
-            pdid->repr = g_strdup_printf(
-                     "Display Id[type=%s, edid=%8s...%8s]", did_type_name, hs, hs+248);
-            free(hs);
-            break;
-      }
-      case(DISP_ID_DISPNO):
-            pdid->repr = g_strdup_printf(
-                     "Display Id[type=%s, dispno=%d]", did_type_name, pdid->dispno);
-            break;
-      case DISP_ID_USB:
-            pdid->repr = g_strdup_printf(
-                     "Display Id[type=%s, usb bus:device=%d.%d]", did_type_name, pdid->usb_bus, pdid->usb_device);;
-            break;
-      case DISP_ID_HIDDEV:
-            pdid->repr = g_strdup_printf(
-                     "Display Id[type=%s, hiddev_devno=%d]", did_type_name, pdid->hiddev_devno);
-            break;
-
-      } // switch
+   char * result = NULL;
+   if (pdid) {
+      if (!pdid->repr) {
+         char * did_type_name = display_id_type_name(pdid->id_type);
+         switch (pdid->id_type) {
+         case(DISP_ID_BUSNO):
+               pdid->repr = g_strdup_printf(
+                        "Display Id[type=%s, bus=/dev/i2c-%d]", did_type_name, pdid->busno);
+               break;
+         case(DISP_ID_ADL):
+               pdid->repr = g_strdup_printf(
+                        "Display Id[type=%s, adlno=%d.%d]", did_type_name, pdid->iAdapterIndex, pdid->iDisplayIndex);
+               break;
+         case(DISP_ID_MONSER):
+               pdid->repr = g_strdup_printf(
+                        "Display Id[type=%s, mfg=%s, model=%s, sn=%s]",
+                        did_type_name, pdid->mfg_id, pdid->model_name, pdid->serial_ascii);
+               break;
+         case(DISP_ID_EDID):
+         {
+               char * hs = hexstring(pdid->edidbytes, 128);
+               pdid->repr = g_strdup_printf(
+                        "Display Id[type=%s, edid=%8s...%8s]", did_type_name, hs, hs+248);
+               free(hs);
+               break;
+         }
+         case(DISP_ID_DISPNO):
+               pdid->repr = g_strdup_printf(
+                        "Display Id[type=%s, dispno=%d]", did_type_name, pdid->dispno);
+               break;
+         case DISP_ID_USB:
+               pdid->repr = g_strdup_printf(
+                        "Display Id[type=%s, usb bus:device=%d.%d]", did_type_name, pdid->usb_bus, pdid->usb_device);;
+               break;
+         case DISP_ID_HIDDEV:
+               pdid->repr = g_strdup_printf(
+                        "Display Id[type=%s, hiddev_devno=%d]", did_type_name, pdid->hiddev_devno);
+               break;
 
+         } // switch
+      } // !pdid->repr
+      result = pdid->repr;
    }
-   return pdid->repr;
+   return result;
 }
 
 
@@ -618,12 +625,12 @@ char * dpath_repr_t(DDCA_IO_Path * dpath) {
 
 // *** Display_Ref ***
 
-
 static Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
    Display_Ref * dref = calloc(1, sizeof(Display_Ref));
    memcpy(dref->marker, DISPLAY_REF_MARKER, 4);
    dref->io_path = io_path;
-   dref->vcp_version = DDCA_VSPEC_UNQUERIED;
+   dref->vcp_version_xdf = DDCA_VSPEC_UNQUERIED;
+   dref->vcp_version_cmdline = DDCA_VSPEC_UNQUERIED;
 
    dref->async_rec  = get_display_async_rec(io_path);    // keep?
 
@@ -631,7 +638,6 @@ static Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
 }
 
 
-
 // PROBLEM: bus display ref getting created some other way
 /** Creates a #Display_Ref for IO mode #DDCA_IO_I2C
  *
@@ -645,7 +651,7 @@ Display_Ref * create_bus_display_ref(int busno) {
    io_path.path.i2c_busno = busno;
    Display_Ref * dref = create_base_display_ref(io_path);
    if (debug) {
-      DBGMSG("Done.  Constructed bus display ref:");
+      DBGMSG("Done.  Constructed bus display ref %s:", dref_repr_t(dref));
       dbgrpt_display_ref(dref,0);
    }
    return dref;
@@ -674,7 +680,7 @@ Display_Ref * create_adl_display_ref(int iAdapterIndex, int iDisplayIndex) {
 
 
 #ifdef USE_USB
-/** Creates a #Display_Ref for IO mode #DDCA_IO_ADL
+/** Creates a #Display_Ref for IO mode #DDCA_IO_USB
  *
  * @param  usb_bus USB bus number
  * @param  usb_device USB device number
@@ -694,7 +700,7 @@ Display_Ref * create_usb_display_ref(int usb_bus, int usb_device, char * hiddev_
    dref->usb_hiddev_name = strdup(hiddev_devname);
 
    if (debug) {
-      DBGMSG("Done.  Constructed ADL display ref:");
+      DBGMSG("Done.  Constructed USB display ref:");
       dbgrpt_display_ref(dref,0);
    }
    return dref;
@@ -721,29 +727,50 @@ Display_Ref * clone_display_ref(Display_Ref * old) {
 #endif
 
 
-// Is it still meaningful to free a display ref?
-
+/** Free a display reference.
+ *
+ *  \param dref  display reference to free
+ *  \retval DDCRC_OK       success
+ *  \retval DDCRC_ARG      invalid display reference
+ *  \retval DDCRC_LOCKED   display reference not marked as transient
+ */
 DDCA_Status free_display_ref(Display_Ref * dref) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "dref=%p -> %s", dref, dref_repr_t(dref));
    DDCA_Status ddcrc = 0;
+   if (!dref || memcmp(dref->marker, DISPLAY_REF_MARKER, 4) != 0) {
+      ddcrc = DDCRC_ARG;
+      DBGMSG("Invalid dref.");
+      if (dref)
+         rpt_hex_dump((Byte*) dref->marker, 4, 2);
+      goto bye;
+   }
    if (dref && (dref->flags & DREF_TRANSIENT) ) {
       if (dref->flags & DREF_OPEN) {
          ddcrc = DDCRC_LOCKED;
       }
       else {
-         assert(memcmp(dref->marker, DISPLAY_REF_MARKER,4) == 0);
          dref->marker[3] = 'x';
-         if (dref->usb_hiddev_name)       // always set by strdup()
+         if (dref->usb_hiddev_name)       // always set using strdup()
             free(dref->usb_hiddev_name);
          if (dref->capabilities_string)   // always a private copy
             free(dref->capabilities_string);
+         if (dref->mmid)                  // always a private copy
+            free(dref->mmid);
          // 9/2017: what about pedid, detail2?
          // what to do with gdl, request_queue?
+         if (dref->dfr)
+            dfr_free(dref->dfr);
+         dref->marker[3] = 'x';
          free(dref);
       }
    }
+bye:
+   DBGTRC_RETURNING(debug, DDCA_TRC_BASE, ddcrc, "");
    return ddcrc;
 }
 
+
 /** Tests if 2 #Display_Ref instances specify the same path to the
  *  display.
  *
@@ -760,41 +787,17 @@ bool dref_eq(Display_Ref* this, Display_Ref* that) {
 }
 
 
-void dbgrpt_dref_flags(Dref_Flags flags, int depth) {
-
-#define RPT_DREF_FLAG(FLAG_NAME) \
-   rpt_vstring(d1, "%s: %s", #FLAG_NAME, sbool(flags&FLAG_NAME) )
-
-   int d0 = depth;
-   int d1 = d0+1;
-   // DBGMSG("d1 = %d", d1);
-   rpt_vstring(d0, "flags:        0x%02x", flags);
-   RPT_DREF_FLAG(DREF_DDC_COMMUNICATION_CHECKED                 );
-   RPT_DREF_FLAG(DREF_DDC_COMMUNICATION_WORKING                 );
-   RPT_DREF_FLAG(DREF_DDC_NULL_RESPONSE_CHECKED                 );
-   RPT_DREF_FLAG(DREF_DDC_IS_MONITOR_CHECKED                    );
-   RPT_DREF_FLAG(DREF_DDC_IS_MONITOR                            );
-   RPT_DREF_FLAG(DREF_TRANSIENT                                 );
-   RPT_DREF_FLAG(DREF_DYNAMIC_FEATURES_CHECKED                  );
-   RPT_DREF_FLAG(DREF_OPEN                                      );
-   RPT_DREF_FLAG(DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED    );
-   RPT_DREF_FLAG(DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED );
-   RPT_DREF_FLAG(DREF_DDC_USES_DDC_FLAG_FOR_UNSUPPORTED         );
-   RPT_DREF_FLAG(DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED         );
-
-#undef RPT_DREF_FLAG
-}
-
-
 /** Reports the contents of a #Display_Ref in a format appropriate for debugging.
  *
  *  \param  dref  pointer to #Display_Ref instance
  *  \param  depth logical indentation depth
  */
 void dbgrpt_display_ref(Display_Ref * dref, int depth) {
-   rpt_structure_loc("DisplayRef", dref, depth );
+   bool debug = false;
+   DBGMSF(debug, "Starting. dref=%p", dref);
+   rpt_structure_loc("Display_Ref", dref, depth );
    int d1 = depth+1;
-   int d2 = depth+2;
+   // int d2 = depth+2;
 
 #ifdef OLD
    // old
@@ -822,7 +825,7 @@ void dbgrpt_display_ref(Display_Ref * dref, int depth) {
 #endif
 
    // alt:
-   rpt_vstring(d1, "io_path:      %s", dpath_repr_t(&(dref->io_path)));
+   rpt_vstring(d1, "io_path:          %s", dpath_repr_t(&(dref->io_path)));
 
    if (dref->io_path.io_mode == DDCA_IO_USB) {
       rpt_int("usb_bus",         NULL, dref->usb_bus,         d1);
@@ -830,22 +833,13 @@ void dbgrpt_display_ref(Display_Ref * dref, int depth) {
       rpt_str("usb_hiddev_name", NULL, dref->usb_hiddev_name, d1);
    }
 
-   // rpt_vstring(d1, "vcp_version:  %d.%d\n", dref->vcp_version.major, dref->vcp_version.minor );
-   rpt_vstring(d1, "vcp_version:  %s", format_vspec(dref->vcp_version) );
-#ifdef OLD
-   rpt_vstring(d1, "flags:        0x%02x", dref->flags);
-   rpt_vstring(d2, "DDC communication checked:                  %s", sbool(dref->flags & DREF_DDC_COMMUNICATION_CHECKED) );
-   if (dref->flags & DREF_DDC_COMMUNICATION_CHECKED)
-   rpt_vstring(d2, "DDC communication working:                  %s", sbool(dref->flags & DREF_DDC_COMMUNICATION_WORKING) );
-   rpt_vstring(d2, "DDC NULL response usage checked:            %s", sbool(dref->flags & DREF_DDC_NULL_RESPONSE_CHECKED) );
-   if (dref->flags & DREF_DDC_NULL_RESPONSE_CHECKED)
-   rpt_vstring(d2, "DDC NULL response may indicate unsupported: %s", sbool(dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
-   rpt_vstring(d2, "DDC normal all byte 0 response may indicate unsupported: %s", sbool(dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED));
-   rpt_vstring(d2, "DDC does not indicate unsupported:          %s", sbool(dref->flags & DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED));
-   rpt_vstring(d2, "Display Ref is open:                        %s", sbool(dref->flags & DREF_OPEN));
-#endif
-   dbgrpt_dref_flags(dref->flags, d1);
-   rpt_vstring(d2, "mmid:                                       %s", (dref->mmid) ? mmk_repr(*dref->mmid) : "NULL");}
+   rpt_vstring(d1, "vcp_version_xdf:  %s", format_vspec(dref->vcp_version_xdf) );
+   // dbgrpt_dref_flags(dref->flags, d1);
+   rpt_vstring(d1, "flags:            %s", interpret_dref_flags_t(dref->flags) );
+   rpt_vstring(d1, "mmid:             %s", (dref->mmid) ? mmk_repr(*dref->mmid) : "NULL");
+
+   DBGMSF(debug, "Done");
+}
 
 
 #ifdef OLD
@@ -918,7 +912,7 @@ char * dref_repr_t(Display_Ref * dref) {
 
    char * buf = get_thread_fixed_buffer(&dref_repr_key, 100);
    if (dref)
-      g_snprintf(buf, 100, "Display_Ref[%s]", dpath_short_name_t(&dref->io_path));
+      g_snprintf(buf, 100, "Display_Ref[%s @%p]", dpath_short_name_t(&dref->io_path), dref);
    else
       strcpy(buf, "Display_Ref[NULL]");
    return buf;
@@ -927,6 +921,8 @@ char * dref_repr_t(Display_Ref * dref) {
 
 // *** Display_Handle ***
 
+#ifdef OLD
+
 /** Creates a #Display_Handle for an I2C #Display_Ref.
  *
  *  \param  fd   Linux file descriptor of open display
@@ -945,31 +941,8 @@ Display_Handle * create_bus_display_handle_from_display_ref(int fd, Display_Ref
    dh->dref = dref;
    // dref->vcp_version = DDCA_VSPEC_UNQUERIED;
    dh->repr = g_strdup_printf(
-                "[i2c: fd=%d, busno=%d]",
-                dh->fd, dh->dref->io_path.path.i2c_busno);
-   return dh;
-}
-
-
-/** Creates a #Display_Handle for an ADL #Display_Ref.
- *
- *  \param  dref pointer to #Display_Ref
- *  \return newly allocated #Display_Handle
- *
- *  \remark
- *  This functions handles the boilerplate of creating a #Display_Handle.
- */
-Display_Handle * create_adl_display_handle_from_display_ref(Display_Ref * dref) {
-   // assert(dref->io_mode == DDCA_IO_ADL);
-   assert(dref->io_path.io_mode == DDCA_IO_ADL);
-   Display_Handle * dh = calloc(1, sizeof(Display_Handle));
-   memcpy(dh->marker, DISPLAY_HANDLE_MARKER, 4);
-   dh->dref = dref;
-   // dref->vcp_version = DDCA_VSPEC_UNQUERIED;   // needed?
-   dh->repr = g_strdup_printf(
-                "[adl: display %d.%d]",
-                // "Display_Handle[adl: display %d.%d]",
-                 dh->dref->io_path.path.adlno.iAdapterIndex, dh->dref->io_path.path.adlno.iDisplayIndex);
+                "[i2c: fd=%d, busno=%d @%p]",
+                dh->fd, dh->dref->io_path.path.i2c_busno, dh);
    return dh;
 }
 
@@ -992,15 +965,59 @@ Display_Handle * create_usb_display_handle_from_display_ref(int fd, Display_Ref
    dh->fd = fd;
    dh->dref = dref;
    dh->repr = g_strdup_printf(
-                "[usb: %d:%d, %s/hiddev%d]",
+                "[usb: %d:%d, %s/hiddev%d @%p]",
                 // "Display_Handle[usb: %d:%d, %s/hiddev%d]",
                 dh->dref->usb_bus, dh->dref->usb_device,
-                usb_hiddev_directory(), dh->dref->io_path.path.hiddev_devno);
+                usb_hiddev_directory(), dh->dref->io_path.path.hiddev_devno,
+                dh);
    // dref->vcp_version = DDCA_VSPEC_UNQUERIED;
    return dh;
 }
 #endif
 
+#endif
+
+
+/** Creates a #Display_Handle for a #Display_Ref.
+ *
+ *  \param  fd    Linux file descriptor of open display
+ *  \param  dref  pointer to #Display_Ref
+ *  \return newly allocated #Display_Handle
+ *
+ *  \remark
+ *  This functions handles the boilerplate of creating a #Display_Handle.
+ */
+Display_Handle * create_base_display_handle(int fd, Display_Ref * dref) {
+   // assert(dref->io_mode == DDCA_IO_USB);
+   Display_Handle * dh = calloc(1, sizeof(Display_Handle));
+   memcpy(dh->marker, DISPLAY_HANDLE_MARKER, 4);
+   dh->fd = fd;
+   dh->dref = dref;
+   if (dref->io_path.io_mode == DDCA_IO_I2C) {
+      dh->repr = g_strdup_printf(
+                     "Display_Handle[i2c: fd=%d, busno=%d @%p]",
+                     dh->fd, dh->dref->io_path.path.i2c_busno, dh);
+   }
+#ifdef USE_USB
+   else if (dref->io_path.io_mode == DDCA_IO_USB) {
+      dh->repr = g_strdup_printf(
+                "Display_Handle[usb: %d:%d, %s/hiddev%d @%p]",
+                // "Display_Handle[usb: %d:%d, %s/hiddev%d]",
+                dh->dref->usb_bus, dh->dref->usb_device,
+                usb_hiddev_directory(), dh->dref->io_path.path.hiddev_devno,
+                dh);
+   }
+#endif
+   else {
+      // DDCA_IO_ADL, DDCA_IO_USB if !USE_USB
+      PROGRAM_LOGIC_ERROR("Unimplemented io_mode = %d", dref->io_path.io_mode);
+      dh->repr = NULL;
+   }
+
+   return dh;
+}
+
+
 
 /** Reports the contents of a #Display_Handle in a format useful for debugging.
  *
@@ -1057,38 +1074,56 @@ void dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth) {
  * \return  string  representation of handle
  */
 char * dh_repr_t(Display_Handle * dh) {
+   // dh_repr is precalculated when display handle is created, returning the
+   // value is thread safe
+   if (!dh)
+      return "Display_Handle[NULL]";
+   return dh->repr;
+
+#ifdef UNNECESSARY
    static GPrivate  dh_buf_key = G_PRIVATE_INIT(g_free);
    const int bufsz = 100;
    char * buf = get_thread_fixed_buffer(&dh_buf_key, bufsz);
 
    if (dh) {
-      assert(dh->dref);
+      if (memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) == 0) {
+         assert(dh->dref);
 
-      switch (dh->dref->io_path.io_mode) {
-      case DDCA_IO_I2C:
-           snprintf(buf, bufsz,
-                    "Display_Handle[i2c: fd=%d, busno=%d]",
-                    dh->fd, dh->dref->io_path.path.i2c_busno);
-           break;
-       case DDCA_IO_ADL:
-           snprintf(buf, bufsz,
-                    "Display_Handle[adl: display %d.%d]",
-                    dh->dref->io_path.path.adlno.iAdapterIndex, dh->dref->io_path.path.adlno.iDisplayIndex);
-           break;
-       case DDCA_IO_USB:
-           snprintf(buf, bufsz,
-                    "Display_Handle[usb: %d:%d, %s/hiddev%d]",
-                    dh->dref->usb_bus, dh->dref->usb_device,
-                    usb_hiddev_directory(), dh->dref->io_path.path.hiddev_devno);
-           break;
-       }
-       buf[bufsz-1] = '\0';
+         switch (dh->dref->io_path.io_mode) {
+         case DDCA_IO_I2C:
+              snprintf(buf, bufsz,
+                       "Display_Handle[i2c: fd=%d, busno=%d @%p]",
+                       dh->fd, dh->dref->io_path.path.i2c_busno, dh);
+              break;
+          case DDCA_IO_ADL:
+              snprintf(buf, bufsz,
+                       "Display_Handle[adl: display %d.%d]",
+                       dh->dref->io_path.path.adlno.iAdapterIndex, dh->dref->io_path.path.adlno.iDisplayIndex);
+              break;
+          case DDCA_IO_USB:
+#ifdef ENABLE_USB
+              snprintf(buf, bufsz,
+                       "Display_Handle[usb: %d:%d, %s/hiddev%d @%p]",
+                       dh->dref->usb_bus, dh->dref->usb_device,
+                       usb_hiddev_directory(), dh->dref->io_path.path.hiddev_devno, dh);
+#else
+              PROGRAM_LOGIC_ERROR("io_path.io_mode == DDCA_IO_USB");
+#endif
+              break;
+          }
+          buf[bufsz-1] = '\0';
+      }
+      else {
+         snprintf(buf, bufsz, "Invalid Display_Handle@%p", dh);
+         //  strcpy(buf, "Invalid Display_Handle");
+      }
    }
    else {
       strcpy(buf, "Display_Handle[NULL]");
    }
 
    return buf;
+#endif
 }
 
 
@@ -1103,7 +1138,7 @@ char * dh_repr(Display_Handle * dh) {
    assert(dh->dref);
    assert(dh->repr);
    // Do not calculate and memoize dh->repr here, due to possible race condition between threads
-   // Instead always precalculate at time of Display_Handle creation
+   // Instead precalculate at time of Display_Handle creation
    return dh->repr;
 }
 
@@ -1113,11 +1148,14 @@ char * dh_repr(Display_Handle * dh) {
  * \param  dh  display handle to free
  */
 void   free_display_handle(Display_Handle * dh) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "dh=%p -> %s", dh, dh_repr_t(dh));
    if (dh && memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) == 0) {
       dh->marker[3] = 'x';
       free(dh->repr);
       free(dh);
    }
+   DBGTRC_DONE(debug, DDCA_TRC_BASE, "");
 }
 
 
@@ -1136,7 +1174,7 @@ Video_Card_Info * create_video_card_info() {
    return card_info;
 }
 
-
+#ifdef USE_USB
 /** Given a hiddev device name, e.g. /dev/usb/hiddev3,
  *  extract its number, e.g. 3.
  *
@@ -1176,14 +1214,52 @@ char * hiddev_number_to_name(int hiddev_number) {
    // DBGMSG("hiddev_number=%d, returning: %s", hiddev_number, s);
    return s;
 }
+#endif
 
 
+Value_Name_Table dref_flags_table = {
+      VN(DREF_DDC_COMMUNICATION_CHECKED),
+ //   VN(DREF_DDC_COMMUNICATION_WORKING),
+      VN(DREF_DDC_NULL_RESPONSE_CHECKED),
+      VN(DREF_DDC_IS_MONITOR_CHECKED),
+      VN(DREF_DDC_IS_MONITOR),
+      VN(DREF_TRANSIENT),
+      VN(DREF_DYNAMIC_FEATURES_CHECKED),
+      VN(DREF_OPEN),
+      VN(DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED),
+      VN(DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED),
+      VN(DREF_DDC_USES_DDC_FLAG_FOR_UNSUPPORTED),
+      VN(DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED),
+      VN(DREF_DDC_BUSY),
+      VN(CALLOPT_NONE),                // special entry
+      VN_END
+};
 
 
-void init_displays() {
-   displays_master_list = g_ptr_array_new();
+/** Interprets a **Dref_Flags** value as a printable string.
+ *  The returned value is valid until the next call of this function in
+ *  the current thread.
+ *
+ *  @param flags  value to interpret
+ *
+ *  @return interpreted value
+ */
+char * interpret_dref_flags_t(Dref_Flags flags) {
+   static GPrivate  buf_key = G_PRIVATE_INIT(g_free);
+   char * buf = get_thread_fixed_buffer(&buf_key, 200);
 
+   char * buftemp = vnt_interpret_flags(flags, dref_flags_table, false, ", ");
+   g_strlcpy(buf, buftemp, 200);    // n. this is a debug msg, truncation benign
+   free(buftemp);
+
+   return buf;
 }
 
 
+void init_displays() {
+   RTTI_ADD_FUNC(free_display_handle);
+   RTTI_ADD_FUNC(free_display_ref);
+
+   displays_master_list = g_ptr_array_new();
+}
 
