@@ -2,7 +2,7 @@
  * Display Specification
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DISPLAYS_H_
@@ -174,7 +174,16 @@ typedef uint16_t Dref_Flags;
 #define DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED 0x0400
 #define DREF_DDC_USES_DDC_FLAG_FOR_UNSUPPORTED         0x0200
 #define DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED         0x0100
+#define DREF_DDC_BUSY                                  0x8000
 
+char * interpret_dref_flags_t(Dref_Flags flags);    // replaces dref_basic_flags()?
+
+// define in ddcutil_types.h?, or perhaps use -1 for generic invalid, put type of invalid in Dref_Flags?
+#define DISPNO_NOT_SET  0
+#define DISPNO_INVALID -1
+#define DISPNO_PHANTOM -2
+#define DISPNO_REMOVED -3
+#define DISPNO_BUSY    -4
 
 #define DISPLAY_REF_MARKER "DREF"
 /** A **Display_Ref** is a logical display identifier.
@@ -187,7 +196,8 @@ typedef struct _display_ref {
    int                      usb_bus;
    int                      usb_device;
    char *                   usb_hiddev_name;
-   DDCA_MCCS_Version_Spec   vcp_version;
+   DDCA_MCCS_Version_Spec   vcp_version_xdf;
+   DDCA_MCCS_Version_Spec   vcp_version_cmdline;
    Dref_Flags               flags;
    char *                   capabilities_string;    // added 4/2017, private copy
    Parsed_Edid *            pedid;                  // added 4/2017
@@ -197,6 +207,7 @@ typedef struct _display_ref {
    Display_Async_Rec *      async_rec;
    Dynamic_Features_Rec *   dfr;                   // user defined feature metadata
    uint64_t                 next_i2c_io_after;     // nanosec
+   struct _display_ref *    actual_display;        // if dispno == -2
 } Display_Ref;
 
 #define ASSERT_DREF_IO_MODE(_dref, _mode)  \
@@ -232,9 +243,14 @@ typedef struct {
    char *       repr;
 } Display_Handle;
 
+#ifdef OLD
 Display_Handle * create_bus_display_handle_from_display_ref(int fd, Display_Ref * dref);
+#ifdef OLD
 Display_Handle * create_adl_display_handle_from_display_ref(Display_Ref * dref);
+#endif
 Display_Handle * create_usb_display_handle_from_display_ref(int fd, Display_Ref * dref);
+#endif
+Display_Handle * create_base_display_handle(int fd, Display_Ref * dref);
 void             dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth);
 char *           dh_repr(Display_Handle * dh);
 char *           dh_repr_t(Display_Handle * dh);
