@@ -3,11 +3,9 @@
  *  Dynamic Feature Record definition, creation, destruction, and conversion
  */
 
-// Copyright (C) 2018-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
-#define _GNU_SOURCE      // for asprintf()
-
 /** \cond */
 #include <assert.h>
 #include <ctype.h>
@@ -21,14 +19,18 @@
 #include "util/report_util.h"
 #include "util/string_util.h"
 
+#include "base/core.h"
 #include "base/feature_metadata.h"
-
-#include "core.h"
-#include "monitor_model_key.h"
-#include "vcp_version.h"
+#include "base/monitor_model_key.h"
+#include "base/vcp_version.h"
 
 #include "dynamic_features.h"
 
+//
+// Trace control
+//
+
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_UDF;
 
 //
 // Generic functions that probably belong elsewhere
@@ -65,47 +67,6 @@ Tokenized first_word(char * s) {
    return result;
 }
 
-
-// TODO: modify parse_vcp_value() in app_setvcp.c to use this function.
-//       Move this function to a more general location.
-
-/** Converts a string containing a (possible) hex value to canonical form.
- *
- *  If the value starts with "x" or "X", or ends with "h" or "H", it is
- *  modified to start with "0x".
- *  Otherwise, the returned value is identical to the input value.
- *
- *  @param string_value  value to convert
- *  @return canonicalized value (caller is responsible for freeing)
- *
- *  @remark
- *  Consider converting to a function that uses a thread-specific buffer, making
- *  the returned value valid until the next call to this function on the current
- *  thread.  Would relieve the caller of responsibility for freeing the value.
- */
-
-char * canonicalize_possible_hex_value(char * string_value) {
-   assert(string_value);
-
-   int bufsz = strlen(string_value) + 1 + 1;  // 1 for possible increased length, 1 for terminating null
-   char * buf = calloc(1, bufsz);
-   if (*string_value == 'X' || *string_value == 'x' ) {
-      // increases string size by 1
-      snprintf(buf, bufsz, "0x%s", string_value+1);
-   }
-   else if (*(string_value + strlen(string_value)-1) == 'H' ||
-            *(string_value + strlen(string_value)-1) == 'h' )
-   {
-      // increases string size by 1
-      int newlen = strlen(string_value)-1;
-      snprintf(buf, bufsz, "0x%.*s", newlen, string_value);
-   }
-   else
-      strcpy(buf, string_value);
-   // DBGMSG("string_value=|%s|, returning |%s|", string_value, buf);
-   return buf;
-}
-
 // End of generic functions
 
 
@@ -137,8 +98,6 @@ void dbgrpt_dynamic_features_rec(
 }
 
 
-
-
 /** Thread safe function that returns a string representation of a #Dynamic_Features_Rec
  *  suitable for diagnostic messages. The returned value is valid until the
  *  next call to this function on the current thread.
@@ -159,8 +118,6 @@ char * dfr_repr_t(Dynamic_Features_Rec * dfr) {
 }
 
 
-
-
 DDCA_Feature_Metadata *
 get_dynamic_feature_metadata(
       Dynamic_Features_Rec * dfr,
@@ -183,10 +140,10 @@ free_feature_metadata(
       gpointer data)    // i.e. DDCA_Feature_Metadata *
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. DDCA_Feature_Metadata * data = %p", data);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "Starting. DDCA_Feature_Metadata * data = %p", data);
 
    DDCA_Feature_Metadata * info = (DDCA_Feature_Metadata*) data;
-   assert(memcmp(info->marker, DDCA_FEATURE_METADATA_MARKER, 4) == 0);
+   assert(info && memcmp(info->marker, DDCA_FEATURE_METADATA_MARKER, 4) == 0);
    // compare vs ddca_free_metadata_contents()
 
    if (debug)
@@ -195,22 +152,24 @@ free_feature_metadata(
    free(info->feature_desc);
    free(info->feature_name);
    if (info->sl_values) {
+      DBGMSF(debug, "Freeing sl_values table at %p", info->sl_values);
       free_sl_value_table(info->sl_values);
-#ifdef OLD
-      DDCA_Feature_Value_Entry * cur = info->sl_values;
-      for ( ;cur->value_code != 0x00 || cur->value_name; cur++ ) {
-         free(cur->value_name);
-      }
-      free(info->sl_values);      // is this right? freeing a hunk allocated by GPtrArray
-#endif
    }
+   // free latest_sl_values ?
    info->marker[3] = 'x';
    free(info);
 
-   DBGMSF(debug, "Done");
+   DBGTRC_DONE(debug, TRACE_GROUP, "");
 }
 
-
+/** Create a #Dynameic_Feature_Rec
+ *
+ *  \param   mfg_id
+ *  \param   model_name
+ *  \param   product_code
+ *  \param   filename
+ *  \return newly allocated #Dynamic_Features_Rec, caller must free
+ */
 Dynamic_Features_Rec *
 dfr_new(
       const char * mfg_id,
@@ -218,6 +177,10 @@ dfr_new(
       uint16_t     product_code,
       const char * filename)
 {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "mfg_id -> %s, model_name -> %s, product_code=%d, filename -> %s",
+         mfg_id, model_name, product_code, filename);
+
    assert(mfg_id);
    assert(model_name);
 
@@ -227,8 +190,11 @@ dfr_new(
    frec->mfg_id       = strdup(mfg_id);
    frec->model_name   = strdup(model_name);
    frec->product_code = product_code;
+   frec->vspec        = DDCA_VSPEC_UNKNOWN;   // redundant, since set by calloc(), but be explicit
    if (filename)
       frec->filename  = strdup(filename);
+
+   DBGTRC_DONE(debug, TRACE_GROUP, "Returning %p", frec);
    return frec;
 }
 
@@ -238,7 +204,7 @@ dfr_free(
       Dynamic_Features_Rec * frec)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. frec=%p", frec);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "frec=%p", frec);
 
    if (frec) {
       assert(memcmp(frec->marker, DYNAMIC_FEATURES_REC_MARKER, 4) == 0);
@@ -256,14 +222,14 @@ dfr_free(
       free(frec);
    }
 
-   DBGMSF(debug, "Done");
+   DBGTRC_DONE(debug, TRACE_GROUP, "");
 }
 
+
 //
 // Functions private to create_monitor_dynamic_featuers()
 //
 
-
 static
 void
 add_error(
@@ -300,9 +266,7 @@ add_error(
    add_error(errors, filename, _linectr, __func__, _fmt, ##__VA_ARGS__)
 
 
-
-static
-bool
+static bool
 attr_keyword(
       DDCA_Feature_Metadata * cur_feature_metadata,
       char *                  keyword)
@@ -335,8 +299,7 @@ attr_keyword(
 }
 
 
-static
-void
+static void
 switch_bits(
       DDCA_Feature_Flags * pflags,
       uint16_t             old_bit,
@@ -347,8 +310,7 @@ switch_bits(
 }
 
 
-static
-void
+static void
 finalize_feature(
       Dynamic_Features_Rec *  frec,
       DDCA_Feature_Metadata * cur_feature_metadata,
@@ -366,6 +328,7 @@ finalize_feature(
       g_array_append_val(cur_feature_values, last_entry);
 
       cur_feature_metadata->sl_values = (DDCA_Feature_Value_Entry*) cur_feature_values->data;
+      // cur_feature_metadata->latest_sl_values = copy_sl_value_table(cur_feature_metadata->sl_values);
       // g_array_free(cur_feature_values, false);
       // cur_feature_values = NULL;
    }
@@ -474,7 +437,7 @@ create_monitor_dynamic_features(
                   ADD_ERROR(linectr, "Unexpected model name \"%s\"", t1.rest);
                }
             }
-            else if (streq(t1.word, "MCCS_VERSION")) {
+            else if (streq(t1.word, "MCCS_VERSION") || streq(t1.word, "VCP_VERSION") ) {
                // mccs_version_seen = true;   // not required for now
                // default as set by calloc() is 0.0, which is DDCA_VSPEC_UNKNOWN
                // returns DDCA_VSPEC_UKNOWN if invalid
