@@ -1,10 +1,9 @@
-
 /** \file parms.h
  *
  *  System configuration and tuning
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef PARMS_H_
@@ -42,20 +41,21 @@
 
 // One of the following 2 defines must be enabled:
 // #define DEFAULT_I2C_IO_STRATEGY  I2C_IO_STRATEGY_IOCTL ///< Use ioctl() calls
-#define DEFAULT_I2C_IO_STRATEGY  I2C_IO_STRATEGY_FILEIO   ///< Use read() and write()
-#define DEFAULT_I2C_READ_BYTEWISE false                   ///< Use single byte reads
+#define DEFAULT_I2C_IO_STRATEGY        I2C_IO_STRATEGY_FILEIO  ///< Use read() and write()
+#define DEFAULT_I2C_READ_BYTEWISE      false                   ///< Use single byte reads
+#define DEFAULT_EDID_WRITE_BEFORE_READ true
+#define DEFAULT_EDID_READ_SIZE           0                     ///< 128, 256, 0=>dynamic
+#define EDID_BUFFER_SIZE               256                     ///< always 256
 
-#define EDID_READ_USES_I2C_LAYER  false
-#define EDID_READ_BYTEWISE  false
+#define DEFAULT_EDID_READ_USES_I2C_LAYER  false
+#define DEFAULT_EDID_READ_BYTEWISE        false
 
 
 // Strategy    Bytewise    read edid uses local i2c call                      read edid uses i2c layer
 // FILEIO      false       ok                                                 ok
 // FILEIO      true        on P2411h and Acer, reads byes 0. 2, 4 of response EDID ok, getvcp fails
-// FILEIO      false       ok                                                 All ok
-// IOCTL       true        on P2411h and Acer, returns corrupte data,         EDID ok, getvcp fails
-
-
+// IOCTL       false       ok                                                 All ok
+// IOCTL       true        on P2411h and Acer, returns corrupt data           EDID ok, getvcp fails
 
 // Parms used only within testcase portion of code:
 
@@ -104,9 +104,22 @@
 // #define MAX_ARGS (MAX_SETVCP_VALUES*2)   // causes CMDID_* undefined
 #define MAX_ARGS 100        // hack
 
-/** Parallelize display checks during initialization if at least this  number of displays */
+/** Parallelize display checks during initialization if at least this number of displays */
 // on banner with 4 displays, async  detect: 1.7 sec, non-async 3.4 sec
-#define DISPLAY_CHECK_ASYNC_THRESHOLD   3
 #define DISPLAY_CHECK_ASYNC_NEVER    0xff
+#define DISPLAY_CHECK_ASYNC_THRESHOLD_STANDARD  3
+#define DISPLAY_CHECK_ASYNC_THRESHOLD_DEFAULT   DISPLAY_CHECK_ASYNC_NEVER
+
+#define DEFAULT_SLEEP_LESS true
+
+#ifdef USE_USB
+#define DEFAULT_ENABLE_USB false
+#else
+#define DEFAULT_ENABLE_USB false
+#endif
+
+#define DEFAULT_ENABLE_CACHED_CAPABILITIES true
+#define DEFAULT_ENABLE_UDF true
+
 
 #endif /* PARMS_H_ */
