@@ -1,8 +1,8 @@
 /* @file rtti.c
- * Primitive runtime type information facilities
+ * Runtime trace information
  */
 
-// Copyright (C) 2018-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -10,6 +10,7 @@
 #include <string.h>
  
 #include "util/report_util.h"
+#include "util/string_util.h"
 #include "base/rtti.h"
 
 
@@ -33,6 +34,23 @@ char * rtti_get_func_name_by_addr(void * ptr) {
    return result;
 }
 
+void * rtti_get_func_addr_by_name(char * name) {
+   void * result = NULL;
+   if (func_name_table) {
+      GHashTableIter iter;
+      gpointer key, value;
+      g_hash_table_iter_init(&iter, func_name_table);
+      while (g_hash_table_iter_next(&iter, &key, &value)) {
+         if (streq(name, (char *) value)) {
+            result = key;
+            break;
+         }
+      }
+   }
+   // printf("(%s) name=%s, returning %s\n", __func__, name, SBOOL(result));
+   return result;
+}
+
 
 void dbgrpt_rtti_func_name_table(int depth) {
    int d1 = depth+1;
