@@ -4,7 +4,7 @@
  *  and applicable multipliers.
  */
 
-// Copyright (C) 2019-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2019-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <assert.h>
@@ -26,9 +26,10 @@
 #include "base/thread_sleep_data.h"
 
 // Experimental suppression of sleeps after reads
-static bool sleep_suppression_enabled = false;
+static bool sleep_suppression_enabled = DEFAULT_SLEEP_LESS;
 
 bool enable_sleep_suppression(bool enable) {
+   // DBGMSG("enable = %s", sbool(enable));
    bool old = sleep_suppression_enabled;
    sleep_suppression_enabled = enable;
    return old;
@@ -42,6 +43,7 @@ bool is_sleep_suppression_enabled() {
 static bool deferred_sleep_enabled = false;
 
 bool enable_deferred_sleep(bool enable) {
+   // DBGMSG("enable = %s", sbool(enable));
    bool old = deferred_sleep_enabled;
    deferred_sleep_enabled = enable;
    return old;
@@ -77,7 +79,7 @@ bool is_deferred_sleep_enabled() {
  *  \todo
  *  Take into account the time since the last monitor return in the
  *  current thread.
- *  \tod
+ *  \todo
  *  Take into account per-display error statistics.  Would require
  *  error statistics be maintained on a per-display basis, either
  *  in the display reference or display handle.
@@ -100,7 +102,9 @@ void tuned_sleep_with_tracex(
       const char *     msg)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. Sleep event type = %s, dh=%s", sleep_event_name(event_type), dh_repr_t(dh));
+   DBGMSF(debug, "Starting. Sleep event type = %s, dh=%s",
+                 sleep_event_name(event_type), dh_repr_t(dh));
+   assert(dh);
    assert( (event_type != SE_SPECIAL && special_sleep_time_millis == 0) ||
            (event_type == SE_SPECIAL && special_sleep_time_millis >  0) );
 
