@@ -3,13 +3,13 @@
  *  VCP (aka MCCS) version specification
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
 /** \cond */
 #include <assert.h>
-#include <glib.h>
+#include <glib-2.0/glib.h>
 #include <stdio.h>
 #include <string.h>
 /** \endcond */
@@ -131,6 +131,11 @@ bool vcp_version_eq(DDCA_MCCS_Version_Spec v1,  DDCA_MCCS_Version_Spec v2){
    return (v1.major == v2.major) && (v1.minor == v2.minor);
 }
 
+bool vcp_version_lt(DDCA_MCCS_Version_Spec v1,  DDCA_MCCS_Version_Spec v2){
+   return vcp_version_gt(v2, v1);
+}
+
+
 
 // use    if (vcp_version_eq(vspec, DDCA_VSPEC_UNQUERIED))
 #ifdef DEPRECATED
@@ -160,15 +165,31 @@ char * format_vspec(DDCA_MCCS_Version_Spec vspec) {
    char * private_buffer = get_thread_fixed_buffer(&format_vspec_key, 20);
 
    if ( vcp_version_eq(vspec, DDCA_VSPEC_UNQUERIED) )
-      g_strlcpy(private_buffer,  "Unqueried", 20);  // g_strlcpy() to quiet coverity
+      strcpy(private_buffer,  "Unqueried");
    else if ( vcp_version_eq(vspec, DDCA_VSPEC_UNKNOWN) )
-      strcpy(private_buffer,  "Unknown");     // will coverity flag this?
+      strcpy(private_buffer,  "Unknown");
    else
       g_snprintf(private_buffer, 20, "%d.%d", vspec.major, vspec.minor);
    // DBGMSG("Returning: |%s|", private_buffer);
    return private_buffer;
 }
 
+char * format_vspec_verbose(DDCA_MCCS_Version_Spec vspec) {
+   bool debug = false;
+   DBGMSF(debug, "Starting. vspec=%d.%d", vspec.major, vspec.minor);
+   static GPrivate  format_vspec_verbose_key = G_PRIVATE_INIT(g_free);
+   char * private_buffer = get_thread_fixed_buffer(&format_vspec_verbose_key, 30);
+
+   if ( vcp_version_eq(vspec, DDCA_VSPEC_UNQUERIED) )
+      g_snprintf(private_buffer, 30, "Unqueried (%d.%d)", vspec.major, vspec.minor);
+   else if ( vcp_version_eq(vspec, DDCA_VSPEC_UNKNOWN) )
+      g_snprintf(private_buffer, 30, "Unknown (%d.%d)", vspec.major, vspec.minor);
+   else
+      g_snprintf(private_buffer, 20, "%d.%d", vspec.major, vspec.minor);
+   DBGMSF(debug, "Returning: |%s|", private_buffer);
+   return private_buffer;
+}
+
 
 Value_Name_Title_Table version_id_table = {
       VNT(DDCA_MCCS_V10,   "1.0"),
@@ -188,6 +209,8 @@ Value_Name_Title_Table version_id_table = {
  * @return value in external form.
  */
 char * format_vcp_version_id(DDCA_MCCS_Version_Id version_id) {
+   char * result2 = vnt_title(version_id_table, version_id);
+#ifndef NDEBUG
    char * result = NULL;
    switch (version_id) {
    case DDCA_MCCS_V10:    result = "1.0";     break;
@@ -198,9 +221,9 @@ char * format_vcp_version_id(DDCA_MCCS_Version_Id version_id) {
    case DDCA_MCCS_VNONE:  result = "unknown"; break;
    case DDCA_MCCS_VANY:   result = "any";     break;
    }
-   char * result2 = vnt_title(version_id_table, version_id);
    assert(streq(result, result2));
-   return result;
+#endif
+   return result2;
 }
 
 
