@@ -39,11 +39,15 @@ static Cmd_Desc cmdinfo[] = {
 #endif
    {CMDID_LOADVCP,      "loadvcp",        3,  1,       1},
    {CMDID_DUMPVCP,      "dumpvcp",        3,  0,       1},
+#ifdef ENABLE_ENVCMDS
    {CMDID_INTERROGATE,  "interrogate",    3,  0,       0},
    {CMDID_ENVIRONMENT,  "environment",    3,  0,       0},
    {CMDID_USBENV,       "usbenvironment", 6,  0,       0},
+#endif
    {CMDID_VCPINFO,      "vcpinfo",        5,  0,       1},
+#ifdef WATCH_COMMAND
    {CMDID_READCHANGES,  "watch",          3,  0,       0},
+#endif
 #ifdef USE_USB
    {CMDID_CHKUSBMON,    "chkusbmon",      3,  1,       1},
 #endif
@@ -131,6 +135,7 @@ bool parse_dot_separated_arg(const char * val, int * piAdapterIndex, int * piDis
    return ok;
 }
 
+
 bool parse_colon_separated_arg(const char * val, int * pv1, int * pv2) {
    int rc = sscanf(val, "%d:%d", pv1, pv2);
    // DBGMSG("val=|%s| sscanf() returned %d  ", val, rc );
@@ -144,6 +149,7 @@ bool parse_int_arg(char * val, int * pIval) {
    return (ct == 1);
 }
 
+
 typedef struct feature_subset_table_entry_s {
    VCP_Feature_Subset   subset_id;
    Cmd_Id_Type          valid_commands;
@@ -279,19 +285,19 @@ bool validate_output_level(Parsed_Cmd* parsed_cmd) {
    // Byte default_output_level = OL_NORMAL;
    switch(parsed_cmd->cmd_id) {
       case (CMDID_DETECT):
-         valid_output_levels = DDCA_OL_TERSE | DDCA_OL_NORMAL | DDCA_OL_VERBOSE;
+         valid_output_levels = DDCA_OL_TERSE | DDCA_OL_NORMAL | DDCA_OL_VERBOSE | DDCA_OL_VV;
          break;
       case (CMDID_GETVCP):
-         valid_output_levels = DDCA_OL_TERSE | DDCA_OL_NORMAL | DDCA_OL_VERBOSE;
+         valid_output_levels = DDCA_OL_TERSE | DDCA_OL_NORMAL | DDCA_OL_VERBOSE | DDCA_OL_VV;
          break;
       case (CMDID_PROBE):
          // don't want to deal with how to report errors, handle write-only features
          // of machine readable output triggered by --terse
-         valid_output_levels =                 DDCA_OL_NORMAL | DDCA_OL_VERBOSE;
+         valid_output_levels =                 DDCA_OL_NORMAL | DDCA_OL_VERBOSE | DDCA_OL_VV;
          break;
       default:
          // default_output_level = OL_NORMAL;
-         valid_output_levels = DDCA_OL_TERSE | DDCA_OL_NORMAL | DDCA_OL_VERBOSE;
+         valid_output_levels = DDCA_OL_TERSE | DDCA_OL_NORMAL | DDCA_OL_VERBOSE | DDCA_OL_VV;
    }
    if (!(parsed_cmd->output_level & valid_output_levels)) {
       printf("Output level invalid for command %s: %s\n",
@@ -319,17 +325,23 @@ char * commands_list_help =
        "   testcase <testcase-number>\n"
        "   listtests\n"
 #endif
+#ifdef ENABLE_ENVCMDS
        "   environment                             Probe execution environment\n"
 #ifdef USE_USB
        "   usbenv                                  Probe for USB connected monitors\n"
+#endif
 #endif
        "   probe                                   Probe monitor abilities\n"
+#ifdef ENABLE_ENVCMDS
        "   interrogate                             Report everything possible\n"
+#endif
 #ifdef USE_USB
        "   chkusbmon                               Check if USB device is monitor (for UDEV)\n"
 #endif
+#ifdef DEPRECATED
        "   watch                                   Watch display for reported changes (under development)\n"
-       "\n";
+#endif
+      "\n";
 
 #ifdef OLD
 char * command_argument_help =
@@ -376,11 +388,7 @@ char * monitor_selection_option_help =
        "  The monitor to be communicated with can be specified using the following options:\n"
        "  --display <display_number>, where <display_number> ranges from 1 to the number of\n"
        "    displays detected\n"
-       "  --bus <bus number>, for /dev/i2c-<bus number>\n"
-#ifdef HAVE_ADL
-       "  --adl <adapter_number>.<display_number>, for monitors connected to an AMD video card\n"
-       "          running AMD's proprietary video driver (ADL is an acronym for AMD Display Library)\n"
-#endif
+       "  --bus <bus number>, for /dev/"I2C"-<bus number>\n"
 #ifdef USE_USB
        "  --usb <usb bus number>.<usb device number>, for monitors communicating via USB\n"
 #endif
@@ -408,7 +416,7 @@ char * tracing_comma_separated_option_help =
 char * tracing_multiple_call_option_help =
        "Trace by trace class:\n"
        "  The argument to --trace is a trace class.  Specify the --trace option multiple\n"
-       "  times to activate multiple trace classes, e.g. \"--trace i2c --trace ddc\"\n"
+       "  times to activate multiple trace classes, e.g. \"--trace "I2C" --trace ddc\"\n"
        "  Valid trace classes are:  BASE, I2C, ADL, DDC, DDCIO, VCP, TOP, ENV, API, UDF, SLEEP, RETRY, ALL.\n"
        "  Trace class names are not case sensitive.\n"
        "  (Some trace classes are more useful than others.)\n"
@@ -444,6 +452,6 @@ char * maxtries_option_help =
       "    maximum write-only exchange count\n"
       "    maximum write-read exchange count\n"
       "    maximum multi-part-read exchange count\n"
-      "  A value of \"\" or \".\" leaves the default value unchanged\n"
-      "  e.g. --retries \",.,15\" changes only the maximum multi-part-read exchange count"
+      "  A value of \"0\" or \".\" leaves the default value unchanged\n"
+      "  e.g. --maxtries \".,.,15\" changes only the maximum multi-part-read exchange count"
       ;
