@@ -3,14 +3,14 @@
  *  Parse the command line using the glib goption functions.
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
 
 #include <assert.h>
 #include <config.h>
-#include <glib.h>
+#include <glib-2.0/glib.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,51 +31,28 @@
 
 
 // Variables used by callback functions
-static char *            adlwork       = NULL;
 static char *            usbwork       = NULL;
 static DDCA_Output_Level output_level  = DDCA_OL_NORMAL;
-static int               iAdapterIndex = -1;
-static int               iDisplayIndex = -1;
 static DDCA_Stats_Type   stats_work    = DDCA_STATS_NONE;
 
 
-// not currently used
-// Callback function for processing an --adl argument
-gboolean adl_arg_func(const gchar* option_name,
-                      const gchar* value,
-                      gpointer     data,
-                      GError**     error)
-{
-   bool debug = false;
-   DBGMSF(debug, "option_name=|%s|, value|%s|, data=%p", option_name, value, data);
-
-   adlwork = strdup(value);   // alt way
-   bool ok = parse_dot_separated_arg(value, &iAdapterIndex, &iDisplayIndex);
-   if (ok) {
-      DBGMSG("parsed adl = %d.%d", iAdapterIndex, iDisplayIndex);
-   }
-   if (!ok) {
-      g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_FAILED, "bad adl" );
-   }
-
-   return ok;
-}
-
-
 // Callback function for processing --terse, --verbose and synonyms
-gboolean output_arg_func(const gchar* option_name,
-                         const gchar* value,
-                         gpointer     data,
-                         GError**     error)
+static gboolean
+output_arg_func(const gchar* option_name,
+                const gchar* value,
+                gpointer     data,
+                GError**     error)
 {
    bool debug = false;
    DBGMSF(debug, "option_name=|%s|, value|%s|, data=%p", option_name, value, data);
    bool ok = true;
 
-   if (streq(option_name, "-v") || streq(option_name, "--verbose") )
+   if (      streq(option_name, "-v") || streq(option_name, "--verbose") )
       output_level = DDCA_OL_VERBOSE;
-   else if (streq(option_name, "-t")  || streq(option_name, "--terse") || streq(option_name, "--brief") )
+   else if ( streq(option_name, "-t")  || streq(option_name, "--terse") || streq(option_name, "--brief") )
       output_level = DDCA_OL_TERSE;
+   else if ( streq(option_name, "--vv") || streq(option_name, "--very-verbose") )
+      output_level = DDCA_OL_VV;
    else {
       PROGRAM_LOGIC_ERROR("Unexpected option_name: %s", option_name);
       g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_FAILED,
@@ -88,10 +65,11 @@ gboolean output_arg_func(const gchar* option_name,
 
 
 // Callback function for processing --stats
-gboolean stats_arg_func(const    gchar* option_name,
-                        const    gchar* value,
-                        gpointer data,
-                        GError** error)
+static gboolean
+stats_arg_func(const    gchar* option_name,
+               const    gchar* value,
+               gpointer data,
+               GError** error)
 {
    bool debug = false;
    DBGMSF(debug,"option_name=|%s|, value|%s|, data=%p", option_name, value, data);
@@ -128,20 +106,42 @@ gboolean stats_arg_func(const    gchar* option_name,
    return ok;
 }
 
+// #define FUTURE
+#ifdef FUTURE
+   gboolean debug_pre_parse_func(
+                        GOptionContext *context,
+                        GOptionGroup *group,
+                        gpointer data,
+                        GError **error)
+   {
+      DBGMSG("Executing");
+     return true;
+   }
+
+   gboolean debug_post_parse_func(
+                        GOptionContext *context,
+                        GOptionGroup *group,
+                        gpointer data,
+                        GError **error)
+   {
+      DBGMSG("Executing");
+     return true;
+   }
+#endif
+
+
 
 /* Primary parsing function
  *
- * Arguments:
- *    argc      number of command line arguments
- *    argv      array of pointers to command line arguments
- *
- * Returns:
- *    pointer to a Parsed_Cmd struct if parsing successful
- *    NULL if execution should be terminated
+ * \param   argc  number of command line arguments
+ * \param   argv  array of pointers to command line arguments
+ * \param   parser_mode called for ddcutil or libddcutil?
+ * \return  pointer to newly allocated Parsed_Cmd struct if parsing successful
+ *          NULL if execution should be terminated
  */
-Parsed_Cmd * parse_command(int argc, char * argv[]) {
+Parsed_Cmd * parse_command(int argc, char * argv[], Parser_Mode parser_mode) {
    bool debug = false;
-   DBGMSF(debug, "Starting" );
+   DBGMSF(debug, "Starting. parser_mode = %d", parser_mode );
    validate_cmdinfo();   // assertions
 
    if (debug) {
@@ -153,9 +153,14 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    }
 
    Parsed_Cmd * parsed_cmd = new_parsed_cmd();
+   parsed_cmd->parser_mode = parser_mode;
    // parsed_cmd->pdid = create_dispno_display_identifier(1);   // default monitor
    // DBGMSG("After new_parsed_cmd(), parsed_cmd->output_level_name = %s", output_level_name(parsed_cmd->output_level));
 
+   gchar * original_command = g_strjoinv(" ",argv);
+   DBGMSF(debug, "original command: %s", original_command);
+   parsed_cmd->raw_command = original_command;
+
 // gboolean stats_flag     = false;
    gboolean ddc_flag       = false;
    gboolean force_flag     = false;
@@ -163,7 +168,9 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    gboolean show_unsupported_flag = false;
    gboolean version_flag   = false;
    gboolean timestamp_trace_flag = false;
+   gboolean wall_timestamp_trace_flag = false;
    gboolean thread_id_trace_flag = false;
+   gboolean syslog_flag    = false;
    gboolean verify_flag    = false;
    gboolean noverify_flag  = false;
    gboolean nodetect_flag  = false;
@@ -173,13 +180,19 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    gboolean rw_only_flag   = false;
    gboolean ro_only_flag   = false;
    gboolean wo_only_flag   = false;
-   gboolean enable_udf_flag = false;
+   gboolean enable_udf_flag = DEFAULT_ENABLE_UDF;
+   const char * enable_udf_expl = (enable_udf_flag) ? "Enable User Define Features (default)" : "Enable User Define Features";
+   const char * disable_udf_expl = (enable_udf_flag) ? "Disable User Defined Features" : "Disable User Define Features (default)";
 #ifdef USE_USB
-   gboolean enable_usb_flag = true;
+   gboolean enable_usb_flag = DEFAULT_ENABLE_USB;
+   const char * enable_usb_expl = (enable_usb_flag) ? "Detect USB devices (default)" : "Detect USB devices";
+   const char * disable_usb_expl = (enable_usb_flag) ? "Ignore USB devices" : "Ignore USB devices (default)";
 #endif
    gboolean timeout_i2c_io_flag = false;
-   gboolean reduce_sleeps_flag  = false;
+   gboolean reduce_sleeps_flag  = DEFAULT_SLEEP_LESS;
    gboolean deferred_sleep_flag = false;
+   gboolean per_thread_stats_flag = false;
+   gboolean show_settings_flag = false;
    gboolean dsa_flag       = false;
    gboolean f1_flag        = false;
    gboolean f2_flag        = false;
@@ -187,7 +200,16 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    gboolean f4_flag        = false;
    gboolean f5_flag        = false;
    gboolean f6_flag        = false;
-   gboolean debug_parse_flag = false;
+   gboolean debug_parse_flag  = false;
+   gboolean x52_no_fifo_flag  = false;
+
+   gboolean enable_cc_flag = DEFAULT_ENABLE_CACHED_CAPABILITIES;
+   const char * enable_cc_expl =  (enable_cc_flag) ? "Enable cached capabilities (default)" : "Enable cached capabilities";
+   const char * disable_cc_expl = (enable_cc_flag) ? "Disable cached capabilities" : "Disable cached capabilities (default)";
+   // gboolean enable_cc_flag_set = false;
+   // gboolean disable_cc_flag_set = false;
+
+   // gboolean ignore_cc_flag = false;
    char *   mfg_id_work    = NULL;
    char *   modelwork      = NULL;
    char *   snwork         = NULL;
@@ -202,59 +224,89 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    gint     hidwork        = -1;
    gint     dispwork       = -1;
    char *   maxtrywork      = NULL;
+   gint     edid_read_size_work = -1;
    gint     i1_work = -1;
    char *   failsim_fn_work = NULL;
    // gboolean enable_failsim_flag = false;
    char *   sleep_multiplier_work = NULL;
 
-   GOptionEntry option_entries[] = {
+   GOptionEntry libddcutil_only_options[] = {
+         {"libddcutil-trace-file",
+                     '\0', 0, G_OPTION_ARG_STRING,   &parsed_cmd->library_trace_file,  "libddcutil trace file",  "file name"},
+         {NULL},
+   };
+
+   GOptionEntry ddcutil_only_options[] = {
+         //  Monitor selection options
+         {"display", 'd',  0, G_OPTION_ARG_INT,      &dispwork,         "Display number",              "number"},
+         {"dis",    '\0',  0, G_OPTION_ARG_INT,      &dispwork,         "Display number",              "number"},
+         {"bus",     'b',  0, G_OPTION_ARG_INT,      &buswork,          "I2C bus number",              "busnum" },
+         {"hiddev", '\0',  0, G_OPTION_ARG_INT,      &hidwork,          "hiddev device number",        "number" },
+         {"usb",     'u',  0, G_OPTION_ARG_STRING,   &usbwork,          "USB bus and device numbers",  "busnum.devicenum"},
+         {"mfg",     'g',  0, G_OPTION_ARG_STRING,   &mfg_id_work,      "Monitor manufacturer code",   "mfg_id"},
+         {"model",   'l',  0, G_OPTION_ARG_STRING,   &modelwork,        "Monitor model",               "model name"},
+         {"sn",      'n',  0, G_OPTION_ARG_STRING,   &snwork,           "Monitor serial number",       "serial number"},
+         {"edid",    'e',  0, G_OPTION_ARG_STRING,   &edidwork,         "Monitor EDID",            "256 char hex string" },
+
+         // Feature selection filters
+         {"show-unsupported",
+                     'U',  0, G_OPTION_ARG_NONE,     &show_unsupported_flag, "Report unsupported features", NULL},
+         {"notable", '\0', G_OPTION_FLAG_HIDDEN,
+                              G_OPTION_ARG_NONE,     &notable_flag,     "Exclude table type feature codes",  NULL},
+         {"no-table",'\0', 0, G_OPTION_ARG_NONE,     &notable_flag,     "Exclude table type feature codes",  NULL},
+         {"show-table",'\0',G_OPTION_FLAG_REVERSE,
+                              G_OPTION_ARG_NONE,     &notable_flag,     "Report table type feature codes",  NULL},
+         {"rw",      '\0', 0, G_OPTION_ARG_NONE,     &rw_only_flag,     "Include only RW features",         NULL},
+         {"ro",      '\0', 0, G_OPTION_ARG_NONE,     &ro_only_flag,     "Include only RO features",         NULL},
+         {"wo",      '\0', 0, G_OPTION_ARG_NONE,     &wo_only_flag,     "Include only WO features",         NULL},
+
+         // Output control
+         {"verbose", 'v',  G_OPTION_FLAG_NO_ARG,
+                              G_OPTION_ARG_CALLBACK, output_arg_func,   "Show extended detail",             NULL},
+         {"terse",   't',  G_OPTION_FLAG_NO_ARG,
+                              G_OPTION_ARG_CALLBACK, output_arg_func,   "Show brief detail",                NULL},
+         {"brief",   '\0', G_OPTION_FLAG_NO_ARG,
+                              G_OPTION_ARG_CALLBACK, output_arg_func,   "Show brief detail",                NULL},
+         {"vv",      '\0', G_OPTION_FLAG_NO_ARG | G_OPTION_FLAG_HIDDEN,
+                              G_OPTION_ARG_CALLBACK, output_arg_func,   "Show extra verbose detail",        NULL},
+         {"very-verbose", '\0', G_OPTION_FLAG_NO_ARG | G_OPTION_FLAG_HIDDEN,
+                              G_OPTION_ARG_CALLBACK, output_arg_func,   "Show extra verbose detail",        NULL},
+  // Program information
+         {"settings",'\0', 0, G_OPTION_ARG_NONE,     &show_settings_flag,"Show current settings",           NULL},
+         {"version", 'V',  0, G_OPTION_ARG_NONE,     &version_flag,     "Show ddcutil version",             NULL},
+      {NULL},
+   };
+
+
+   GOptionEntry common_options[] = {
    //  long_name short flags option-type          gpointer           description                    arg description
-   //  monitor selection options
-      {"display", 'd',  0, G_OPTION_ARG_INT,      &dispwork,         "Display number",              "number"},
-      {"dis",    '\0',  0, G_OPTION_ARG_INT,      &dispwork,         "Display number",              "number"},
-      {"bus",     'b',  0, G_OPTION_ARG_INT,      &buswork,          "I2C bus number",              "busnum" },
-      {"hiddev", '\0',  0, G_OPTION_ARG_INT,      &hidwork,          "hiddev device number",        "number" },
-#ifdef HAVE_ADL
-//    {"adl",     'a',  0, G_OPTION_ARG_CALLBACK, adl_arg_func,      "ADL adapter and display indexes", "adapterIndex.displayIndex"},
-      {"adl",     'a',  0, G_OPTION_ARG_STRING,   &adlwork,          "ADL adapter and display indexes", "adapterIndex.displayIndex"},
-#endif
-      {"usb",     'u',  0, G_OPTION_ARG_STRING,   &usbwork,          "USB bus and device numbers",  "busnum.devicenum"},
-      {"mfg",     'g',  0, G_OPTION_ARG_STRING,   &mfg_id_work,      "Monitor manufacturer code",   "mfg_id"},
-      {"model",   'l',  0, G_OPTION_ARG_STRING,   &modelwork,        "Monitor model",               "model name"},
-      {"sn",      'n',  0, G_OPTION_ARG_STRING,   &snwork,           "Monitor serial number",       "serial number"},
-      {"edid",    'e',  0, G_OPTION_ARG_STRING,   &edidwork,         "Monitor EDID",            "256 char hex string" },
-#ifdef USE_USB
-//    {"disable-usb", '\0', G_OPTION_FLAG_REVERSE,
-//                             G_OPTION_ARG_NONE, &enable_usb_flag,  "Do not detect USB devices", NULL},
-      {"nousb",   '\0', G_OPTION_FLAG_REVERSE,
-                               G_OPTION_ARG_NONE, &enable_usb_flag,  "Do not detect USB devices", NULL},
 
-#endif
-      // output control
+
+
+      // Diagnostic output
       {"ddc",     '\0', 0, G_OPTION_ARG_NONE,     &ddc_flag,         "Report DDC protocol and data errors", NULL},
-      {"verbose", 'v',  G_OPTION_FLAG_NO_ARG,
-                           G_OPTION_ARG_CALLBACK, output_arg_func,   "Show extended detail",             NULL},
-      {"terse",   't',  G_OPTION_FLAG_NO_ARG,
-                           G_OPTION_ARG_CALLBACK, output_arg_func,   "Show brief detail",                NULL},
-      {"brief",   '\0', G_OPTION_FLAG_NO_ARG,
-                           G_OPTION_ARG_CALLBACK, output_arg_func,   "Show brief detail",                NULL},
-
-      // feature selection filters
-      {"show-unsupported",
-                  'U',  0, G_OPTION_ARG_NONE,     &show_unsupported_flag, "Report unsupported features", NULL},
-      {"notable", '\0', G_OPTION_FLAG_HIDDEN,
-                           G_OPTION_ARG_NONE,     &notable_flag,     "Exclude table type feature codes",  NULL},
-      {"no-table",'\0', 0, G_OPTION_ARG_NONE,     &notable_flag,     "Exclude table type feature codes",  NULL},
-      {"show-table",'\0',G_OPTION_FLAG_REVERSE,
-                           G_OPTION_ARG_NONE,     &notable_flag,     "Report table type feature codes",  NULL},
-      {"rw",      '\0', 0, G_OPTION_ARG_NONE,     &rw_only_flag,     "Include only RW features",         NULL},
-      {"ro",      '\0', 0, G_OPTION_ARG_NONE,     &ro_only_flag,     "Include only RO features",         NULL},
-      {"wo",      '\0', 0, G_OPTION_ARG_NONE,     &wo_only_flag,     "Include only WO features",         NULL},
-
-      // tuning
-      {"maxtries",'\0', 0, G_OPTION_ARG_STRING,   &maxtrywork,       "Max try adjustment",  "comma separated list" },
       {"stats",   's',  G_OPTION_FLAG_OPTIONAL_ARG,
                            G_OPTION_ARG_CALLBACK, stats_arg_func,    "Show performance statistics",  "stats type"},
+      {"per-thread-stats",
+                  '\0', 0, G_OPTION_ARG_NONE,     &per_thread_stats_flag, "Include per-thread statistics",   NULL},
+
+
+
+      // Behavior options
+#ifdef USE_USB
+      {"enable-usb", '\0', G_OPTION_FLAG_NONE,
+                               G_OPTION_ARG_NONE, &enable_usb_flag,  enable_usb_expl, NULL},
+      {"disable-usb",'\0', G_OPTION_FLAG_REVERSE,
+                               G_OPTION_ARG_NONE, &enable_usb_flag,  disable_usb_expl, NULL},
+
+      {"nousb",   '\0', G_OPTION_FLAG_REVERSE,
+                               G_OPTION_ARG_NONE, &enable_usb_flag,  disable_usb_expl, NULL},
+#endif
+      {"mccs",    '\0', 0, G_OPTION_ARG_STRING,   &mccswork,         "MCCS version",            "major.minor" },
+      {"timeout-i2c-io",'\0', 0, G_OPTION_ARG_NONE, &timeout_i2c_io_flag, "Wrap I2C IO in timeout",  NULL},
+//    {"no-timeout-ddc-io",'\0',G_OPTION_FLAG_REVERSE,
+//                            G_OPTION_ARG_NONE,  &timeout_i2c_io_flag,   "Do not wrap DDC IO in timeout (default)",  NULL},
+
       {"force-slave-address",
                   '\0', 0, G_OPTION_ARG_NONE,     &force_slave_flag, "Force I2C slave address",         NULL},
       {"force",   'f',  G_OPTION_FLAG_HIDDEN,
@@ -263,65 +315,114 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       {"noverify",'\0', 0, G_OPTION_ARG_NONE,     &noverify_flag,    "Do not read VCP value after setting it", NULL},
 //    {"nodetect",'\0', 0, G_OPTION_ARG_NONE,     &nodetect_flag,    "Skip initial monitor detection",  NULL},
       {"async",   '\0', 0, G_OPTION_ARG_NONE,     &async_flag,       "Enable asynchronous display detection", NULL},
+      {"enable-capabilities-cache",
+                  '\0', 0, G_OPTION_ARG_NONE,     &enable_cc_flag,   enable_cc_expl,     NULL},
+      {"disable-capabilities-cache", '\0', G_OPTION_FLAG_REVERSE,
+                           G_OPTION_ARG_NONE,     &enable_cc_flag,   disable_cc_expl ,   NULL},
 
-      {"udf",     '\0', 0, G_OPTION_ARG_NONE,     &enable_udf_flag,  "Enable user defined feature support", NULL},
+      {"udf",     '\0', 0, G_OPTION_ARG_NONE,     &enable_udf_flag,  enable_udf_expl,    NULL},
+      {"enable-udf",'\0',0,G_OPTION_ARG_NONE,     &enable_udf_flag,  enable_udf_expl,    NULL},
       {"noudf",   '\0', G_OPTION_FLAG_REVERSE,
-                           G_OPTION_ARG_NONE,     &enable_udf_flag,  "Disable user defined feature support", NULL},
+                           G_OPTION_ARG_NONE,     &enable_udf_flag,  disable_udf_expl,   NULL},
+      {"disable-udf",'\0', G_OPTION_FLAG_REVERSE,
+                           G_OPTION_ARG_NONE,     &enable_udf_flag,  disable_udf_expl,   NULL},
+      {"x52-no-fifo",'\0',0,G_OPTION_ARG_NONE,    &x52_no_fifo_flag, "Feature x52 does have a FIFO queue", NULL},
 
-      // performance and retry
+      // Performance and retry
+      {"maxtries",'\0', 0, G_OPTION_ARG_STRING,   &maxtrywork,       "Max try adjustment",  "comma separated list" },
       {"sleep-multiplier", '\0', 0,
                            G_OPTION_ARG_STRING,   &sleep_multiplier_work, "Multiplication factor for DDC sleeps", "number"},
-      {"timeout-i2c-io",'\0', 0, G_OPTION_ARG_NONE, &timeout_i2c_io_flag, "Wrap I2C IO in timeout",  NULL},
-//    {"no-timeout-ddc-io",'\0',G_OPTION_FLAG_REVERSE,
-//                            G_OPTION_ARG_NONE,  &timeout_i2c_io_flag,   "Do not wrap DDC IO in timeout (default)",  NULL},
-      {"less-sleep" ,'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps",  NULL},
-      {"sleep-less" ,'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps",  NULL},
+
+      {"less-sleep" ,'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps (default)",  NULL},
+      {"sleep-less" ,'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps (default)",  NULL},
+      {"enable-sleep-less" ,'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps (default)",  NULL},
+      {"disable-sleep-less",'\0',G_OPTION_FLAG_REVERSE,
+                                     G_OPTION_ARG_NONE,  &reduce_sleeps_flag, "Do not eliminate any sleeps",  NULL},
 //    {"reduce-sleeps",'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps",  NULL},
 //    {"no-reduce-sleeps",'\0',G_OPTION_FLAG_REVERSE,
 //                               G_OPTION_ARG_NONE,  &reduce_sleeps_flag, "Do not eliminate any sleeps (default)",  NULL},
-      {"lazy-sleep", '\0', 0, G_OPTION_ARG_NONE, &deferred_sleep_flag, "Delay sleeps if possible",  NULL},
+
+      {"lazy-sleep",  '\0', 0, G_OPTION_ARG_NONE, &deferred_sleep_flag, "Delay sleeps if possible",  NULL},
 //    {"defer-sleeps",'\0', 0, G_OPTION_ARG_NONE, &deferred_sleep_flag, "Delay sleeps if possible",  NULL},
+
       {"dynamic-sleep-adjustment",'\0', 0, G_OPTION_ARG_NONE, &dsa_flag, "Enable dynamic sleep adjustment",  NULL},
       {"dsa",                     '\0', 0, G_OPTION_ARG_NONE, &dsa_flag, "Enable dynamic sleep adjustment",  NULL},
+      {"edid-read-size",
+                      '\0', 0, G_OPTION_ARG_INT,         &edid_read_size_work, "Number of EDID bytes to read", "128,256" },
+      {NULL},
+   };
+
+   GOptionEntry debug_options[] = {
+      // Debugging
+      {"excp",       '\0', 0, G_OPTION_ARG_NONE,         &report_freed_excp_flag, "Report freed exceptions", NULL},
+      {"trace",      '\0', 0, G_OPTION_ARG_STRING_ARRAY, &trace_classes,        "Trace classes",  "trace class name" },
+//    {"trace",      '\0', 0, G_OPTION_ARG_STRING,       &tracework,            "Trace classes",  "comma separated list" },
+      {"trcfunc",    '\0', 0, G_OPTION_ARG_STRING_ARRAY, &trace_functions,      "Trace functions","function name" },
+      {"trcfile",    '\0', 0, G_OPTION_ARG_STRING_ARRAY, &trace_filenames,      "Trace files",    "file name" },
+      {"timestamp",  '\0', 0, G_OPTION_ARG_NONE,         &timestamp_trace_flag, "Prepend trace msgs with elapsed time",  NULL},
+      {"ts",         '\0', 0, G_OPTION_ARG_NONE,         &timestamp_trace_flag, "Prepend trace msgs with elapsed time",  NULL},
+      {"wall-timestamp",
+                     '\0', 0, G_OPTION_ARG_NONE,         &wall_timestamp_trace_flag, "Prepend trace msgs with wall time",  NULL},
+      {"wts",        '\0', 0, G_OPTION_ARG_NONE,         &wall_timestamp_trace_flag, "Prepend trace msgs with wall time",  NULL},
+      {"thread-id",  '\0', 0, G_OPTION_ARG_NONE,         &thread_id_trace_flag, "Prepend trace msgs with thread id",  NULL},
+      {"tid",        '\0', 0, G_OPTION_ARG_NONE,         &thread_id_trace_flag, "Prepend trace msgs with thread id",  NULL},
+      {"syslog",     '\0', 0, G_OPTION_ARG_NONE,         &syslog_flag,           "Write trace messages to system log",  NULL},
+      {"debug-parse",'\0', 0,  G_OPTION_ARG_NONE,        &debug_parse_flag,     "Report parsed command",    NULL},
+      {"failsim",    '\0', 0,  G_OPTION_ARG_FILENAME,    &failsim_fn_work,      "Enable simulation", "control file name"},
 
 
-      // debugging
-      {"excp",    '\0', 0, G_OPTION_ARG_NONE,     &report_freed_excp_flag,  "Report freed exceptions", NULL},
-      {"trace",   '\0', 0, G_OPTION_ARG_STRING_ARRAY, &trace_classes, "Trace classes",         "trace class name" },
-//    {"trace",   '\0', 0, G_OPTION_ARG_STRING,   &tracework,        "Trace classes",          "comma separated list" },
-      {"trcfunc", '\0',0, G_OPTION_ARG_STRING_ARRAY, &trace_functions, "Trace functions",     "function name" },
-      {"trcfile", '\0',0, G_OPTION_ARG_STRING_ARRAY, &trace_filenames,    "Trace files",     "file name" },
-
-      {"timestamp",'\0',  0, G_OPTION_ARG_NONE,   &timestamp_trace_flag, "Prepend trace msgs with elapsed time",  NULL},
-      {"ts",      '\0',   0, G_OPTION_ARG_NONE,   &timestamp_trace_flag, "Prepend trace msgs with elapsed time",  NULL},
-      {"thread-id",'\0',  0, G_OPTION_ARG_NONE,   &thread_id_trace_flag, "Prepend trace msgs with thread id",  NULL},
-      {"tid",     '\0',   0, G_OPTION_ARG_NONE,   &thread_id_trace_flag, "Prepend trace msgs with thread id",  NULL},
-
-//    {"myusage", '\0', 0, G_OPTION_ARG_NONE,     &myusage_flag,     "Show usage", NULL},
-//    {"myhelp",  '\0', 0,  G_OPTION_ARG_NONE,     &myhelp_flag,      "Show usage", NULL},
-      {"i1",      '\0', 0,  G_OPTION_ARG_INT,      &i1_work, "special", "non-negative number" },
+      // Generic options to aid development
+      {"i1",      '\0', 0,  G_OPTION_ARG_INT,      &i1_work,         "special", "non-negative number" },
       {"f1",      '\0', 0,  G_OPTION_ARG_NONE,     &f1_flag,         "Special flag 1",    NULL},
       {"f2",      '\0', 0,  G_OPTION_ARG_NONE,     &f2_flag,         "Special flag 2",    NULL},
       {"f3",      '\0', 0,  G_OPTION_ARG_NONE,     &f3_flag,         "Special flag 3",    NULL},
       {"f4",      '\0', 0,  G_OPTION_ARG_NONE,     &f4_flag,         "Special flag 4",    NULL},
       {"f5",      '\0', 0,  G_OPTION_ARG_NONE,     &f5_flag,         "Special flag 5",    NULL},
       {"f6",      '\0', 0,  G_OPTION_ARG_NONE,     &f6_flag,         "Special flag 6",    NULL},
-      {"debug-parse", '\0', 0,  G_OPTION_ARG_NONE,  &debug_parse_flag,"Report parsed command",    NULL},
-      {"failsim", '\0', 0,  G_OPTION_ARG_FILENAME, &failsim_fn_work, "Enable simulation", "control file name"},
-
-      // other
-      {"version", 'V',  0, G_OPTION_ARG_NONE,     &version_flag,     "Show version information", NULL},
-      {"mccs",    '\0', 0, G_OPTION_ARG_STRING,   &mccswork,         "MCCS version",            "major.minor" },
+      {"s1",      '\0', 0,  G_OPTION_ARG_STRING,   &parsed_cmd->s1,  "Special string 1",  "string"},
+      {"s2",      '\0', 0,  G_OPTION_ARG_STRING,   &parsed_cmd->s2,  "Special string 2",  "string"},
+      {"s3",      '\0', 0,  G_OPTION_ARG_STRING,   &parsed_cmd->s3,  "Special string 3",  "string"},
+      {"s4",      '\0', 0,  G_OPTION_ARG_STRING,   &parsed_cmd->s4,  "Special string 4",  "string"},
+      {NULL},
+   };
 
+   GOptionEntry final_options[] = {
       {G_OPTION_REMAINING,
                  '\0', 0,  G_OPTION_ARG_STRING_ARRAY, &cmd_and_args, "ARGUMENTS description",   "command [arguments...]"},
       { NULL }
    };
 
+   GOptionGroup * all_options = g_option_group_new(
+         "group name", "group description", "help description", NULL, NULL);
+   if (parser_mode == MODE_DDCUTIL) {
+      g_option_group_add_entries(all_options, ddcutil_only_options);
+   }
+   g_option_group_add_entries(all_options, common_options);
+   if (parser_mode == MODE_LIBDDCUTIL) {
+      g_option_group_add_entries(all_options, libddcutil_only_options);
+   }
+#ifndef FUTURE
+   g_option_group_add_entries(all_options, debug_options);
+#endif
+   g_option_group_add_entries(all_options, final_options);
+
+#ifdef FUTURE
+   GOptionGroup * debug_option_group =
+   g_option_group_new("debug","Debug Options", "(debug options description)",
+                      NULL,   // user data
+                      NULL);  // function to free user data
+   g_option_group_add_entries(debug_option_group, debug_options);
+   g_option_group_set_parse_hooks(debug_option_group, debug_pre_parse_func, debug_post_parse_func);
+#endif
+
+
    GError* error = NULL;
    GOptionContext* context = g_option_context_new("- DDC query and manipulation");
-   g_option_context_add_main_entries(context, option_entries, NULL);
-   // g_option_context_add_group(context,  gtk_get_option_group(TRUE));
+   // g_option_context_add_main_entries(context, option_entries, NULL);
+   g_option_context_set_main_group(context, all_options);
+#ifdef FUTURE
+   g_option_context_add_group(context, debug_option_group);
+#endif
 
    // comma delimited list of trace identifiers:
    // char * trace_group_string = strjoin(trace_group_names, trace_group_ct, ", ");
@@ -364,10 +465,29 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    g_option_context_set_help_enabled(context, true);
    // bool ok = false;
 
-   bool ok = g_option_context_parse(context, &argc, &argv, &error);
+   /* From g_option_parse_documentation():
+      If the parsing is successful, any parsed arguments are removed from the
+      array and argc and argv are updated accordingly.
+
+      From g_option_parse_strv() documentation:
+      This function is similar to g_option_context_parse() except that it
+      respects the normal memory rules when dealing with a strv instead of
+      assuming that the passed-in array is the argv of the main function.
+      In particular, strings that are removed from the arguments list will
+      be freed using g_free().
+
+      Pass a mangleable copy of argv to g_option_context_parse_strv().
+   */
+   Null_Terminated_String_Array temp_argv = ntsa_copy(argv, true);
+   bool ok = g_option_context_parse_strv(context, &temp_argv, &error);
    if (!ok) {
-      fprintf(stderr, "Option parsing failed: %s\n", error->message);
+      char * mode_name = (parser_mode == MODE_DDCUTIL) ? "ddcutil" : "libddcutil";
+      if (error)
+         fprintf(stderr, "%s option parsing failed: %s\n", mode_name, error->message);
+      else
+         fprintf(stderr, "%s option parsing failed\n", mode_name);
    }
+   ntsa_free(temp_argv, true);
 
    // DBGMSG("buswork=%d", buswork);
    // DBGMSG("dispwork=%d", dispwork);
@@ -386,20 +506,29 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       ok = false;
    }
 
-
 #define SET_CMDFLAG(_bit, _flag) \
    do { \
       if (_flag) \
          parsed_cmd->flags |= _bit; \
    } while(0)
 
+#define SET_CLR_CMDFLAG(_bit, _flag) \
+   do { \
+      if (_flag) \
+         parsed_cmd->flags |= _bit; \
+      else \
+         parsed_cmd->flags &= ~_bit; \
+   } while(0)
+
    parsed_cmd->output_level     = output_level;
    parsed_cmd->stats_types      = stats_work;
    parsed_cmd->i1               = i1_work;
    SET_CMDFLAG(CMD_FLAG_DDCDATA,           ddc_flag);
    SET_CMDFLAG(CMD_FLAG_FORCE_SLAVE_ADDR,  force_slave_flag);
    SET_CMDFLAG(CMD_FLAG_TIMESTAMP_TRACE,   timestamp_trace_flag);
+   SET_CMDFLAG(CMD_FLAG_WALLTIME_TRACE,    wall_timestamp_trace_flag);
    SET_CMDFLAG(CMD_FLAG_THREAD_ID_TRACE,   thread_id_trace_flag);
+   SET_CMDFLAG(CMD_FLAG_SYSLOG,            syslog_flag);
    SET_CMDFLAG(CMD_FLAG_VERIFY,            verify_flag || !noverify_flag);
    // if (verify_flag || !noverify_flag)
    //    parsed_cmd->flags |= CMD_FLAG_VERIFY;
@@ -412,7 +541,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    SET_CMDFLAG(CMD_FLAG_RO_ONLY,           ro_only_flag);
    SET_CMDFLAG(CMD_FLAG_WO_ONLY,           wo_only_flag);
    SET_CMDFLAG(CMD_FLAG_FORCE,             force_flag);
-   SET_CMDFLAG(CMD_FLAG_ENABLE_UDF,        enable_udf_flag);
+   SET_CLR_CMDFLAG(CMD_FLAG_ENABLE_UDF,    enable_udf_flag);
 #ifdef USE_USB
    SET_CMDFLAG(CMD_FLAG_ENABLE_USB,        enable_usb_flag);
 #endif
@@ -426,6 +555,11 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    SET_CMDFLAG(CMD_FLAG_F4,                f4_flag);
    SET_CMDFLAG(CMD_FLAG_F5,                f5_flag);
    SET_CMDFLAG(CMD_FLAG_F6,                f6_flag);
+   SET_CMDFLAG(CMD_FLAG_X52_NO_FIFO,       x52_no_fifo_flag);
+   SET_CMDFLAG(CMD_FLAG_PER_THREAD_STATS,  per_thread_stats_flag);
+   SET_CMDFLAG(CMD_FLAG_SHOW_SETTINGS,     show_settings_flag);
+
+   SET_CLR_CMDFLAG(CMD_FLAG_ENABLE_CACHED_CAPABILITIES, enable_cc_flag);
 
    if (failsim_fn_work) {
 #ifdef ENABLE_FAILSIM
@@ -439,37 +573,15 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    }
 
 #undef SET_CMDFLAG
+#undef SET_CLR_CMDFLAG
 
+     // rpt_vstring(1, "(%s) show settings: %s",
+   //                             __func__, sbool(parsed_cmd->flags & CMD_FLAG_SHOW_SETTINGS));
 
    // Create display identifier
    //
    // n. at this point parsed_cmd->pdid == NULL
 
-   if (adlwork) {
-#ifdef HAVE_ADL
-      if (debug)
-         DBGMSG("adlwork = |%s|", adlwork);
-      int iAdapterIndex;
-      int iDisplayIndex;
-      bool adlok = parse_dot_separated_arg(adlwork, &iAdapterIndex, &iDisplayIndex);
-      if (!adlok) {
-          fprintf(stderr, "Invalid ADL argument: %s\n", adlwork );
-          ok = false;
-          // DBGMSG("After ADL parse, ok=%d", ok);
-      }
-      else {
-         // parsedCmd->dref = createAdlDisplayRef(iAdapterIndex, iDisplayIndex);
-         // free(parsed_cmd->pdid);
-         parsed_cmd->pdid = create_adlno_display_identifier(iAdapterIndex, iDisplayIndex);  // new way
-      }
-      explicit_display_spec_ct++;
-#else
-      fprintf(stderr, "ddcutil not built with support for AMD proprietary driver.  --adl option invalid.\n");
-      ok = false;
-#endif
-   }
-
-
    if (usbwork) {
 #ifdef USE_USB
       DBGMSF(debug, "usbwork = |%s|", usbwork);
@@ -585,6 +697,11 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
                    fprintf(stderr, "--maxtries value %d exceeds %d\n", ival, MAX_MAX_TRIES);
                    ok = false;
                 }
+                else if (ival < 0) {
+                   fprintf(stderr, "negative --maxtries value: %d\n", ival);
+                   ok = false;
+                }
+
                 else {
                    parsed_cmd->max_tries[ndx] = ival;
                 }
@@ -615,9 +732,8 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
           ok = false;
       }
       else {
-         // which?
          parsed_cmd->mccs_vspec = vspec;
-         parsed_cmd->mccs_version_id = mccs_version_spec_to_id(vspec);
+         // parsed_cmd->mccs_version_id = mccs_version_spec_to_id(vspec);
       }
    }
 
@@ -639,6 +755,17 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       }
    }
 
+   DBGMSF(debug, "edid_read_size_work = %d", edid_read_size_work);
+   if (edid_read_size_work !=  -1 &&
+       edid_read_size_work != 128 &&
+       edid_read_size_work !=   0 &&
+       edid_read_size_work != 256)
+   {
+      fprintf(stderr, "Invalid EDID read size: %d\n", edid_read_size_work);
+      ok = false;
+   }
+   else
+      parsed_cmd->edid_read_size = edid_read_size_work;
 
 #ifdef COMMA_DELIMITED_TRACE
    if (tracework) {
@@ -726,14 +853,9 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    }
 
    if (version_flag) {
-      printf("ddcutil %s\n", BUILD_VERSION);
+      printf("ddcutil %s\n", get_full_ddcutil_version());
       // TODO: patch values at link time
       // printf("Built %s at %s\n", BUILD_DATE, BUILD_TIME);
-#ifdef HAVE_ADL
-      printf("Built with support for AMD Display Library (AMD proprietary driver).\n");
-#else
-      printf("Built without support for AMD Display Library (AMD proprietary driver).\n");
-#endif
 #ifdef USE_USB
       printf("Built with support for USB connected displays.\n");
 #else
@@ -752,7 +874,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       puts("");
       // if no command specified, include license in version information and terminate
       if (rest_ct == 0) {
-         puts("Copyright (C) 2015-2020 Sanford Rockowitz");
+         puts("Copyright (C) 2015-2021 Sanford Rockowitz");
          puts("License GPLv2: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>");
          puts("This is free software: you are free to change and redistribute it.");
          puts("There is NO WARRANTY, to the extent permitted by law.");
@@ -771,13 +893,16 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    // else if (explicit_display_spec_ct == 0)
    //   parsed_cmd->pdid = create_dispno_display_identifier(1);   // default monitor
 
-
-   if (rest_ct == 0) {
+   if (parser_mode == MODE_LIBDDCUTIL && rest_ct > 0) {
+         fprintf(stderr, "Unrecognized configuration file options: %s\n", cmd_and_args[0]);
+         ok = false;
+   }
+   else if (ok && parser_mode == MODE_DDCUTIL && rest_ct == 0) {
       fprintf(stderr, "No command specified\n");
       ok = false;
    }
-   else {
-      char * cmd = cmd_and_args[0];;
+   if (ok && parser_mode == MODE_DDCUTIL) {
+      char * cmd = cmd_and_args[0];
       if (debug)
          printf("cmd=|%s|\n", cmd);
       Cmd_Desc * cmdInfo = find_command(cmd);
@@ -838,6 +963,49 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
             parsed_cmd->flags &= ~CMD_FLAG_WO_ONLY;
          }
 
+         // new way
+         if (ok && parsed_cmd->cmd_id == CMDID_SETVCP) {
+            // for (int argpos = 0; argpos < parsed_cmd->argct; argpos+=2) {
+            int argpos = 0;
+            while(argpos < parsed_cmd->argct) {
+               Parsed_Setvcp_Args psv;
+               bool feature_code_ok = any_one_byte_hex_string_to_byte_in_buf(
+                             parsed_cmd->args[argpos],
+                             &psv.feature_code);
+               if (!feature_code_ok) {
+                  fprintf(stderr, "Invalid feature code: %s\n", parsed_cmd->args[argpos]);
+                  ok = false;
+                  break;
+               }
+
+               argpos++;
+               if (argpos >= parsed_cmd->argct) {
+                  fprintf(stderr, "Missing feature value\n");
+                  ok = false;
+                  break;
+               }
+               psv.feature_value_type = VALUE_TYPE_ABSOLUTE;
+               if ( streq(parsed_cmd->args[argpos], "+") || streq(parsed_cmd->args[argpos], "-") )
+               {
+                  if ( streq(parsed_cmd->args[argpos], "+") )
+                     psv.feature_value_type = VALUE_TYPE_RELATIVE_PLUS;
+                  else
+                     psv.feature_value_type = VALUE_TYPE_RELATIVE_MINUS;
+                  argpos++;
+                  if (argpos >= parsed_cmd->argct) {
+                     fprintf(stderr, "Missing feature value\n");
+                     ok = false;
+                     break;
+                  }
+               }
+
+               psv.feature_value = strdup(parsed_cmd->args[argpos]);
+               g_array_append_val(parsed_cmd->setvcp_values, psv);
+               argpos++;
+            }
+         }
+
+#ifdef OLD
          if (ok && parsed_cmd->cmd_id == CMDID_SETVCP) {
             for (int argpos = 0; argpos < parsed_cmd->argct; argpos+=2) {
                // DBGMSG("argpos=%d, argct=%d", argpos, parsed_cmd->argct);
@@ -869,6 +1037,8 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
                }
             }
          }
+#endif
+
       }  // recognized command
    }
 
@@ -892,6 +1062,13 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       parsed_cmd = NULL;
    }
 
+   if (debug) {
+      DBGMSG("Before return: argc=%d", argc);
+      int ndx = 0;
+      for (; ndx < argc; ndx++) {
+         DBGMSG("argv[%d] = |%s|", ndx, argv[ndx]);
+      }
+   }
    DBGMSF(debug, "Returning: %p", parsed_cmd);
    return parsed_cmd;
 }
