@@ -3,7 +3,7 @@
  *  opened simultaneously from multiple threads.
  */
 
-// Copyright (C) 2018-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /*
@@ -29,6 +29,7 @@
 #include "util/string_util.h"
 
 #include "base/displays.h"
+#include "base/rtti.h"
 #include "base/status_code_mgt.h"
 
 #include "ddc/ddc_display_lock.h"
@@ -36,6 +37,8 @@
 #include "ddcutil_types.h"
 #include "ddcutil_status_codes.h"
 
+// Trace class for this file
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDCIO;
 
 #define DISTINCT_DISPLAY_DESC_MARKER "DDSC"
 typedef struct {
@@ -76,7 +79,7 @@ bool io_path_eq(DDCA_IO_Path path1, DDCA_IO_Path path2) {
 #endif
 
 
-bool display_desc_matches(Distinct_Display_Desc * ddesc, Display_Ref * dref) {
+static bool display_desc_matches(Distinct_Display_Desc * ddesc, Display_Ref * dref) {
    bool result = false;
    if (dpath_eq(ddesc->io_path, dref->io_path))
       result = true;
@@ -92,7 +95,7 @@ bool display_desc_matches(Distinct_Display_Desc * ddesc, Display_Ref * dref) {
       else {
          // pathological case, should be impossible but user report
          // re X260 laptop and Ultradock indicates possible (see note in ddc_open_display()
-         DBGMSG0("Null EDID");
+         DBGMSG("Null EDID");
       }
    }
 #endif
@@ -105,25 +108,22 @@ static GMutex descriptors_mutex;                // single threads access to disp
 static GMutex master_display_lock_mutex;
 
 
-void init_ddc_display_lock(void) {
-   display_descriptors= g_ptr_array_new();
-}
-
 // must be called when lock not held by current thread, o.w. deadlock
-char * distinct_display_ref_repr_t(Distinct_Display_Ref id) {
+static char * distinct_display_ref_repr_t(Distinct_Display_Ref id) {
    static GPrivate  repr_key = G_PRIVATE_INIT(g_free);
    char * buf = get_thread_fixed_buffer(&repr_key, 100);
    g_mutex_lock(&descriptors_mutex);
    Distinct_Display_Desc * ref = (Distinct_Display_Desc *) id;
    assert(memcmp(ref->marker, DISTINCT_DISPLAY_DESC_MARKER, 4) == 0);
-   g_snprintf(buf, 100, "Distinct_Display_Ref[%s]", dpath_repr_t(&ref->io_path));
+   g_snprintf(buf, 100, "Distinct_Display_Ref[%s @%p]", dpath_repr_t(&ref->io_path), ref);
    g_mutex_unlock(&descriptors_mutex);
    return buf;
 }
 
+
 Distinct_Display_Ref get_distinct_display_ref(Display_Ref * dref) {
    bool debug = false;
-   DBGMSF(debug, "Starting. dref=%s", dref_repr_t(dref));
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dref=%s", dref_repr_t(dref));
 
    void * result = NULL;
    g_mutex_lock(&descriptors_mutex);
@@ -151,7 +151,7 @@ Distinct_Display_Ref get_distinct_display_ref(Display_Ref * dref) {
 
    g_mutex_unlock(&descriptors_mutex);
 
-   DBGMSF(debug, "Done.  Returning: %p -> %s", result,  distinct_display_ref_repr_t(result));
+   DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %p -> %s", result,  distinct_display_ref_repr_t(result));
    return result;
 }
 
@@ -172,11 +172,11 @@ lock_distinct_display(
 {
    DDCA_Status ddcrc = 0;
    bool debug = false;
-   DBGMSF(debug, "Starting. id=%p -> %s", id, distinct_display_ref_repr_t(id));
+   DBGTRC_STARTING(debug, TRACE_GROUP, "id=%p -> %s", id, distinct_display_ref_repr_t(id));
 
    Distinct_Display_Desc * ddesc = (Distinct_Display_Desc *) id;
    // TODO:  If this function is exposed in API, change assert to returning illegal argument status code
-   assert(memcmp(ddesc->marker, DISTINCT_DISPLAY_DESC_MARKER, 4) == 0);
+   TRACED_ASSERT(memcmp(ddesc->marker, DISTINCT_DISPLAY_DESC_MARKER, 4) == 0);
    bool self_thread = false;
    g_mutex_lock(&master_display_lock_mutex);  //wrong - will hold lock during wait
    if (ddesc->display_mutex_thread == g_thread_self() )
@@ -200,19 +200,24 @@ lock_distinct_display(
          ddesc->display_mutex_thread = g_thread_self();
    }
    // need a new DDC status code
-   DBGMSF(debug, "Done.     id=%p -> %s, Returning: %s",
-                 id,distinct_display_ref_repr_t(id),  psc_desc(ddcrc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, ddcrc, "id=%p -> %s", id, distinct_display_ref_repr_t(id));
    return ddcrc;
 }
 
 
+/** Unlocks a distinct display.
+ *
+ *  \param id  distinct display identifier
+ *  \retval DDCRC_LOCKED attempting to unlock a display owned by a different thread
+ *  \retval DDCRC_OK
+ */
 DDCA_Status unlock_distinct_display(Distinct_Display_Ref id) {
    bool debug = false;
-   DBGMSF(debug, "Starting. id=%p -> %s", id, distinct_display_ref_repr_t(id));
+   DBGTRC_STARTING(debug, TRACE_GROUP, "id=%p -> %s", id, distinct_display_ref_repr_t(id));
    DDCA_Status ddcrc = 0;
    Distinct_Display_Desc * ddesc = (Distinct_Display_Desc *) id;
    // TODO:  If this function is exposed in API, change assert to returning illegal argument status code
-   assert(memcmp(ddesc->marker, DISTINCT_DISPLAY_DESC_MARKER, 4) == 0);
+   TRACED_ASSERT(memcmp(ddesc->marker, DISTINCT_DISPLAY_DESC_MARKER, 4) == 0);
    g_mutex_lock(&master_display_lock_mutex);
    if (ddesc->display_mutex_thread != g_thread_self()) {
       DBGMSG("Attempting to unlock display lock owned by different thread");
@@ -223,12 +228,38 @@ DDCA_Status unlock_distinct_display(Distinct_Display_Ref id) {
       g_mutex_unlock(&ddesc->display_mutex);
    }
    g_mutex_unlock(&master_display_lock_mutex);
-   DBGMSF(debug, "Done.     id=%p -> %s, Returning %s",
-                 id, distinct_display_ref_repr_t(id), psc_desc(ddcrc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, ddcrc, "id=%p -> %s", id, distinct_display_ref_repr_t(id));
    return ddcrc;
 }
 
 
+/** Unlocks all distinct displays.
+ *
+ *  The function is used during reinitialization.
+ */
+void unlock_all_distinct_displays() {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
+   g_mutex_lock(&master_display_lock_mutex);   // are both locks needed?
+   g_mutex_lock(&descriptors_mutex);
+    for (int ndx=0; ndx < display_descriptors->len; ndx++) {
+       Distinct_Display_Desc * cur = g_ptr_array_index(display_descriptors, ndx);
+       DBGTRC_NOPREFIX(debug, TRACE_GROUP, "%2d - %p  %-28s",
+                                           ndx, cur,
+                                           dpath_repr_t(&cur->io_path) );
+
+       g_mutex_unlock(&cur->display_mutex);
+    }
+    g_mutex_unlock(&descriptors_mutex);
+    g_mutex_unlock(&master_display_lock_mutex);
+    DBGTRC_DONE(debug, TRACE_GROUP, "");
+ }
+
+
+/** Emits a report of all distinct display descriptors.
+ *
+ *  \param depth logical indentation depth
+ */
 void dbgrpt_distinct_display_descriptors(int depth) {
    rpt_vstring(depth, "display_descriptors@%p", display_descriptors);
    g_mutex_lock(&descriptors_mutex);
@@ -250,3 +281,13 @@ void dbgrpt_distinct_display_descriptors(int depth) {
    g_mutex_unlock(&descriptors_mutex);
 }
 
+
+/** Initializes this module */
+void init_ddc_display_lock(void) {
+   display_descriptors= g_ptr_array_new();
+
+   RTTI_ADD_FUNC(get_distinct_display_ref);
+   RTTI_ADD_FUNC(lock_distinct_display);
+   RTTI_ADD_FUNC(unlock_distinct_display);
+   RTTI_ADD_FUNC(unlock_all_distinct_displays);
+}
