@@ -3,15 +3,16 @@
  * Load/store VCP settings from/to file.
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
+#include "config.h"
+
 /** \cond */
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <glib.h>
-#include <linux/limits.h>    // PATH_MAX, NAME_MAX
+#include <glib-2.0/glib.h>
 #include <stdbool.h>
 #include <string.h>
 #include <time.h>
@@ -31,6 +32,7 @@
 #include "base/displays.h"
 #include "base/monitor_model_key.h"
 #include "base/parms.h"
+#include "base/rtti.h"
 #include "base/status_code_mgt.h"
 #include "base/vcp_version.h"
 
@@ -117,7 +119,9 @@ create_dumpload_data_from_g_ptr_array(
    *dumpload_data_loc = NULL;
 
    Dumpload_Data * data = calloc(1, sizeof(Dumpload_Data));
+#ifndef NDEBUG
    bool valid_data = true;
+#endif
    // default:
    data->vcp_version.major = 2;
    data->vcp_version.minor = 0;
@@ -147,7 +151,9 @@ create_dumpload_data_from_g_ptr_array(
             //                       "Invalid data at line %d: %s", linectr, line);
             // errinfo_add_cause(errs, err);
             ADD_DATA_ERROR("Invalid data");
+#ifndef NDEBUG
             valid_data = false;
+#endif
          }
          else {
             rest = head + strlen(s0);;
@@ -172,20 +178,22 @@ create_dumpload_data_from_g_ptr_array(
                bool ok = str_to_int(s1, &ival, 10);
                if (ok && ival >= 0 && ival <= 65535)
                   data->product_code = (uint16_t) ival;
+#ifndef NDEBUG
                else
                   valid_data = false;
+#endif
             }
             else if (streq(s0, "EDID") || streq(s0, "EDIDSTR")) {
-               g_strlcpy(data->edidstr, s1, sizeof(data->edidstr));
+               STRLCPY(data->edidstr, s1, sizeof(data->edidstr));
             }
             else if (streq(s0, "MFG_ID")) {
-               g_strlcpy(data->mfg_id, s1, sizeof(data->mfg_id));
+               memcpy(data->mfg_id, s1, sizeof(data->mfg_id));
             }
             else if (streq(s0, "MODEL")) {
-               g_strlcpy(data->model, rest, sizeof(data->model));
+               STRLCPY(data->model, rest, sizeof(data->model));
             }
             else if (streq(s0, "SN")) {
-               g_strlcpy(data->serial_ascii, rest, sizeof(data->serial_ascii));
+               STRLCPY(data->serial_ascii, rest, sizeof(data->serial_ascii));
             }
             else if (streq(s0, "VCP_VERSION")) {
                data->vcp_version = parse_vspec(s1);
@@ -199,7 +207,9 @@ create_dumpload_data_from_g_ptr_array(
                   //                  DDCRC_BAD_DATA, __func__,
                   //                  "Invalid VCP VERSION at line %d: %s\n", linectr, line) );
                   ADD_DATA_ERROR("Invalid VCP VERSION");
+#ifndef NDEBUG
                   valid_data = false;
+#endif
                }
             }
             else if (streq(s0, "TIMESTAMP_TEXT")   ||
@@ -212,7 +222,9 @@ create_dumpload_data_from_g_ptr_array(
                if (ct != 3) {
                   // f0printf(ferr(), "Invalid VCP data at line %d: %s\n", linectr, line);
                   ADD_DATA_ERROR("Invalid VCP data");
+#ifndef NDEBUG
                   valid_data = false;
+#endif
                }
                else {   // found feature id and value
                   Byte feature_id;
@@ -220,7 +232,9 @@ create_dumpload_data_from_g_ptr_array(
                   if (!ok) {
                      // f0printf(ferr(), "Invalid opcode at line %d: %s", linectr, s1);
                      ADD_DATA_ERROR("Invalid  opcode");
+#ifndef NDEBUG
                      valid_data = false;
+#endif
                   }
                   else {     // valid opcode
                      DDCA_Any_Vcp_Value * valrec = NULL;
@@ -237,7 +251,7 @@ create_dumpload_data_from_g_ptr_array(
                            data->mfg_id, data->model, data->product_code);
 
                      Display_Feature_Metadata * dfm =
-                                              dyn_get_feature_metadata_by_mmk_and_vspec_dfm(
+                                              dyn_get_feature_metadata_by_mmk_and_vspec(
                                                    feature_id,
                                                    mmk,
                                                    data->vcp_version,
@@ -253,7 +267,9 @@ create_dumpload_data_from_g_ptr_array(
                            //          "Invalid hex string value for opcode at line %d: %s\n",
                            //          linectr, line);
                            ADD_DATA_ERROR("Invalid hex string value for opcode");
+#ifndef NDEBUG
                            valid_data = false;
+#endif
                         }
                         else {
                            valrec = create_table_vcp_value_by_bytes(
@@ -269,7 +285,9 @@ create_dumpload_data_from_g_ptr_array(
                         if (ct == 0) {
                            // f0printf(ferr(), "Invalid value for opcode at line %d: %s\n", linectr, line);
                            ADD_DATA_ERROR("Invalid value for opcode");
+#ifndef NDEBUG
                            valid_data = false;
+#endif
                         }
                         else {
                            // good opcode and value
@@ -297,7 +315,9 @@ create_dumpload_data_from_g_ptr_array(
                           errinfo_new2(
                                 DDCRC_BAD_DATA, __func__,
                                 "Unexpected field \"%s\" at line %d: %s", s0, linectr, line) );
+#ifndef NDEBUG
                valid_data = false;
+#endif
             }
          }    // more than 1 field on line
       }       // non-comment line
@@ -540,11 +560,11 @@ loadvcp_by_string(
 
 
 //
-// Dumpvcp
+// DUMPVCP
 //
 
 //
-// Support for dumpvcp command and returning profile info as string in API
+// Support for the DUMPVCP command and for returning profile info in the API
 //
 
 /** Formats a timestamp in a way usable in a filename, specifically:
@@ -561,9 +581,12 @@ loadvcp_by_string(
  */
 char *
 format_timestamp(time_t time_millis, char * buf, int bufsz) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_NONE, "buf=%p, bufsz=%d", buf, bufsz);
    if (bufsz == 0 || buf == NULL) {
       bufsz = 128;
       buf = calloc(1, bufsz);
+      DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Allocated new buffer: %p", buf);
    }
    struct tm tm = *localtime(&time_millis);
    snprintf(buf, bufsz, "%4d%02d%02d-%02d%02d%02d",
@@ -574,6 +597,7 @@ format_timestamp(time_t time_millis, char * buf, int bufsz) {
                   tm.tm_min,
                   tm.tm_sec
                  );
+   DBGTRC_DONE(debug, DDCA_TRC_NONE, "Returning: %p", buf);
    return buf;
 }
 
@@ -592,7 +616,7 @@ void collect_machine_readable_monitor_id(Display_Handle * dh, GPtrArray * vals)
    char buf[400];
    int bufsz = sizeof(buf)/sizeof(char);
 
-   Parsed_Edid * edid = ddc_get_parsed_edid_by_display_handle(dh);
+   Parsed_Edid * edid = ddc_get_parsed_edid_by_dh(dh);
    snprintf(buf, bufsz, "MFG_ID  %s",  edid->mfg_id);
    g_ptr_array_add(vals, strdup(buf));
    snprintf(buf, bufsz, "MODEL   %s",  edid->model_name);
@@ -619,6 +643,8 @@ void collect_machine_readable_monitor_id(Display_Handle * dh, GPtrArray * vals)
  */
 void
 collect_machine_readable_timestamp(time_t time_millis, GPtrArray* vals) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_NONE, "");
    // temporarily use same output format as filename, but format the
    // date separately here for flexibility
    char timestamp_buf[30];
@@ -634,6 +660,8 @@ collect_machine_readable_timestamp(time_t time_millis, GPtrArray* vals) {
    // Value is ignored on input, so just don't output it and avoid the issues. (11/2018)
    // snprintf(buf, bufsz, "TIMESTAMP_MILLIS %jd", time_millis);
    // g_ptr_array_add(vals, strdup(buf));
+
+   DBGTRC_DONE(debug, DDCA_TRC_NONE, "Appended %s", buf);
 }
 
 #ifdef UNUSED
@@ -686,33 +714,35 @@ collect_profile_related_values(
 
 /** Primary function for the DUMPVCP command.
  *
- * Writes DUMPVCP data to the in-core Dumpload_Data structure
+ *  Writes DUMPVCP data to the in-core Dumpload_Data structure
  *
- * \param   dh              display handle for connected display
- * \param   pdumpload_data  address at which to return pointer to newly allocated
- *                    Dumpload_Data struct.  It is the responsibility of the
- *                    caller to free this data structure.
- * \return status code
+ *  \param   dh                 display handle for connected display
+ *  \param   dumpload_data_loc  address at which to return pointer to newly allocated
+ *                             Dumpload_Data struct.  It is the responsibility of the
+ *                             caller to free this data structure.
+ *  \return status code
  */
 Public_Status_Code
 dumpvcp_as_dumpload_data(
       Display_Handle * dh,
-      Dumpload_Data** pdumpload_data)
+      Dumpload_Data** dumpload_data_loc)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting");
+   DBGMSF(debug, "Starting. dh=%s", dh_repr_t(dh));
    Public_Status_Code psc = 0;
    Dumpload_Data * dumped_data = calloc(1, sizeof(Dumpload_Data));
 
    // timestamp:
    dumped_data->timestamp_millis = time(NULL);
 
-   dumped_data->vcp_version = get_vcp_version_by_display_handle(dh);  // use function to ensure set
+   DBGMSF(debug, "Before get_vcp_version_by_dh()");
+   dumped_data->vcp_version = get_vcp_version_by_dh(dh);  // use function to ensure set
 
    // identification information from edid:
-   // Parsed_Edid * edid = ddc_get_parsed_edid_by_display_handle(dh);
+   // Parsed_Edid * edid = ddc_get_parsed_edid_by_dh(dh);
    Parsed_Edid * edid = dh->dref->pedid;
    assert(edid);
+   DBGMSF(debug, "Have EDID");
 
    dumped_data->product_code = edid->product_code;
    memcpy(dumped_data->mfg_id, edid->mfg_id, sizeof(dumped_data->mfg_id));
@@ -741,10 +771,11 @@ dumpvcp_as_dumpload_data(
    if (psc != 0 && dumped_data)
       free(dumped_data);
    else
-      *pdumpload_data = dumped_data;
+      *dumpload_data_loc = dumped_data;
+
    if (debug) {
-      DBGMSG("Returning: %s, *pdumpload_data=%p", psc_desc(psc), *pdumpload_data);
-      dbgrpt_dumpload_data(*pdumpload_data, 1);
+      DBGMSG("Returning: %s, *pdumpload_data=%p", psc_desc(psc), *dumpload_data_loc);
+      dbgrpt_dumpload_data(*dumpload_data_loc, 1);
    }
    return psc;
 }
@@ -808,32 +839,37 @@ convert_dumpload_data_to_string_array(Dumpload_Data * data) {
 }
 
 
-/** Returns the output of the DUMPVCP command a single string.
+/** Returns the output of the DUMPVCP command in a single string.
  *  Each field is separated by a semicolon.
  *
  *  The caller is responsible for freeing the returned string.
  *
- *  \param  dh       display handle of open monitor
- *  \param  pstring  location at which to return string
+ *  \param  dh          display handle of open monitor
+ *  \param  result_loc  location at which to return string
  *  \return status code
  */
-// n. called from ddct_public.c
-// move to glib_util.c?
+// n. called from api_feature_access.c
 Public_Status_Code
-dumpvcp_as_string(Display_Handle * dh, char ** pstring) {
+dumpvcp_as_string(Display_Handle * dh, char ** result_loc) {
    bool debug = false;
-   DBGMSF(debug, "Starting");
+   DBGMSF(debug, "Starting, dh=%s", dh_repr_t(dh));
 
    Public_Status_Code psc    = 0;
    Dumpload_Data *    data   = NULL;
-   *pstring = NULL;
+   *result_loc = NULL;
 
    psc = dumpvcp_as_dumpload_data(dh, &data);
    if (psc == 0) {
       GPtrArray * strings = convert_dumpload_data_to_string_array(data);
-      *pstring = join_string_g_ptr_array(strings, ";");
+      *result_loc = join_string_g_ptr_array(strings, ";");
       free_dumpload_data(data);
    }
-   DBGMSF(debug, "Returning: %s, *pstring=|%s|", psc_desc(psc), *pstring);
+   DBGMSF(debug, "Returning: %s, *result_loc=|%s|", psc_desc(psc), *result_loc);
    return psc;
 }
+
+
+void init_ddc_dumpload() {
+   RTTI_ADD_FUNC(format_timestamp);
+   RTTI_ADD_FUNC(collect_machine_readable_timestamp);
+}
