@@ -4,7 +4,7 @@
  * Capabilities.
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -58,8 +58,8 @@ try_multi_part_read(
       Buffer *         accumulator)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP,
-          "Starting. request_type=0x%02x, request_subtype=x%02x, all_zero_response_ok=%s, accumulator=%p",
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+          "request_type=0x%02x, request_subtype=x%02x, all_zero_response_ok=%s, accumulator=%p",
           request_type, request_subtype, sbool(all_zero_response_ok), accumulator);
 
    const int MAX_FRAGMENT_SIZE = 32;
@@ -79,7 +79,7 @@ try_multi_part_read(
    int  cur_offset = 0;
    bool complete   = false;
    while (!complete && !excp) {         // loop over fragments
-      DBGTRC(debug, DDCA_TRC_NONE, "Top of fragment loop");
+      DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Top of fragment loop");
 
       int fragment_size;
       update_ddc_multi_part_read_request_packet_offset(request_packet_ptr, cur_offset);
@@ -98,7 +98,7 @@ try_multi_part_read(
            &response_packet_ptr
           );
       psc = (excp) ? excp->status_code : 0;
-      DBGTRC(debug, DDCA_TRC_NONE,
+      DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
              "ddc_write_read_with_retry() request_type=0x%02x, request_subtype=0x%02x, returned %s",
              request_type, request_subtype, errinfo_summary(excp));
       TUNED_SLEEP_WITH_TRACE(dh, SE_AFTER_EACH_CAP_TABLE_SEGMENT, NULL);
@@ -122,17 +122,17 @@ try_multi_part_read(
 
       int display_current_offset = aux_data_ptr->fragment_offset;
       if (display_current_offset != cur_offset) {
-         DBGTRC(debug, DDCA_TRC_NONE,
+         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
                 "display_current_offset %d != cur_offset %d", display_current_offset, cur_offset);
          psc = DDCRC_MULTI_PART_READ_FRAGMENT;
          excp = errinfo_new(psc, __func__);
          COUNT_STATUS_CODE(psc);
       }
       else {
-         DBGTRC(debug, DDCA_TRC_NONE, "display_current_offset = %d matches cur_offset", display_current_offset);
+         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "display_current_offset = %d matches cur_offset", display_current_offset);
 
          fragment_size = aux_data_ptr->fragment_length;         // ***
-         DBGTRC(debug, DDCA_TRC_NONE, "fragment_size = %d", fragment_size);
+         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "fragment_size = %d", fragment_size);
          if (fragment_size == 0) {
             complete = true;   // redundant
          }
@@ -152,7 +152,7 @@ try_multi_part_read(
 
    free_ddc_packet(request_packet_ptr);
 
-   DBGTRC(debug, TRACE_GROUP, "Returning %s", errinfo_summary(excp));
+   DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, excp, "");
    return excp;
 }
 
@@ -181,8 +181,8 @@ multi_part_read_with_retry(
 {
    bool debug = false;
    Retry_Op_Value max_multi_part_read_tries = try_data_get_maxtries2(MULTI_PART_READ_OP);
-   DBGTRC(debug, TRACE_GROUP,
-          "Starting.  request_type=0x%02x, request_subtype=0x%02x, all_zero_response_ok=%s"
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+          "request_type=0x%02x, request_subtype=0x%02x, all_zero_response_ok=%s"
           ", max_multi_part_read_tries=%d",
           request_type, request_subtype, sbool(all_zero_response_ok), max_multi_part_read_tries);
 
@@ -197,7 +197,7 @@ multi_part_read_with_retry(
    Buffer * accumulator = buffer_new(2048, "multi part read buffer");
 
    while (tryctr < max_multi_part_read_tries && rc < 0 && can_retry) {
-      DBGTRC(debug, DDCA_TRC_NONE,
+      DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
              "Start of while loop. try_ctr=%d, max_multi_part_read_tries=%d",
              tryctr, max_multi_part_read_tries);
 
@@ -242,7 +242,7 @@ multi_part_read_with_retry(
       tryctr++;
    }
    assert( (rc<0 && ddc_excp) || (rc==0 && !ddc_excp) );
-   DBGTRC(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, rc=%d. ddc_excp=%s",
+   DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, rc=%d. ddc_excp=%s",
                             tryctr, rc, errinfo_summary(ddc_excp));
 
    if (debug) {
@@ -273,7 +273,8 @@ multi_part_read_with_retry(
    try_data_record_tries2(MULTI_PART_READ_OP, rc, tryctr);
 
    *buffer_loc = accumulator;
-   DBGTRC(debug, TRACE_GROUP, "Returning: %s", errinfo_summary(ddc_excp));
+   ASSERT_IFF(ddc_excp, !*buffer_loc);
+   DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, ddc_excp, "*buffer_loc=%p", *buffer_loc);
    return ddc_excp;
 }
 
@@ -295,8 +296,8 @@ try_multi_part_write(
    bool debug = false;
    Byte request_type = DDC_PACKET_TYPE_TABLE_WRITE_REQUEST;
    Byte request_subtype = vcp_code;
-   DBGTRC(debug, TRACE_GROUP,
-          "Starting. request_type=0x%02x, request_subtype=x%02x, accumulator=%p",
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+          "request_type=0x%02x, request_subtype=x%02x, accumulator=%p",
           request_type, request_subtype, value_to_set);
 
    Public_Status_Code psc = 0;
@@ -332,7 +333,7 @@ try_multi_part_write(
       }
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", errinfo_summary(ddc_excp));
+   DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, ddc_excp, "");
    assert( (ddc_excp && psc<0) || (!ddc_excp && psc==0) );
    return ddc_excp;
 }
@@ -355,7 +356,7 @@ multi_part_write_with_retry(
    bool debug = false;
    if (IS_TRACING())
       puts("");
-   DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s, vcp_code=0x%02x",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s, vcp_code=0x%02x",
                               dh_repr_t(dh), vcp_code);
 
    Public_Status_Code rc = -1;   // dummy value for first call of while loop
@@ -367,7 +368,7 @@ multi_part_write_with_retry(
    bool can_retry = true;
 
    while (tryctr < max_multi_part_write_tries && rc < 0 && can_retry) {
-      DBGTRC(debug, TRACE_GROUP,
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP,
              "Start of while loop. try_ctr=%d, max_multi_part_write_tries=%d",
              tryctr, max_multi_part_write_tries);
 
@@ -400,7 +401,7 @@ multi_part_write_with_retry(
       }
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", errinfo_summary(ddc_excp));
+   DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, ddc_excp, "");
    return ddc_excp;
 }
 
