@@ -1,7 +1,7 @@
 /** \file ddc_output.c
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -10,16 +10,14 @@
 /** \cond */
 #include <assert.h>
 #include <errno.h>
-#include <glib.h>
+#include <glib-2.0/glib.h>
 #include <string.h>
 #include <time.h>
 
-#include "../i2c/i2c_strategy_dispatcher.h"
 #include "util/error_info.h"
 #include "util/report_util.h"
 /** \endcond */
 
-#include "base/adl_errors.h"
 #include "base/ddc_errno.h"
 #include "base/ddc_packets.h"
 #include "base/linux_errno.h"
@@ -28,7 +26,7 @@
 #include "base/sleep.h"
 
 #include "i2c/i2c_bus_core.h"
-#include "adl/adl_shim.h"
+#include "i2c/i2c_strategy_dispatcher.h"
 
 #ifdef USE_USB
 #include "usb/usb_displays.h"
@@ -77,13 +75,13 @@ const char* FMT_CODE_NAME_DETAIL_W_NL  = "VCP code 0x%02x (%-30s): %s\n";
  *    false if not
  */
 bool
-is_table_feature_by_display_handle(
+is_table_feature_by_dh(
       VCP_Feature_Table_Entry *  frec,
       Display_Handle *           dh)
 {
    // bool debug = false;
    bool result = false;
-   DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
+   DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_dh(dh);
    DDCA_Version_Feature_Flags feature_flags = get_version_sensitive_feature_flags(frec, vcp_version);
    assert(feature_flags);
    result = (feature_flags & DDCA_TABLE);
@@ -124,7 +122,7 @@ check_valid_operation_by_feature_id_and_dh(
    if (!frec)
       result = DDCRC_UNKNOWN_FEATURE;
    else {
-      DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
+      DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_dh(dh);
       result = check_valid_operation_by_feature_rec_and_version(frec, vcp_version, operation_flags);
    }
    return result;
@@ -163,17 +161,17 @@ get_raw_value_for_feature_metadata(
    assert(dh->dref);
 
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. frec=%p");
+   DBGTRC_STARTING(debug, TRACE_GROUP, "frec=%p, feature_code=0x%02x", frec, (frec) ? frec->feature_code : 0x00);
 
    // Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
 
-   // DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
+   // DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_dh(dh);
    // char * feature_name = get_version_sensitive_feature_name(frec, vspec);
    char * feature_name = frec->feature_name;
 
    Byte feature_code = frec->feature_code;
-   // bool is_table_feature = is_table_feature_by_display_handle(frec, dh);
+   // bool is_table_feature = is_table_feature_by_dh(frec, dh);
    bool is_table_feature = frec->feature_flags & DDCA_TABLE;
    DDCA_Vcp_Value_Type feature_type = (is_table_feature) ? DDCA_TABLE_VCP_VALUE : DDCA_NON_TABLE_VCP_VALUE;
    DDCA_Output_Level output_level = get_output_level();
@@ -306,15 +304,15 @@ get_raw_value_for_feature_metadata(
    *pvalrec = valrec;
    ASSERT_IFF(!ddc_excp, *pvalrec);;
 
-   if (debug || IS_TRACING()) {
-      if (ddc_excp) {
-         DBGMSG("Done.     Returning %s", errinfo_summary(ddc_excp));
-      }
-      else {
-         DBGMSG("Done.     Returning NULL, *pvalrec -> ");
-         dbgrpt_single_vcp_value(*pvalrec, 3);
-      }
+   if (ddc_excp) {
+      DBGTRC_DONE(debug, TRACE_GROUP, "Returning %s", errinfo_summary(ddc_excp));
    }
+   else {
+      DBGTRC_DONE(debug, TRACE_GROUP, "Returning NULL, *pvalrec -> ");
+      if (debug || IS_TRACING())
+         dbgrpt_single_vcp_value(*pvalrec, 2);
+   }
+
    return ddc_excp;
 }
 
@@ -337,12 +335,12 @@ get_raw_value_for_feature_metadata_dfm(
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
 
-   // DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
+   // DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_dh(dh);
    // char * feature_name = get_version_sensitive_feature_name(frec, vspec);
    char * feature_name = frec->feature_name;
 
    Byte feature_code = frec->feature_code;
-   // bool is_table_feature = is_table_feature_by_display_handle(frec, dh);
+   // bool is_table_feature = is_table_feature_by_dh(frec, dh);
    bool is_table_feature = frec->feature_flags & DDCA_TABLE;
    DDCA_Vcp_Value_Type feature_type = (is_table_feature) ? DDCA_TABLE_VCP_VALUE : DDCA_NON_TABLE_VCP_VALUE;
    DDCA_Output_Level output_level = get_output_level();
@@ -460,22 +458,22 @@ get_raw_value_for_feature_metadata_dfm(
 Public_Status_Code
 collect_raw_feature_set_values2_dfm(
       Display_Handle *      dh,
-      Dyn_Feature_Set*       feature_set,
+      Dyn_Feature_Set*      feature_set,
       Vcp_Value_Set         vset,
       bool                  ignore_unsupported,  // if false, is error if unsupported
       FILE *                msg_fh)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting.");
+   DBGMSF(debug, "Starting. dh=%s, msg_fh=%p", dh_repr_t(dh), msg_fh);
 
    Public_Status_Code master_status_code = 0;
-   int features_ct = dyn_get_feature_set_size2_dfm(feature_set);
+   int features_ct = dyn_get_feature_set_size2(feature_set);
    // needed when called from C API, o.w. get get NULL response for first feature
    // DBGMSG("Inserting sleep() before first call to get_raw_value_for_feature_table_entry()");
    // sleep_millis_with_trace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, "initial");
    int ndx;
    for (ndx=0; ndx< features_ct; ndx++) {
-      Display_Feature_Metadata * dfm = dyn_get_feature_set_entry2_dfm(feature_set, ndx);
+      Display_Feature_Metadata * dfm = dyn_get_feature_set_entry2(feature_set, ndx);
       DBGMSF(debug,"ndx=%d, feature = 0x%02x", ndx, dfm->feature_code);
       DDCA_Any_Vcp_Value *  pvalrec;
       // DDCA_Feature_Metadata * ddca_meta = dfm_to_ddca_feature_metadata(dfm);
@@ -508,6 +506,7 @@ collect_raw_feature_set_values2_dfm(
       }
    }
 
+   DBGMSF(debug, "Done.  Returning: %s", psc_desc(master_status_code));
    return master_status_code;
 }
 
@@ -533,25 +532,20 @@ ddc_collect_raw_subset_values(
         FILE *              msg_fh)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting.  subset=%d  dh=%s", subset, dh_repr(dh) );
+   DBGMSF(debug, "Starting.  subset=%s  dh=%s, msg_fn=%p",
+                 feature_subset_name(subset), dh_repr(dh), msg_fh );
 
    assert(subset == VCP_SUBSET_PROFILE);  // currently the only use of this function,
                                           // will need to reconsider handling of Feature_Set_Flags if other
                                           // uses arise
-
    Public_Status_Code psc = 0;
-
-   // DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
-   // DBGMSG("VCP version = %d.%d", vcp_version.major, vcp_version.minor);
-
    Feature_Set_Flags flags = FSF_NOTABLE;
    if (subset == VCP_SUBSET_PROFILE)
       flags |= FSF_RW_ONLY;
-   Dyn_Feature_Set * feature_set = dyn_create_feature_set2_dfm(
+   Dyn_Feature_Set * feature_set = dyn_create_feature_set2(
                                      subset,
                                      dh->dref,          // vcp_version,
-                                     FSF_NOTABLE);
-                                //   false);      // exclude_table_features
+                                     flags);
    if (debug)
       dbgrpt_dyn_feature_set(feature_set, true, 0);
 
@@ -574,12 +568,12 @@ ddc_collect_raw_subset_values(
  *  a formatted interpretation of the value.
  *
  * \param  dh         handle for open display
- * \param  internal_metadata
+ * \param  dfm        feature metadata
  * \param  suppress_unsupported
  *                    if true, do not report unsupported features
  * \param  prefix_value_with_feature_code
  *                    include feature code in formatted value
- * \param  pformatted_value
+ * \param  formatted_value_loc
  *                    where to return pointer to formatted value
  * \param msg_fh      where to write extended messages for verbose
  *                    value retrieval, etc.
@@ -590,7 +584,7 @@ ddc_collect_raw_subset_values(
  * earlier code.  It needs refactoring.
  */
 Public_Status_Code
-ddc_get_formatted_value_for_display_feature_metadata(
+ddc_get_formatted_value_for_dfm(
       Display_Handle *            dh,
       Display_Feature_Metadata *  dfm,
       bool                        suppress_unsupported,
@@ -599,18 +593,21 @@ ddc_get_formatted_value_for_display_feature_metadata(
       FILE *                      msg_fh)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. suppress_unsupported=%s", sbool(suppress_unsupported));
+   DBGTRC_STARTING(debug, TRACE_GROUP, "suppress_unsupported=%s", sbool(suppress_unsupported));
 
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp;
    *formatted_value_loc = NULL;
 
-   DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
+   DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_dh(dh);
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "vspec=%d.%d", vspec.major, vspec.minor);
    // DDCA_Feature_Metadata* extmeta = dfm_to_ddca_feature_metadata(dfm);
    Byte feature_code = dfm->feature_code;
    char * feature_name = dfm->feature_name;
    bool is_table_feature = dfm->feature_flags & DDCA_TABLE;
+#ifndef NDEBUG
    DDCA_Vcp_Value_Type feature_type = (is_table_feature) ? DDCA_TABLE_VCP_VALUE : DDCA_NON_TABLE_VCP_VALUE;
+#endif
    DDCA_Output_Level output_level = get_output_level();
    if (output_level >= DDCA_OL_VERBOSE) {
       fprintf(msg_fh, "\nGetting data for %s VCP code 0x%02x - %s:\n",
@@ -689,17 +686,28 @@ ddc_get_formatted_value_for_display_feature_metadata(
          bool ok;
          char * formatted_data = NULL;
 
-         ok = dyn_format_feature_detail_dfm(
+         ok = dyn_format_feature_detail(
                  dfm,
                  vspec,
                  pvalrec,
                  &formatted_data);
          // DBGMSG("vcp_format_feature_detail set formatted_data=|%s|", formatted_data);
          if (!ok) {
+            char msg[100];
+            if (pvalrec->value_type == DDCA_NON_TABLE_VCP_VALUE) {
+               g_snprintf(
+                     msg, 100,
+                     "!!! UNABLE TO FORMAT OUTPUT. mh=0x%02x, ml=0x%02x, sh=0x%02x, sl=0x%02x",
+                     pvalrec->val.c_nc.mh,  pvalrec->val.c_nc.ml,
+                     pvalrec->val.c_nc.sh,  pvalrec->val.c_nc.sl);
+            }
+            else {
+               strcpy(msg,  "!!! UNABLE TO FORMAT OUTPUT");
+            }
             f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
-                            feature_code, feature_name, "!!! UNABLE TO FORMAT OUTPUT");
+                            feature_code, feature_name, msg);
             psc = DDCRC_INTERPRETATION_FAILED;
-            ddc_excp = errinfo_new(DDCRC_INTERPRETATION_FAILED, __func__);
+            ddc_excp = errinfo_new2(DDCRC_INTERPRETATION_FAILED, __func__, msg);
             // TODO: retry with default output function
          }
 
@@ -728,9 +736,9 @@ ddc_get_formatted_value_for_display_feature_metadata(
    if (pvalrec)
       free_single_vcp_value(pvalrec);
 
-   DBGTRC(debug, TRACE_GROUP,
-          "Done.      Returning: %s, *formatted_value_loc=%p",
-          psc_desc(psc), formatted_value_loc);
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc,
+          "*formatted_value_loc=%p -> %s",
+          *formatted_value_loc, *formatted_value_loc);
 
    ASSERT_IFF(psc == 0, !ddc_excp);
    ERRINFO_FREE_WITH_REPORT(ddc_excp, debug || IS_TRACING() || report_freed_exceptions);
@@ -764,14 +772,14 @@ show_feature_set_values2_dfm(
        show_unsupported = true;
    bool suppress_unsupported = !show_unsupported;
 
-   // DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
+   // DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_dh(dh);
    bool prefix_value_with_feature_code = true;    // TO FIX
    FILE * msg_fh = outf;                        // TO FIX
-   int features_ct = dyn_get_feature_set_size2_dfm(feature_set);
+   int features_ct = dyn_get_feature_set_size2(feature_set);
    DBGMSF(debug, "features_ct=%d", features_ct);
    int ndx;
    for (ndx=0; ndx< features_ct; ndx++) {
-      Display_Feature_Metadata * dfm = dyn_get_feature_set_entry2_dfm(feature_set, ndx);
+      Display_Feature_Metadata * dfm = dyn_get_feature_set_entry2(feature_set, ndx);
       // DDCA_Feature_Metadata * extmeta = ifm->external_metadata;
       DBGMSF(debug,"ndx=%d, feature = 0x%02x", ndx, dfm->feature_code);
       if ( !(dfm->feature_flags & DDCA_READABLE) ) {
@@ -797,7 +805,7 @@ show_feature_set_values2_dfm(
 
             char * formatted_value = NULL;
             Public_Status_Code psc =
-            ddc_get_formatted_value_for_display_feature_metadata(
+            ddc_get_formatted_value_for_dfm(
                   dh,
                   dfm,
                   suppress_unsupported,
@@ -879,17 +887,15 @@ ddc_show_vcp_values(
         Byte_Bit_Flags      features_seen)
 {
    bool debug = false;
-   if (debug || IS_TRACING()) {
-      char * s0 = feature_set_flag_names_t(flags);
-      DBGMSG("Starting.  subset=%d, flags=%s,  dh=%s", subset, s0, dh_repr(dh) );
-   }
+   DBGTRC_STARTING(debug, TRACE_GROUP, "subset=%d, flags=%s,  dh=%s",
+                   subset, feature_set_flag_names_t(flags), dh_repr(dh) );
 
    Public_Status_Code psc = 0;
 
-   // DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
+   // DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_dh(dh);
    // DBGMSG("VCP version = %d.%d", vcp_version.major, vcp_version.minor);
 
-   Dyn_Feature_Set* feature_set = dyn_create_feature_set2_dfm(
+   Dyn_Feature_Set* feature_set = dyn_create_feature_set2(
                                     subset,
                                     dh->dref,   // vcp_version,
                                     flags);
@@ -911,14 +917,15 @@ ddc_show_vcp_values(
    psc = show_feature_set_values2_dfm(
             dh, feature_set, collector, flags, features_seen);
    dyn_free_feature_set(feature_set);
-   DBGTRC(debug, TRACE_GROUP, "Done. Returning %s", psc_desc(psc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc, "");
    return psc;
 }
 
+
 static void init_ddc_output_func_name_table() {
 #define ADD_FUNC(_NAME) rtti_func_name_table_add(_NAME, #_NAME);
    ADD_FUNC(get_raw_value_for_feature_metadata);
-   ADD_FUNC(ddc_get_formatted_value_for_display_feature_metadata);
+   ADD_FUNC(ddc_get_formatted_value_for_dfm);
 #undef ADD_FUNC
 }
 
