@@ -4,7 +4,7 @@
  *  or the ADL API, as appropriate.  Handles I2C bus retry.
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 // N. ddc_open_display() and ddc_close_display() handle case USB, but the
@@ -16,21 +16,21 @@
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <i2c/i2c_strategy_dispatcher.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-// #include <sys/ioctl.h>
 #include <unistd.h>
 
 #include "public/ddcutil_types.h"
 
 #include "util/debug_util.h"
+#include "util/report_util.h"
 #include "util/string_util.h"
 #include "util/utilrpt.h"
 /** \endcond */
 
+#include "base/core.h"
 #include "base/ddc_errno.h"
 #include "base/displays.h"
 #include "base/dynamic_sleep.h"
@@ -42,7 +42,7 @@
 #include "base/thread_sleep_data.h"
 
 #include "i2c/i2c_bus_core.h"
-#include "adl/adl_shim.h"
+#include "i2c/i2c_strategy_dispatcher.h"
 
 #ifdef USE_USB
 #include "usb/usb_displays.h"
@@ -57,6 +57,8 @@
 // Trace class for this file
 static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDCIO;
 
+static GHashTable * open_displays = NULL;
+
 #ifdef DEPRECATED
 // Deprecated - use all_bytes_zero() in string_util.c
 // Tests if a range of bytes is entirely 0
@@ -85,6 +87,34 @@ bool is_ddc_null_message(Byte * packet) {
 #endif
 
 
+bool
+ddc_is_valid_display_handle(Display_Handle * dh) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%p", dh);
+   assert(open_displays);
+   bool result = g_hash_table_contains(open_displays, dh);
+   DBGTRC_DONE(debug, TRACE_GROUP, "Rreturning %s. dh=%p", sbool(result), dh);
+   return result;
+}
+
+
+void ddc_dbgrpt_valid_display_handles(int depth) {
+   rpt_vstring(depth, "Valid display handle = open_displays:");
+   assert(open_displays);
+   GList * display_handles = g_hash_table_get_keys(open_displays);
+   if (g_list_length(display_handles) > 0) {
+      for (GList * cur = display_handles; cur; cur = cur->next) {
+         Display_Handle * dh = cur->data;
+         rpt_vstring(depth+1, "%p -> %s", dh, dh_repr_t(dh));
+      }
+   }
+   else {
+      rpt_vstring(depth+1, "None");
+   }
+   g_list_free(display_handles);
+}
+
+
 //
 // Open/Close Display
 //
@@ -97,6 +127,7 @@ bool is_ddc_null_message(Byte * packet) {
  *  \return status code     as from #i2c_open_bus(), #usb_open_hiddev_device()
  *  \retval DDCRC_LOCKED    display open in another thread
  *  \retval DDCRC_ALREADY_OPEN display already open in current thread
+ *  \retval -EBUSY          from i2c_set_addr()
  *
  *  **Call_Option** flags recognized:
  *  - CALLOPT_WAIT
@@ -109,9 +140,10 @@ ddc_open_display(
       Display_Handle** dh_loc)
 {
    bool debug = false;
-   DBGMSF(debug, "Opening display %s, callopts=%s",
-                 dref_repr_t(dref), interpret_call_options_t(callopts) );
-   assert(dh_loc);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "Opening display %s, callopts=%s, dh_loc=%p",
+                      dref_repr_t(dref), interpret_call_options_t(callopts), dh_loc );
+   TRACED_ASSERT(dh_loc);
+   // TRACED_ASSERT(1==5);    // for testing
 
    Display_Handle * dh = NULL;
    DDCA_Status ddcrc = 0;
@@ -126,27 +158,40 @@ ddc_open_display(
       ddcrc = DDCRC_LOCKED;          // is there an appropriate errno value?  EBUSY? EACCES?
       goto bye;
    }
+   // DBGMSF(debug, "lockrc = %s, DREF_OPEN = %s", psc_desc(lockrc), sbool(dref->flags&DREF_OPEN));
+   // assumes there is only one Display_Ref for a display
+   // DREF_OPEN flag will not be set if caller used a different Display_Ref on this open call
+   // TRACED_ASSERT_IFF( ddcrc == DDCRC_ALREADY_OPEN, dref->flags & DREF_OPEN);
 
-   if (dref->flags & DREF_OPEN) {
-      assert(ddcrc == DDCRC_ALREADY_OPEN);   // from lock_distinct_display()
+  //  if (dref->flags & DREF_OPEN) {
+  //     ddcrc = DDCRC_ALREADY_OPEN;
+  //     goto bye;
+  //  }
+
+   if (lockrc == DDCRC_ALREADY_OPEN) {
       ddcrc = DDCRC_ALREADY_OPEN;
       goto bye;
    }
 
-   assert(ddcrc == 0);
    switch (dref->io_path.io_mode) {
 
    case DDCA_IO_I2C:
       {
+         I2C_Bus_Info * bus_info = dref->detail;
+         TRACED_ASSERT(bus_info);   // need to convert to a test?
+         TRACED_ASSERT( bus_info && memcmp(bus_info, I2C_BUS_INFO_MARKER, 4) == 0);
+
          int fd = i2c_open_bus(dref->io_path.path.i2c_busno, callopts);
          if (fd < 0) {
             ddcrc = fd;
          }
          else {
-            DBGMSF(debug, "Calling set_addr(0x37) for %s", dref_repr_t(dref));
+            // DBGMSF(debug, "Calling set_addr(0x37) for %s", dref_repr_t(dref));
             ddcrc =  i2c_set_addr(fd, 0x37, callopts);
             if (ddcrc != 0) {
-               assert(ddcrc < 0);
+               TRACED_ASSERT(ddcrc < 0);
+               if (ddcrc == -EBUSY)
+                  bus_info->flags |= I2C_BUS_BUSY;
                close(fd);
             }
 
@@ -154,19 +199,14 @@ ddc_open_display(
                // Is this needed?
                // 10/24/15, try disabling:
                // sleepMillisWithTrace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, NULL);
-
-               dh = create_bus_display_handle_from_display_ref(fd, dref);    // n. sets dh->dref = dref
-
-               I2C_Bus_Info * bus_info = dref->detail;
-               assert(bus_info);   // need to convert to a test?
-               assert( memcmp(bus_info, I2C_BUS_INFO_MARKER, 4) == 0);
+               dh = create_base_display_handle(fd, dref);    // n. sets dh->dref = dref
 
                dref->pedid = bus_info->edid;
                if (!dref->pedid) {
                   // How is this even possible?
                   // 1/2017:  Observed with x260 laptop and Ultradock, See ddcutil user report.
                   //          close(fd) fails
-                  DBGMSG("No EDID for device on bus /dev/i2c-%d", dref->io_path.path.i2c_busno);
+                  DBGMSG("No EDID for device on bus /dev/"I2C"-%d", dref->io_path.path.i2c_busno);
                   close(fd);
                   ddcrc = DDCRC_EDID;
                   free_display_handle(dh);
@@ -178,39 +218,43 @@ ddc_open_display(
       break;
 
    case DDCA_IO_ADL:
-      dh = create_adl_display_handle_from_display_ref(dref);  // n. sets dh->dref = dref
-      dref->pedid = adlshim_get_parsed_edid_by_display_handle(dh);
+      PROGRAM_LOGIC_ERROR("Case DDCA_IO_ADL");
       break;
 
    case DDCA_IO_USB:
 #ifdef USE_USB
       {
-         DBGMSF(debug, "Opening USB device: %s", dref->usb_hiddev_name);
-         assert(dref->usb_hiddev_name);
+         DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Opening USB device: %s", dref->usb_hiddev_name);
+         TRACED_ASSERT(dref && dref->usb_hiddev_name);
          // if (!dref->usb_hiddev_name) { // HACK
          //    DBGMSG("HACK FIXUP.  dref->usb_hiddev_name");
-         //    dref->usb_hiddev_name = get_hiddev_devname_by_display_ref(dref);
+         //    dref->usb_hiddev_name = get_hiddev_devname_by_dref(dref);
          // }
          int fd = usb_open_hiddev_device(dref->usb_hiddev_name, callopts);
          if (fd < 0) {
             ddcrc = fd;
          }
          else {
-            dh = create_usb_display_handle_from_display_ref(fd, dref);
-            dref->pedid = usb_get_parsed_edid_by_display_handle(dh);
+            dh = create_base_display_handle(fd, dref);
+            dref->pedid = usb_get_parsed_edid_by_dh(dh);
          }
       }
 #else
       PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
+      assert(false);   // avoid coverity error re null dreference
 #endif
       break;
    } // switch
-   assert(!dh || dh->dref->pedid);
+   TRACED_ASSERT_IFF(ddcrc == 0, dh);
+   TRACED_ASSERT(!dh || dh->dref->pedid);
 
    if (ddcrc == 0) {
       if (dref->io_path.io_mode != DDCA_IO_USB)
          TUNED_SLEEP_WITH_TRACE(dh, SE_POST_OPEN, NULL);
       dref->flags |= DREF_OPEN;
+      // protect with lock?
+      TRACED_ASSERT(open_displays);
+      g_hash_table_add(open_displays, dh);
    }
    else {
       unlock_distinct_display(ddisp_ref);
@@ -221,10 +265,10 @@ bye:
       COUNT_STATUS_CODE(ddcrc);
    }
    *dh_loc = dh;
-   assert(ddcrc <= 0);
-   assert( (ddcrc == 0 && *dh_loc) || (ddcrc < 0 && !*dh_loc) );
+   TRACED_ASSERT(ddcrc <= 0);
+   TRACED_ASSERT( (ddcrc == 0 && *dh_loc) || (ddcrc < 0 && !*dh_loc) );
    // dbgrpt_distinct_display_descriptors(0);
-   DBGMSF(debug, "Done.  Returning: %s, *dh_loc=%s", psc_desc(ddcrc), dh_repr_t(*dh_loc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, ddcrc, "*dh_loc=%s", dh_repr_t(*dh_loc));
    return ddcrc;
 }
 
@@ -240,8 +284,9 @@ bye:
 Status_Errno
 ddc_close_display(Display_Handle * dh) {
    bool debug = false;
-   DBGMSF(debug, "Starting. dh=%s, dref=%s, fd=%d, dpath=%s",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s, dref=%s, fd=%d, dpath=%s",
               dh_repr_t(dh), dref_repr_t(dh->dref), dh->fd, dpath_short_name_t(&dh->dref->io_path) ) ;
+   Display_Ref * dref = dh->dref;
    Status_Errno rc = 0;
    if (dh->fd == -1) {
       rc = DDCRC_INVALID_OPERATION;    // or DDCRC_ARG?
@@ -250,9 +295,9 @@ ddc_close_display(Display_Handle * dh) {
       switch(dh->dref->io_path.io_mode) {
       case DDCA_IO_I2C:
          {
-            rc = i2c_close_bus(dh->fd, dh->dref->io_path.path.i2c_busno,  CALLOPT_NONE);    // return error if failure
+            rc = i2c_close_bus(dh->fd, CALLOPT_NONE);
             if (rc != 0) {
-               assert(rc < 0);
+               TRACED_ASSERT(rc < 0);
                DBGMSG("i2c_close_bus returned %d, errno=%s", rc, psc_desc(errno) );
                COUNT_STATUS_CODE(rc);
             }
@@ -267,7 +312,7 @@ ddc_close_display(Display_Handle * dh) {
          {
             rc = usb_close_device(dh->fd, dh->dref->usb_hiddev_name, CALLOPT_NONE); // return error if failure
             if (rc != 0) {
-               assert(rc < 0);
+               TRACED_ASSERT(rc < 0);
                DBGMSG("usb_close_device returned %d", rc);
                COUNT_STATUS_CODE(rc);
             }
@@ -283,12 +328,28 @@ ddc_close_display(Display_Handle * dh) {
    dh->dref->flags &= (~DREF_OPEN);
    Distinct_Display_Ref display_id = get_distinct_display_ref(dh->dref);
    unlock_distinct_display(display_id);
+   assert(open_displays);
+   g_hash_table_remove(open_displays, dh);
 
    free_display_handle(dh);
-   DBGMSF(debug, "Done.     Returning: %s", psc_desc(rc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, rc, "dref=%s", dref_repr_t(dref));
    return rc;
 }
 
+void ddc_close_all_displays() {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
+   assert(open_displays);
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Closing %d open displays", g_hash_table_size(open_displays));
+   GList * display_handles = g_hash_table_get_keys(open_displays);
+   for (GList * cur = display_handles; cur; cur = cur->next) {
+      Display_Handle * dh = cur->data;
+      ddc_close_display(dh);
+   }
+   // open_displays should be empty at this point
+   TRACED_ASSERT(g_hash_table_size(open_displays) == 0);
+   DBGTRC_DONE(debug, TRACE_GROUP, "");
+}
 
 
 
@@ -338,21 +399,26 @@ DDCA_Status ddc_i2c_write_read_raw(
         )
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s, readbuf=%p",dh_repr_t(dh), readbuf);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s, readbuf=%p",dh_repr_t(dh), readbuf);
    // DBGMSG("request_packet_ptr=%p", request_packet_ptr);
    // dump_packet(request_packet_ptr);
 
-   assert(dh);
-   assert(dh->dref);
-   assert(dh->dref->io_path.io_mode == DDCA_IO_I2C);
-   // ASSERT_DISPLAY_IO_MODE(dh, DDCA_IO_DEVI2C);
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "request_packet_ptr->raw_bytes: %s",
+                              hexstring3_t(request_packet_ptr->raw_bytes->bytes,
+                                           request_packet_ptr->raw_bytes->len,
+                                           " ", 1, false) );
+   TRACED_ASSERT(dh);
+   TRACED_ASSERT(dh->dref);
+   TRACED_ASSERT(dh && dh->dref && dh->dref->io_path.io_mode == DDCA_IO_I2C);
 
 #ifdef TEST_THAT_DIDNT_WORK
    bool single_byte_reads = false;   // doesn't work
 #endif
 
+#ifndef NDEBUG
    Byte slave_addr = request_packet_ptr->raw_bytes->bytes[0];      // 0x6e
-   assert(slave_addr >> 1 == 0x37);
+   TRACED_ASSERT(slave_addr >> 1 == 0x37);
+#endif
 
    CHECK_DEFERRED_SLEEP(dh);
    Status_Errno_DDC rc =
@@ -361,7 +427,7 @@ DDCA_Status ddc_i2c_write_read_raw(
                            0x37,
                            get_packet_len(request_packet_ptr)-1,
                            get_packet_start(request_packet_ptr)+1 );
-   DBGMSF(debug, "invoke_i2c_writer() returned %d\n", rc);
+   DBGMSF(debug, "invoke_i2c_writer() returned %d", rc);
    if (rc == 0) {
       TUNED_SLEEP_WITH_TRACE(dh, SE_WRITE_TO_READ, NULL);
       // tuned_sleep_i2c_with_trace(SE_WRITE_TO_READ, __func__, NULL);
@@ -376,6 +442,9 @@ DDCA_Status ddc_i2c_write_read_raw(
       // try adding sleep to see if improves capabilities read for P2411H
       // tuned_sleep_i2c_with_trace(SE_POST_READ, __func__, NULL);
       TUNED_SLEEP_WITH_TRACE(dh, SE_POST_READ, NULL);
+      if (rc == 0)
+         DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Response bytes: %s",
+                                hexstring3_t(readbuf, max_read_bytes, " ", 1, false) );
 
       if (rc == 0 && all_bytes_zero(readbuf, max_read_bytes)) {
          DDCMSG(debug, "All zero response detected in %s", __func__);
@@ -389,88 +458,12 @@ DDCA_Status ddc_i2c_write_read_raw(
       COUNT_STATUS_CODE(rc);
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Done. psc=%s", psc_desc(rc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, rc, "");
    return rc;
 }
 
 
-/* Writes a DDC request packet to an ADL display,
- * and returns the raw response.
- *
- * Arguments:
- *   dh               display handle ADL device
- *   request_packet_ptr   DDC packet to write
- *   max_read_bytes   maximum number of bytes to read
- *   readbuf          where to return response
- *   pbytes_received  where to write count of bytes received
- *
- * Returns:
- *   0 if success
- *   modulated ADL status code otherwise
- *
- *   Negative ADL status codes indicate errors
- *   Positive values indicate success but with
- *   additional information.  Never seen.  How to handle?
- */
-
-static DDCA_Status ddc_adl_write_read_raw(
-      Display_Handle * dh,
-      DDC_Packet *     request_packet_ptr,
-      int              max_read_bytes,
-      Byte *           readbuf,
-      int *            pbytes_received
-     )
-{
-   bool debug = false;
-   DBGTRC(debug, TRACE_GROUP,
-          "Starting. Using adl_ddc_write_only() and adl_ddc_read_only() dh=%s",
-          dh_repr_t(dh));
-   assert(dh && dh->dref && dh->dref->io_path.io_mode == DDCA_IO_ADL);
-   // ASSERT_DISPLAY_IO_MODE(dh, DDCA_IO_ADL);
-
-   DDCA_Status psc = adlshim_ddc_write_only(
-                               dh,
-                               get_packet_start(request_packet_ptr),   // n. no adjustment, unlike i2c version
-                               get_packet_len(request_packet_ptr)
-                              );
-   if (psc < 0) {
-      DBGTRC(debug, TRACE_GROUP, "adl_ddc_write_only() returned gsc=%d\n", psc);
-   }
-   else {
-      // tuned_sleep(DDCA_IO_ADL, SE_WRITE_TO_READ);
-      TUNED_SLEEP_WITH_TRACE(dh, SE_WRITE_TO_READ, NULL);
-      psc = adlshim_ddc_read_only(
-            dh,
-            readbuf,
-            pbytes_received);
-      // note_io_event(IE_READ_AFTER_WRITE, __func__);
-      if (psc < 0) {
-         DBGTRC(debug, TRACE_GROUP, "adl_ddc_read_only() returned %d\n", psc);
-      }
-      else {
-         if ( all_bytes_zero(readbuf+1, max_read_bytes-1)) {
-                 psc = DDCRC_READ_ALL_ZERO;
-                 DDCMSG(debug, "All zero response.");
-                 COUNT_STATUS_CODE(psc);
-         }
-         else if (memcmp(get_packet_start(request_packet_ptr), readbuf, get_packet_len(request_packet_ptr)) == 0) {
-            // is this a DDC error or a programming bug?
-            DDCMSG(debug, "Bytes read same as bytes written.", __func__ );
-            psc = COUNT_STATUS_CODE(DDCRC_DDC_DATA);   // was DDCRC_READ_EQUALS_WRITE
-         }
-         else {
-            psc = 0;
-         }
-      }
-   }
-
-   if (psc < 0)
-      log_status_code(psc, __func__);
-   DBGTRC(debug, TRACE_GROUP, "Done. rc=%s\n", psc_desc(psc));
-   return psc;
-}
-
-
+// TODO: eliminate this function, used to route I2C vs ADL calls
 // static  // allow function to appear in backtrace
 DDCA_Status ddc_write_read_raw(
       Display_Handle * dh,
@@ -482,45 +475,32 @@ DDCA_Status ddc_write_read_raw(
      )
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s, readbuf=%p, max_read_bytes=%d",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s, readbuf=%p, max_read_bytes=%d",
                               dh_repr_t(dh), readbuf, max_read_bytes);
    if (debug) {
-      // DBGMSG0("request_packet_ptr->raw_bytes:");
+      // DBGMSG("request_packet_ptr->raw_bytes:");
       // dbgrpt_buffer(request_packet_ptr->raw_bytes, 1);
       char * s =  hexstring3_t(request_packet_ptr->raw_bytes->bytes,
                               request_packet_ptr->raw_bytes->len, " ", 1, false );
       DBGMSG("request_packet_ptr->raw_bytes: %s", s);
    }
 
-   DDCA_Status psc;
-
    // This function should not be called for USB
-   assert(dh->dref->io_path.io_mode == DDCA_IO_I2C || dh->dref->io_path.io_mode == DDCA_IO_ADL);
+   TRACED_ASSERT(dh && dh->dref && dh->dref->io_path.io_mode == DDCA_IO_I2C);
 
-   if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
-        psc =  ddc_i2c_write_read_raw(
-              dh,
-              request_packet_ptr,
-              read_bytewise,
-              max_read_bytes,
-              readbuf,
-              p_rcvd_bytes_ct
-       );
-   }
-   else {
-      psc =  ddc_adl_write_read_raw(
-              dh,
-              request_packet_ptr,
-              max_read_bytes,
-              readbuf,
-              p_rcvd_bytes_ct
-       );
-   }
+   DDCA_Status psc =  ddc_i2c_write_read_raw(
+                 dh,
+                 request_packet_ptr,
+                 read_bytewise,
+                 max_read_bytes,
+                 readbuf,
+                 p_rcvd_bytes_ct
+              );
 
-   DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", psc_desc(psc));
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc, "");
    if (psc == 0) {
-      DBGTRC(debug, TRACE_GROUP,
-             "      readbuf: %s",
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP,
+             "readbuf: %s",
              hexstring3_t(readbuf, *p_rcvd_bytes_ct, " ", 4, false));
    }
    return psc;
@@ -553,7 +533,8 @@ ddc_write_read(
      )
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s", dh_repr_t(dh) );
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s, read_bytewise=%s, max_read_bytes=%d",
+                 dh_repr_t(dh), sbool(read_bytewise), max_read_bytes );
 
    Byte * readbuf = calloc(1, max_read_bytes);
    int    bytes_received = max_read_bytes;
@@ -578,7 +559,7 @@ ddc_write_read(
               expected_subtype,
               __func__,
               response_packet_ptr_loc);
-       DBGTRC(debug, TRACE_GROUP,
+       DBGTRC_NOPREFIX(debug, TRACE_GROUP,
               "create_ddc_typed_response_packet() returned %s, *response_packet_ptr_loc=%p",
               ddcrc_desc_t(psc), *response_packet_ptr_loc );
 
@@ -600,14 +581,13 @@ ddc_write_read(
    if (psc < 0)
       excp = errinfo_new(psc, __func__);
 
-   if (debug || IS_TRACING()) {
-      if (excp) {
-         DBGMSG("Done.     Returning: %s", errinfo_summary(excp)  );
-      }
-      else {
-         DBGMSG("Done.     Returning: NULL, *response_packet_ptr_loc ->");
-         dbgrpt_packet(*response_packet_ptr_loc, 3);
-      }
+   if (excp) {
+      DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %s", errinfo_summary(excp)  );
+   }
+   else {
+      DBGTRC_DONE(debug, TRACE_GROUP, "Returning: NULL, *response_packet_ptr_loc ->");
+      if (debug || IS_TRACING())
+         dbgrpt_packet(*response_packet_ptr_loc, 2);
    }
 
    return excp;
@@ -648,9 +628,9 @@ ddc_write_read_with_retry(
         )
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s, all_zero_response_ok=%s",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s, all_zero_response_ok=%s",
           dh_repr_t(dh), sbool(all_zero_response_ok)  );
-   assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
+   TRACED_ASSERT(dh->dref->io_path.io_mode != DDCA_IO_USB);
    // show_backtrace(1);
 
    // if (debug)
@@ -671,9 +651,9 @@ ddc_write_read_with_retry(
           sbool(retry_null_response), ddcrc_null_response_max);
    Error_Info * try_errors[MAX_MAX_TRIES];
 
-   // assert(max_write_read_exchange_tries > 0);   // to avoid clang warning
+   // TRACED_ASSERT(max_write_read_exchange_tries > 0);   // to avoid clang warning
    int max_tries = try_data_get_maxtries2(WRITE_READ_TRIES_OP);
-   assert(max_tries >= 0);
+   TRACED_ASSERT(max_tries >= 0);
    for (tryctr=0, psc=-999, retryable=true;
         tryctr < max_tries && psc < 0 && retryable;
         tryctr++)
@@ -700,7 +680,7 @@ ddc_write_read_with_retry(
       try_errors[tryctr] = cur_excp;
 
       if (psc == 0 && ddcrc_null_response_ct > 0) {
-         DBGTRC(debug, TRACE_GROUP | DDCA_TRC_RETRY,
+         DBGTRC_NOPREFIX(debug, TRACE_GROUP | DDCA_TRC_RETRY,
                 "%s, ddc_write_read() succeeded after %d sleep and retry for DDC Null Response",
                 dh_repr_t(dh),
                 ddcrc_null_response_ct);
@@ -710,7 +690,8 @@ ddc_write_read_with_retry(
          DBGMSF(debug, "ddc_write_read() returned %s", psc_desc(psc) );
          COUNT_RETRYABLE_STATUS_CODE(psc);
 
-         if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
+         TRACED_ASSERT(dh->dref->io_path.io_mode == DDCA_IO_I2C);
+
             // The problem: Does NULL response indicate an error condition, or
             // is the monitor using NULL response to indicate unsupported?
             // Acer monitor uses NULL response instead of setting the unsupported
@@ -757,24 +738,14 @@ ddc_write_read_with_retry(
             // if (retryable)
             //    call_dynamic_tuned_sleep_i2c(SE_DDC_NULL, tryctr+1);
                   }
-         }  // DDCA_IO_I2C
 
-         else {   // DDC_IO_ADL
-            // TODO more detailed tests
-            if (psc == DDCRC_NULL_RESPONSE)
-               retryable = false;
-            else if (psc == DDCRC_READ_ALL_ZERO)
-               retryable = true;
-            else
-               retryable = false;
-         }
 
          if (psc == DDCRC_READ_ALL_ZERO)
             ddcrc_read_all_zero_ct++;
       }    // rc < 0
       // DBGMSG("Bottom of try loop. psc=%d, tryctr=%d, retryable=%s", psc, tryctr, sbool(retryable));
    }
-   DBGTRC(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, psc=%d, retryable=%s, read_bytewise=%s",
+   DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, psc=%d, retryable=%s, read_bytewise=%s",
          tryctr, psc, sbool(retryable), sbool(read_bytewise));
 
    int errct = (psc == 0) ? tryctr-1 : tryctr;
@@ -794,7 +765,7 @@ ddc_write_read_with_retry(
          char * s0 = (psc == 0) ? "Succeeded" : "Failed";
          char * s1 = (errct == 1) ? "" : "s";
          char * s = errinfo_array_summary(try_errors, errct);
-         DBGTRC(debug, TRACE_GROUP | DDCA_TRC_RETRY, "%s after %d error%s: %s", s0, errct, s1, s);
+         DBGTRC_NOPREFIX(debug, TRACE_GROUP | DDCA_TRC_RETRY, "%s after %d error%s: %s", s0, errct, s1, s);
          free(s);
 
    }
@@ -807,7 +778,7 @@ ddc_write_read_with_retry(
 
    if (psc < 0) {
       // int last_try_index = tryctr-1;
-      DBGTRC(debug, TRACE_GROUP, "After try loop. tryctr=%d, retryable=%s", tryctr, sbool(retryable));
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "After try loop. tryctr=%d, retryable=%s", tryctr, sbool(retryable));
 
       if (retryable)
          psc = DDCRC_RETRIES;
@@ -830,7 +801,7 @@ ddc_write_read_with_retry(
 
    try_data_record_tries2(WRITE_READ_TRIES_OP, psc, tryctr);
 
-   DBGTRC(debug, TRACE_GROUP, "Done.  Total Tries (tryctr): %d. Returning: %s", tryctr, errinfo_summary(ddc_excp));
+   DBGTRC_DONE(debug, TRACE_GROUP, "Total Tries (tryctr): %d. Returning: %s", tryctr, errinfo_summary(ddc_excp));
    return ddc_excp;
 }
 
@@ -853,9 +824,14 @@ ddc_i2c_write_only(
 {
    bool debug = false;
    int fh = dh->fd;
-   DBGTRC0(debug, TRACE_GROUP, "Starting.");
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
    if (debug)
-      dbgrpt_packet(request_packet_ptr, 1);
+      dbgrpt_packet(request_packet_ptr, 2);
+
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "request_packet_ptr->raw_bytes: %s",
+                              hexstring3_t(request_packet_ptr->raw_bytes->bytes,
+                                           request_packet_ptr->raw_bytes->len,
+                                           " ", 1, false) );
 
    // Byte slave_address = request_packet_ptr[0];
    // assert(slave_address == 0x37);
@@ -875,7 +851,7 @@ ddc_i2c_write_only(
             : SE_POST_WRITE;
    // tuned_sleep_i2c_with_trace(sleep_type, __func__, NULL);
    TUNED_SLEEP_WITH_TRACE(dh, sleep_type, NULL);
-   DBGTRC(debug, TRACE_GROUP, "Done. rc=%s", psc_desc(rc) );
+   DBGTRC_RETURNING(debug, TRACE_GROUP, rc, "");
    return rc;
 }
 
@@ -885,6 +861,10 @@ ddc_i2c_write_only(
  * \param  dh                  Display_Handle for open I2C or ADL device
  * \param  request_packet_ptr  DDC packet to write
  * \return NULL if success, #Error_Info struct if error
+ *
+ * @todo
+ * Eliminate this function, it used to route to the ADL version as
+ * well as ddc_i2c_write_only()
  */
 Error_Info *
 ddc_write_only(
@@ -892,32 +872,16 @@ ddc_write_only(
       DDC_Packet *     request_packet_ptr)
 {
    bool debug = false;
-   DBGTRC0(debug, TRACE_GROUP, "Starting.");
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
 
-   DDCA_Status psc = 0;
-   assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
-   if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
-      psc = ddc_i2c_write_only(dh, request_packet_ptr);
-   }
-   else {
-      psc = adlshim_ddc_write_only(
-              dh,
-              get_packet_start(request_packet_ptr),
-              get_packet_len(request_packet_ptr)
-              // get_packet_start(request_packet_ptr)+1,
-              // get_packet_len(request_packet_ptr)-1
-             );
-      if (psc > 0) {
-         DBGMSG("Unusual positive return code from ADL: %d", psc);
-         psc = 0;
-      }
-   }
+   TRACED_ASSERT(dh->dref->io_path.io_mode == DDCA_IO_I2C);
 
+   DDCA_Status psc = ddc_i2c_write_only(dh, request_packet_ptr);
    Error_Info *  ddc_excp = NULL;
    if (psc)
       ddc_excp = errinfo_new(psc, __func__);
 
-   DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", errinfo_summary(ddc_excp));
+   DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %s", errinfo_summary(ddc_excp));
    return ddc_excp;
 }
 
@@ -937,9 +901,9 @@ ddc_write_only_with_retry(
       DDC_Packet *     request_packet_ptr)
 {
    bool debug = false;
-   DBGTRC0(debug, TRACE_GROUP, "Starting." );
+   DBGTRC_STARTING(debug, TRACE_GROUP, "" );
 
-   assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
+   TRACED_ASSERT(dh->dref->io_path.io_mode == DDCA_IO_I2C);
 
    DDCA_Status        psc;
    int                tryctr;
@@ -947,7 +911,7 @@ ddc_write_only_with_retry(
    Error_Info *       try_errors[MAX_MAX_TRIES];
 
    int max_tries = try_data_get_maxtries2(WRITE_ONLY_TRIES_OP);
-   assert(max_tries > 0);
+   TRACED_ASSERT(max_tries > 0);
    for (tryctr=0, psc=-999, retryable=true;
        tryctr < max_tries && psc < 0 && retryable;
        tryctr++)
@@ -960,26 +924,9 @@ ddc_write_only_with_retry(
       psc = (cur_excp) ? cur_excp->status_code : 0;
       try_errors[tryctr] = cur_excp;
 
-      if (psc < 0) {
-         COUNT_RETRYABLE_STATUS_CODE(psc);
-         if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
-            if (psc < 0) {
-               if (psc != -EIO)
-                   retryable = false;
-            }
-         }
-         else {
-            if (psc < 0) {
-                // no logic in ADL case to test for continuing to retry, should there be ???
-                // is it even meaningful to retry for ADL?
-                   // retryable = true;    // *** TEMP ***
-            }
-         }
-      }   // rc < 0
       // try_status_codes[tryctr] = psc;   // for future Ddc_Error mechanism
    }
 
-
    Error_Info * ddc_excp = NULL;
 
    if (psc < 0) {
@@ -991,7 +938,8 @@ ddc_write_only_with_retry(
       //   tryctr == max_tries && !retryable
 
       // int last_try_index = tryctr-1;
-      DBGTRC(debug, TRACE_GROUP, "After try loop. tryctr=%d, retryable=%s", tryctr, sbool(retryable));
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "After try loop. tryctr=%d, retryable=%s",
+                                           tryctr, sbool(retryable));
 
       if (retryable)
          psc = DDCRC_RETRIES;
@@ -1013,24 +961,29 @@ ddc_write_only_with_retry(
 
    try_data_record_tries2(WRITE_ONLY_TRIES_OP, psc, tryctr);
 
-   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", errinfo_summary(ddc_excp));
+   DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %s", errinfo_summary(ddc_excp));
    return ddc_excp;
 }
 
 
 static void
 init_ddc_packet_io_func_name_table() {
-#define ADD_FUNC(_NAME) rtti_func_name_table_add(_NAME, #_NAME);
-   ADD_FUNC(ddc_write_read_raw);
-   ADD_FUNC(ddc_write_read);
-   ADD_FUNC(ddc_write_read_with_retry);
-   ADD_FUNC(ddc_write_only);
-   ADD_FUNC(ddc_write_only_with_retry);
-#undef ADD_FUNC
+   RTTI_ADD_FUNC(ddc_open_display);
+   RTTI_ADD_FUNC(ddc_close_display);
+   RTTI_ADD_FUNC(ddc_i2c_write_read_raw);
+   RTTI_ADD_FUNC(ddc_i2c_write_only);
+   RTTI_ADD_FUNC(ddc_write_read_raw);
+   RTTI_ADD_FUNC(ddc_write_read);
+   RTTI_ADD_FUNC(ddc_write_read_with_retry);
+   RTTI_ADD_FUNC(ddc_write_only);
+   RTTI_ADD_FUNC(ddc_write_only_with_retry);
+   RTTI_ADD_FUNC(ddc_is_valid_display_handle);
 }
 
 
 void
 init_ddc_packet_io() {
    init_ddc_packet_io_func_name_table();
+
+   open_displays = g_hash_table_new(g_direct_hash, NULL);
 }
