@@ -3,17 +3,17 @@
  * ddc layer initialization and configuration, statistics management
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
+#include "config.h"
+
 /** \cond */
-#include <i2c/i2c_strategy_dispatcher.h>
 #include <stdio.h>
-/** \endcond */
 
 #include "util/report_util.h"
+/** \endcond */
 
-#include "base/adl_errors.h"
 #include "base/base_init.h"
 #include "base/feature_metadata.h"
 #include "base/parms.h"
@@ -24,21 +24,27 @@
 #include "base/thread_sleep_data.h"
 
 #include "vcp/vcp_feature_codes.h"
+#include "vcp/persistent_capabilities.h"
 
 #include "dynvcp/dyn_feature_codes.h"
+#include "dynvcp/dyn_feature_files.h"
 
 #include "i2c/i2c_bus_core.h"
-#include "adl/adl_shim.h"
-
+#include "i2c/i2c_strategy_dispatcher.h"
+#ifdef USE_USB
 #include "usb/usb_displays.h"
+#endif
 
 #include "ddc/ddc_display_lock.h"
 #include "ddc/ddc_displays.h"
+#include "ddc/ddc_dumpload.h"
 #include "ddc/ddc_multi_part_io.h"
 #include "ddc/ddc_output.h"
 #include "ddc/ddc_packet_io.h"
+#include "ddc/ddc_read_capabilities.h"
 #include "ddc/ddc_try_stats.h"
 #include "ddc/ddc_vcp.h"
+#include "ddc/ddc_watch_displays.h"
 
 #include "ddc/ddc_services.h"
 
@@ -144,17 +150,10 @@ void ddc_report_stats_main(DDCA_Stats_Type stats, bool show_per_thread_stats, in
  */
 void init_ddc_services() {
    bool debug = false;
-   DBGMSF(debug, "Executing");
+   DBGMSF(debug, "Starting");
 
    // i2c:
    i2c_set_io_strategy(DEFAULT_I2C_IO_STRATEGY);
-
-   // adl:
-   init_adl_errors();
-   adl_debug = debug;      // turn on adl initialization tracing
-   adlshim_initialize();
-
-   // i2c
    init_i2c_bus_core();
 
    // usb
@@ -164,14 +163,20 @@ void init_ddc_services() {
 
    // ddc:
    try_data_init();
+   init_persistent_capabilities();
    init_vcp_feature_codes();
    init_dyn_feature_codes();    // must come after init_vcp_feature_codes()
+   init_dyn_feature_files();
    init_ddc_display_lock();
    init_ddc_displays();
+   init_ddc_dumpload();
    init_ddc_output();
    init_ddc_packet_io();
+   init_ddc_read_capabilities();
    init_ddc_multi_part_io();
    init_ddc_vcp();
+   init_ddc_watch_displays();
 
    // dbgrpt_rtti_func_name_table(1);
+   DBGMSF(debug, "Done");
 }
