@@ -1,7 +1,7 @@
 /** \file ddc_watch_displays.h - Watch for monitor addition and removal
  */
 
-// Copyright (C) 2019-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2019-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DDC_WATCH_DISPLAYS_H_
@@ -14,7 +14,8 @@
 
 #include <glib-2.0/glib.h>
 
-#include "util/error_info.h"
+#include "ddcutil_status_codes.h"
+
 /** \endcond */
 
 typedef enum {Changed_None    = 0,
@@ -30,21 +31,15 @@ typedef void (*Display_Change_Handler)(
                  GPtrArray *          removed,
                  GPtrArray *          added);
 
-#define WATCH_DISPLAYS_DATA_MARKER "WDDM"
-typedef struct {
-   char                   marker[4];
-   Display_Change_Handler display_change_handler;
-   pid_t                  main_process_id;
-   pid_t                  main_thread_id;
-} Watch_Displays_Data;
-
-
-Error_Info * ddc_start_watch_displays();
-
 void dummy_display_change_handler(
         Displays_Change_Type change_type,
         GPtrArray *          removed,
         GPtrArray *          added);
 
+DDCA_Status ddc_start_watch_displays();
+DDCA_Status ddc_stop_watch_displays();
+void init_ddc_watch_displays();
+
+// GPtrArray *  get_sysfs_drm_displays();
 
 #endif /* DDC_WATCH_DISPLAYS_H_ */
