@@ -1,7 +1,7 @@
 /** @file dyn_feature_set.c
  */
 
-// Copyright (C) 2018-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <string.h>
@@ -140,7 +140,7 @@ dyn_create_feature_set0(
       GPtrArray *          members_dfm)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. subset_id=%d, number of members=%d",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "subset_id=%d, number of members=%d",
                               subset_id, (members_dfm) ? members_dfm->len : -1);
 
    Dyn_Feature_Set * fset = calloc(1,sizeof(Dyn_Feature_Set));
@@ -148,13 +148,13 @@ dyn_create_feature_set0(
    fset->subset = subset_id;
    fset->members_dfm = members_dfm;
 
-   DBGTRC(debug, TRACE_GROUP, "Returning %p", fset);
+   DBGTRC_DONE(debug, TRACE_GROUP, "Returning %p", fset);
    return fset;
 }
 
 
 Dyn_Feature_Set *
-dyn_create_feature_set2_dfm(
+dyn_create_feature_set2(
       VCP_Feature_Subset     subset_id,
       DDCA_Display_Ref       display_ref,
       Feature_Set_Flags      feature_set_flags)
@@ -213,14 +213,15 @@ dyn_create_feature_set2_dfm(
        VCP_Feature_Set * vcp_feature_set =
           create_feature_set(
              subset_id,
-             dref->vcp_version,
+             // dref->vcp_version,
+             get_vcp_version_by_dref(dref),
              feature_set_flags);
        int ct = get_feature_set_size(vcp_feature_set);
        for (int ndx = 0; ndx < ct; ndx++) {
            VCP_Feature_Table_Entry * vfte = get_feature_set_entry(vcp_feature_set, ndx);
            DDCA_Vcp_Feature_Code feature_code = vfte->code;
            Display_Feature_Metadata * dfm =
-                 dyn_get_feature_metadata_by_dref_dfm(
+                 dyn_get_feature_metadata_by_dref(
                        feature_code,
                        dref,
                        true);    // with_default
@@ -242,14 +243,15 @@ dyn_create_feature_set2_dfm(
        VCP_Feature_Set * vcp_feature_set =
           create_feature_set(
              subset_id,
-             dref->vcp_version,
+             // dref->vcp_version,
+             get_vcp_version_by_dref(dref),
              feature_set_flags);
        int ct = get_feature_set_size(vcp_feature_set);
        for (int ndx = 0; ndx < ct; ndx++) {
           VCP_Feature_Table_Entry * vfte = get_feature_set_entry(vcp_feature_set, ndx);
           DDCA_Vcp_Feature_Code feature_code = vfte->code;
           Display_Feature_Metadata * dfm =
-                dyn_get_feature_metadata_by_dref_dfm(
+                dyn_get_feature_metadata_by_dref(
                       feature_code,
                       dref,
                       true);    // with_default
@@ -283,6 +285,9 @@ dyn_create_single_feature_set_by_hexid2(
       DDCA_Display_Ref       display_ref,
       bool                  force)
 {
+   bool debug = false;
+   DBGMSF(debug, "feature_code=0x%02x, display_ref=%s, force=%s",
+                 feature_code, dref_repr_t(display_ref), sbool(force));
    Display_Ref * dref = (Display_Ref *) display_ref;
    assert( dref && memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0);
 
@@ -307,7 +312,10 @@ dyn_create_single_feature_set_by_hexid2(
        else
           vcp_entry = vcp_find_feature_by_hexid(feature_code);
        if (vcp_entry)
-          dfm = dyn_create_dynamic_feature_from_vcp_feature_table_entry_dfm(vcp_entry, dref->vcp_version);
+          dfm = dyn_create_dynamic_feature_from_vcp_feature_table_entry_dfm(
+                   vcp_entry,
+                   get_vcp_version_by_dref(dref) );
+                   // dref->vcp_version);
    }
 
    if (dfm)
@@ -318,12 +326,13 @@ dyn_create_single_feature_set_by_hexid2(
       // result = NULL  ??
    }
 
+   DBGMSF(debug, "Done. Returning %p", result);
    return result;
 }
 
 
 Display_Feature_Metadata *
-dyn_get_feature_set_entry2_dfm(
+dyn_get_feature_set_entry2(
       Dyn_Feature_Set * feature_set,
       unsigned          index)
 {
@@ -336,7 +345,7 @@ dyn_get_feature_set_entry2_dfm(
 
 
 int
-dyn_get_feature_set_size2_dfm(
+dyn_get_feature_set_size2(
       Dyn_Feature_Set * feature_set)
 {
    // show_backtrace(2);
@@ -363,7 +372,7 @@ dyn_create_feature_set_from_feature_set_ref2(
       result = dyn_create_single_feature_set_by_hexid2(fsref->specific_feature, dref, flags & FSF_FORCE);
    }
    else {
-      result = dyn_create_feature_set2_dfm(fsref->subset, dref, flags);
+      result = dyn_create_feature_set2(fsref->subset, dref, flags);
    }
 
    if (debug || IS_TRACING()) {
