@@ -2,7 +2,7 @@
  *
  *  I2C bus detection and inspection
  */
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef I2C_BUS_CORE_H_
@@ -41,7 +41,7 @@ extern bool i2c_force_bus;
 
 // Basic I2C bus operations
 int           i2c_open_bus(int busno, Call_Options callopts);
-Status_Errno  i2c_close_bus(int fd, int busno, Call_Options callopts);
+Status_Errno  i2c_close_bus(int fd, Call_Options callopts);
 Status_Errno  i2c_set_addr(int fd, int addr, Call_Options callopts);
 
 // EDID inspection
@@ -53,12 +53,14 @@ Status_Errno_DDC i2c_get_parsed_edid_by_fd(int fd, Parsed_Edid ** edid_ptr_loc);
 #define I2C_BUS_EXISTS               0x80
 #define I2C_BUS_ACCESSIBLE           0x40
 #define I2C_BUS_ADDR_0X50            0x20      ///< detected I2C bus address 0x50
-#define I2C_BUS_ADDR_0X37            0x10
+#define I2C_BUS_ADDR_0X37            0x10      ///< detected I2C bus address 0x37
 #define I2C_BUS_ADDR_0X30            0x08      ///< detected write-only addr to specify EDID block number
 #define I2C_BUS_EDP                  0x04      ///< bus associated with eDP display
+#define I2C_BUS_LVDS                 0x02      ///< bus associated with LVDS display
 #define I2C_BUS_PROBED               0x01      ///< has bus been checked?
 #define I2C_BUS_VALID_NAME_CHECKED 0x0800
 #define I2C_BUS_HAS_VALID_NAME     0x0400
+#define I2C_BUS_BUSY               0x0200      ///< for possible future use
 
 #define I2C_BUS_INFO_MARKER "BINF"
 /** Information about one I2C bus */
@@ -77,6 +79,7 @@ void i2c_report_active_display(I2C_Bus_Info * businfo, int depth);
 // Simple bus detection, no side effects
 bool i2c_device_exists(int busno);
 int  i2c_device_count();           // simple /dev/i2c-n count, no side effects
+Byte_Value_Array get_i2c_devices_by_existence_test();
 
 // Bus inventory - detect and probe buses
 int i2c_detect_buses();            // creates internal array of Bus_Info for I2C buses
@@ -85,7 +88,7 @@ I2C_Bus_Info * i2c_detect_single_bus(int busno);
 void i2c_free_bus_info(I2C_Bus_Info * bus_info);
 
 // Simple Bus_Info retrieval
-I2C_Bus_Info * i2c_get_bus_info_by_index(int busndx);
+I2C_Bus_Info * i2c_get_bus_info_by_index(uint busndx);
 I2C_Bus_Info * i2c_find_bus_info_by_busno(int busno);
 
 
