@@ -30,30 +30,32 @@ typedef Status_Errno_DDC (*I2C_Reader)(
       int    bytect,
       Byte * readbuf);
 
-Status_Errno_DDC fileio_writer(
+Status_Errno_DDC i2c_fileio_writer(
       int    fd,
       Byte   slave_address,
       int    bytect,
       Byte * pbytes);
 
-Status_Errno_DDC fileio_reader (
+Status_Errno_DDC i2c_fileio_reader (
       int    fd,
       Byte   slave_address,
       bool   read_bytewise,
       int    bytect,
       Byte * readbuf);
 
-Status_Errno_DDC ioctl_writer(
+Status_Errno_DDC i2c_ioctl_writer(
       int    fd,
       Byte   slave_address,
       int    bytect,
       Byte * pbytes);
 
-Status_Errno_DDC ioctl_reader(
+Status_Errno_DDC i2c_ioctl_reader(
       int    fd,
       Byte   slave_address,
       bool   read_bytewise,
       int    bytect,
       Byte * readbuf);
 
+void init_i2c_execute_func_name_table();
+
 #endif /* I2C_EXECUTE_H_ */
