@@ -1,32 +1,45 @@
-/** api_base.c
+/** \file api_base.c
  *
  *  C API base functions.
  */
 
-// Copyright (C) 2015-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2015-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
 
 #include <assert.h>
 #include <errno.h>
+#include <glib-2.0/glib.h>
+#include <signal.h>
 #include <string.h>
- 
+#include <syslog.h>
+
 #include "public/ddcutil_c_api.h"
 
+#include "util/ddcutil_config_file.h"
+#include "util/debug_util.h"
+#include "util/file_util.h"
+#include "util/report_util.h"
+#include "util/sysfs_filter_functions.h"
+#include "util/xdg_util.h"
+
 #include "base/base_init.h"
 #include "base/build_info.h"
 #include "base/core.h"
+#include "base/core_per_thread_settings.h"
 #include "base/parms.h"
+#include "base/per_thread_data.h"
 #include "base/thread_retry_data.h"
 #include "base/thread_sleep_data.h"
 #include "base/tuned_sleep.h"
-#include "base/per_thread_data.h"
 
-#include "adl/adl_shim.h"
+#include "cmdline/cmd_parser.h"
+#include "cmdline/parsed_cmd.h"
 
 // #include "i2c/i2c_bus_core.h"   // for testing watch_devices
 
+#include "ddc/ddc_displays.h"
 #include "ddc/ddc_multi_part_io.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_services.h"
@@ -34,7 +47,11 @@
 #include "ddc/ddc_vcp.h"
 #include "ddc/ddc_watch_displays.h"
 
+#include "ddc/common_init.h"
+
+#include "libmain/api_error_info_internal.h"
 #include "libmain/api_base_internal.h"
+#include "libmain/api_services_internal.h"
 
 
 //
@@ -69,30 +86,32 @@ ddca_ddcutil_version(void) {
    static bool vspec_init = false;
 
    if (!vspec_init) {
-      int ct = sscanf(BUILD_VERSION, "%hhu.%hhu.%hhu", &vspec.major, &vspec.minor, &vspec.micro);
+#ifndef NDEBUG
+      int ct =
+#endif
+            sscanf(get_base_ddcutil_version(),
+                      "%hhu.%hhu.%hhu", &vspec.major, &vspec.minor, &vspec.micro);
       assert(ct == 3);
       vspec_init = true;
    }
-   // DBGMSG("Returning: %d.%d.%d", vspec.major, vspec.minor, vspec.micro);
+   DBGMSG("Returning: %d.%d.%d", vspec.major, vspec.minor, vspec.micro);
    return vspec;
 }
 
 
-//  Returns the ddcutil version as a string in the form "major.minor.micro".
+/**  Returns the ddcutil version as a string in the form "major.minor.micro".
+ *
+ */
 const char *
 ddca_ddcutil_version_string(void) {
-   return BUILD_VERSION;
+   return get_base_ddcutil_version();
 }
 
 
-// Indicates whether the ddcutil library was built with ADL support. .
-bool
-ddca_built_with_adl(void) {
-#ifdef HAVE_ADL
-   return true;
-#else
-   return false;
-#endif
+// Returns the full ddcutil version as a string that may be suffixed with an extension
+const char *
+ddca_ddcutil_extended_version_string(void) {
+   return get_full_ddcutil_version();
 }
 
 
@@ -114,9 +133,6 @@ ddca_built_with_usb(void) {
 DDCA_Build_Option_Flags
 ddca_build_options(void) {
    uint8_t result = 0x00;
-#ifdef HAVE_ADL
-   result |= DDCA_BUILT_WITH_ADL;
-#endif
 #ifdef USE_USB
          result |= DDCA_BUILT_WITH_USB;
 #endif
@@ -127,19 +143,118 @@ ddca_build_options(void) {
    return result;
 }
 
-
-// Indicates whether ADL successfully initialized.
-// (would fail e.g. fglrx driver not found)
-bool
-ddca_adl_is_available(void) {
-   return adlshim_is_available();
+#ifdef FUTURE
+char * get_library_filename() {
+   intmax_t pid = get_process_id();
+   return NULL;
 }
+#endif
+
 
 
 //
 // Initialization
 //
 
+
+static
+Parsed_Cmd * get_parsed_libmain_config() {
+   bool debug = false;
+   DBGF(debug, "Starting");
+
+   Parsed_Cmd * parsed_cmd = NULL;
+
+   // dummy initial argument list so libddcutil is not a special case
+   Null_Terminated_String_Array cmd_name_array = calloc(2, sizeof(char*));
+   cmd_name_array[0] = "libddcutil";
+   cmd_name_array[1] = NULL;
+   DBGF(debug, "cmd_name_array=%p, cmd_name_array[1]=%p -> %s",
+                cmd_name_array, cmd_name_array[0], cmd_name_array[0]);
+
+   GPtrArray* errmsgs = g_ptr_array_new_with_free_func(g_free);
+   char ** new_argv = NULL;
+   int     new_argc = 0;
+   char *  untokenized_option_string = NULL;
+   char *  config_fn;
+   DBGF(debug, "Calling apply_config_file()...");
+   int apply_config_rc = apply_config_file(
+                                 "libddcutil",  // use this section of config file
+                                 1, cmd_name_array,
+                                 &new_argc,
+                                 &new_argv,
+                                 &untokenized_option_string,
+                                 &config_fn,
+                                 errmsgs);
+   // DBGF(debug, "Calling ntsa_free(cmd_name_array=%p", cmd_name_array);
+   // ntsa_free(cmd_name_array, false);
+   DBGF(debug, "apply_config_file() returned: %d, new_argc=%d, new_argv=%p: ",
+                 apply_config_rc, new_argc, new_argv);
+   assert(apply_config_rc <= 0);
+   assert( new_argc == ntsa_length(new_argv) );
+   if (debug)
+      ntsa_show(new_argv);
+
+   if (errmsgs->len > 0) {
+      f0printf(ferr(), "Errors reading libddcutil configuration file %s:\n", config_fn);
+      for (int ndx = 0; ndx < errmsgs->len; ndx++) {
+         f0printf(fout(), "   %s\n", (char*) g_ptr_array_index(errmsgs, ndx));
+      }
+   }
+   g_ptr_array_free(errmsgs, true);
+   if (untokenized_option_string && strlen(untokenized_option_string) > 0)
+      fprintf(fout(), "Applying libddcutil options from %s: %s\n", config_fn,
+            untokenized_option_string);
+
+   // Continue even if config file errors
+   // if (apply_config_rc < 0)
+   //    goto bye;
+
+
+   assert(new_argc >= 1);
+   DBGF(debug, "Calling parse_command()");
+   parsed_cmd = parse_command(new_argc, new_argv, MODE_LIBDDCUTIL);
+   if (!parsed_cmd) {
+      syslog(LOG_WARNING, "Ignoring invalid configuration file options: %s",
+            untokenized_option_string);
+      fprintf(ferr(), "Ignoring invalid configuration file options: %s\n",
+                      untokenized_option_string);
+      DBGF(debug, "Retrying parse_command() with no options");
+      parsed_cmd = parse_command(1, cmd_name_array, MODE_LIBDDCUTIL);
+   }
+   if (debug)
+      dbgrpt_parsed_cmd(parsed_cmd, 1);
+   DBGF(debug, "Calling ntsa_free(cmd_name_array=%p", cmd_name_array);
+   ntsa_free(cmd_name_array, false);
+   ntsa_free(new_argv, true);
+   free(untokenized_option_string);
+   free(config_fn);
+
+   DBGF(debug, "Done.     Returning %p", parsed_cmd);
+   return parsed_cmd;
+}
+
+
+#ifdef TESTING_CLEANUP
+void done() {
+   printf("(%s) Starting\n", __func__);
+   _ddca_terminate();
+   syslog(LOG_INFO, "(%s) executing done()", __func__);
+   printf("(%s) Done.\n", __func__);
+}
+
+void dummy_sigterm_handler() {
+   printf("(%s) Executing. library_initialized = %s\n",
+         __func__, SBOOL(library_initialized));
+}
+
+void atexit_func() {
+   printf("(%s) Executing. library_initalized = %s\n",
+         __func__, SBOOL(library_initialized));
+}
+#endif
+
+static FILE * flog = NULL;
+
 bool library_initialized = false;
 
 /** Initializes the ddcutil library module.
@@ -150,12 +265,62 @@ bool library_initialized = false;
  */
 void __attribute__ ((constructor))
 _ddca_init(void) {
-   // Note: Until init_msg_control() is called within init_base_services(),
-   // FOUT is null, so DBGMSG() causes a segfault
    bool debug = false;
+   if (debug)
+      printf("(%s) Starting. library_initialized=%s\n", __func__, sbool(library_initialized));
    if (!library_initialized) {
+      openlog("libddcutil", LOG_CONS|LOG_PID, LOG_USER);
+      syslog(LOG_INFO, "Initializing.  ddcutil version %s", get_full_ddcutil_version());
+#ifdef TESTING_CLEANUP
+      // signal(SIGTERM, dummy_sigterm_handler);
+      // atexit(atexit_func);  // TESTING CLAEANUP
+#endif
       init_base_services();
-      init_ddc_services();
+      Parsed_Cmd* parsed_cmd = get_parsed_libmain_config();
+      init_tracing(parsed_cmd);
+
+      if (parsed_cmd->library_trace_file) {
+         char * trace_file = (parsed_cmd->library_trace_file[0] != '/')
+                ? xdg_state_home_file("ddcutil", parsed_cmd->library_trace_file)
+                : strdup(parsed_cmd->library_trace_file);
+         if (debug)
+            printf("(%s) Setting trace destination %s\n", __func__, trace_file);
+         syslog(LOG_INFO, "Trace destination: %s", trace_file);
+
+         fopen_mkdir(trace_file, "a", stderr, &flog);
+
+         // flog = fopen(trace_file, "a");
+         if (flog) {
+            time_t trace_start_time = time(NULL);
+            char * trace_start_time_s = asctime(localtime(&trace_start_time));
+            if (trace_start_time_s[strlen(trace_start_time_s)-1] == 0x0a)
+                 trace_start_time_s[strlen(trace_start_time_s)-1] = 0;
+            fprintf(flog, "%s tracing started %s\n", "libddcutil", trace_start_time_s);
+            if (debug) {
+               fprintf(stdout, "Writing %s trace output to %s\n", "libddcutil",trace_file);
+            }
+            set_default_thread_output_settings(flog, flog);
+            set_fout(flog);
+            set_ferr(flog);
+
+            rpt_set_default_output_dest(flog);    // for future threads
+            rpt_push_output_dest(flog);           // for this thread
+         }
+         else {
+            fprintf(stderr, "Error opening libddcutil trace file %s: %s\n",
+                            trace_file, strerror(errno));
+         }
+         free(trace_file);
+      }
+
+      init_api_services();
+      submaster_initializer(parsed_cmd);
+      free_parsed_cmd(parsed_cmd);
+
+     //  explicitly set the async threshold for testing
+     //  int threshold = DISPLAY_CHECK_ASYNC_THRESHOLD_STANDARD;
+     //  int threshold = DISPLAY_CHECK_ASYNC_NEVER; //
+     //  ddc_set_async_threshold(threshold);
 
       // no longer needed, values are initialized on first use per-thread
       // set_output_level(DDCA_OL_NORMAL);
@@ -165,11 +330,46 @@ _ddca_init(void) {
       ddc_start_watch_displays();
 
       library_initialized = true;
-      DBGMSF(debug, "library initialization executed");
+
+#ifdef TESTING_CLEANUP
+      // int atexit_rc = atexit(done);   // TESTING CLEANUP
+      // printf("(%s) atexit() returned %d\n", __func__, atexit_rc);
+#endif
+
+      DBGTRC_DONE(debug, DDCA_TRC_API, "library initialization executed");
+      syslog(LOG_INFO, "Library initialization complete.");
    }
    else {
-      DBGMSF(debug, "library was already initialized");
+      DBGTRC_DONE (debug, DDCA_TRC_API, "library was already initialized");
    }
+   // TRACED_ASSERT(1==5); for testing
+}
+
+
+/** Cleanup at library termination
+ *
+ *  - Terminates thread that watches for display addition or removal.
+ *  - Releases heap memory to avoid error reports from memory analyzers.
+ */
+void __attribute__ ((destructor))
+_ddca_terminate(void) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API, "library_initialized = %s", SBOOL(library_initialized));
+   if (library_initialized) {
+      ddc_discard_detected_displays();
+      release_base_services();
+      ddc_stop_watch_displays();
+      free_regex_hash_table();
+      library_initialized = false;
+      if (flog)
+         fclose(flog);
+      DBGTRC_DONE(debug, DDCA_TRC_API, "library termination complete");
+   }
+   else {
+      DBGTRC_DONE(debug, DDCA_TRC_API, "library was already terminated");   // should be impossible
+   }
+   syslog(LOG_INFO, "Terminating.");
+   closelog();
 }
 
 
@@ -185,7 +385,7 @@ ddca_get_error_detail() {
    DDCA_Error_Detail * result = dup_error_detail(get_thread_error_detail());
 
    if (debug) {
-      DBGMSG("Done.  Returning: %p", result);
+      DBGMSG("Done.     Returning: %p", result);
       if (result)
          report_error_detail(result, 2);
    }
@@ -421,29 +621,13 @@ ddca_max_max_tries(void) {
 
 int
 ddca_get_max_tries(DDCA_Retry_Type retry_type) {
-#ifdef OLD
-   int result = 0;
-   switch(retry_type) {
-      case (WRITE_ONLY_TRIES_OP):
-         result = ddc_get_max_write_only_exchange_tries();
-      break;
-   case (WRITE_READ_TRIES_OPE):
-      result = ddc_get_max_write_read_exchange_tries();
-      break;
-   case (MULTI_PART_READ_OP):
-      result = ddc_get_max_multi_part_read_tries();
-      break;
-   case (MULTI_PART_WRITE_OP):
-      result = ddc_get_max_multi_part_write_tries();
-      break;
-   }
+   // stats for multi part writes and reads are separate, but the
+   // max tries for both are identical
+#ifndef NDEBUG
+   Retry_Op_Value result3 = try_data_get_maxtries2((Retry_Operation) retry_type);
 #endif
-
-
-
-   Retry_Op_Value result3 = try_data_get_maxtries2(retry_type);
    // new way using retry_mgt
-   Retry_Op_Value result2 = trd_get_thread_max_tries(retry_type);
+   Retry_Op_Value result2 = trd_get_thread_max_tries((Retry_Operation) retry_type);
    // assert(result == result2);
    assert(result2 == result3);
    return result2;
@@ -460,33 +644,13 @@ ddca_set_max_tries(
    if (max_tries < 1 || max_tries > MAX_MAX_TRIES)
       rc = DDCRC_ARG;
    else {
-
-#ifdef OLD
-      switch(retry_type) {
-      case (WRITE_ONLY_TRIES_OP):
-         ddc_set_max_write_only_exchange_tries(max_tries);   // sets in Try_Data
-
-         break;
-      case (WRITE_READ_TRIES_OPE):
-         ddc_set_max_write_read_exchange_tries(max_tries);   // sets in Try_Data
-         break;
-      case (MULTI_PART_READ_OP):
-         ddc_set_max_multi_part_read_tries(max_tries);       // sets in Try_Data  , sets vars in ddc_muti_part.io
-      case (MULTI_PART_WRITE_OP):
-         ddc_set_max_multi_part_write_tries(max_tries);      // TODO: Separate constant
-         break;
-      }
-#endif
-
-
-
-      try_data_set_maxtries2(retry_type, max_tries);
+      try_data_set_maxtries2((Retry_Operation) retry_type, max_tries);
       // for DDCA_MULTI_PART_TRIES, set both  MULTI_PART_WRITE_OP and MULTI_PART_READ_OP
       if (retry_type == DDCA_MULTI_PART_TRIES)
          try_data_set_maxtries2(MULTI_PART_WRITE_OP, max_tries);
 
       // new way, set in retry_mgt
-      trd_set_thread_max_tries(retry_type, max_tries);
+      trd_set_thread_max_tries((Retry_Operation) retry_type, max_tries);
       if (retry_type == DDCA_MULTI_PART_TRIES)
            trd_set_thread_max_tries(MULTI_PART_WRITE_OP, max_tries);
    }
@@ -529,10 +693,12 @@ ddca_is_sleep_suppression_enabled() {
 }
 
 
-void
+double
 ddca_set_default_sleep_multiplier(double multiplier)
 {
+   double result = tsd_get_default_sleep_multiplier_factor();
    tsd_set_default_sleep_multiplier_factor(multiplier);
+   return result;
 }
 
 double
@@ -545,6 +711,7 @@ void
 ddca_set_global_sleep_multiplier(double multiplier)
 {
    ddca_set_default_sleep_multiplier(multiplier);
+   return;
 }
 
 double
@@ -554,19 +721,21 @@ ddca_get_global_sleep_multiplier()
 }
 
 // for current thread
-void
+double
 ddca_set_sleep_multiplier(double multiplier)
 {
-   // bool debug = true;
+   // bool debug = false;
+   double result = tsd_get_sleep_multiplier_factor();
    // DBGMSF(debug, "Setting %5.2f", multiplier);
    tsd_set_sleep_multiplier_factor(multiplier);
    // DBGMSF(debug, "Done");
+   return result;
 }
 
 double
 ddca_get_sleep_multiplier()
 {
-   // bool debug = true;
+   // bool debug = false;
    // DBGMSF(debug, "Starting");
    double result = tsd_get_sleep_multiplier_factor();
    // DBGMSF(debug, "Returning %5.2f", result);
@@ -600,6 +769,21 @@ ddca_set_timeout_millis(
 #endif
 
 
+bool
+ddca_enable_force_slave_address(bool onoff) {
+   bool old = i2c_force_slave_addr_flag;
+   i2c_force_slave_addr_flag = onoff;
+   return old;
+}
+
+
+bool
+ddca_is_force_slave_address_enabled(void) {
+   return i2c_force_slave_addr_flag;
+}
+
+
+
 //
 // Tracing
 //
@@ -618,7 +802,13 @@ ddca_add_traced_file(const char * filename) {
 
 void
 ddca_set_trace_groups(DDCA_Trace_Group trace_flags) {
-   set_trace_levels(trace_flags);
+   set_trace_groups(trace_flags);
+}
+
+
+void
+ddca_add_trace_groups(DDCA_Trace_Group trace_flags) {
+   add_trace_groups(trace_flags);
 }
 
 
@@ -627,18 +817,21 @@ ddca_trace_group_name_to_value(char * name) {
    return trace_class_name_to_value(name);
 }
 
+
 void
 ddca_set_trace_options(DDCA_Trace_Options  options) {
-   // global variables in core.c
    // DBGMSG("options = 0x%02x", options);
-   dbgtrc_show_time = options & DDCA_TRCOPT_TIMESTAMP;
-   dbgtrc_show_thread_id = options & DDCA_TRCOPT_THREAD_ID;
+   // global variables in core.c
+
+   if (options & DDCA_TRCOPT_TIMESTAMP)
+      dbgtrc_show_time = true;
+   if (options & DDCA_TRCOPT_WALLTIME)
+      dbgtrc_show_time = true;
+   if (options & DDCA_TRCOPT_THREAD_ID)
+      dbgtrc_show_thread_id = true;
 }
 
 
-
-
-
 //
 // Statistics
 //
@@ -672,8 +865,6 @@ ddca_get_thread_descripton() {
    return ptd_get_thread_description_t();
 }
 
-
-
 void
 ddca_reset_stats(void) {
    // DBGMSG("Executing");
@@ -687,11 +878,8 @@ ddca_show_stats(
       bool            by_thread,
       int             depth)
 {
-   // DBGMSG("stats_types = %d, depth=%d", stats_types, depth);
-
-   ddc_report_stats_main( stats_types,    // stats to show
-                          by_thread,
-                          depth);         // logical indentation depth
+   if (stats_types)
+      ddc_report_stats_main( stats_types, by_thread, depth);
 }
 
 
