@@ -3,7 +3,7 @@
  *  Capabilities related functions of the API
  */
 
-// Copyright (C) 2015-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2015-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -22,6 +22,7 @@
 
 #include "base/displays.h"
 #include "base/feature_metadata.h"
+#include "base/rtti.h"
 #include "base/vcp_version.h"
 
 #include "vcp/ddc_command_codes.h"
@@ -39,6 +40,7 @@
 #include "libmain/api_displays_internal.h"
 #include "libmain/api_metadata_internal.h"
 
+#include "libmain/api_error_info_internal.h"
 #include "libmain/api_capabilities_internal.h"
  
 
@@ -52,16 +54,15 @@ ddca_get_capabilities_string(
       char**               pcaps_loc)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. ddca_dh=%s", dh_repr((Display_Handle *) ddca_dh ));
+   DBGTRC_STARTING(debug, DDCA_TRC_API, "ddca_dh=%s", dh_repr((Display_Handle *) ddca_dh ) );
    free_thread_error_detail();
-   // assert(pcaps_loc);
-   PRECOND(pcaps_loc);
+   API_PRECOND(pcaps_loc);
    *pcaps_loc = NULL;
    Error_Info * ddc_excp = NULL;
-   WITH_DH(ddca_dh,
+   WITH_VALIDATED_DH2(ddca_dh,
       {
          char * p_cap_string = NULL;
-         ddc_excp = get_capabilities_string(dh, &p_cap_string);
+         ddc_excp = ddc_get_capabilities_string(dh, &p_cap_string);
          psc = (ddc_excp) ? ddc_excp->status_code : 0;
          save_thread_error_detail(error_info_to_ddca_detail(ddc_excp));
          errinfo_free(ddc_excp);
@@ -69,12 +70,12 @@ ddca_get_capabilities_string(
             // make copy to prevent caller from mucking around in ddcutil's
             // internal data structures
             *pcaps_loc = strdup(p_cap_string);
-            // DBGMSF(debug, "*pcaps_loc=%p", *pcaps_loc);
          }
-        //  DBGMSF(debug, "psc=%s", ddca_rc_desc(psc));
-         assert( (psc==0 && *pcaps_loc) || (psc!=0 && !*pcaps_loc));
-         DBGMSF(debug, "Done.     ddca_dh=%s, *pcaps_loc=%p, Returning: %s",
-                        dh_repr((Display_Handle *) ddca_dh ), *pcaps_loc, ddca_rc_desc(psc));
+         ASSERT_IFF(psc==0, *pcaps_loc);
+         DBGTRC_RETURNING(debug, DDCA_TRC_API, psc,
+               "ddca_dh=%s, *pcaps_loc=%p -> |%s|",
+               dh_repr_t((Display_Handle *) ddca_dh),
+               *pcaps_loc, *pcaps_loc);
       }
    );
 }
@@ -89,7 +90,6 @@ dbgrpt_ddca_cap_vcp(DDCA_Cap_Vcp * cap, int depth) {
    rpt_vstring(d1, "value_ct:        %d", cap->value_ct);
    if (cap->value_ct > 0) {
       rpt_label(d1, "Values: ");
-
       for (int ndx = 0; ndx < cap->value_ct; ndx++) {
          rpt_vstring(d2, "Value:   0x%02x", cap->values[ndx]);
       }
@@ -129,10 +129,11 @@ ddca_parse_capabilities_string(
       DDCA_Capabilities **     parsed_capabilities_loc)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. capabilities_string: |%s|", capabilities_string);
+   DBGTRC_STARTING(debug, DDCA_TRC_API, "parsed_capabilities_loc=%p, capabilities_string: |%s|",
+                                       parsed_capabilities_loc, capabilities_string);
    // assert(parsed_capabilities_loc);
    free_thread_error_detail();
-   PRECOND(parsed_capabilities_loc);
+   API_PRECOND(parsed_capabilities_loc);
    DDCA_Status ddcrc = DDCRC_BAD_DATA;
    DBGMSF(debug, "ddcrc initialized to %s", psc_desc(ddcrc));
    DDCA_Capabilities * result = NULL;
@@ -209,11 +210,12 @@ ddca_parse_capabilities_string(
    }
 
    *parsed_capabilities_loc = result;
-   DBGMSF(debug, "Done.    *parsed_capabilities_loc=%p, Returning: %d", *parsed_capabilities_loc, ddcrc);
-   if (debug && *parsed_capabilities_loc)
+
+   DBGTRC_RETURNING(debug, DDCA_TRC_API, ddcrc, "*parsed_capabilities_loc=%p", *parsed_capabilities_loc);
+   if ( IS_DBGTRC(debug, DDCA_TRC_API) && *parsed_capabilities_loc)
       dbgrpt_ddca_capabilities(*parsed_capabilities_loc, 2);
+
    ASSERT_IFF(ddcrc==0, *parsed_capabilities_loc);
-   assert( (ddcrc==0 && *parsed_capabilities_loc) || (ddcrc!=0 && !*parsed_capabilities_loc));
    return ddcrc;
 }
 
@@ -223,10 +225,10 @@ ddca_free_parsed_capabilities(
       DDCA_Capabilities * pcaps)
 {
    bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API, "pcaps=%p", pcaps);
    if (pcaps) {
       assert(memcmp(pcaps->marker, DDCA_CAPABILITIES_MARKER, 4) == 0);
       free(pcaps->unparsed_string);
-
       DBGMSF(debug, "vcp_code_ct = %d", pcaps->vcp_code_ct);
       for (int ndx = 0; ndx < pcaps->vcp_code_ct; ndx++) {
          DDCA_Cap_Vcp * cur_vcp = &pcaps->vcp_codes[ndx];
@@ -234,11 +236,13 @@ ddca_free_parsed_capabilities(
          cur_vcp->marker[3] = 'x';
          free(cur_vcp->values);
       }
-
+      free(pcaps->vcp_codes);
+      free(pcaps->cmd_codes);
       ntsa_free(pcaps->messages, true);
       pcaps->marker[3] = 'x';
       free(pcaps);
    }
+   DBGTRC_DONE(debug, DDCA_TRC_API, "");
 }
 
 
@@ -254,19 +258,16 @@ ddca_report_parsed_capabilities_by_dref(
    free_thread_error_detail();
    DDCA_Status ddcrc = 0;
 
-   PRECOND(p_caps);
-#ifdef ALT
-   // no need to check marker since DDCA_Capabilities not opaque
-   if (!p_caps) {
-      ddcrc = DDCRC_ARG;
-      goto bye;
-   }
-#endif
+   API_PRECOND(p_caps);   // no need to check marker, DDCA_CAPABILITIES not opaque
 
-   Display_Ref * dref = (Display_Ref *) ddca_dref;
-   if (dref != NULL && memcmp(dref->marker, DISPLAY_REF_MARKER, 4) != 0 ) {
-      ddcrc = DDCRC_ARG;
-      goto bye;
+   Display_Ref * dref = NULL;
+   // dref may be NULL, but if not it must be valid
+   if (ddca_dref) {
+      dref = validated_ddca_display_ref(ddca_dref);
+      if (!dref) {
+         ddcrc = DDCRC_ARG;
+         goto bye;
+      }
    }
 
    int d0 = depth;
@@ -302,7 +303,7 @@ ddca_report_parsed_capabilities_by_dref(
       assert( memcmp(cur_vcp->marker, DDCA_CAP_VCP_MARKER, 4) == 0);
 
       Display_Feature_Metadata * dfm =
-         dyn_get_feature_metadata_by_dref_dfm(
+         dyn_get_feature_metadata_by_dref(
                cur_vcp->feature_code,
                ddca_dref,
                true);    // create_default_if_not_found);
@@ -378,14 +379,14 @@ ddca_report_parsed_capabilities_by_dh(
 
    // Ensure dh->dref->vcp_version is not unqueried,
    // ddca_report_parsed_capabilities_by_dref() will fail trying to lock the already open device
-   get_vcp_version_by_display_handle(dh);
-   DBGMSF(debug, "After get_vcp_version_by_display_handle(), dh->dref->vcp_version=%s",
-                 format_vspec(dh->dref->vcp_version) );
+   get_vcp_version_by_dh(dh);
+   DBGMSF(debug, "After get_vcp_version_by_dh(), dh->dref->vcp_version_df=%s",
+                 format_vspec_verbose(dh->dref->vcp_version_xdf));
 
    ddca_report_parsed_capabilities_by_dref(p_caps, dh->dref, depth);
 
 bye:
-   DBGMSF(debug, "Done. Returning %s", ddca_rc_desc(ddcrc));
+   DBGMSF(debug, "Done.     Returning %s", ddca_rc_desc(ddcrc));
    return ddcrc;
 }
 
@@ -415,4 +416,9 @@ ddca_feature_list_from_capabilities(
    return result;
 }
 
-
+#ifdef NOT_NEEDED
+void init_api_capabilities() {
+   printf("(%s) Executing\n", __func__);
+   RTTI_ADD_FUNC(ddca_get_capabilities_string);
+}
+#endif
