@@ -3,7 +3,7 @@
  *  For use only by other api_... files.
  */
 
-// Copyright (C) 2015-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2015-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
  
 
@@ -14,13 +14,26 @@
 #include "private/ddcutil_types_private.h"
 
 
-#define WITH_DR(ddca_dref, action) \
+Display_Ref * validated_ddca_display_ref(DDCA_Display_Ref ddca_dref);
+Display_Handle * validated_ddca_display_handle(DDCA_Display_Handle ddca_dh);
+
+#define VALIDATE_DDCA_DREF(_ddca_dref, _dref, _debug) \
+   do { \
+      _dref = validated_ddca_display_ref(_ddca_dref); \
+      if (!_dref) { \
+         DBGTRC_DONE(_debug, DDCA_TRC_API, "Returning DDCRC_ARG"); \
+         return DDCRC_ARG; \
+      } \
+   } while(0)
+
+#ifdef OLD
+#define WITH_VALIDATED_DR(ddca_dref, action) \
    do { \
       assert(library_initialized); \
       DDCA_Status psc = 0; \
       free_thread_error_detail(); \
-      Display_Ref * dref = (Display_Ref *) ddca_dref; \
-      if (dref == NULL || memcmp(dref->marker, DISPLAY_REF_MARKER, 4) != 0 )  { \
+      Display_Ref * dref = validated_ddca_display_ref(ddca_dref); \
+      if (!dref)  { \
          psc = DDCRC_ARG; \
       } \
       else { \
@@ -28,27 +41,43 @@
       } \
       return psc; \
    } while(0);
+#endif
+
+#define WITH_VALIDATED_DR3(_ddca_dref, _ddcrc, _action) \
+   do { \
+      assert(library_initialized); \
+      _ddcrc = 0; \
+      free_thread_error_detail(); \
+      Display_Ref * dref = validated_ddca_display_ref(_ddca_dref); \
+      if (!dref)  { \
+         _ddcrc = DDCRC_ARG; \
+      } \
+      else { \
+         (_action); \
+      } \
+   } while(0);
 
 
-#define WITH_DH(_ddca_dh_, _action_) \
+#define WITH_VALIDATED_DH2(ddca_dh, action) \
    do { \
       assert(library_initialized); \
       DDCA_Status psc = 0; \
       free_thread_error_detail(); \
-      Display_Handle * dh = (Display_Handle *) _ddca_dh_; \
-      if ( !dh || memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) != 0 )  { \
+      Display_Handle * dh = validated_ddca_display_handle(ddca_dh); \
+      if (!dh)  { \
          psc = DDCRC_ARG; \
+         DBGTRC_RETURNING(debug, DDCA_TRC_API, psc, "ddca_dh=%p", ddca_dh); \
       } \
       else { \
-         (_action_); \
+         (action); \
       } \
+      /* DBGTRC_RETURNING(debug, DDCA_TRC_API, psc, ""); */ \
       return psc; \
    } while(0);
 
+
 // extern DDCA_Monitor_Model_Key DDCA_UNDEFINED_MONITOR_MODEL_KEY;
 
-
-
 // Monitor Model Key - UNPUBLISHED, USED INTERNALLY
 
 //
