@@ -3,7 +3,7 @@
  *  Get, set, and format feature values
  */
 
-// Copyright (C) 2015-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2015-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -18,6 +18,7 @@
 #include "util/error_info.h"
 #include "util/report_util.h"
 
+#include "base/core.h"
 #include "base/displays.h"
 #include "base/monitor_model_key.h"
 
@@ -25,11 +26,11 @@
 
 #include "dynvcp/dyn_feature_codes.h"
 
-#include "ddc/ddc_async.h"
 #include "ddc/ddc_dumpload.h"
 #include "ddc/ddc_vcp_version.h"
 #include "ddc/ddc_vcp.h"
 
+#include "libmain/api_error_info_internal.h"
 #include "libmain/api_base_internal.h"
 #include "libmain/api_displays_internal.h"
 
@@ -83,8 +84,11 @@ ddca_get_non_table_vcp_value(
       DDCA_Vcp_Feature_Code      feature_code,
       DDCA_Non_Table_Vcp_Value*  valrec)
 {
-   assert(valrec);
-   WITH_DH(ddca_dh,  {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API, "ddca_dh=%p, feature_code=0x%02x, valrec=%p",
+                               ddca_dh, feature_code, valrec );
+   API_PRECOND(valrec);
+   WITH_VALIDATED_DH2(ddca_dh,  {
        Error_Info * ddc_excp = NULL;
        Parsed_Nontable_Vcp_Response * code_info;
        ddc_excp = ddc_get_nontable_vcp_value(
@@ -100,12 +104,16 @@ ddca_get_non_table_vcp_value(
           // DBGMSG("valrec:  mh=0x%02x, ml=0x%02x, sh=0x%02x, sl=0x%02x",
           //        valrec->mh, valrec->ml, valrec->sh, valrec->sl);
           free(code_info);
+          DBGTRC_RETURNING(debug, DDCA_TRC_API, psc,
+                "valrec:  mh=0x%02x, ml=0x%02x, sh=0x%02x, sl=0x%02x",
+                valrec->mh, valrec->ml, valrec->sh, valrec->sl);
        }
        else {
           psc = ddc_excp->status_code;
           save_thread_error_detail(error_info_to_ddca_detail(ddc_excp));
           // errinfo_free(ddc_excp, report_freed_exceptions, __func__);
           errinfo_free(ddc_excp);
+          DBGTRC_RETURNING(debug, DDCA_TRC_API, psc, "");
        }
     } );
 }
@@ -114,11 +122,16 @@ ddca_get_non_table_vcp_value(
 // untested
 DDCA_Status
 ddca_get_table_vcp_value(
-      DDCA_Display_Handle    ddca_dh,
-      DDCA_Vcp_Feature_Code  feature_code,
-      DDCA_Table_Vcp_Value **    table_value_loc)
+      DDCA_Display_Handle     ddca_dh,
+      DDCA_Vcp_Feature_Code   feature_code,
+      DDCA_Table_Vcp_Value ** table_value_loc)
 {
-   WITH_DH(ddca_dh,
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API,
+         "ddca_dh=%p, feature_code=0x%02x, table_value_loc=%p",
+         ddca_dh, feature_code, table_value_loc);
+   API_PRECOND(table_value_loc);
+   WITH_VALIDATED_DH2(ddca_dh,
       {
          assert(table_value_loc);
          Error_Info * ddc_excp = NULL;
@@ -137,10 +150,12 @@ ddca_get_table_vcp_value(
                memcpy(tv->bytes, p_table_bytes->bytes, len);
             }
             *table_value_loc = tv;
-
             buffer_free(p_table_bytes, __func__);
          }
-         assert( (psc==0 && *table_value_loc) || (psc!=0 && !*table_value_loc));
+         TRACED_ASSERT( (psc==0 && *table_value_loc) || (psc!=0 && !*table_value_loc));
+         DBGTRC_RETURNING(debug, DDCA_TRC_API, psc,
+                "ddca_dh=%p->%s, feature_code=0x%02x, *table_value_loc=%p",
+                ddca_dh, dh_repr_t(ddca_dh), feature_code, *table_value_loc);
       }
    );
 }
@@ -154,19 +169,21 @@ ddca_get_vcp_value(
       DDCA_Vcp_Value_Type    call_type,   // why is this needed?   look it up from dh and feature_code
       DDCA_Any_Vcp_Value **  pvalrec)
 {
-   Error_Info * ddc_excp = NULL;
 
-   WITH_DH(ddca_dh,
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API,
+          "ddca_dh=%p, feature_code=0x%02x, call_type=%d, pvalrec=%p",
+          ddca_dh, feature_code, call_type, pvalrec);
+
+   Error_Info * ddc_excp = NULL;
+   WITH_VALIDATED_DH2(ddca_dh,
          {
-               bool debug = false;
-               DBGMSF(debug, "Starting. ddca_dh=%p, feature_code=0x%02x, call_type=%d, pvalrec=%p",
-                      ddca_dh, feature_code, call_type, pvalrec);
                *pvalrec = NULL;
                ddc_excp = ddc_get_vcp_value(dh, feature_code, call_type, pvalrec);
                psc = (ddc_excp) ? ddc_excp->status_code : 0;
                save_thread_error_detail(error_info_to_ddca_detail(ddc_excp));
                errinfo_free(ddc_excp);
-               DBGMSF(debug, "*pvalrec=%p", *pvalrec);
+               DBGTRC_RETURNING(debug, DDCA_TRC_API, psc, "*pvalrec=%p", *pvalrec);
          }
    );
 }
@@ -182,7 +199,7 @@ get_value_type(
    DBGMSF(debug, "Starting. ddca_dh=%p, feature_code=0x%02x", ddca_dh, feature_code);
 
    DDCA_Status ddcrc = DDCRC_NOT_FOUND;
-   DDCA_MCCS_Version_Spec vspec     = get_vcp_version_by_display_handle(ddca_dh);
+   DDCA_MCCS_Version_Spec vspec     = get_vcp_version_by_dh(ddca_dh);
    VCP_Feature_Table_Entry * pentry = vcp_find_feature_by_hexid(feature_code);
    if (pentry) {
       DDCA_Version_Feature_Flags flags = get_version_sensitive_feature_flags(pentry, vspec);
@@ -216,7 +233,7 @@ ddca_get_any_vcp_value_using_explicit_type(
       *valrec_loc = valrec2;
    }
 
-   DBGMSF(debug, "Done. Returning %s, *valrec_loc=%p", psc_desc(rc), *valrec_loc);
+   DBGMSF(debug, "Done.     Returning %s, *valrec_loc=%p", psc_desc(rc), *valrec_loc);
    assert( (rc==0 && *valrec_loc) || (rc!=0 && !*valrec_loc) );
    return rc;
 }
@@ -249,7 +266,7 @@ ddca_get_any_vcp_value_using_explicit_type_new(
          *pvalrec = valrec;
       }
    }
-   DBGMSF(debug, "Done. Returning %s, *pvalrec=%p", psc_desc(rc), *pvalrec);
+   DBGMSF(debug, "Done.     Returning %s, *pvalrec=%p", psc_desc(rc), *pvalrec);
    return rc;
 }
 #endif
@@ -339,12 +356,15 @@ ddca_get_formatted_vcp_value(
       char**                 formatted_value_loc)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. feature_code=0x%02x", feature_code);
+   DBGTRC_STARTING(debug, DDCA_TRC_API, "feature_code=0x%02x, formatted_value_loc=%p",
+                 feature_code, formatted_value_loc);
+
+   API_PRECOND(formatted_value_loc);
    Error_Info * ddc_excp = NULL;
-   WITH_DH(ddca_dh,
+   WITH_VALIDATED_DH2(ddca_dh,
          {
                *formatted_value_loc = NULL;
-               DDCA_MCCS_Version_Spec vspec      = get_vcp_version_by_display_handle(dh);
+               DDCA_MCCS_Version_Spec vspec      = get_vcp_version_by_dh(dh);
                // DDCA_MCCS_Version_Id   version_id = mccs_version_spec_to_id(vspec);
 
                VCP_Feature_Table_Entry * pentry = vcp_find_feature_by_hexid(feature_code);
@@ -394,6 +414,7 @@ ddca_get_formatted_vcp_value(
                       }
                   }
                }
+               DBGTRC_RETURNING(debug, DDCA_TRC_API, psc, "");
          }
    )
 }
@@ -408,7 +429,7 @@ ddca_format_any_vcp_value(
       char **                  formatted_value_loc)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. feature_code=0x%02x, vspec=%d.%d, mmid=%p -> %s",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "feature_code=0x%02x, vspec=%d.%d, mmid=%p -> %s",
                  feature_code,
                  vspec.major, vspec.minor,
                  mmid,
@@ -427,7 +448,7 @@ ddca_format_any_vcp_value(
       goto bye;
    }
 
-   dfm = dyn_get_feature_metadata_by_mmk_and_vspec_dfm(
+   dfm = dyn_get_feature_metadata_by_mmk_and_vspec(
             feature_code, *mmid, vspec, /*with_default=*/ true);
    if (!dfm) {
       ddcrc = DDCRC_ARG;
@@ -442,7 +463,7 @@ ddca_format_any_vcp_value(
                                            feature_code, vspec.major, vspec.minor);
       else
          *formatted_value_loc = g_strdup_printf("Feature %02x is not readable", feature_code);
-      DBGTRC(debug, TRACE_GROUP, "%s", *formatted_value_loc);
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "%s", *formatted_value_loc);
       ddcrc = DDCRC_INVALID_OPERATION;
       goto bye;
    }
@@ -458,7 +479,7 @@ ddca_format_any_vcp_value(
        ddcrc = DDCRC_ARG;
        goto bye;
    }
-   bool ok = dyn_format_feature_detail_dfm(dfm, vspec, anyval,formatted_value_loc);
+   bool ok = dyn_format_feature_detail(dfm, vspec, anyval,formatted_value_loc);
    if (!ok) {
        ddcrc = DDCRC_ARG;    // ??
        assert(!*formatted_value_loc);
@@ -468,7 +489,7 @@ ddca_format_any_vcp_value(
 bye:
    if (dfm)
       dfm_free(dfm);
-   DBGTRC(debug, TRACE_GROUP, "Returning: %s, formatted_value_loc -> %s", psc_desc(ddcrc), *formatted_value_loc);
+   DBGTRC_RETURNING(debug, TRACE_GROUP, ddcrc, "formatted_value_loc -> %s", *formatted_value_loc);
    // 7/2019: wrong, *formatted_value_loc always set, why did this ever work?
    // assert( (ddcrc==0 && *formatted_value_loc) || (ddcrc!=0 &&!*formatted_value_loc) );
    return ddcrc;
@@ -482,20 +503,23 @@ ddca_format_any_vcp_value_by_dref(
       DDCA_Any_Vcp_Value *    valrec,
       char **                 formatted_value_loc)
 {
-   WITH_DR(ddca_dref,
+   bool debug = true;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "feature_code=0x%02x, ddca_dref=%p, valrec=%s",
+             feature_code,
+             ddca_dref,
+             summarize_single_vcp_value(valrec) );
+   assert(formatted_value_loc);
+   DDCA_Status ddcrc = 0;
+   WITH_VALIDATED_DR3(ddca_dref, ddcrc,
          {
-               assert(formatted_value_loc);
-               bool debug = false;
-               if (debug) {
-                  DBGMSG("feature_code=0x%02x, ddca_dref=%s, valrec=%s",
-                         feature_code,
-                         dref_repr_t(dref),
-                         summarize_single_vcp_value(valrec) );
-                  dbgrpt_display_ref( dref, 1);
+               if (debug || IS_TRACING()) {
+                  DBGTRC_NOPREFIX(debug, TRACE_GROUP, "dref = %s", dref_repr_t(dref));
+                  dbgrpt_display_ref(dref,1);
                }
-               psc = ddca_format_any_vcp_value(
+               ddcrc = ddca_format_any_vcp_value(
                          feature_code,
-                         dref->vcp_version,
+                         get_vcp_version_by_dref(dref),
+                         // dref->vcp_version,
                          dref->mmid,
                          valrec,
                          formatted_value_loc);
@@ -503,6 +527,9 @@ ddca_format_any_vcp_value_by_dref(
                // assert( (psc==0 && *formatted_value_loc) || (psc!=0 &&!*formatted_value_loc) );
          }
    )
+   DBGTRC_RETURNING(debug, DDCA_TRC_API, ddcrc, "*formatted_value_loc = %p -> |%s|",
+                                         *formatted_value_loc, *formatted_value_loc);
+   return ddcrc;
 }
 
 
@@ -515,13 +542,13 @@ ddca_format_non_table_vcp_value(
       char **                     formatted_value_loc)
 {
    bool debug = false;
-   if (debug) {
-      DBGMSG("feature_code=0x%02x, vspec=%d.%d, mmid=%s",
+   DBGTRC_STARTING(debug, DDCA_TRC_API,"feature_code=0x%02x, vspec=%d.%d, mmid=%s, formatted_value_loc=%p",
              feature_code,
              vspec.major, vspec.minor,
-             (mmid) ? mmk_repr(*mmid) : "NULL");
-   }
-   assert(formatted_value_loc);
+             (mmid) ? mmk_repr(*mmid) : "NULL",
+             formatted_value_loc);
+   API_PRECOND(formatted_value_loc);
+   *formatted_value_loc = NULL;
 
    // free_thread_error_detail();   // unnecessary, done by ddca_format_any_vcp_value();
    DDCA_Any_Vcp_Value anyval;
@@ -532,8 +559,16 @@ ddca_format_non_table_vcp_value(
    anyval.val.c_nc.sh = valrec->sh;
    anyval.val.c_nc.sl = valrec->sl;
 
-   DDCA_Status ddcrc =  ddca_format_any_vcp_value(feature_code, vspec, mmid, &anyval, formatted_value_loc);
+   DDCA_Status ddcrc =  ddca_format_any_vcp_value(
+                          feature_code, vspec, mmid, &anyval, formatted_value_loc);
    // assert( (ddcrc==0 &&*formatted_value_loc) || (ddcrc!=0 && !*formatted_value_loc) );
+
+   if (ddcrc == 0)
+      DBGTRC_RETURNING(debug, DDCA_TRC_API, ddcrc,
+             "*formatted_value_loc=%p->%s", *formatted_value_loc, *formatted_value_loc);
+   else
+      DBGTRC_RETURNING(debug, DDCA_TRC_API, ddcrc,  "*formatted_value_loc=%p", *formatted_value_loc);
+
    return ddcrc;
 }
 
@@ -544,26 +579,30 @@ ddca_format_non_table_vcp_value_by_dref(
       DDCA_Non_Table_Vcp_Value *  valrec,
       char **                     formatted_value_loc)
 {
-   WITH_DR(ddca_dref,
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "feature_code=0x%02x, ddca_dref=%p",
+                          feature_code, ddca_dref);
+   assert(formatted_value_loc);
+   DDCA_Status ddcrc = 0;
+   WITH_VALIDATED_DR3(ddca_dref, ddcrc,
          {
-               bool debug = false;
-               if (debug) {
-                  DBGMSG("feature_code=0x%02x, ddca_dref=%s",
-                         feature_code,
-                         dref_repr_t(dref) );
-                         // summarize_single_vcp_value(valrec) );
-                  dbgrpt_display_ref( dref, 1);
+               if (debug || IS_TRACING()) {
+                  DBGTRC_NOPREFIX(debug, TRACE_GROUP, "dref = %s", dref_repr_t(dref));
+                  dbgrpt_display_ref(dref,1);
                }
-               assert(formatted_value_loc);
-               psc = ddca_format_non_table_vcp_value(
+               ddcrc = ddca_format_non_table_vcp_value(
                          feature_code,
-                         dref->vcp_version,
+                         // dref->vcp_version,
+                         get_vcp_version_by_dref(dref),
                          dref->mmid,
                          valrec,
                          formatted_value_loc);
                // assert( (psc==0 &&*formatted_value_loc) || (psc!=0 && !*formatted_value_loc) );
          }
    )
+   DBGTRC_RETURNING(debug, TRACE_GROUP, ddcrc, "*formatted_value_loc = %p -> |%s|",
+                                               *formatted_value_loc, *formatted_value_loc);
+   return ddcrc;
 }
 
 
@@ -594,31 +633,49 @@ ddca_format_table_vcp_value_by_dref(
       DDCA_Table_Vcp_Value *  table_value,
       char **                 formatted_value_loc)
 {
-   WITH_DR(ddca_dref,
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+                          "feature_code=0x%02x, ddca_dref=%p", feature_code, ddca_dref);
+   assert(formatted_value_loc);
+   DDCA_Status ddcrc = 0;
+   WITH_VALIDATED_DR3(ddca_dref, ddcrc,
          {
-               return ddca_format_table_vcp_value(
+               if (debug || IS_TRACING()) {
+                  DBGTRC_NOPREFIX(debug, TRACE_GROUP, "dref = %s", dref_repr_t(dref));
+                  dbgrpt_display_ref(dref,1);
+               }
+              ddcrc = ddca_format_table_vcp_value(
                          feature_code,
-                         dref->vcp_version,
+                         // dref->vcp_version,
+                         get_vcp_version_by_dref(dref),
                          dref->mmid,
                          table_value,
                          formatted_value_loc);
          }
    )
+   DBGTRC_RETURNING(debug, TRACE_GROUP, ddcrc, "*formatted_value_loc = %p -> |%s|",
+                                               *formatted_value_loc, *formatted_value_loc);
+   return ddcrc;
 }
 
 
 static
 DDCA_Status
 set_single_vcp_value(
-      DDCA_Display_Handle  ddca_dh,
+      DDCA_Display_Handle    ddca_dh,
       DDCA_Any_Vcp_Value *   valrec,
-      DDCA_Any_Vcp_Value **  verified_value_loc)
+      DDCA_Any_Vcp_Value **  verified_value_loc)  // NULL => do not return value
 {
+      bool debug = false;
+      DBGTRC_STARTING(debug, DDCA_TRC_API, "ddca_dh=%p, valrec=%p, verified_value_loc = %p",
+                                  ddca_dh, valrec, verified_value_loc);
+
       Error_Info * ddc_excp = NULL;
-      WITH_DH(ddca_dh,  {
+      WITH_VALIDATED_DH2(ddca_dh,  {
             ddc_excp = ddc_set_vcp_value(dh, valrec, verified_value_loc);
             psc = (ddc_excp) ? ddc_excp->status_code : 0;
             errinfo_free(ddc_excp);
+            DBGTRC_RETURNING(debug, DDCA_TRC_API, psc, "");
          } );
 }
 
@@ -734,6 +791,10 @@ ddca_set_non_table_vcp_value_verify(
       Byte *                 verified_hi_byte_loc,
       Byte *                 verified_lo_byte_loc)
 {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API,
+          "ddca_dh=%p, feature_code=0x%02x, hi_byte=0x%02x, lo_byte=0x%02x",
+          ddca_dh, feature_code, hi_byte, lo_byte);
    free_thread_error_detail();
    if ( ( verified_hi_byte_loc && !verified_lo_byte_loc) ||
         (!verified_hi_byte_loc &&  verified_lo_byte_loc )
@@ -758,6 +819,7 @@ ddca_set_non_table_vcp_value_verify(
                           NULL);
    }
 
+   DBGTRC_RETURNING(debug, DDCA_TRC_API, rc, "");
    return rc;
 }
 
@@ -891,19 +953,20 @@ ddca_get_profile_related_values(
       DDCA_Display_Handle ddca_dh,
       char**              profile_values_string_loc)
 {
-   WITH_DH(ddca_dh,
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API,
+          "ddca_dh=%p, profile_values_string_loc=%p",
+          ddca_dh, profile_values_string_loc);
+   API_PRECOND(profile_values_string_loc);
+
+   WITH_VALIDATED_DH2(ddca_dh,
       {
-         bool debug = false;
-         DBGMSF(debug, "Before dumpvcp_to_string_by_display_handle(), pprofile_values_string=%p,"
-                       " *profile_values_string_loc=%p",
-               profile_values_string_loc, *profile_values_string_loc);
-         assert(profile_values_string_loc);
          psc = dumpvcp_as_string(dh, profile_values_string_loc);
-         DBGMSF(debug, "After dumpvcp_to_string_by_display_handle(), pprofile_values_string=%p,"
-                       " *profile_values_string_loc=%p",
-               profile_values_string_loc, *profile_values_string_loc);
-         DBGMSF(debug, "*profile_values_string_loc = |%s|", *profile_values_string_loc);
-         assert( (psc==0  && *profile_values_string_loc) || (psc!=0 && !*profile_values_string_loc) );
+         TRACED_ASSERT_IFF(psc==0, *profile_values_string_loc);
+         DBGTRC_RETURNING(debug, TRACE_GROUP, psc,
+                                 "*profile_values_string_loc=%p -> %s",
+                                 *profile_values_string_loc,
+                                 *profile_values_string_loc);
       }
    );
 }
@@ -912,9 +975,13 @@ ddca_get_profile_related_values(
 DDCA_Status
 ddca_set_profile_related_values(
       DDCA_Display_Handle  ddca_dh,
-      char * profile_values_string)
+      char *               profile_values_string)
 {
-   WITH_DH(ddca_dh,
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API,
+          "ddca_h=%p, profile_values_string = %s",
+           ddca_dh, profile_values_string);
+   WITH_VALIDATED_DH2(ddca_dh,
       {
          free_thread_error_detail();
          Error_Info * ddc_excp = loadvcp_by_string(profile_values_string, dh);
@@ -923,13 +990,15 @@ ddca_set_profile_related_values(
             save_thread_error_detail(error_info_to_ddca_detail(ddc_excp));
             errinfo_free(ddc_excp);
          }
+         DBGTRC_RETURNING(debug, DDCA_TRC_API, psc, "");
       }
    );
 }
 
 
 //
-// Async operation - experimental
+// Vestiges of old experimental async API.
+// Never published, retained for ABI compatibility.
 //
 
 DDCA_Status
@@ -939,19 +1008,7 @@ ddca_start_get_any_vcp_value(
       DDCA_Vcp_Value_Type         call_type,
       DDCA_Notification_Func      callback_func)
 {
-   bool debug = false;
-   DBGMSF(debug, "Starting. ddca_dh=%p, feature_code=0x%02x, call_type=%d",
-                 ddca_dh, feature_code, call_type);
-   // DDCA_Status rc = DDCRC_ARG;
-   Error_Info * ddc_excp = NULL;
-
-   WITH_DH(ddca_dh,
-       {
-          ddc_excp = start_get_vcp_value(dh, feature_code, call_type, callback_func);
-          psc = (ddc_excp) ? ddc_excp->status_code : 0;
-          errinfo_free(ddc_excp);
-       }
-      );
+   return DDCRC_UNIMPLEMENTED;
 }
 
 
@@ -971,8 +1028,9 @@ ddca_queue_get_non_table_vcp_value(
    return DDCRC_UNIMPLEMENTED;
 }
 
-
+//
 // CFFI
+//
 DDCA_Status
 ddca_pass_callback(
       Simple_Callback_Func  func,
@@ -984,6 +1042,11 @@ ddca_pass_callback(
    return callback_rc;
 }
 
+
+
+
+
+
 #ifdef FUTURE
 // which header file would this go in?
 void init_api_access_feature_codes() {
