@@ -1,6 +1,6 @@
 // api_metadata.c
 
-// Copyright (C) 2018-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -25,11 +25,12 @@
 #include "ddc/ddc_vcp_version.h"
 
 #include "dynvcp/dyn_feature_codes.h"
+#include "dynvcp/dyn_feature_files.h"
 #include "dynvcp/dyn_feature_set.h"
-#include "dynvcp/dyn_dynamic_features.h"
 
 #include "private/ddcutil_c_api_private.h"
 
+#include "libmain/api_error_info_internal.h"
 #include "libmain/api_base_internal.h"
 #include "libmain/api_displays_internal.h"
 #include "libmain/api_metadata_internal.h"
@@ -41,6 +42,8 @@
 // TODO: Move most functions into directory src/base
 //
 
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_API;
+
 const DDCA_Feature_List DDCA_EMPTY_FEATURE_LIST = {{0}};
 
 
@@ -174,26 +177,29 @@ ddca_get_feature_list_by_dref(
       bool                    include_table_features,
       DDCA_Feature_List*      feature_list_loc)
 {
-   WITH_DR(
-         ddca_dref,
-         {
-               bool debug = false;
-               DBGMSF(debug, "Starting. feature_subset_id=%d=0x%08x=%s, dref=%p=%s, "
-                             "include_table_features=%s, feature_list_loc=%p",
-                      feature_set_id, feature_set_id, ddca_feature_list_id_name(feature_set_id),
-                      dref, dref_repr_t(dref),
-                      sbool(include_table_features),
-                      feature_list_loc);
-               assert(feature_list_loc);
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "feature_subset_id=%d=0x%08x=%s, ddca_dref=%p, "
+              "include_table_features=%s, feature_list_loc=%p",
+              feature_set_id, feature_set_id, ddca_feature_list_id_name(feature_set_id),
+          ddca_dref,
+          sbool(include_table_features),
+          feature_list_loc);
+   assert(feature_list_loc);
+   DDCA_Status psc = 0;
+   VCP_Feature_Subset subset = VCP_SUBSET_NONE;  // pointless initialization to avoid compile warning
 
-               DDCA_MCCS_Version_Spec vspec = dref->vcp_version;
-               DBGMSF(debug, "vspec=%p=%s=%d.%d", &dref->vcp_version, format_vspec(dref->vcp_version), dref->vcp_version.major, dref->vcp_version.minor);
+   WITH_VALIDATED_DR3(
+         ddca_dref, psc,
+         {
+               DDCA_MCCS_Version_Spec vspec = // dref->vcp_version;
+                                             get_vcp_version_by_dref(dref);
+               DBGMSF(debug, "vspec=%%", format_vspec_verbose(vspec) );
                // redundant:
                // assert( !vcp_version_eq( vspec, DDCA_VSPEC_UNQUERIED) );
                // Whether a feature is a table feature can vary by version, so can't
                // specify VCP_SPEC_ANY to request feature ids in any version
                assert(vcp_version_is_valid(vspec, /* allow unknown */ false));
-               VCP_Feature_Subset subset = VCP_SUBSET_NONE;  // pointless initialization to avoid compile warning
+
                switch (feature_set_id) {
                case DDCA_SUBSET_KNOWN:
                   subset = VCP_SUBSET_KNOWN;
@@ -228,24 +234,22 @@ ddca_get_feature_list_by_dref(
                Feature_Set_Flags flags = 0x00;
                if (!include_table_features)
                   flags |= FSF_NOTABLE;
-               Dyn_Feature_Set * fset = dyn_create_feature_set2_dfm(subset, dref, flags);
+               Dyn_Feature_Set * fset = dyn_create_feature_set2(subset, dref, flags);
                // VCP_Feature_Set fset = create_feature_set(subset, vspec, !include_table_features);
 
                // TODO: function variant that takes result location as a parm, avoid memcpy
                DDCA_Feature_List result = feature_list_from_dyn_feature_set(fset);
                memcpy(feature_list_loc, &result, 32);
                dyn_free_feature_set(fset);
-
-            // bye:
-               DBGMSF(debug, "Done. feature_set_id=%d=0x%08x=%s, subset=%d=%s, Returning: %s",
-                     feature_set_id, feature_set_id, ddca_feature_list_id_name(feature_set_id),
-                     subset, feature_subset_name(subset), psc_desc(psc));
-               if (debug) {
-                  DBGMSG(    "      Feature list: %s", feature_list_string(feature_list_loc, "", ","));
-                  // rpt_hex_dump((Byte*) p_feature_list, 32, 1);
-               }
          }
-      );
+   );
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc, "feature_set_id=%d=0x%08x=%s, subset=%d=%s",
+         feature_set_id, feature_set_id, ddca_feature_list_id_name(feature_set_id),
+         subset, feature_subset_name(subset));
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP,
+          "Feature list: %s", feature_list_string(feature_list_loc, "", ","));
+      // rpt_hex_dump((Byte*) p_feature_list, 32, 1);
+   return psc;
 }
 
 
@@ -396,7 +400,7 @@ ddca_get_feature_flags_by_vspec(
    free_thread_error_detail();
    DDCA_Status psc = DDCRC_ARG;
    // assert(feature_flags);
-   PRECOND(feature_flags);
+   API_PRECOND(feature_flags);
    if (vcp_version_is_valid(vspec, /*unknown_ok*/ true)) {
 //    DDCA_Version_Feature_Info * full_info =  get_version_feature_info_by_vspec(
       Display_Feature_Metadata * dfm =  get_version_feature_info_by_vspec_dfm(
@@ -442,6 +446,25 @@ ddca_get_feature_flags_by_version_id(
 }
 #endif
 
+#ifdef NO
+DDCA_Status
+ddca_get_highest_version_sl_values(
+      DDCA_Vcp_Feature_Code       feature_code,
+      DDCA_Feature_Value_Entry ** sl_table_loc)
+{
+   DDCA_Status rc = DDCRC_NOT_FOUND;
+   DDCA_Feature_Value_Entry * result = NULL;
+   VCP_Feature_Table_Entry * vfte = vcp_find_feature_by_hexid(feature_code);
+   if (vfte) {
+      result = get_highest_version_sl_values(vfte);
+      rc = DDCRC_OK;
+   }
+   *sl_table_loc = result;
+   return rc;
+}
+#endif
+
+
 
 DDCA_Status
 ddca_get_feature_metadata_by_vspec(
@@ -451,8 +474,8 @@ ddca_get_feature_metadata_by_vspec(
       DDCA_Feature_Metadata **    info_loc) //
 {
    bool debug = false;
-   DBGMSF(debug, "feature_code=0x%02x, vspec=%d.%d, create_default_if_not_found=%s, info_loc=%p",
-                 feature_code, vspec.major, vspec.minor, sbool(create_default_if_not_found), info_loc);
+   DBGMSF(debug, "feature_code=0x%02x, vspec=%s, create_default_if_not_found=%s, info_loc=%p",
+                 feature_code, format_vspec_verbose(vspec), sbool(create_default_if_not_found), info_loc);
    assert(info_loc);
    free_thread_error_detail();
    DDCA_Feature_Metadata * meta = NULL;
@@ -489,17 +512,18 @@ ddca_get_feature_metadata_by_dref(
       bool                        create_default_if_not_found,
       DDCA_Feature_Metadata **    metadata_loc)
 {
-   WITH_DR(
-         ddca_dref,
-         {
-               bool debug = false;
-               DBGMSF(debug, "feature_code=0x%02x, dref=%s, create_default_if_not_found=%s, meta_loc=%p",
-                             feature_code, dref_repr_t(dref), sbool(create_default_if_not_found), metadata_loc);
-               assert(metadata_loc);
+   bool debug = false;
+   DBGTRC(debug, TRACE_GROUP, "feature_code=0x%02x, ddca_dref=%p, create_default_if_not_found=%s, meta_loc=%p",
+                 feature_code, ddca_dref, sbool(create_default_if_not_found), metadata_loc);
+   assert(metadata_loc);
 
+   DDCA_Status psc = 0;
+   WITH_VALIDATED_DR3(
+         ddca_dref, psc,
+         {
                DDCA_Feature_Metadata * external_metadata = NULL;
                Display_Feature_Metadata * internal_metadata =
-                  dyn_get_feature_metadata_by_dref_dfm(feature_code, dref, create_default_if_not_found);
+                  dyn_get_feature_metadata_by_dref(feature_code, dref, create_default_if_not_found);
                if (!internal_metadata) {
                   psc = DDCRC_NOT_FOUND;
                }
@@ -508,15 +532,11 @@ ddca_get_feature_metadata_by_dref(
                   dfm_free(internal_metadata);
                }
                *metadata_loc = external_metadata;
-
-               if (debug) {
-                  DBGMSG("Returning: %s", psc_desc(psc));
-                  if (psc == 0)
-                     dbgrpt_ddca_feature_metadata(external_metadata, 2);
-               }
-               assert( (psc==0 && *metadata_loc) || (psc!=0 &&!*metadata_loc) );
          }
-      );
+   );
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc, "");
+   ASSERT_IFF(psc==0, *metadata_loc);
+   return psc;
 }
 
 
@@ -527,21 +547,22 @@ ddca_get_feature_metadata_by_dh(
       bool                        create_default_if_not_found,
       DDCA_Feature_Metadata **    metadata_loc)
 {
-   WITH_DH(
+   bool debug = false;
+   // if (feature_code == 0xca)
+   //    debug = true;
+   DBGTRC_STARTING(debug, DDCA_TRC_API,
+          "feature_code=0x%02x, ddca_dh=%p, create_default_if_not_found=%s, metadata_loc=%p",
+          feature_code, ddca_dh, sbool(create_default_if_not_found), metadata_loc);
+   API_PRECOND(metadata_loc);
+   WITH_VALIDATED_DH2(
          ddca_dh,
          {
-               bool debug = false;
-               // if (feature_code == 0xca)
-               //    debug = true;
-               DBGMSF(debug, "Starting.  feature_code=0x%02x, ddca_dh=%s, create_default_if_not_found=%s, metadata_loc=%p",
-                             feature_code, ddca_dh_repr(ddca_dh), sbool(create_default_if_not_found), metadata_loc);
                if (debug)
                   dbgrpt_display_ref(dh->dref, 1);
-               assert(metadata_loc);
 
                DDCA_Feature_Metadata * external_metadata = NULL;
                Display_Feature_Metadata * internal_metadata =
-                  dyn_get_feature_metadata_by_dh_dfm(feature_code, dh, create_default_if_not_found);
+                  dyn_get_feature_metadata_by_dh(feature_code, dh, create_default_if_not_found);
                if (!internal_metadata) {
                   psc = DDCRC_NOT_FOUND;
                }
@@ -550,12 +571,12 @@ ddca_get_feature_metadata_by_dh(
                   dfm_free(internal_metadata);
                }
                *metadata_loc = external_metadata;
-
-                DBGMSF(debug, "Done.  Returning: %s", ddca_rc_desc(psc));
+               ASSERT_IFF(psc == 0, *metadata_loc);
+               DBGTRC_RETURNING(debug, DDCA_TRC_API, psc,
+                                        "ddca_dh=%p->%s", ddca_dh, dh_repr_t(ddca_dh));
                 if (psc == 0 && debug) {
                    dbgrpt_ddca_feature_metadata(external_metadata, 5);
                 }
-                assert( (psc==0 && *metadata_loc) || (psc!=0 &&!*metadata_loc) );
          }
       );
 }
@@ -630,14 +651,18 @@ ddca_get_feature_name_by_dref(
       DDCA_Display_Ref       ddca_dref,
       char **                name_loc)
 {
-   WITH_DR(ddca_dref,
+   DDCA_Status psc = 0;
+   WITH_VALIDATED_DR3(ddca_dref, psc,
          {
                //*name_loc = ddca_feature_name_by_vspec(feature_code, dref->vcp_version, dref->mmid);
-               *name_loc = get_feature_name_by_id_and_vcp_version(feature_code, dref->vcp_version);
+               *name_loc = get_feature_name_by_id_and_vcp_version(feature_code,
+                     get_vcp_version_by_dref(dref)    //dref->vcp_version
+                     );
                if (!*name_loc)
                   psc = DDCRC_ARG;
          }
    )
+   return psc;
 }
 
 
@@ -645,6 +670,7 @@ ddca_get_feature_name_by_dref(
 // Display Inquiry
 //
 
+#ifdef UNUSED
 // unpublished
 DDCA_Status
 ddca_get_simple_sl_value_table_by_vspec(
@@ -678,6 +704,7 @@ ddca_get_simple_sl_value_table_by_vspec(
      }
      else  {
         DDCA_Feature_Value_Entry * table = get_version_sensitive_sl_values(pentry, vspec);
+        // DDCA_Feature_Value_Entry * table = get_highest_version_sl_values(pentry);
         DDCA_Feature_Value_Entry * table2 = (DDCA_Feature_Value_Entry*) table;    // identical definitions
         *value_table_loc = table2;
         rc = 0;
@@ -692,12 +719,14 @@ ddca_get_simple_sl_value_table_by_vspec(
   }
 
 bye:
-   DBGMSF(debug, "Done. *pvalue_table=%p, returning %s", *value_table_loc, psc_desc(rc));
+   DBGMSF(debug, "Done.     *pvalue_table=%p, returning %s", *value_table_loc, psc_desc(rc));
    assert ( (rc==0 && *value_table_loc) || (rc!=0 && !*value_table_loc) );
    return rc;
 }
+#endif
 
 
+#ifdef UNUSED
 // for now, just gets SL value table based on the vspec of the display ref,
 // eventually handle dynamically assigned monitor specs
 DDCA_Status
@@ -710,13 +739,15 @@ ddca_get_simple_sl_value_table_by_dref(
       {
          assert(value_table_loc);
          psc = ddca_get_simple_sl_value_table_by_vspec(
-                  feature_code, dref->vcp_version, dref->mmid, value_table_loc);
+                  feature_code, dref->vcp_version_old, dref->mmid, value_table_loc);
          assert ( (psc==0 && *value_table_loc) || (psc!=0 && !*value_table_loc) );
       }
    )
 }
+#endif
 
 
+#ifdef UNUSED
 DDCA_Status
 ddca_get_simple_sl_value_table(
       DDCA_Vcp_Feature_Code      feature_code,
@@ -734,14 +765,16 @@ ddca_get_simple_sl_value_table(
    rc = ddca_get_simple_sl_value_table_by_vspec(
            feature_code, vspec, &DDCA_UNDEFINED_MONITOR_MODEL_KEY,  value_table_loc);
 
-   DBGMSF(debug, "Done. *value_table_loc=%p, returning %s", *value_table_loc, psc_desc(rc));
+   DBGMSF(debug, "Done.     *value_table_loc=%p, returning %s", *value_table_loc, psc_desc(rc));
    assert ( (rc==0 && *value_table_loc) || (rc!=0 && !*value_table_loc) );
    return rc;
 }
 
+#endif
 
 // typedef void * Feature_Value_Table;   // temp
 
+
 DDCA_Status
 ddca_get_simple_nc_feature_value_name_by_table(
       DDCA_Feature_Value_Entry *  feature_value_table,
@@ -762,6 +795,7 @@ ddca_get_simple_nc_feature_value_name_by_table(
 }
 
 
+#ifdef UNUSED
 DDCA_Status
 ddca_get_simple_nc_feature_value_name_by_vspec(
       DDCA_Vcp_Feature_Code    feature_code,
@@ -784,8 +818,9 @@ ddca_get_simple_nc_feature_value_name_by_vspec(
    assert ( (rc==0 && *feature_name_loc) || (rc!=0 && !*feature_name_loc) );
    return rc;
 }
+#endif
 
-
+#ifdef UNUSED
 // deprecated
 DDCA_Status
 ddca_get_simple_nc_feature_value_name_by_display(
@@ -795,20 +830,26 @@ ddca_get_simple_nc_feature_value_name_by_display(
       char**                 feature_name_loc)
 {
    WITH_DH(ddca_dh,  {
-         DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
+         DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_dh(dh);
          DDCA_Monitor_Model_Key * p_mmid = dh->dref->mmid;
          return ddca_get_simple_nc_feature_value_name_by_vspec(
                    feature_code, vspec, p_mmid, feature_value, feature_name_loc);
       }
    );
 }
+#endif
 
 void
 ddca_dbgrpt_feature_metadata(
       DDCA_Feature_Metadata * md,
       int                     depth)
 {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
+   // rpt_push_output_dest(stdout);
    dbgrpt_ddca_feature_metadata(md, depth);
+   // rpt_pop_output_dest();
+   DBGTRC_DONE(debug, TRACE_GROUP, "");
 }
 
 
@@ -835,35 +876,38 @@ ddca_is_udf_enabled(void)
 DDCA_Status
 ddca_dfr_check_by_dref(DDCA_Display_Ref ddca_dref)
 {
-   WITH_DR(ddca_dref,
-      {
-            bool debug = false;
-            DBGMSF(debug, "dref=%s", dref_repr_t(dref));
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "ddca_dref=%p", ddca_dref);
 
+   DDCA_Status psc = 0;
+   WITH_VALIDATED_DR3(ddca_dref, psc,
+      {
             free_thread_error_detail();
             Error_Info * ddc_excp = dfr_check_by_dref(dref);
             if (ddc_excp) {
                psc = ddc_excp->status_code;
-               DBGMSF(debug, "ddc_excp->status_code=%s, psc=%s", ddca_rc_name(ddc_excp->status_code), ddca_rc_name(psc));
                save_thread_error_detail(error_info_to_ddca_detail(ddc_excp));
                errinfo_free(ddc_excp);
             }
-            DBGMSF(debug, "Returning: %s", ddca_rc_name(psc));
       }
    );
+   DBGTRC_RETURNING(debug, TRACE_GROUP, psc, "");
+   return psc;
 }
 
 DDCA_Status
 ddca_dfr_check_by_dh(DDCA_Display_Handle ddca_dh)
 {
-   WITH_DH(ddca_dh,
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API, "ddca_dh=%p", ddca_dh);
+   WITH_VALIDATED_DH2(ddca_dh,
       {
-            bool debug = false;
             DBGMSF(debug, "dref=%s", dh_repr_t(dh));
 
             psc = ddca_dfr_check_by_dref(dh->dref);
 
-            DBGMSF(debug, "Returning: %s", ddca_rc_name(psc));
+            DBGTRC_DONE(debug, DDCA_TRC_API, "ddca_dh=%p->%s. Returning: %s(%d)",
+                   ddca_dh, dh_repr_t(ddca_dh), ddca_rc_name(psc), psc);
       }
    );
 }
