@@ -3,7 +3,7 @@
  *  Public C API for ddcutil
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DDCUTIL_C_API_H_
@@ -39,7 +39,7 @@ extern "C" {
  * for the ddcutil command line program.
  *
  * The operation of these functions can be tweaked in two ways.
- * - The "depth" parameter is a logical indentation depth.  This allows
+ * - The "depth" parameter is a logical indentation depth. This enables
  *   reports that invoke other reports to indent the subreports
  *   sensibly.  At the level of the ddcutil_c_api(), one unit of
  *   logical indentation depth translates to 3 spaces.
@@ -63,16 +63,26 @@ ddca_ddcutil_version(void);
 /**
  * Returns the ddcutil version as a string in the form "major.minor.micro".
  *
- * @return version string.
+ * @return version string (caller must not free)
  */
 const char *
 ddca_ddcutil_version_string(void);
 
+
+/** Returns the full ddcutil version as a string, possibly with a suffix,
+ *
+ * @return string in form "1.3.0" or "1.3.0-dev" (caller must not free)
+ *
+ * @since 1.2.0
+ */
+const char *
+ddca_ddcutil_extended_version_string(void);
+
+
 // Bit ids for ddca_get_build_options() - how to make connection in doxygen?
 #ifdef ALT
  * Defined Bits
  * <table>
- * <tr><td>#DDCA_BUILT_WITH_ADL</td><td>built with ADL support</td>
  * <tr><td>#DDCA_BUILT_WITH_USB</td><td>built with USB support
  * <tr><td>#DDCA_BUILT_WITH_FAILSIM</td><td> built with failure simulation
  * </table>
@@ -84,7 +94,6 @@ ddca_ddcutil_version_string(void);
  *
  * | Defined Bits: | |
  * |:-------| :--------------
- * |#DDCA_BUILT_WITH_ADL  | built with ADL support
  * |#DDCA_BUILT_WITH_USB  | built with USB support
  * |#DDCA_BUILT_WITH_FAILSIM | built with failure simulation
  *
@@ -110,7 +119,7 @@ ddca_build_options(void);
 
  /** Frees a detailed error information record
   *
-  *  @param  ddca_erec  error information to free
+  *  @param[in]  ddca_erec  error information to free
   *
   *  @since 0.9.0
   */
@@ -120,8 +129,8 @@ ddca_build_options(void);
 
  /** Issues a detailed report of a #DDCA_Error_Detail instance.
   *
-  *  @param ddca_erec  error information record
-  *  @param depth      logical indentation depth
+  *  @param[in] ddca_erec  error information record
+  *  @param[in] depth      logical indentation depth
   */
  void
  ddca_report_error_detail(
@@ -233,13 +242,77 @@ bool
 ddca_is_verify_enabled(void);
 
 
+/** Controls the force I2C slave address setting.
+ *
+ *  Normally, ioctl operation I2C_SLAVE is used to set the I2C slave address.
+ *  If that returns EBUSY and this setting is in effect, slave address setting
+ *  is retried using operation I2C_SLAVE_FORCE.
+ *
+ *  \param[in] onoff true/false
+ *  \return  prior value
+ *  \since 1.2.2
+ */
+bool
+ddca_enable_force_slave_address(bool onoff);
+
+/** Query the force I2C slave address setting.
+ *
+ *  \return true/false
+ *  \since 1.2.2
+ */
+bool
+ddca_is_force_slave_address_enabled(void);
+
+
+/** Sets the sleep multiplier factor to be used for new threads.
+ *
+ *  \param[in]  multiplier
+ *  \return     old multiplier
+ *
+ *  \remark
+ *  This function is intended for use only during program initialization,
+ *  typically from a value passed on the command line.
+ *  Consequently there are no associated lock/unlock functions for the value.
+ */
+double
+ddca_set_default_sleep_multiplier(double multiplier);
+
+/** Gets the sleep multiplier factor used for new threads
+ *
+ * \return multiplier
+ */
+double
+ddca_get_default_sleep_multiplier();
+
+/** \deprecated use #ddca_set_default_sleep_multiplier()
+ */
+__attribute__ ((deprecated ("use ddca_set_default_sleep_multiplier")))
 void
 ddca_set_global_sleep_multiplier(double multiplier);
 
+/** \deprecated use #ddca_get_default_display_multiplier()
+ */
+__attribute__ ((deprecated ("use ddca_get_default_sleep_multiplier")))
 double
 ddca_get_global_sleep_multiplier();
 
 
+/** Sets the sleep multiplier factor for the current thread.
+ *
+ *  @param[in] multiplier
+ *  @return    old multiplier
+ */
+double
+ddca_set_sleep_multiplier(double multiplier);
+
+/** Gets the sleep multiplier for the current thread
+ *
+ *  @return[in] sleep multiplier
+ */
+double
+ddca_get_sleep_multiplier();
+
+
 //
 // Output Redirection
 //
@@ -285,7 +358,7 @@ ddca_start_capture(
  *  If error output was also being captured, error output is redirected
  *  to **stderr**.
  *
- *  @return captured output as a string.
+ *  @return captured output as a string, caller responsible for freeing
  *
  *  @note
  *  If output is not currently being captured, returns a 0 length string.
@@ -348,7 +421,7 @@ ddca_is_report_ddc_errors_enabled(void);
 
 /** Turn on tracing for a specific function.
  *
- *  \param  funcname   function name
+ *  \param[in]  funcname   function name
  *
  *  \remark
  *  The function must include trace calls.
@@ -359,36 +432,62 @@ ddca_add_traced_function(
 
 /** Turn on all tracing in a specific source file.
  *
- *  \param filename  simple file name, with or without the ".c" extension,
- *                   e.g. vcp_feature_set.c, vcp_feature_set
+ *  \param[in] filename  simple file name, with or without the ".c" extension,
+ *                        e.g. vcp_feature_set.c, vcp_feature_set
  */
 void
 ddca_add_traced_file(
       const char * filename);
 
-/* Specify one or more trace groups.
+/** Replaces the groups being traced
  *
- *  \param trace_flags  bitfield indicating groups to trace
+ *  \param[in] trace_flags  bitfield indicating groups to trace
  */
 void
 ddca_set_trace_groups(
       DDCA_Trace_Group trace_flags);
 
 
+/** Adds to the groups being traced
+ *
+ *  \param[in] trace_flags  bitfield indicating groups to trace
+ *
+ *  @since 1.2.0
+ */
+void
+ddca_add_trace_groups(
+      DDCA_Trace_Group trace_flags);
+
+
 /** Given a trace group name, returns its identifier.
  *  Case is ignored.
  *
- *  \param   name trace group name
- *  \return  trace group identifier
- *  \retval  TRC_NEVER unrecognized name
+ *  \param[in] name trace group name
+ *  \return    trace group identifier
+ *  \retval    TRC_NEVER unrecognized name
  */
 DDCA_Trace_Group
 ddca_trace_group_name_to_value(char * name);
 
+/** Sets tracing options
+ *
+ *  \param[in] options  enum that can be used as bit flags
+ */
 void
 ddca_set_trace_options(DDCA_Trace_Options  options);
 
 
+//
+// Performance Options
+//
+
+bool
+ddca_enable_sleep_suppression(bool newval); ;
+
+bool
+ddca_is_sleep_suppression_enabled();
+
+
 //
 // Statistics and Diagnostics
 //
@@ -399,21 +498,38 @@ ddca_set_trace_options(DDCA_Trace_Options  options);
 void
 ddca_reset_stats(void);
 
+/** Assigns a description to the the current thread.
+ *
+ *  \param[in] description
+ */
+void
+ddca_set_thread_description(const char * description);
+
+/** Appends text to the current thread description.
+ *
+ *  \param[in] description]
+ */
+void
+ddca_append_thread_description(const char * description);
+const char * ddca_get_thread_descripton();
+
 /** Show execution statistics.
  *
  *  \param[in] stats  bitflags of statistics types to show
+ *  \param[in] include_per_thread_data include per thread detail
  *  \param[in] depth  logical indentation depth
  */
 void
 ddca_show_stats(
       DDCA_Stats_Type stats,
+      bool            include_per_thread_data,
       int             depth);
 
 // TODO: Add functions to get stats
 
 /** Enable display of internal exception reports (Error_Info).
  *
- *  @param  enable   true/false
+ *  @param[in] enable  true/false
  *  @return prior value
  */
 bool
@@ -429,10 +545,10 @@ ddca_enable_error_info(
  *
  *  Must be called before any API call that triggers display detection.
  *
- *  @param onoff
- *  @retval  DDCRC_OK                success
- *  @retval  DDCRC_INVALID_OPERATION display detection has already occurred
- *  @retval  DDCRC_UNIMPLEMENTED     ddcutil not built with USB monitor support
+ *  @param[in] onoff
+ *  @retval    DDCRC_OK                success
+ *  @retval    DDCRC_INVALID_OPERATION display detection has already occurred
+ *  @retval    DDCRC_UNIMPLEMENTED     ddcutil not built with USB monitor support
  *
  *  @remark
  *  The default is to check USB devices.
@@ -445,11 +561,52 @@ ddca_enable_usb_display_detection(bool onoff);
 /** Reports whether USB devices are checked as part of display detection
  *
  *  @retval true  USB devices are checked
- *  @retval valse UBB devices are not checked
+ *  @retval false USB devices are not checked
  */
 bool
 ddca_ddca_is_usb_display_detection_enabled();
 
+/** Gets display references list for all detected displays.
+ *
+ *  @param[in]  include_invalid_displays if true, displays that do not support DDC are included
+ *  @param[out] drefs_loc where to return pointer to null-terminated array of #DDCA_Display_Ref
+ *  @retval     0  always succeeds
+ *
+ *  @since 1.2.0
+ */
+DDCA_Status
+ddca_get_display_refs(
+      bool                include_invalid_displays,
+      DDCA_Display_Ref**  drefs_loc);
+
+/** Gets publicly visible information about a display reference
+ *
+ *  The returned struct can simply be free()'d by the client.
+ *
+ *  @param  ddca_dref display reference
+ *  @param  dinfo_loc where to return pointer to newly allocated #DDCA_Display_Info
+ *
+ *  @since 1.2.0
+ */
+DDCA_Status
+ddca_get_display_info(
+      DDCA_Display_Ref  ddca_dref,
+      DDCA_Display_Info ** dinfo_loc);
+
+/** Frees a #DDCA_Display_Info struct.
+ *
+ *  This is a convenience function. #DDCA_Display_Info is copied to
+ *  the client and contains no pointers.  It can simply be free()'d
+ *  by the client.
+ *
+ *  @param info_rec pointer to instance to free
+ *
+ *  @since 1.2.0
+ */
+
+void
+ddca_free_display_info(DDCA_Display_Info * info_rec);
+
 /** Gets a list of the detected displays.
  *
  *  @param[in]  include_invalid_displays if true, displays that do not support DDC are included
@@ -463,9 +620,9 @@ ddca_get_display_info_list2(
 
 /** Frees a list of detected displays.
  *
- *  This function understands which fields in the list
- *  point to permanently allocated data structures and should
- *  not be freed.
+ *  This is a convenience function. #DDCA_Display_Info_List
+ *  contains no pointers and is copied to the client, so the
+ *  list can simply be free'd by the client.
  *
  *  \param[in] dlist pointer to #DDCA_Display_Info_List
  */
@@ -511,6 +668,20 @@ ddca_report_displays(
       bool include_invalid_displays,
       int  depth);
 
+/** Reinitializes detected displays
+ *
+ *  - closes all open displays, releasing any display locks
+ *  - n. all existing display handles become invalid (TODO: validate dh similarly to dref)
+ *  - releases display refs (all existing display refs become invalid)
+ *  - releases i2c bus info
+ *  - rescans i2c buses
+ *  - redetects displays
+ *
+ *  @since 1.2.0
+ */
+DDCA_Status
+ddca_redetect_displays();
+
 
 //
 // Display Identifier
@@ -520,7 +691,6 @@ ddca_report_displays(
  * @param[in]  dispno  display number
  * @param[out] did_loc    where to return display identifier handle
  * @retval     0
- *
  * \ingroup api_display_spec
  * */
 DDCA_Status
@@ -540,20 +710,6 @@ ddca_create_busno_display_identifier(
       int                      busno,
       DDCA_Display_Identifier* did_loc);
 
-/** Creates a display identifier using an ADL (adapter index, display index) pair
- * @param[in]  iAdapterIndex ADL adapter index
- * @param[in]  iDisplayIndex ADL display index
- * @param[out] did_loc          where to return display identifier handle
- * @return     status code
- *
- * \ingroup api_display_spec
- */
-DDCA_Status
-ddca_create_adlno_display_identifier(
-      int                      iAdapterIndex,
-      int                      iDisplayIndex,
-      DDCA_Display_Identifier* did_loc);
-
 /** Creates a display identifier using some combination of the manufacturer id,
  * model name string and serial number string.  At least 1 of the 3 must be specified.
  * @param[in]  mfg_id    3 letter manufacturer id
@@ -685,7 +841,7 @@ ddca_get_display_ref(
  *
  * Use this function to safely release a #DDCA_Display_Ref.
  * If the display reference was dynamically created, it is freed.
- * If the display references was permanently allocated (normal case), does nothing.
+ * If the display reference was permanently allocated (normal case), does nothing.
  * @param[in] dref  display reference to free
  * @retval DDCRC_OK     success, or dref == NULL
  * @retval DDCRC_ARG    dref does not point to a valid display reference
@@ -770,7 +926,7 @@ ddca_dh_repr(
 // CHANGE NAME?
 /** Returns the display reference for display handle.
  *
- *  @param  ddca_dh   display handle
+ *  @param[in] ddca_dh   display handle
  *  @return #DDCA_Display_Ref of the handle,
  *          NULL if invalid display handle
  *
@@ -818,8 +974,8 @@ ddca_parse_capabilities_string(
 
 /** Frees a DDCA_Capabilities struct
  *
- *  @param[in]  parsed_capabilities  pointer to struct to free.
- *                                   Does nothing if NULL.
+ *  @param[in] parsed_capabilities  pointer to struct to free,
+ *                                  does nothing if NULL.
  */
 void
 ddca_free_parsed_capabilities(
@@ -891,7 +1047,7 @@ ddca_report_parsed_capabilities(
 /** Returns the VCP feature codes defined in a
  *  parsed capabilities record as a #DDCA_Feature_LIst
  *
- *  @param  parsed_caps  parsed capabilities
+ *  @param[in] parsed_caps  parsed capabilities
  *  @return bitfield of feature ids
  *  @since 0.9.0
  */
@@ -972,8 +1128,6 @@ ddca_dfr_check_by_dh(DDCA_Display_Handle ddca_dh);
 /**
  * Gets metadata for a VCP feature.
  *
- * Note that VCP characteristics (C vs NC, RW vs RO, etc) can vary by MCCS version.
- *
  * @param[in]  vspec            VCP version
  * @param[in]  feature_code     VCP feature code
  * @param[in]  create_default_if_not_found
@@ -986,9 +1140,12 @@ ddca_dfr_check_by_dh(DDCA_Display_Handle ddca_dh);
  *  It is the responsibility of the caller to free the returned DDCA_Feature_Metadata instance.
  *
  * @remark
+ * Note that VCP characteristics (C vs NC, RW vs RO, etc) can vary by MCCS version.
+ * @remark
  * Only takes into account VCP version.  Useful for reporting display agnostic
  * feature information.  For display sensitive feature information, i.e. taking
- * into account the specific monitor model, use #ddca_get_feature_metdata_by_dref().
+ * into account the specific monitor model, use #ddca_get_feature_metdata_by_dref()
+ * or #ddca_get_feature_metadata_by_dh().
  *
  * @since 0.9.3
  */
@@ -1014,6 +1171,13 @@ ddca_get_feature_metadata_by_vspec(
  *                              !create_default_if_not_found
  *
  * It is the responsibility of the caller to free the returned DDCA_Feature_Metadata instance.
+ *
+ * @remark
+ * This function first checks if there is a user supplied feature definition
+ * for the monitor.  If not, it looks up feature metadata based on the
+ * VCP version of the monitor.
+ * @remark
+ * Note that feature characteristics (C vs NC, RW vs RO, etc) can vary by MCCS version.
  * @since 0.9.3
  */
 DDCA_Status
@@ -1026,8 +1190,6 @@ ddca_get_feature_metadata_by_dref(
 /**
  * Gets metadata for a VCP feature.
  *
- * Note that VCP characteristics (C vs NC, RW vs RO, etc) can vary by MCCS version.
- *
  * @param[in]  ddca_dh          display handle
  * @param[in]  feature_code     VCP feature code
  * @param[in]  create_default_if_not_found
@@ -1038,6 +1200,13 @@ ddca_get_feature_metadata_by_dref(
  *                              !create_default_if_not_found
  *
  * It is the responsibility of the caller to free the returned DDCA_Feature_Metadata instance.
+ *
+ * @remark
+ * This function first checks if there is a user supplied feature definition
+ * for the monitor.  If not, it looks up feature metadata based on the
+ * VCP version of the monitor.
+ * @remark
+ * Note that feature characteristics (C vs NC, RW vs RO, etc) can vary by MCCS version.
  * @since 0.9.3
  */
 DDCA_Status
@@ -1109,8 +1278,8 @@ ddca_get_simple_nc_feature_value_name_by_table(
 
 /** Outputs a debugging report of the \DDCA_Feature_Metadata data structure.
  *
- *  @param md    pointer to \DDCA_Feature_Metadata instance
- *  @param depth logical indentation depth
+ *  @param[in] md    pointer to \DDCA_Feature_Metadata instance
+ *  @param[in] depth logical indentation depth
  */
 void
 ddca_dbgrpt_feature_metadata(
@@ -1126,8 +1295,8 @@ ddca_dbgrpt_feature_metadata(
  *
  * Output is written using report functions
  *
- * \param dref       pointer to display reference
- * \param depth      logical indentation depth
+ * \param[in] dref       pointer to display reference
+ * \param[in] depth      logical indentation depth
  * \retval DDCRC_ARG invalid display ref
  * \retval 0         success
  *
@@ -1153,7 +1322,7 @@ extern const DDCA_Feature_List DDCA_EMPTY_FEATURE_LIST;
 
 /** Returns feature list symbolic name (for debug messages)
  *
- *  @param feature_set_id
+ *  @param[in] feature_set_id
  *  @return symbolic name (do not free)
  */
 const char *
@@ -1177,33 +1346,40 @@ ddca_get_feature_list_by_dref(
       DDCA_Feature_Subset_Id  feature_set_id,
       DDCA_Display_Ref        dref,
       bool                    include_table_features,
-      DDCA_Feature_List*      p_feature_list);
+      DDCA_Feature_List*      feature_list_loc);
 
 /** Empties a #DDCA_Feature_List
  *
  *  @param[in]  vcplist pointer to feature list
  *
+ *  @remark
+ *  Alternatively, just set vcplist = DDCA_EMPTY_FEATURE_LIST
  *  @since 0.9.0
  */
 void
 ddca_feature_list_clear(
       DDCA_Feature_List* vcplist);
 
+
 /** Adds a feature code to a #DDCA_Feature_List
  *
  *  @param[in]  vcplist   pointer to feature list
  *  @param[in]  vcp_code  VCP feature code
+ *  #return     modified feature list
+ *
+ *  @remark
+ *  The feature list is modified in place and also returned.
  *
  *  @since 0.9.0
  */
-void
+DDCA_Feature_List
 ddca_feature_list_add(
       DDCA_Feature_List* vcplist,
       uint8_t vcp_code);
 
 /** Tests if a #DDCA_Feature_List contains a VCP feature code
  *
- *  @param[in]  vcplist   pointer to feature list
+ *  @param[in]  vcplist   feature list
  *  @param[in]  vcp_code  VCP feature code
  *  @return     true/false
  *
@@ -1211,15 +1387,30 @@ ddca_feature_list_add(
  */
 bool
 ddca_feature_list_contains(
-      DDCA_Feature_List* vcplist,
+      DDCA_Feature_List vcplist,
       uint8_t vcp_code);
 
+/** Tests if 2 feature lists are equal.
+ *
+ *  @param[in] vcplist1   first feature list
+ *  @param[in] vcplist2   second feature list
+ *  @return true if they contain the same features, false if not
+ *
+ *  @remark
+ *  The input feature lists are not modified.
+ *  @since 0.9.9
+ */
+bool
+ddca_feature_list_eq(
+      DDCA_Feature_List vcplist1,
+      DDCA_Feature_List vcplist2);
+
 /** Creates a union of 2 feature lists.
  *
- *  @param[in] vcplist1   pointer to first feature list
- *  @param[in] vcplist2   pointer to second feature list
+ *  @param[in] vcplist1   first feature list
+ *  @param[in] vcplist2   second feature list
  *  @return feature list in which a feature is set if it is in either
- *          or the 2 input feature lists
+ *          of the 2 input feature lists
  *
  *  @remark
  *  The input feature lists are not modified.
@@ -1227,13 +1418,13 @@ ddca_feature_list_contains(
  */
 DDCA_Feature_List
 ddca_feature_list_or(
-      DDCA_Feature_List* vcplist1,
-      DDCA_Feature_List * vcplist2);
+      DDCA_Feature_List vcplist1,
+      DDCA_Feature_List vcplist2);
 
 /** Creates the intersection of 2 feature lists.
  *
- *  @param[in] vcplist1   pointer to first feature list
- *  @param[in] vcplist2   pointer to second feature list
+ *  @param[in] vcplist1   first feature list
+ *  @param[in] vcplist2   second feature list
  *  @return feature list in which a feature is set if it is in both
  *          of the 2 input feature lists
  *
@@ -1243,14 +1434,14 @@ ddca_feature_list_or(
  */
 DDCA_Feature_List
 ddca_feature_list_and(
-      DDCA_Feature_List* vcplist1,
-      DDCA_Feature_List * vcplist2);
+      DDCA_Feature_List vcplist1,
+      DDCA_Feature_List vcplist2);
 
 /** Returns a feature list consisting of all the features in the
  *  first list that are not in the second.
  *
- *  @param[in] vcplist1   pointer to first feature list
- *  @param[in] vcplist2   pointer to second feature list
+ *  @param[in] vcplist1   first feature list
+ *  @param[in] vcplist2   second feature list
  *  @return feature list in which a feature is set if it is in **vcplist1** but
  *          not **vcplist2**
  *
@@ -1260,24 +1451,24 @@ ddca_feature_list_and(
  */
 DDCA_Feature_List
 ddca_feature_list_and_not(
-      DDCA_Feature_List* vcplist1,
-      DDCA_Feature_List* vcplist2);
+      DDCA_Feature_List vcplist1,
+      DDCA_Feature_List vcplist2);
 
 /** Returns the number of features in a feature list
  *
- *  @param[in] feature_list   pointer to feature list
+ *  @param[in] feature_list   feature list
  *  @return  number of features, 0 if feature_list == NULL
  *
  *  @since 0.9.0
  */
 int
 ddca_feature_list_count(
-      DDCA_Feature_List * feature_list);
+      DDCA_Feature_List feature_list);
 
 /** Returns a string representation of a feature list as a
  *  sequence of 2 character hex values.
  *
- *  @param[in] feature_list   pointer to feature list
+ *  @param[in] feature_list   feature list
  *  @param[in] value_prefix   precede each value with this string, e.g. "0x"
  *                            if NULL, then no preceding string
  *  @param[in] sepstr         separator string between pair of values, e.g. ", "
@@ -1285,15 +1476,13 @@ ddca_feature_list_count(
  *  @return    string representation; The value is valid until the next call
  *             to this function in the current thread.  Caller should not free.
  *
- *  @remark
- *  Returns NULL if feature_list == NULL
  *  @since 0.9.0
  */
 const char *
 ddca_feature_list_string(
-      DDCA_Feature_List * feature_list,
-      char * value_prefix,
-      char * sepstr);
+      DDCA_Feature_List feature_list,
+      const char * value_prefix,
+      const char * sepstr);
 
 
 //
@@ -1343,8 +1532,8 @@ ddca_free_any_vcp_value(
 
 /** Produces a debugging report of a #DDCA_Any_Vcp_Value instance.
  *  The report is written to the current FOUT device.
- *  @param  valrec  instance to report
- *  @param  depth   logical indentation depth
+ *  @param[in]  valrec  instance to report
+ *  @param[in]  depth   logical indentation depth
  *  @since 0.9.0
  */
 void
