@@ -7,7 +7,7 @@
  *  within the code that implements the API.
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DDCUTIL_TYPES_H_
@@ -121,7 +121,8 @@ typedef enum{
 typedef enum {
    DDCA_OL_TERSE  =0x04,         /**< Brief   output  */
    DDCA_OL_NORMAL =0x08,         /**< Normal  output */
-   DDCA_OL_VERBOSE=0x10          /**< Verbose output */
+   DDCA_OL_VERBOSE=0x10,         /**< Verbose output */
+   DDCA_OL_VV=0x20               /**< Very verbose output */
 } DDCA_Output_Level;
 
 
@@ -145,7 +146,7 @@ typedef enum {
  DDCA_TRC_VCP   = 0x0200,       /**< VCP layer, feature definitions */
  DDCA_TRC_DDCIO = 0x0400,       /**< DDC IO functions */
  DDCA_TRC_SLEEP = 0x0800,       /**< low level sleeps */
- DDCA_TRC_RETRY = 0x1000,       /**< successful retries, subset of DDCA_TRC_SLEEP */
+ DDCA_TRC_RETRY = 0x1000,       /**< successful retries, subset of DDCA_TRC_DDCIO */
 
  DDCA_TRC_NONE = 0x0000,        /**< all tracing disabled    */
  DDCA_TRC_ALL  = 0xffff         /**< all tracing enabled     */
@@ -155,6 +156,7 @@ typedef enum {
 typedef enum {
    DDCA_TRCOPT_TIMESTAMP = 0x01,
    DDCA_TRCOPT_THREAD_ID = 0x02,
+   DDCA_TRCOPT_WALLTIME  = 0x04
 } DDCA_Trace_Options;
 
 
@@ -345,7 +347,10 @@ typedef struct {
 
 
 #define DDCA_DISPLAY_INFO_MARKER "DDIN"
-/** Describes one monitor detected by ddcutil. */
+/** Describes one monitor detected by ddcutil.
+ *
+ *  This struct is copied to the caller and can simply be freed.
+ */
 typedef struct {
    char                   marker[4];        ///< always "DDIN"
    int                    dispno;           ///< ddcutil assigned display number
@@ -459,6 +464,7 @@ struct {
    DDCA_MCCS_Version_Spec                vcp_version;    /**< MCCS version    */
    DDCA_Feature_Flags                    feature_flags;  /**< feature type description */
    DDCA_Feature_Value_Entry *            sl_values;      /**< valid when DDCA_SIMPLE_NC set */
+   DDCA_Feature_Value_Entry *            latest_sl_values;  /** no longer used */
    char *                                feature_name;   /**< feature name */
    char *                                feature_desc;   /**< feature description */
    // possibly add pointers to formatting functions
