@@ -1,7 +1,7 @@
 /** @file usb_displays.c
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -17,12 +17,12 @@
 #include "util/device_id_util.h"
 #include "util/report_util.h"
 #include "util/string_util.h"
-#include "util/udev_util.h"
 #include "util/udev_usb_util.h"
+#include "util/udev_util.h"
 
-#include "usb_util/usb_hid_common.h"
 #include "usb_util/hiddev_reports.h"
 #include "usb_util/hiddev_util.h"
+#include "usb_util/usb_hid_common.h"
 
 #include "base/core.h"
 #include "base/ddc_errno.h"
@@ -77,7 +77,8 @@ static GPtrArray * usb_monitors = NULL;    // array of Usb_Monitor_Info
  *
  * Returns:   nothing
  */
-static void report_usb_monitor_vcp_rec(Usb_Monitor_Vcp_Rec * vcprec, int depth) {
+static void
+dbgrpt_usb_monitor_vcp_rec(Usb_Monitor_Vcp_Rec * vcprec, int depth) {
    const int d1 = depth+1;
    rpt_structure_loc("Usb_Monitor_Vcp_Rec", vcprec, depth);
    rpt_vstring(d1, "%-20s:    %-4.4s", "marker",       vcprec->marker);
@@ -101,7 +102,8 @@ static void report_usb_monitor_vcp_rec(Usb_Monitor_Vcp_Rec * vcprec, int depth)
  *
  * Returns:       nothing
  */
-void dbgrpt_usb_monitor_info(Usb_Monitor_Info * moninfo, int depth) {
+void
+dbgrpt_usb_monitor_info(Usb_Monitor_Info * moninfo, int depth) {
    const int d1 = depth+1;
    const int d2 = depth+2;
    rpt_structure_loc("Usb_Monitor_Info", moninfo, d1);
@@ -116,7 +118,7 @@ void dbgrpt_usb_monitor_info(Usb_Monitor_Info * moninfo, int depth) {
       if (monrecs) {
          rpt_vstring(d1, "vcp feature code 0x%02x has %d records:", feature_code, monrecs->len);
          for (int ndx=0; ndx<monrecs->len; ndx++) {
-            report_usb_monitor_vcp_rec( g_ptr_array_index(monrecs, ndx), d2);
+            dbgrpt_usb_monitor_vcp_rec( g_ptr_array_index(monrecs, ndx), d2);
          }
       }
    }
@@ -132,7 +134,8 @@ void dbgrpt_usb_monitor_info(Usb_Monitor_Info * moninfo, int depth) {
  * Returns:      nothing
  */
 // static
-void report_usb_monitors(GPtrArray * monitors, int depth) {
+void
+report_usb_monitors(GPtrArray * monitors, int depth) {
    const int d1 = depth+1;
 
    rpt_vstring(depth, "GPtrArray of %d Usb_Monitor_Info at %p", monitors->len, monitors);
@@ -150,7 +153,8 @@ void report_usb_monitors(GPtrArray * monitors, int depth) {
  *
  * Returns:  array of Usb_Monitor_Vcp_Rec for each usage
  */
-GPtrArray * collect_vcp_reports(int fd) {
+GPtrArray *
+collect_vcp_reports(int fd) {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Starting");
    GPtrArray * vcp_reports = g_ptr_array_new();
@@ -252,7 +256,8 @@ GPtrArray * collect_vcp_reports(int fd) {
  *
  *  It is the responsibility of the caller to free the returned string.
  */
-static char * usb_synthesize_capabilities_string(Usb_Monitor_Info * moninfo) {
+static char *
+usb_synthesize_capabilities_string(Usb_Monitor_Info * moninfo) {
    assert(moninfo);
    char buf[1000];
    strcpy(buf,"(vcp(");
@@ -325,7 +330,8 @@ avoid_device_by_usb_interfaces_property_string(const char * interfaces) {
  *  @return   true/false
  */
 
-bool is_possible_monitor_by_hiddev_name(const char * hiddev_name) {
+bool
+is_possible_monitor_by_hiddev_name(const char * hiddev_name) {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Starting.  hiddev_name = %s", hiddev_name);
 
@@ -364,7 +370,8 @@ bool is_possible_monitor_by_hiddev_name(const char * hiddev_name) {
  *
  *  The result is cached in global variable usb_monitors
  */
-GPtrArray * get_usb_monitor_list() {
+GPtrArray *
+get_usb_monitor_list() {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Starting...");
    DDCA_Output_Level ol = get_output_level();
@@ -481,7 +488,7 @@ GPtrArray * get_usb_monitor_list() {
       }  // monitor opened
    } // loop over device names
 
-   g_ptr_array_set_free_func(hiddev_names, free);
+   g_ptr_array_set_free_func(hiddev_names, g_free);
    g_ptr_array_free(hiddev_names, true);
 
    if ( debug || IS_TRACING() ) {
@@ -498,7 +505,8 @@ GPtrArray * get_usb_monitor_list() {
 // Functions to find Usb_Monitor_Info for a display
 //
 
-static Usb_Monitor_Info * usb_find_monitor_by_busnum_devnum(int busnum, int devnum) {
+static Usb_Monitor_Info *
+usb_find_monitor_by_busnum_devnum(int busnum, int devnum) {
    bool debug = false;
    DBGMSF(debug, "Starting. busnum=%d, devnum=%d", busnum, devnum);
    assert(usb_monitors);
@@ -518,7 +526,8 @@ static Usb_Monitor_Info * usb_find_monitor_by_busnum_devnum(int busnum, int devn
 }
 
 
-static Usb_Monitor_Info * usb_find_monitor_by_display_ref(Display_Ref * dref) {
+static Usb_Monitor_Info *
+usb_find_monitor_by_dref(Display_Ref * dref) {
    bool debug = false;
    DBGMSF(debug, "Starting. dref = %s", dref_repr_t(dref));
    assert(dref->io_path.io_mode == DDCA_IO_USB);
@@ -528,7 +537,8 @@ static Usb_Monitor_Info * usb_find_monitor_by_display_ref(Display_Ref * dref) {
 }
 
 
-Usb_Monitor_Info * usb_find_monitor_by_display_handle(Display_Handle * dh) {
+Usb_Monitor_Info *
+usb_find_monitor_by_dh(Display_Handle * dh) {
    // printf("(%s) Starting. dh=%p\n", __func__, dh);
    bool debug = false;
    DBGMSF(debug, "Starting. dh = %s", dh_repr(dh));
@@ -542,8 +552,8 @@ Usb_Monitor_Info * usb_find_monitor_by_display_handle(Display_Handle * dh) {
 
 
 #ifdef APPARENTLY_UNUSED
-char * get_hiddev_devname_by_display_ref(Display_Ref * dref) {
-   Usb_Monitor_Info * moninfo = usb_find_monitor_by_display_ref(dref);
+char * get_hiddev_devname_by_dref(Display_Ref * dref) {
+   Usb_Monitor_Info * moninfo = usb_find_monitor_by_dref(dref);
    char * result = moninfo->hiddev_device_name;
    DBGMSG("dref=%s, returning: %s", dref_short_name(dref), result);
    return result;
@@ -601,113 +611,10 @@ Display_Info_List usb_get_valid_displays() {
 
 //  *** Functions to return a Display_Ref for a USB monitor ***
 
-#ifdef PRE_DISPLAY_REF
-
-static Display_Ref *
-create_display_ref_from_usb_monitor_info(Usb_Monitor_Info * moninfo) {
-   // hacky - to be cleaned up
-   Display_Ref * dref = create_usb_display_ref(moninfo->hiddev_devinfo->busnum,
-                                               moninfo->hiddev_devinfo->devnum,
-                                               moninfo->hiddev_device_name);
-   return dref;
-}
-
-
-Display_Ref *
-usb_find_display_by_mfg_model_sn(const char * mfg_id, const char * model, const char * sn) {
-   Display_Ref * result = NULL;
-
-   Usb_Monitor_Info * found_monitor = NULL;
-   GPtrArray * all_usb_monitors = get_usb_monitor_list();
-   for (int ndx=0; ndx<all_usb_monitors->len; ndx++) {
-      Usb_Monitor_Info * curmon = g_ptr_array_index(all_usb_monitors, ndx);
-      bool some_test_passed = false;
-      bool some_test_failed = false;
-
-      if (mfg_id && strlen(mfg_id) > 0) {
-         if ( streq(mfg_id, curmon->edid->mfg_id) )
-            some_test_passed = true;
-         else
-            some_test_failed = false;
-      }
-
-      if (model && strlen(model) > 0) {
-         if ( streq(model, curmon->edid->model_name) )
-            some_test_passed = true;
-         else
-            some_test_failed = false;
-      }
-
-      if (sn && strlen(sn) > 0) {
-         if ( streq(sn, curmon->edid->serial_ascii) )
-            some_test_passed = true;
-         else
-            some_test_failed = false;
-      }
-
-      // if ( strcmp(model, curmon->edid->model_name)   == 0 &&
-      //      strcmp(sn,    curmon->edid->serial_ascii) == 0
-      //    )
-      if (some_test_passed && !some_test_failed)
-      {
-          found_monitor = curmon;
-          break;
-       }
-    }
-    if (found_monitor)
-       result = create_display_ref_from_usb_monitor_info(found_monitor);
-
-   return result;
-}
-
-
-Display_Ref *
-usb_find_display_by_busnum_devnum(int busnum, int devnum) {
-   Display_Ref * result = NULL;
-
-   Usb_Monitor_Info * found_monitor = NULL;
-   GPtrArray * all_usb_monitors = get_usb_monitor_list();
-   for (int ndx=0; ndx<all_usb_monitors->len; ndx++) {
-      Usb_Monitor_Info * curmon = g_ptr_array_index(all_usb_monitors, ndx);
-      if ( curmon->hiddev_devinfo->busnum == busnum &&
-           curmon->hiddev_devinfo->devnum == devnum
-         )
-      {
-          found_monitor = curmon;
-          break;
-       }
-    }
-    if (found_monitor)
-       result = create_display_ref_from_usb_monitor_info(found_monitor);
-
-   return result;
-}
-
-
-Display_Ref *
-usb_find_display_by_edid(const Byte * edidbytes) {
-   Display_Ref * result = NULL;
-
-   Usb_Monitor_Info * found_monitor = NULL;
-    GPtrArray * all_usb_monitors = get_usb_monitor_list();
-    for (int ndx=0; ndx<all_usb_monitors->len; ndx++) {
-       Usb_Monitor_Info * curmon = g_ptr_array_index(all_usb_monitors, ndx);
-       if ( memcmp(edidbytes, curmon->edid->bytes, 128) == 0) {
-          found_monitor = curmon;
-          break;
-       }
-    }
-    if (found_monitor)
-       result = create_display_ref_from_usb_monitor_info(found_monitor);
-
-   return result;
-}
-#endif
-
-
-bool usb_is_valid_display_ref(Display_Ref * dref, bool emit_error_msg) {
+bool
+usb_is_valid_display_ref(Display_Ref * dref, bool emit_error_msg) {
    bool result = true;
-   if (!usb_find_monitor_by_display_ref(dref)) {
+   if (!usb_find_monitor_by_dref(dref)) {
       result = false;
       if (emit_error_msg)
          fprintf(stderr, "Invalid Display_Ref\n");
@@ -716,11 +623,12 @@ bool usb_is_valid_display_ref(Display_Ref * dref, bool emit_error_msg) {
 }
 
 
-void usb_show_active_display_by_display_ref(Display_Ref * dref, int depth) {
+void
+usb_show_active_display_by_dref(Display_Ref * dref, int depth) {
    DDCA_Output_Level output_level = get_output_level();
    rpt_vstring(depth, "USB bus:device:      %d:%d", dref->usb_bus, dref->usb_device);
 
-   Usb_Monitor_Info * moninfo = usb_find_monitor_by_display_ref(dref);
+   Usb_Monitor_Info * moninfo = usb_find_monitor_by_dref(dref);
 
 #ifdef OLD
    if (output_level == DDCA_OL_TERSE || output_level == OL_PROGRAM)
@@ -760,19 +668,23 @@ void usb_show_active_display_by_display_ref(Display_Ref * dref, int depth) {
 // (for hiding Usb_Monitor_Info from higher software levels)
 //
 
-Parsed_Edid * usb_get_parsed_edid_by_display_ref(Display_Ref * dref) {
-   Usb_Monitor_Info * moninfo = usb_find_monitor_by_display_ref(dref);
-   return moninfo->edid;
-}
-
-Parsed_Edid * usb_get_parsed_edid_by_display_handle(Display_Handle * dh) {
-   Usb_Monitor_Info * moninfo = usb_find_monitor_by_display_handle(dh);
+Parsed_Edid *
+usb_get_parsed_edid_by_dref(Display_Ref * dref) {
+   Usb_Monitor_Info * moninfo = usb_find_monitor_by_dref(dref);
    return moninfo->edid;
 }
 
 
-char * usb_get_capabilities_string_by_display_handle(Display_Handle * dh) {
-   Usb_Monitor_Info * moninfo = usb_find_monitor_by_display_handle(dh);
+Parsed_Edid *
+usb_get_parsed_edid_by_dh(Display_Handle * dh) {
+   Usb_Monitor_Info * moninfo = usb_find_monitor_by_dh(dh);
+   return moninfo->edid;
+}
+
+
+char *
+usb_get_capabilities_string_by_dh(Display_Handle * dh) {
+   Usb_Monitor_Info * moninfo = usb_find_monitor_by_dh(dh);
    assert(dh);
    return usb_synthesize_capabilities_string(moninfo);
 }
@@ -797,7 +709,8 @@ char * usb_get_capabilities_string_by_display_handle(Display_Handle * dh) {
  * Note that messages will not appear when this function runs as part
  * of normal udev execution.  They are intended to aid in debugging.
  */
-bool check_usb_monitor( char * device_name ) {
+bool
+check_usb_monitor( char * device_name ) {
    assert(device_name);
    bool debug = false;
    DDCA_Output_Level ol = get_output_level();
@@ -830,7 +743,8 @@ bool check_usb_monitor( char * device_name ) {
  }
 
 
-void init_usb_displays() {
+void
+init_usb_displays() {
    rtti_func_name_table_add(get_usb_monitor_list, "get_usb_monitor_list");
    rtti_func_name_table_add(avoid_device_by_usb_interfaces_property_string,
                             "avoid_device_by_usb_interfaces_property_string");
