@@ -3,7 +3,7 @@
  *  Functions to get EDID for USB connected monitors
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -12,7 +12,7 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <glib.h>
+#include <glib-2.0/glib.h>
 #include <linux/hiddev.h>
 #include <linux/limits.h>
 #include <stddef.h>
@@ -21,7 +21,6 @@
 #include <string.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
-// #include <wchar.h>
 
 #include "util/device_id_util.h"
 #include "util/report_util.h"
@@ -40,7 +39,9 @@
 
 #include "i2c/i2c_bus_core.h"     // for EDID fallback
 #include "i2c/i2c_bus_selector.h" // for EDID fallback
+#ifdef ADL
 #include "adl/adl_shim.h"         // for EDID fallback
+#endif
 
 #include "usb/usb_base.h"
 
@@ -245,7 +246,7 @@ Parsed_Edid * get_x11_edid_by_model_sn(char * model_name, char * sn_ascii) {
          if (streq(parsed_edid->model_name, model_name) &&
                streq(parsed_edid->serial_ascii, sn_ascii) )
          {
-            g_strlcpy(parsed_edid->edid_source, "X11", EDID_SOURCE_FIELD_SIZE);
+            STRLCPY(parsed_edid->edid_source, "X11", EDID_SOURCE_FIELD_SIZE);
             DBGMSF(debug, "Found matching EDID from X11");
             break;
          }
@@ -302,7 +303,7 @@ Parsed_Edid * get_fallback_hiddev_edid(int fd, struct hiddev_devinfo * dev_info)
             DBGMSG("Using EDID for /dev/i2c-%d", bus_info->busno);
             parsed_edid = bus_info->edid;
             edid_source = "I2C";
-            // g_strlcpy(parsed_edid->edid_source, "I2C", EDID_SOURCE_FIELD_SIZE);
+            // STRLCPY(parsed_edid->edid_source, "I2C", EDID_SOURCE_FIELD_SIZE);
             // result = NULL;   // for testing - both i2c and X11 methods work
          }
          else {    // ADL
@@ -311,8 +312,7 @@ Parsed_Edid * get_fallback_hiddev_edid(int fd, struct hiddev_devinfo * dev_info)
                                     NULL,              // mfg_id
                                     model_sn->model,
                                     model_sn->sn);
-            parsed_edid = adlshim_get_parsed_edid_by_display_ref(dref);
-#endif
+            parsed_edid = adlshim_get_parsed_edid_by_dref(dref);
             DDCA_Adlno adlno = adlshim_find_adlno_by_mfg_model_sn(
                                     NULL,              // mfg_id
                                     model_sn->model,
@@ -320,10 +320,12 @@ Parsed_Edid * get_fallback_hiddev_edid(int fd, struct hiddev_devinfo * dev_info)
             if (adlno.iAdapterIndex >= 0)   {    //  {-1,-1} means unset
                parsed_edid = adlshim_get_parsed_edid_by_adlno(adlno.iAdapterIndex, adlno.iDisplayIndex);
                edid_source = "ADL";
-                  // g_strlcpy(parsed_edid->edid_source, "ADL", EDID_SOURCE_FIELD_SIZE);
+                  // STRLCPY(parsed_edid->edid_source, "ADL", EDID_SOURCE_FIELD_SIZE);
             }
             // memory leak: not freeing dref because don't want to clobber parsed_edid
             // need to review Display_Ref lifecycle
+#endif
+            PROGRAM_LOGIC_ERROR("ADL implementation removed");
          }
       }
    }
@@ -338,7 +340,7 @@ Parsed_Edid * get_fallback_hiddev_edid(int fd, struct hiddev_devinfo * dev_info)
    if (model_sn)
       free_model_sn_pair(model_sn);
    if (parsed_edid)
-      g_strlcpy(parsed_edid->edid_source, edid_source, EDID_SOURCE_FIELD_SIZE);
+      STRLCPY(parsed_edid->edid_source, edid_source, EDID_SOURCE_FIELD_SIZE);
    DBGMSF(debug, "Returning: %p", parsed_edid);
    return parsed_edid;
 }
@@ -386,7 +388,7 @@ Parsed_Edid * get_hiddev_edid_with_fallback(int fd, struct hiddev_devinfo * dev_
           // }
        }
        else
-          g_strlcpy(parsed_edid->edid_source, "USB", EDID_SOURCE_FIELD_SIZE);
+          STRLCPY(parsed_edid->edid_source, "USB", EDID_SOURCE_FIELD_SIZE);
 
        buffer_free(edid_buffer, __func__);
        edid_buffer = NULL;
