@@ -1,11 +1,11 @@
 /** @file hid_report_descriptor.c
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <assert.h>
-#include <glib.h>
+#include <glib-2.0/glib.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -443,7 +443,8 @@ void free_cur_report_locals(Cur_Report_Locals * locals) {
 }
 
 
-Parsed_Hid_Report * find_hid_report(Parsed_Hid_Collection * col, Byte report_type, uint16_t report_id) {
+Parsed_Hid_Report *
+find_hid_report(Parsed_Hid_Collection * col, Byte report_type, uint16_t report_id) {
    Parsed_Hid_Report * result = NULL;
 
    if (col->reports->len) {
