@@ -2,19 +2,20 @@
  *  General purpose data structures
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
 #include <glib-2.0/glib.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/param.h>     // for MIN, MAX
 /** \endcond */
 
-#include "debug_util.h"
+// #include "debug_util.h"
 #include "string_util.h"
 
 #include "data_structures.h"
@@ -48,7 +49,7 @@ Byte_Value_Array bva_create() {
 Byte_Value_Array bva_filter(Byte_Value_Array  bva, IFilter filter_func) {
    GByteArray * src = (GByteArray*) bva;
    GByteArray * result = g_byte_array_new();
-   for (int ndx=0; ndx < src->len;  ndx++) {
+   for (guint ndx=0; ndx < src->len;  ndx++) {
       guint8 v = src->data[ndx];
       // Byte v1 = v;
       if (filter_func(v))
@@ -76,8 +77,12 @@ int bva_length(Byte_Value_Array bva) {
  */
 void bva_append(Byte_Value_Array bva, Byte item) {
    GByteArray* ga = (GByteArray*) bva;
+#ifdef NDEBUG
+   g_byte_array_append(ga, &item, 1);
+#else
    GByteArray * ga2 = g_byte_array_append(ga, &item, 1);
    assert(ga2 == ga);
+#endif
 }
 
 
@@ -90,9 +95,9 @@ void bva_append(Byte_Value_Array bva, Byte item) {
  *  **ndx** must be a valid value.
  *  The only check is by assert().
  */
-Byte bva_get(Byte_Value_Array bva, int ndx) {
+Byte bva_get(Byte_Value_Array bva, guint ndx) {
    GByteArray* ga = (GByteArray*) bva;
-   assert(0 <= ndx && ndx < ga->len);
+   assert(ndx < ga->len);
 
    guint8 v = ga->data[ndx];
    Byte v1 = v;
@@ -108,7 +113,7 @@ Byte bva_get(Byte_Value_Array bva, int ndx) {
  */
 bool bva_contains(Byte_Value_Array bva, Byte item) {
    GByteArray* ga = (GByteArray*) bva;
-   int ndx;
+   guint ndx;
    bool result = false;
    // printf("(%s) item=0x%02x, ga->len=%d\n", __func__, item, ga->len);
    for (ndx=0; ndx < ga->len; ndx++) {
@@ -255,6 +260,7 @@ void bva_report(Byte_Value_Array bva, char * title) {
 }
 
 
+#ifdef TESTS
 // Tests and sample code
 
 int egmain(int argc, char** argv) {
@@ -288,6 +294,7 @@ void test_value_array() {
    }
    bva_free(bva);
 }
+#endif
 
 
 //
@@ -486,6 +493,7 @@ char * bbf_to_string(Byte_Bit_Flags bbflags, char * buffer, int buflen) {
    else
       buffer = malloc(reqd_size);
    char * pos = buffer;
+   *pos = '\0';
    unsigned int flagno = 0;
    // printf("(%s) bbflags->byte=0x%s\n", __func__, hexstring(flags->byte,32));
    for (flagno = 0; flagno < 256; flagno++) {
@@ -513,8 +521,10 @@ int bbf_to_bytes(Byte_Bit_Flags bbflags, Byte * buffer, int buflen) {
    BYTE_BIT_UNOPAQUE(flags, bbflags);
    BYTE_BIT_VALIDATE(flags);
 
+#ifndef NDEBUG
    int bit_set_ct = bbf_count_set(flags);
    assert(buflen >= bit_set_ct);
+#endif
 
    unsigned int bufpos = 0;
    unsigned int flagno = 0;
@@ -853,21 +863,23 @@ Buffer * buffer_dup(Buffer * srcbuf, const char * trace_msg) {
 void buffer_free(Buffer * buffer, const char * trace_msg) {
    if (trace_buffer_malloc_free)
       printf("(%s) Starting. buffer = %p\n", __func__, (void*) buffer);
-   ASSERT_WITH_BACKTRACE(buffer);
-   ASSERT_WITH_BACKTRACE(memcmp(buffer->marker, BUFFER_MARKER, 4) == 0);
 
-      if (buffer->bytes) {
-        if (trace_buffer_malloc_free)
-            printf("(%s) Freeing buffer->bytes = %p, &buffer->bytes=%p\n",
-                   __func__, buffer->bytes, (void*)&(buffer->bytes));
-        free(buffer->bytes);
-      }
-      if (trace_buffer_malloc_free)
-         printf("(%s) Freeing buffer = %p, %s\n", __func__, (void*)buffer, trace_msg);
-      buffer->marker[3] = 'x';
-      free(buffer);
-      if (trace_buffer_malloc_free)
-         printf("(%s) Done\n", __func__);
+   // ASSERT_WITH_BACKTRACE(buffer);
+   // ASSERT_WITH_BACKTRACE(memcmp(buffer->marker, BUFFER_MARKER, 4) == 0);
+
+   if (buffer->bytes) {
+     if (trace_buffer_malloc_free)
+         printf("(%s) Freeing buffer->bytes = %p, &buffer->bytes=%p\n",
+                __func__, buffer->bytes, (void*)&(buffer->bytes));
+     free(buffer->bytes);
+   }
+   if (trace_buffer_malloc_free)
+      printf("(%s) Freeing buffer = %p, %s\n", __func__, (void*)buffer, trace_msg);
+   buffer->marker[3] = 'x';
+   free(buffer);
+
+   if (trace_buffer_malloc_free)
+      printf("(%s) Done\n", __func__);
 }
 
 
@@ -1243,3 +1255,416 @@ void vnt_debug_table(Value_Name_Title * table) {
    }
 }
 
+
+//
+// Circular_String_Buffer
+//
+
+/** Allocates a new #Circular_String_Buffer
+ *
+ *  @param  size  buffer size (number of entries)
+ *  @return newly allocated #Circular_String_Buffer
+ */
+Circular_String_Buffer *
+csb_new(int size) {
+   Circular_String_Buffer * csb = calloc(1, sizeof(Circular_String_Buffer));
+   csb->lines = calloc(size, sizeof(char*));
+   csb->size = size;
+   csb->ct = 0;
+   return csb;
+}
+
+
+/** Appends a string to a #Circular_String_Buffer.
+ *
+ *  \param   csb   #Circular_String_Buffer
+ *  \param   line  string to append
+ *  \param   copy  if true, a copy of the string is appended to the buffer
+ *                 if false, the string itself is appended
+ */
+void
+csb_add(Circular_String_Buffer * csb, char * line, bool copy) {
+    int nextpos = csb->ct % csb->size;
+    // printf("(%s) Adding at ct %d, pos %d, line |%s|\n", __func__, csb->ct, nextpos, line);
+    if (csb->lines[nextpos])
+       free(csb->lines[nextpos]);
+    if (copy)
+       csb->lines[nextpos] = g_strdup(line);
+    else
+       csb->lines[nextpos] = line;
+    csb->ct++;
+}
+
+
+/** All the strings in a #Circular_String_Buffer are moved to  a newly
+ *  allocated GPtrArray. The count of lines in the now empty #Circular_String_Buffer
+ *  is set to 0.
+ *
+ *   \param csb #Circular_String_Buffer to convert
+ *   \return    newly allocated #GPtrArray
+ */
+GPtrArray *
+csb_to_g_ptr_array(Circular_String_Buffer * csb) {
+   // printf("(%s) csb->size=%d, csb->ct=%d\n", __func__, csb->size, csb->ct);
+   GPtrArray * pa = g_ptr_array_sized_new(csb->ct);
+
+   int first = 0;
+   if (csb->ct > csb->size)
+      first = csb->ct % csb->size;
+   // printf("(%s) first=%d\n", __func__, first);
+
+   for (int ndx = 0; ndx < csb->ct; ndx++) {
+      int pos = (first + ndx) % csb->size;
+      char * s = csb->lines[pos];
+      // printf("(%s) line %d, |%s|\n", __func__, ndx, s);
+
+      g_ptr_array_add(pa, s);
+   }
+   csb->ct = 0;
+   return pa;
+}
+
+
+//
+// bs256 - Bit_Set_256
+//
+// A data structure containing 256 flags
+//
+
+const Bit_Set_256 EMPTY_BIT_SET_256 = {{0}};
+
+/** Sets a flag in a #Bit_Set_256
+ *
+ *  \param  flags   existing #Bit_Set_256 value
+ *  \param  flagno  flag number to set (0 based)
+ *  \return updated set
+ */
+Bit_Set_256 bs256_add(
+    Bit_Set_256 bitset,
+    Byte        bitno)
+{
+    bool debug = false;
+
+    Bit_Set_256 result = bitset;
+    int bytendx   = bitno >> 3;
+    int shiftct   = bitno & 0x07;
+    Byte flagbit  = 0x01 << shiftct;
+    if (debug) {
+       printf("(%s) bitno=0x%02x, bytendx=%d, shiftct=%d, flagbit=0x%02x\n",
+              __func__, bitno, bytendx, shiftct, flagbit);
+    }
+    result.bytes[bytendx] |= flagbit;
+
+    if (debug) {
+       char * bs1 = strdup(bs256_to_string(bitset,  "",""));
+       char * bs2 = strdup(bs256_to_string(result, "",""));
+       printf("(%s) old bitstring=%s, value %d, returning: %s\n",
+              __func__, bs1, bitno, bs2);
+       free( bs1);
+       free(bs2);
+    }
+
+    return result;
+}
+
+
+/** Tests if a bit is set in a #Bit_Set_256.
+ *
+ *  \param bitset  #Bit_Set_256 to check
+ *  \param bitno   bit number to test (0 based)
+ *  \return true/false
+ */
+bool bs256_contains(
+    Bit_Set_256 bitset,
+    Byte        bitno)
+{
+    bool debug = false;
+
+    int flagndx   = bitno >> 3;
+    int shiftct   = bitno  & 0x07;
+    Byte flagbit  = 0x01 << shiftct;
+    // printf("(%s) bitno=0x%02x, flagndx=%d, shiftct=%d, flagbit=0x%02x\n",
+    //        __func__, bitno, flagndx, shiftct, flagbit);
+    bool result = bitset.bytes[flagndx] & flagbit;
+    if (debug) {
+       printf("(%s) bitset:\n   ",__func__);
+       for (int ndx = 0; ndx < 32; ndx++) {
+          printf("%02x", bitset.bytes[ndx]);
+       }
+       printf("\n");
+       printf("(%s)  bit %d, returning: %d\n",  __func__, bitno, result);
+    }
+    return result;
+}
+
+
+bool bs256_eq(
+    Bit_Set_256 set1,
+    Bit_Set_256 set2)
+{
+   return memcmp(&set1, &set2, 32) == 0;
+}
+
+
+Bit_Set_256 bs256_or(
+   Bit_Set_256 set1,
+   Bit_Set_256 set2)
+{
+   Bit_Set_256 result;
+   for (int ndx = 0; ndx < 32; ndx++) {
+      result.bytes[ndx] =  set1.bytes[ndx] | set2.bytes[ndx];
+   }
+   return result;
+}
+
+
+Bit_Set_256 bs256_and(
+   Bit_Set_256 set1,
+   Bit_Set_256 set2)
+{
+   Bit_Set_256 result;
+   for (int ndx = 0; ndx < 32; ndx++) {
+      result.bytes[ndx] =  set1.bytes[ndx] & set2.bytes[ndx];
+   }
+   return result;
+}
+
+
+Bit_Set_256 bs256_and_not(
+      Bit_Set_256 set1,
+      Bit_Set_256 set2)
+{
+   // DBGMSG("Starting. vcplist1=%p, vcplist2=%p", vcplist1, vcplist2);
+   Bit_Set_256 result;
+   for (int ndx = 0; ndx < 32; ndx++) {
+      result.bytes[ndx] =  set1.bytes[ndx] & ~set2.bytes[ndx];
+   }
+
+   // char * s = ddca_bs256_string(&result, "0x",", ");
+   // DBGMSG("Returning: %s", s);
+   // free(s);
+   return result;
+}
+
+
+#define BB256_REPR_BUF_SZ (3*32+1)
+/** Represents a #Bit_Set_256 value as a sequence of 32 hex values.
+ *
+ *  \param buf   buffer in which to return value
+ *  \param bufsz buffer size, must be at least #BB256_REPR_BUF_SZ
+ *  \param bbset value to represent
+ */
+void bb256_repr(char * buf, int bufsz, Bit_Set_256 bbset) {
+   assert(bufsz >= BB256_REPR_BUF_SZ);
+   g_snprintf(buf, bufsz,
+              "%02x %02x %02x %02x %02x %02x %02x %02x "
+              "%02x %02x %02x %02x %02x %02x %02x %02x "
+              "%02x %02x %02x %02x %02x %02x %02x %02x "
+              "%02x %02x %02x %02x %02x %02x %02x %02x",
+              bbset.bytes[ 0], bbset.bytes[ 1], bbset.bytes[ 2], bbset.bytes[ 3],
+              bbset.bytes[ 4], bbset.bytes[ 5], bbset.bytes[ 6], bbset.bytes[ 7],
+              bbset.bytes[ 8], bbset.bytes[ 9], bbset.bytes[10], bbset.bytes[11],
+              bbset.bytes[12], bbset.bytes[13], bbset.bytes[14], bbset.bytes[15],
+              bbset.bytes[16], bbset.bytes[17], bbset.bytes[18], bbset.bytes[19],
+              bbset.bytes[20], bbset.bytes[21], bbset.bytes[22], bbset.bytes[23],
+              bbset.bytes[24], bbset.bytes[25], bbset.bytes[26], bbset.bytes[27],
+              bbset.bytes[28], bbset.bytes[29], bbset.bytes[30], bbset.bytes[31] );
+}
+
+
+/** Returns the number of bits set in a #Bit_Set_256 instance.
+ *
+ *  \param  bbset  value to examine
+ *  \return number of bits set
+ */
+int bs256_count(
+   Bit_Set_256 bbset)
+{
+   bool debug = false;
+
+   int result = 0;
+   int flagndx;
+   int bitndx;
+   for (flagndx=0; flagndx < 32; flagndx++) {
+      for (bitndx = 0; bitndx < 8; bitndx++) {
+         unsigned char flagbit = (0x80 >> bitndx);
+         if (bbset.bytes[flagndx] & flagbit)
+            result += 1;
+      }
+   }
+   if (debug) {
+      char buf[BB256_REPR_BUF_SZ];
+      bb256_repr(buf, sizeof(buf), bbset);
+      printf("(%s) Returning %d. bbset: %s\n", __func__, result, buf);
+   }
+   return result;
+}
+
+
+#ifdef COMPILE_ERRORS
+int bs256_count(
+      Bit_Set_256 bbset)
+{
+   // regard the array of 32 bytes as an array of 8 4-byte unsigned integers
+   uint64_t  list2 = (uint64_t) bbset.bytes;
+   unsigned int ct = 0;
+   for (int ndx = 0; ndx < 4; ndx++) {
+      // clever algorithm for counting number of bits per Brian Kernihgan
+      uint64_t v = list2[ndx];
+      for (; v; ct++) {
+        v &= v - 1; // clear the least significant bit set
+      }
+      // DBGMSG("feature_list_count() returning: %d", ct);
+   }
+// #ifdef OLD
+   assert(ct == bs256_count0(bbset));
+// #endif
+   return ct;
+}
+#endif
+
+
+/** Returns a string representation of a #Bit_Set_256 as a list of hex numbers.
+ *
+ *  The value returned is valid until the next call to this function in the
+ *  current thread.
+ *
+ *  \param  bitset value to represent
+ *  \param  value_prefix  prefix for each hex number, typically "0x" or ""
+ *  \param  sepstr        string to insert between each value, typically "", ",", or " "
+ *  \return string representation
+ */
+char *
+bs256_to_string(
+      Bit_Set_256  bitset,
+      const char * value_prefix,
+      const char * sepstr)
+{
+   bool debug = false;
+   if (debug) {
+      printf("(%s) value_prefix=|%s|, sepstr=|%s| bitset: ",
+             __func__, value_prefix, sepstr);
+      for (int ndx = 0; ndx < 32; ndx++) {
+         printf("%02x", bitset.bytes[ndx]);
+      }
+      printf("\n");
+      // rpt_hex_dump((Byte*)feature_list, 32, 2);
+   }
+
+   static GPrivate  key =     G_PRIVATE_INIT(g_free);
+   static GPrivate  len_key = G_PRIVATE_INIT(g_free);
+
+   if (!value_prefix)
+      value_prefix = "";
+   if (!sepstr)
+      sepstr = "";
+   int vsize = strlen(value_prefix) + 2 + strlen(sepstr);
+   int bit_ct = bs256_count(bitset);
+   int reqd_size = (bit_ct*vsize)+1;   // +1 for trailing null
+
+   char * buf = get_thread_dynamic_buffer(&key, &len_key, reqd_size);
+   // char * buf = calloc(1, reqd_size);
+
+   buf[0] = '\0';
+   // printf("(%s) feature_ct=%d, vsize=%d, buf size = %d",
+   //          __func__, feature_ct, vsize, vsize*feature_ct);
+
+   for (int ndx = 0; ndx < 256; ndx++) {
+      if ( bs256_contains(bitset, ndx) )
+         sprintf(buf + strlen(buf), "%s%02x%s", value_prefix, ndx, sepstr);
+   }
+
+   if (bit_ct > 0)
+      buf[ strlen(buf)-strlen(sepstr)] = '\0';
+
+   // printf("(%s) wolf 4\n", __func__);
+   // DBGMSG("Returned string length: %d", strlen(buf));
+   // DBGMSG("Returning %p - %s", buf, buf);
+   if (debug)
+   printf("(%s) Returning: len=%d, %s\n", __func__, (int) strlen(buf), buf);
+
+   return buf;
+}
+
+
+#define BBF_ITER_MARKER "BBFI"
+typedef struct {
+   char        marker[4];
+   Bit_Set_256 bbflags;
+   int         lastpos;
+} _Bit_Set_256_Iterator;
+
+
+/** Creates an iterator for a #Bit_Set_256 instance.
+ *  The iterator is an opaque object.
+ */
+Bit_Set_256_Iterator
+bs256_iter_new(Bit_Set_256 bbflags) {
+   _Bit_Set_256_Iterator * result = malloc(sizeof(_Bit_Set_256_Iterator));
+   memcpy(result->marker, BBF_ITER_MARKER, 4);
+   result->bbflags = bbflags;   // TODO: save pointer to unopaque _BitByteFlags
+   result->lastpos = -1;
+   return result;
+}
+
+
+/** Free a #Bit_Set_256_Iterator.
+ *
+ * \param bs256_iter handle to iterator (may be NULL)
+ */
+void
+bs256_iter_free(
+      Bit_Set_256_Iterator bs256_iter)
+{
+   _Bit_Set_256_Iterator * iter = (_Bit_Set_256_Iterator *) bs256_iter;
+
+   if (bs256_iter) {
+      assert(memcmp(iter->marker, BBF_ITER_MARKER, 4) == 0);
+      iter->marker[3] = 'x';
+      free(iter);
+   }
+}
+
+
+/** Reinitializes an iterator.  Sets the current position before the first
+ *  value.
+ *
+ * \param bs256_iter handle to iterator
+ */
+void
+bs256_iter_reset(
+      Bit_Set_256_Iterator bs256_iter)
+{
+   _Bit_Set_256_Iterator * iter = (_Bit_Set_256_Iterator *) bs256_iter;
+   assert(iter && memcmp(iter->marker, BBF_ITER_MARKER, 4) == 0);
+
+   iter->lastpos = -1;
+}
+
+
+/** Returns the number of the next bit that is set.
+ *
+ * \param bs256_iter handle to iterator
+ * \return number of next bit that is set, -1 if no more
+ */
+int
+bs256_iter_next(
+      Bit_Set_256_Iterator
+      bs256_iter)
+{
+   _Bit_Set_256_Iterator * iter = (_Bit_Set_256_Iterator *) bs256_iter;
+   assert( iter && memcmp(iter->marker, BBF_ITER_MARKER, 4) == 0);
+   // printf("(%s) Starting. lastpos = %d\n", __func__, iter->lastpos);
+
+   int result = -1;
+   for (int ndx = iter->lastpos + 1; ndx < 256; ndx++) {
+      if (bs256_contains(iter->bbflags, ndx)) {
+         result = ndx;
+         iter->lastpos = ndx;
+         break;
+      }
+   }
+   // printf("(%s) Returning: %d\n", __func__, result);
+   return result;
+}
+
