@@ -10,7 +10,7 @@
  *  to **ddcutil** are interpreted.
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -76,6 +76,17 @@ bool is_valid_edid_header(Byte * edidbytes) {
 }
 
 
+bool is_valid_raw_edid(Byte * edid, int len) {
+   return (len >= 128) && is_valid_edid_header(edid) && is_valid_edid_checksum(edid);
+}
+
+bool is_valid_raw_cea861_extension_block(Byte * edid, int len) {
+   bool result = len >= 128 && edid[0] == 0x02 && is_valid_edid_checksum(edid);
+
+   return result;
+}
+
+
 /** Unpacks the 2 byte manufacturer id field from the EDID into a 3 character
  * string.
  *
@@ -215,7 +226,9 @@ static void get_edid_descriptor_strings(
 Parsed_Edid * create_parsed_edid(Byte* edidbytes) {
    assert(edidbytes);
    // bool debug = false;
+#ifdef UNNEEDED
    bool ok = true;
+#endif
    Parsed_Edid* parsed_edid = NULL;
 
    if ( !is_valid_edid_header(edidbytes) || !is_valid_edid_checksum(edidbytes) )
@@ -247,6 +260,7 @@ Parsed_Edid * create_parsed_edid(Byte* edidbytes) {
 
    parsed_edid->year = edidbytes[17] + 1990;
    parsed_edid->is_model_year = edidbytes[16] == 0xff;
+   parsed_edid->manufacture_week = edidbytes[16];
    parsed_edid->edid_version_major = edidbytes[18];
 #ifdef UNNEEDED
    if (parsed_edid->edid_version_major != 1 && parsed_edid->edid_version_major != 2) {
@@ -274,10 +288,12 @@ Parsed_Edid * create_parsed_edid(Byte* edidbytes) {
    parsed_edid->supported_features = edidbytes[0x18];
    parsed_edid->extension_flag = edidbytes[0x7e];
 
+#ifdef UNNEEDED
    if (!ok) {
       free(parsed_edid);
       parsed_edid = NULL;
    }
+#endif
 
 bye:
    return parsed_edid;
@@ -297,19 +313,23 @@ void free_parsed_edid(Parsed_Edid * parsed_edid) {
 }
 
 
-
 /** Writes EDID summary to the current report output destination.
  * (normally stdout, but may be changed by rpt_push_output_dest())
  *
- *  @param  edid       pointer to parsed edid struct
- *  @param  verbose    show additional detail
- *  @param  show_raw   include hex dump of EDID
- *  @param  depth      logical indentation depth
+ *  @param  edid              pointer to parsed edid struct
+ *  @param  verbose_synopsis  show additional EDID detail
+ *  @param  show_raw          include hex dump of EDID
+ *  @param  depth             logical indentation depth
  *
  *  @remark
  *  Output is written using rpt_ functions.
  */
-void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, int depth) {
+void report_parsed_edid_base(
+      Parsed_Edid *  edid,
+      bool           verbose_synopsis,
+      bool           show_raw,
+      int            depth)
+{
    bool debug = false;
    if (debug)
       printf("(%s) Starting. edid=%p", __func__, edid);
@@ -319,22 +339,21 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
    // verbose = true;
    if (edid) {
       rpt_vstring(depth,"EDID synopsis:");
-
-      rpt_vstring(d1,"Mfg id:           %s",          edid->mfg_id);
-      rpt_vstring(d1,"Model:            %s",          edid->model_name);
-      rpt_vstring(d1,"Serial number:    %s",          edid->serial_ascii);
-      char * title = (edid->is_model_year) ? "Model year" : "Manufacture year";
-      rpt_vstring(d1,"%-16s: %d", title, edid->year);
-      rpt_vstring(d1,"EDID version:     %d.%d", edid->edid_version_major, edid->edid_version_minor);
-
-      if (verbose) {
-      // rpt_vstring(d1,"Product code:     0x%04x (%u)",      edid->product_code, edid->product_code);
-         rpt_vstring(d1,"Product code:     %u",          edid->product_code);
-         // Binary serial number is typically 0x00000000 or 0x01010101, but occasionally
-         // useful for differentiating displays using a generic ASCII serial number
-         rpt_vstring(d1,"Binary sn:        %"PRIu32" (0x%08x)", edid->serial_binary, edid->serial_binary);
-         rpt_vstring(d1,"Extra descriptor: %s",          edid->extra_descriptor_string);
-
+      rpt_vstring(d1,"Mfg id:               %s",          edid->mfg_id);
+      rpt_vstring(d1,"Model:                %s",          edid->model_name);
+   // rpt_vstring(d1,"Product code:         0x%04x (%u)", edid->product_code, edid->product_code);
+      rpt_vstring(d1,"Product code:         %u",          edid->product_code);
+      rpt_vstring(d1,"Serial number:        %s",          edid->serial_ascii);
+      // Binary serial number is typically 0x00000000 or 0x01010101, but occasionally
+      // useful for differentiating displays that share a generic ASCII "serial number"
+      rpt_vstring(d1,"Binary serial number: %"PRIu32" (0x%08x)", edid->serial_binary, edid->serial_binary);
+      if (edid->is_model_year)
+      rpt_vstring(d1,"Model year:           %d", edid->year);
+      else
+      rpt_vstring(d1,"Manufacture year:     %d,  Week: %d", edid->year, edid->manufacture_week);
+      if (verbose_synopsis) {
+         rpt_vstring(d1,"EDID version:         %d.%d",       edid->edid_version_major, edid->edid_version_minor);
+         rpt_vstring(d1,"Extra descriptor:        %s",          edid->extra_descriptor_string);
          char explbuf[100];
          explbuf[0] = '\0';
          if (edid->video_input_definition & 0x80) {
@@ -362,13 +381,38 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
                default:
                   strcat(explbuf, " (Invalid DVI standard)");
                }
+               strcat(explbuf, ", Bit depth: ");
+               switch ( (edid->video_input_definition & 0x70) >> 4) {
+               case 0x00:
+                  strcat(explbuf, "undefined");
+                  break;
+               case 0x01:
+                  strcat(explbuf, "6");
+                  break;
+               case 0x02:
+                  strcat(explbuf, "8");
+                  break;
+               case 0x03:
+                  strcat(explbuf, "10");
+                  break;
+               case 0x04:
+                  strcat(explbuf, "12");
+                  break;
+               case 0x05:
+                  strcat(explbuf, "14");
+                  break;
+               case 0x06:
+                  strcat(explbuf, "16");
+                  break;
+               case 0x07:
+                  strcat(explbuf, " (x07 reserved)");
+               }
             }
          }
          else {
             strcpy(explbuf, "Analog Input");
          }
-         rpt_vstring(d1,"Video input definition: 0x%02x - %s", edid->video_input_definition, explbuf);
-      // rpt_vstring(d1,"Video input:      %s",          (edid->is_digital_input) ? "Digital" : "Analog");
+         rpt_vstring(d1,"Video input definition:    0x%02x - %s", edid->video_input_definition, explbuf);
          // end, video_input_definition interpretation
 
          rpt_vstring(d1, "Supported features:");
@@ -394,7 +438,6 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
                rpt_vstring(d2, "Digital display type: RGB 4:4:4 + YCrCb 4:4:4 + YCrCb 4:2:2");
                break;
             default:
-               // should be PROGRAM_LOGIC_ERROR, but that would violdate layering
                rpt_vstring(d2, "Invalid digital display type: 0x%02x", display_type);
             }
          }
@@ -413,7 +456,7 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
                rpt_vstring(d2, "Undefined analog display type");
                break;
             default:
-               // should be PROGRAM_LOGIC_ERROR, but that would violdate layering
+               // should be PROGRAM_LOGIC_ERROR, but that would violate layering
                rpt_vstring(d2, "Invalid analog display type: 0x%02x", display_type);
             }
          }
@@ -428,17 +471,17 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
          // restrict to EDID version >= 1.3?
          rpt_vstring(d1,"Extension blocks: %u",    edid->extension_flag);
 
-      //  if (strlen(edid->edid_source) > 0)
-         rpt_vstring(depth,"EDID source: %s",        edid->edid_source);
-      }
+         if (strlen(edid->edid_source) > 0)   // will be set only for USB devices, values "USB", "X11"
+            rpt_vstring(depth,"EDID source: %s",        edid->edid_source);
+      }  // if (verbose_synopsis)
       if (show_raw) {
          rpt_vstring(depth,"EDID hex dump:");
          rpt_hex_dump(edid->bytes, 128, d1);
       }
 
-   }
+   }  // if (edid)
    else {
-       if (verbose)
+       if (verbose_synopsis)
          rpt_vstring(d1,"No EDID");
    }
 
