@@ -4,7 +4,7 @@
  * the bytes of the EDID are obtained.
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef EDID_H_
@@ -15,7 +15,7 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>    // normally included by stdlib.h, but explicitly require for Alpint Linux
+#include <sys/types.h>    // normally included by stdlib.h, but explicitly required for Alpine Linux
 /** \endcond */
 
 #include "coredefs.h"
@@ -36,6 +36,8 @@ Byte edid_checksum(Byte * edid);
 
 bool is_valid_edid_checksum(Byte * edidbytes);
 bool is_valid_edid_header(Byte * edidbytes);
+bool is_valid_raw_edid(Byte * edidbytes, int len);
+bool is_valid_raw_cea861_extension_block(Byte * edid, int len);
 
 void parse_mfg_id_in_buffer(Byte * mfgIdBytes, char * buffer, int bufsize);
 
@@ -58,6 +60,7 @@ struct {
    char         extra_descriptor_string[EDID_EXTRA_STRING_FIELD_SIZE];  ///< (tag 0xfe)
    int          year;                    ///< can be year of manufacture or model
    bool         is_model_year;           ///< if true, year is model year, if false, is manufacture year
+   uint8_t      manufacture_week;        ///< xFF if is_model_year == true
    Byte         edid_version_major;      ///< EDID major version number
    Byte         edid_version_minor;      ///< EDID minor version number
    ushort       wx;                      ///< whitepoint x coordinate
@@ -69,7 +72,7 @@ struct {
    ushort       bx;                      ///< blue  x coordinate
    ushort       by;                      ///< blue  y coordinate
    Byte         video_input_definition;  /// EDID byte 20 (x14)
-   // bool         is_digital_input;      // from byte 20 (x14), but 7
+   // bool         is_digital_input;      // from byte 20 (x14), bit 7
    Byte         supported_features;      ///< EDID byte 24 (x18) supported features bitmap
    uint8_t      extension_flag;          ///< number of optional extension blocks
    char         edid_source[EDID_SOURCE_FIELD_SIZE];  ///< describes source of EDID
@@ -77,7 +80,7 @@ struct {
 
 
 Parsed_Edid * create_parsed_edid(Byte* edidbytes);
-void          report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, int depth);
+void          report_parsed_edid_base(Parsed_Edid * edid, bool verbose_synopsis, bool show_raw, int depth);
 void          report_parsed_edid(Parsed_Edid * edid, bool verbose, int depth);
 void          free_parsed_edid(Parsed_Edid * parsed_edid);
 bool          is_embedded_parsed_edid(Parsed_Edid * parsed_edid);
