@@ -9,7 +9,7 @@
  *  error is retained for use by higher levels in the call stack.
  */
 
-// Copyright (C) 2017-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2017-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -462,6 +462,25 @@ errinfo_new_with_causes2(
 }
 
 
+Error_Info *
+errinfo_new_with_causes3(
+      int            status_code,
+      Error_Info **  causes,
+      int            cause_ct,
+      const char *   func,
+      char *         detail,
+      ...)
+{
+   va_list ap;
+   va_start(ap, detail);
+   Error_Info * result = errinfo_newv(status_code, func, detail, ap);
+   va_end(ap);
+   for (int ndx = 0; ndx < cause_ct; ndx++) {
+      errinfo_add_cause(result, causes[ndx]);
+   }
+   return result;
+}
+
 
 #ifdef UNUSED
 
@@ -734,13 +753,13 @@ errinfo_report(Error_Info * erec, int depth) {
    // rpt_vstring(depth, "(%s) Status code: %d", __func__, erec->status_code);
    // rpt_vstring(depth, "(%s) Location: %s", __func__, (erec->func) ? erec->func : "not set");
    // rpt_vstring(depth, "(%s) errinfo_name_func=%p, errinfo_desc_func=%p", __func__, errinfo_name_func, errinfo_desc_func);
-   rpt_push_output_dest(stderr);
+   // rpt_push_output_dest(stderr);
    rpt_vstring(depth, "Exception in function %s: status=%s",
          (erec->func) ? erec->func : "not set",
          errinfo_desc_func(erec->status_code) );  // can't call psc_desc(), violates layering
    if (erec->detail)
       rpt_label(depth+1, erec->detail);
-   rpt_pop_output_dest();
+   // rpt_pop_output_dest();
 
    if (erec->cause_ct > 0) {
       rpt_vstring(depth, "Caused by: ");
@@ -760,6 +779,30 @@ errinfo_report(Error_Info * erec, int depth) {
    // rpt_vstring(depth, "(%s) Done", __func__);
 }
 
+void
+errinfo_report_details(Error_Info * erec, int depth) {
+   assert(erec);
+   int d0 = depth;
+   int d1 = depth+1;
+
+   // rpt_vstring(depth, "Exception in function %s: status=%s",
+   //       (erec->func) ? erec->func : "not set",
+   //       errinfo_desc_func(erec->status_code) );  // can't call psc_desc(), violates layering
+   if (erec->detail)
+      rpt_label(d0, erec->detail);
+   else
+      rpt_vstring(d0, "Error %d in function %s",erec->status_code, erec->func);
+
+   // printf("%s) cause_ct = %d\n", __func__, erec->cause_ct);
+   if (erec->cause_ct > 0) {
+      // rpt_vstring(depth, "Caused by: ");
+      for (int ndx = 0; ndx < erec->cause_ct; ndx++) {
+         errinfo_report_details(erec->causes[ndx], d1);
+      }
+   }
+
+}
+
 
 /** Returns a string summary of the specified #Error_Info.
  *  The returned value is valid until the next call to this function in the
@@ -796,7 +839,7 @@ errinfo_summary(Error_Info * erec) {
    int required_size = strlen(buf1) + 1;
 
    char * buf = get_thread_dynamic_buffer(&esumm_key, &esumm_len_key, required_size);
-   g_strlcpy(buf, buf1, required_size);
+   strcpy(buf, buf1);
    free(buf1);
    return buf;
 }
