@@ -9,7 +9,8 @@
 #ifndef I2C_UTIL_H_
 #define I2C_UTIL_H_
 
-int           i2c_name_to_busno(char * name);
+int           i2c_name_to_busno(const char * name);
+int           i2c_compare(const void * v1, const void * v2);
 
 unsigned long i2c_get_functionality_flags_by_fd(int fd);
 char *        i2c_interpret_functionality_flags(unsigned long functionality);
