@@ -3,7 +3,7 @@
  *  Report utility package
  */
 
-// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef REPORT_UTIL_H_
@@ -16,7 +16,9 @@
 /** \endcond */
 
 #include "coredefs.h"
+#include "string_util.h"
 
+void rpt_set_default_output_dest(FILE* output_dest);
 void rpt_push_output_dest(FILE* new_dest);
 void rpt_pop_output_dest();
 FILE * rpt_cur_output_dest();
@@ -38,6 +40,7 @@ void rpt_vstring(int depth, char * format, ...) ;
 void rpt_2col(char * s1,  char * s2,  int col2offset, bool offset_absolute, int depth);
 void rpt_structure_loc(const char * name, const void * ptr, int depth);
 void rpt_hex_dump(const Byte * data, int size, int depth);
+void rpt_ntsa(Null_Terminated_String_Array ntsa, int depth);
 int rpt_file_contents(const char * fn, bool verbose, int depth);
 
 // Remaining rpt_ functions share common formatting
