@@ -2,7 +2,7 @@
  *  String utility functions
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -12,13 +12,13 @@
 
 #include <assert.h>
 #include <ctype.h>
-#include <glib-2.0/glib.h>
 #include <errno.h>
-#include <stdio.h>
+#include <glib-2.0/glib.h>
+#include <stdarg.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdarg.h>
 /** \endcond */
 
 #include "glib_util.h"
@@ -77,7 +77,7 @@ bool streq(const char * s1, const char * s2) {
 bool is_abbrev(const char * value, const char * longname, size_t  minchars) {
    bool result = false;
    if (value && longname) {
-      int vlen = strlen(value);
+      size_t vlen = strlen(value);
       if ( vlen >= minchars &&
            vlen <= strlen(longname) &&
            memcmp(value, longname, vlen) == 0   // n. returns 0 if vlen == 0
@@ -155,7 +155,7 @@ int str_contains(const char * value_to_test, const char * segment) {
 bool str_all_printable(const char * s) {
    bool result = true;
    if (s) {
-      for (int ndx = 0; ndx < strlen(s); ndx++) {
+      for (size_t ndx = 0; ndx < strlen(s); ndx++) {
          if (!isprint(s[ndx])) {
             result = false;
             break;
@@ -256,6 +256,20 @@ char * strtrim_r(const char * s, char * buffer, int bufsz) {
 }
 
 
+/** Returns a pointer to the first non-whitespace character in a string.
+ *
+ *  @param string to trim
+ *  @return pointer to first non-whitespace character,
+ *          to the ending \0 if all characters are whitespace
+ */
+char * ltrim_in_place(char * s) {
+   char * p = s;
+   while (*p && isspace(*p))
+      p++;
+   return p;
+}
+
+
 /** Trims trailing whitespace from a string.
  *
  * @param s string to trim
@@ -274,6 +288,23 @@ char * rtrim_in_place(char * s) {
 }
 
 
+/** Trims leading and trailing whitespace from a string.
+ *  Trailing whitespace characters are replaced by \0.
+ *
+ * @param s string to trim
+ * @return pointer to first non-whitespcace character
+ */
+char * trim_in_place(char * s) {
+   char * p = s;
+   while (*p && isspace(*p))
+      p++;
+   int len = strlen(p);
+   while (len > 0 && isspace(p[len-1]))
+      s[--len] = '\0';
+   return p;
+}
+
+
 /** Trims leading and trailing whitespace from a string and
  * returns the result in newly allocated memory.
  * It is the caller's responsibility to free this memory.
@@ -298,9 +329,7 @@ char * strtrim(const char * s) {
  *                  the string length, ct is reduced accordingly
  * @return extracted substring, in newly allocated memory
  */
-char * substr(const char * s, int startpos, int ct) {
-   assert(startpos >= 0);
-   assert(ct>=0);
+char * substr(const char * s, size_t startpos, size_t ct) {
    if (startpos + ct > strlen(s))
       ct = strlen(s) - startpos;
    char * result = calloc(ct+1, sizeof(char));
@@ -309,6 +338,7 @@ char * substr(const char * s, int startpos, int ct) {
    return result;
 }
 
+
 /** Returns the initial portion of a string
  *
  * @param s         string to process
@@ -316,7 +346,7 @@ char * substr(const char * s, int startpos, int ct) {
  *                  the string length, ct is reduced accordingly
  * @return extracted substring, in newly allocated memory
  */
-char * lsub(const char * s, int ct) {
+char * lsub(const char * s, size_t ct) {
    return substr(s, 0, ct);
 }
 
@@ -331,6 +361,10 @@ char * lsub(const char * s, int ct) {
  *
  * The returned string has been malloc'd.  It is the responsibility of
  * the caller to free it.
+ *
+ * @remark
+ * If ct < 0, i.e. pieces is null terminated, at most the first 9999 strings in
+ * the pieces array are joined.
  */
 char * strjoin( const char ** pieces, const int ct0, const char * sepstr) {
    // printf("(%s) ct0=%d, sepstr=|%s|\n", __func__, ct0, sepstr);
@@ -349,6 +383,7 @@ char * strjoin( const char ** pieces, const int ct0, const char * sepstr) {
 
    // printf("(%s) ct=%d, total_length=%d\n", __func__, ct, total_length);
    char * result = malloc(total_length);
+   result[0] = '\0';
    char * end = result;
    for (ndx=0; ndx<ct; ndx++) {
       if (ndx > 0 && seplen > 0) {
@@ -394,9 +429,10 @@ String_Array* new_string_array(int size) {
  */
 Null_Terminated_String_Array strsplit(const char * str_to_split, const char * delims) {
    bool debug = false;
-   int max_pieces = (strlen(str_to_split)+1);
+   size_t max_pieces = (strlen(str_to_split)+1);
    if (debug)
-      printf("(%s) str_to_split=|%s|, delims=|%s|, max_pieces=%d\n", __func__, str_to_split, delims, max_pieces);
+      printf("(%s) str_to_split=|%s|, delims=|%s|, max_pieces=%zu\n",
+             __func__, str_to_split, delims, max_pieces);
 
    char** workstruct = calloc(sizeof(char *), max_pieces+1);
    int piecect = 0;
@@ -408,7 +444,8 @@ Null_Terminated_String_Array strsplit(const char * str_to_split, const char * de
    // complaining about uninitialized variable, trying to figure out why
    token = strsep(&rest, delims);      // n. overwrites character found
    while (token) {
-      // printf("(%s) token: |%s|\n", __func__, token);
+      if (debug)
+         printf("(%s) token: |%s|\n", __func__, token);
       if (strlen(token) > 0)
          workstruct[piecect++] = strdup(token);
       token = strsep(&rest, delims);
@@ -509,15 +546,27 @@ strsplit_maxlength(
 /** Frees a null terminated array of strings.
  *
  *  @param string_array null terminated array of pointers to strings
- *  @param free_strings if try, each string in the array is freed as well
+ *  @param free_strings if set, each string in the array is freed as well
+ *
+ *  @remark
+ *  g_strfreev(string_array) equivalent to ntsa_free(string_array, true)
  */
 void ntsa_free(Null_Terminated_String_Array string_array, bool free_strings) {
+   bool debug = false;
+   if (debug)
+      printf("(%s) Freeing NTSA %p, free_strings=%s\n", __func__, string_array, sbool(free_strings) );
    if (string_array) {
       if (free_strings) {
-      int ndx = 0;
-      while (string_array[ndx] != NULL)
-         free(string_array[ndx++]);
+         int ndx = 0;
+         while (string_array[ndx] != NULL) {
+            if (debug)
+               printf("(%s) Freeing string %d, %p->%s\n",
+                      __func__, ndx, string_array[ndx], string_array[ndx]);
+            free(string_array[ndx++]);
+         }
       }
+      if (debug)
+         printf("(%s) Freeing string_array=%p\n", __func__, string_array);
       free(string_array);
    }
 }
@@ -525,8 +574,11 @@ void ntsa_free(Null_Terminated_String_Array string_array, bool free_strings) {
 
 /** Returns the number of strings in a null terminated array of strings.
  *
- * @param  string_array null terminated array of pointers to strings
- * @return number of strings in the array
+ *  @param  string_array null terminated array of pointers to strings
+ *  @return number of strings in the array
+ *
+ *  @remark
+ *  Equivalent to g_strv_length()
  */
 int ntsa_length(Null_Terminated_String_Array string_array) {
    assert(string_array);
@@ -583,6 +635,65 @@ Null_Terminated_String_Array ntsa_join(
 }
 
 
+
+/** Creates copy of a #Null_Terminated_String_Array.
+ *
+ *  The pointers in the new array point to newly allocated copies of the
+ *  original array.
+ *
+ *  @param a1  instance to copy
+ *  @param dup if true, copy the strings as well as pointer array
+ *  @return newly allocated #Null_Terminated_String_Array
+ */
+Null_Terminated_String_Array ntsa_copy(Null_Terminated_String_Array a1, bool dup)
+{
+   assert(a1);
+   int ct = ntsa_length(a1);
+   Null_Terminated_String_Array result = calloc((ct+1), sizeof(char *));
+   char ** to = result;
+   char ** from = a1;
+   while (*from) {
+      if (dup)
+         *to = strdup(*from);
+      else
+         *to = *from;
+      to++;
+      from++;
+   }
+   return result;
+}
+
+
+Null_Terminated_String_Array  ntsa_prepend(
+      char *                       value,
+      Null_Terminated_String_Array old_array,
+      bool                         dup)
+{
+   int old_ct = ntsa_length(old_array);
+   int new_ct = old_ct + 1;
+   Null_Terminated_String_Array new_array = calloc((new_ct+1), sizeof(char *));
+   char ** to = new_array;
+   *to++ = (dup) ? strdup(value) : value;
+   char ** from = old_array;
+   while (*from) {
+      *to++ = (dup) ? strdup(*from) : *from;
+      from++;
+   }
+   *to = NULL;
+
+   // ntsa_show(argv);
+   return new_array;
+}
+
+
+Null_Terminated_String_Array ntsa_create_empty_array() {
+   Null_Terminated_String_Array new_array = calloc(1, sizeof(char *));
+   *new_array = NULL;
+   return new_array;
+}
+
+
+
 /** Searches a #Null_Terminated_String_Array for an entry that matches a given
  *  value using a match function provided.
  *
@@ -610,7 +721,6 @@ int ntsa_findx(
    }
    // printf("(%s) Returning: %d\n", __func__, result);
    return result;
-
 }
 
 
@@ -676,7 +786,7 @@ g_ptr_array_to_ntsa(
 {
    assert(gparray);
    Null_Terminated_String_Array ntsa = calloc(gparray->len+1, sizeof(char *));
-   for (int ndx=0; ndx < gparray->len; ndx++) {
+   for (guint ndx=0; ndx < gparray->len; ndx++) {
       if (duplicate)
          ntsa[ndx] = strdup(g_ptr_array_index(gparray,ndx));
       else
@@ -703,6 +813,23 @@ char * strupper(char * s) {
 }
 
 
+/** Converts an ASCII string to lower case.  The original string is converted in place.
+ *
+ * @param  s string to force to lower case
+ * @return converted string
+ */
+char * strlower(char * s) {
+   if (s) {     // check s not a null pointer
+      char * p = s;
+      while(*p) {
+         *p = tolower(*p);
+         p++;
+      }
+   }
+   return s;
+}
+
+
 /** Creates an upper case copy of an ASCII string
  *
  * @param  s  string to copy
@@ -777,6 +904,24 @@ char * chars_to_string(const char * start, int len)
 }
 
 
+/** qsort() style string comparison function
+ *
+ *  @param a pointer to a pointer to a string
+ *  @param b pointer to a pointer to a string
+ *  @return -1 if the first string sorts before the second
+ *           0 if the strings are identical
+ *           1 if the first string sorts after the second
+ *
+ *  @remark
+ *  Satisfies GCompareFunc
+ */
+int indirect_strcmp(const void * a, const void * b) {
+   char * alpha = *(char **) a;
+   char * beta  = *(char **) b;
+   return strcmp(alpha, beta);
+}
+
+
 /** Appends a value to a string in a buffer.
  *
  * @param buf     pointer to character buffer
@@ -795,9 +940,9 @@ bool sbuf_append(char * buf, int bufsz, char * sepstr, char * nextval)
 {
    assert(buf && (bufsz > 4) );   //avoid handling pathological case
    bool truncated = false;
-   int seplen = (sepstr) ? strlen(sepstr) : 0;
-   int maxchars = bufsz-1;
-   int newlen = ( strlen(buf) == 0 )
+   size_t seplen = (sepstr) ? strlen(sepstr) : 0;
+   size_t maxchars = bufsz-1;
+   size_t newlen = ( strlen(buf) == 0 )
                      ? strlen(nextval)
                      : ( strlen(buf) + seplen + strlen(nextval));
    if (newlen <= maxchars) {
@@ -848,13 +993,15 @@ bool str_to_int(const char * sval, int * p_ival, int base)
 
    char * endptr;
    bool ok = false;
-   if ( *sval != '\0') {
-      long result = strtol(sval, &endptr, base); // allow hex
-      // printf("(%s) sval=%p, endptr=%p, *endptr=|%c| (0x%02x), result=%ld\n",
-      //        __func__, sval, endptr, *endptr, *endptr, result);
-      if (*endptr == '\0') {
-         *p_ival = result;
-         ok = true;
+   if (sval) {
+      if ( *sval != '\0') {
+         long result = strtol(sval, &endptr, base); // allow hex
+         // printf("(%s) sval=%p, endptr=%p, *endptr=|%c| (0x%02x), result=%ld\n",
+         //        __func__, sval, endptr, *endptr, *endptr, result);
+         if (*endptr == '\0') {
+            *p_ival = result;
+            ok = true;
+         }
       }
    }
 
@@ -955,6 +1102,9 @@ bool hhs_to_byte_in_buf(const char * s, Byte * result)
  *  the value to begin with "0x" or "x", or end with "h".  The allowed
  *  prefix or suffix is case-insensitive.
  *
+ *  Note that if the string need not be prefixed with "0X" or suffixed with "h"
+ *  to be regarded as a hex value.
+ *
  *  @param  s         pointer to hex string
  *  @param  result    pointer to byte in which result will be returned
  *  @retval **true**  successful conversion,
@@ -1092,7 +1242,7 @@ char * hexstring2(
           const char *          sepstr,
           bool                  uppercase,
           char *                buffer,
-          int                   bufsz)
+          size_t                bufsz)
 {
    // if (len > 1)
    // printf("(%s) bytes=%p, len=%d, sepstr=|%s|, uppercase=%s, buffer=%p, bufsz=%d\n", __func__,
@@ -1101,7 +1251,7 @@ char * hexstring2(
    if (sepstr) {
       sepsize = strlen(sepstr);
    }
-   int required_size =   2*len             // hex rep of bytes
+   size_t required_size =   2*len             // hex rep of bytes
                        + (len-1)*sepsize   // for separators
                        + 1;                // terminating null
    // if (len > 1)
@@ -1155,8 +1305,8 @@ char * hexstring2(
  * Note that if the returned pointer is referenced after another call to
  * this function, the results are unpredictable.
  *
- * This function is intended to simplify formatting of diagnostic messages, since
- * the caller needn't be concerned with buffer size and allocation.
+ * This function is intended to simplify formatting of diagnostic messages.
+ * The caller needn't be concerned with buffer size and allocation.
  */
 char * hexstring3_t(
           const unsigned char * bytes,      // bytes to convert
@@ -1165,11 +1315,13 @@ char * hexstring3_t(
           uint8_t               hunk_size,  // separator string frequency
           bool                  uppercase)  // use upper case hex characters
 {
+   bool debug = false;
    static GPrivate  hexstring3_key = G_PRIVATE_INIT(g_free);
    static GPrivate  hexstring3_len_key = G_PRIVATE_INIT(g_free);
 
-   // printf("(%s) bytes=%p, len=%d, sepstr=|%s|, uppercase=%s\n", __func__,
-   //       bytes, len, sepstr, sbool(uppercase));
+   if (debug)
+      printf("(%s) bytes=%p, len=%d, sepstr=|%s|, uppercase=%s\n", __func__,
+             bytes, len, sepstr, sbool(uppercase));
    if (hunk_size == 0)
       sepstr = NULL;
    else if (sepstr == NULL)
@@ -1179,13 +1331,15 @@ char * hexstring3_t(
    if (sepstr) {
       sepsize = strlen(sepstr);
    }
-   int required_size = 1;    // special case if len == 0
+   size_t required_size = 1;    // special case if len == 0
    // excessive if hunk_size > 1, but not worth the effort to be accurate
    if (len > 0)
       required_size =   2*len             // hex rep of bytes
                        + (len-1)*sepsize   // for separators
                        + 1;                // terminating null
-   // printf("(%s) sepstr=|%s|, hunk_size=%d, required_size=%d\n", __func__, sepstr, hunk_size, required_size);
+   if (debug)
+      printf("(%s) sepstr=|%s|, hunk_size=%d, required_size=%zu\n",
+             __func__, sepstr, hunk_size, required_size);
 
    char * buf = get_thread_dynamic_buffer(&hexstring3_key, &hexstring3_len_key, required_size);
    // char * buf = get_thread_private_buffer(&hexstring3_key, NULL, required_size);
@@ -1203,11 +1357,14 @@ char * hexstring3_t(
       if (insert_sepstr)
          strcat(buf, sepstr);
    }
-   // printf("(%s) strlen(buffer) = %ld, required_size=%d   \n", __func__, strlen(buffer), required_size );
-   // printf("(%s)  buffer=|%s|\n", __func__, buffer );
+   if (debug) {
+      printf("(%s) strlen(buf) = %ld, required_size=%zu\n", __func__, strlen(buf), required_size );
+      printf("(%s)  buf=|%s|\n", __func__, buf );
+   }
    assert(strlen(buf) <= required_size-1);
 
-   // printf("(%s) Returning: %p\n", __func__, buf);
+   if (debug)
+      printf("(%s) Returning: %p -> |%s|\n", __func__, buf, buf);
    return buf;
 }
 
@@ -1486,3 +1643,44 @@ bool apply_filter_terms(const char * text, char ** terms, bool ignore_case)
    return result;
 }
 
+
+/** Converts a string containing a (possible) hex value to canonical form.
+ *
+ *  If the value starts with "x" or "X", or ends with "h" or "H", or starts
+ *  with "0X", it is modified to start with "0x".
+ *  Otherwise, the returned value is identical to the input value.
+ *
+ *  @param string_value  value to convert
+ *  @return canonicalized value (caller is responsible for freeing)
+ *
+ *  @remark
+ *  Consider converting to a function that uses a thread-specific buffer, making
+ *  the returned value valid until the next call to this function on the current
+ *  thread.  Would relieve the caller of responsibility for freeing the value.
+ */
+
+char * canonicalize_possible_hex_value(char * string_value) {
+   assert(string_value);
+
+   int bufsz = strlen(string_value) + 1 + 1;  // 1 for possible increased length, 1 for terminating null
+   char * buf = calloc(1, bufsz);
+   if (*string_value == 'X' || *string_value == 'x' ) {
+      // increases string size by 1
+      snprintf(buf, bufsz, "0x%s", string_value+1);
+   }
+   else if (*(string_value + strlen(string_value)-1) == 'H' ||
+            *(string_value + strlen(string_value)-1) == 'h' )
+   {
+      // increases string size by 1
+      int newlen = strlen(string_value)-1;
+      snprintf(buf, bufsz, "0x%.*s", newlen, string_value);
+   }
+   else if (str_starts_with(string_value, "0X")) {
+      snprintf(buf, bufsz, "0x%s", string_value+2);
+   }
+   else
+      strcpy(buf, string_value);
+   // DBGMSG("string_value=|%s|, returning |%s|", string_value, buf);
+   return buf;
+}
+
