@@ -2,7 +2,7 @@
  *  String utility functions header file
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef STRINGUTIL_H_
@@ -15,40 +15,23 @@
 #include <stdio.h>
 /** \endcond */
 
-#include "coredefs.h"
+#include "coredefs_base.h"
 #include "glib_util.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 
 //
 // General
 //
 
-static inline char * sbool(int val) {  return (val)  ? "true" : "false"; }
+static inline const char * sbool(uint64_t val) {  return (val)  ? "true" : "false"; }
 
 // A macro alternative to sbool()
 #define SBOOL(val) ( (val) ? "true" : "false" )
 
-
-#ifdef DEPRECATED
-// use glib function g_strlcpy() instead
-#define SAFE_STRNCPY(dest, src, buflen) \
-   do { \
-      strncpy(dest, src, (buflen) ); \
-      if (buflen > 0) \
-         dest[buflen-1] = '\0'; \
-   } while(0)
-#endif
-
-#ifdef DEPRECATED
-// use glib function g_snprintf()
-#define SAFE_SNPRINTF(buf, bufsz, fmt, ...) \
-   do { \
-      snprintf(buf, bufsz, fmt, __VA_ARGS__ ); \
-      if (bufsz > 0) \
-         buf[bufsz-1] = '\0'; \
-   } while(0)
-#endif
-
 //
 // String functions (other than hex)
 //
@@ -60,18 +43,22 @@ bool   str_ends_with(const char * value_to_test, const char * end_part);
 int    str_contains(const char * value_to_test, const char * segment);
 bool   str_all_printable(const char * s);
 char * strupper(char * s);
+char * strlower(char * s);
 char * strdup_uc(const char* s);
 char * strjoin( const char ** pieces, const int ct, const char * sepstr);
 char * chars_to_string(const char * start, int len);
 char * strtrim(const char * s);
 char * strtrim_r(const char * s, char * buffer, int bufsz);
+char * ltrim_in_place(char * s);
 char * rtrim_in_place(char * s);
-char * substr(const char * s, int startpos, int ct);
-char * lsub(const char * s, int ct);
+char * trim_in_place(char * s);
+char * substr(const char * s, size_t startpos, size_t ct);
+char * lsub(const char * s, size_t ct);
 char * str_replace_char(char * s, char old_char, char new_char);
 char * strcat_new(char * s1, char * s2);
 bool   sbuf_append(char * buf, int bufsz, char * sepstr, char * nextval);
 char * ascii_strcasestr(const char * haystack, const char * needle);
+int    indirect_strcmp(const void * a, const void * b);
 
 typedef bool (*String_Comp_Func)(const char * a, const char * b);
 int matches_by_func(    const char * word, const char ** match_list, String_Comp_Func  comp_func);
@@ -79,13 +66,16 @@ int exactly_matches_any(const char * word, const char ** match_list);
 int starts_with_any(    const char * word, const char ** match_list);
 
 /** pointer to null-terminated array of strings */
-typedef char** Null_Terminated_String_Array;
+typedef char** Null_Terminated_String_Array;    // equivalent to GStrv
 void ntsa_free(  Null_Terminated_String_Array string_array, bool free_strings);
 int  ntsa_length(Null_Terminated_String_Array string_array);
 void ntsa_show(  Null_Terminated_String_Array string_array);
 int  ntsa_findx( Null_Terminated_String_Array string_array, char * value, String_Comp_Func func);
 int  ntsa_find(  Null_Terminated_String_Array string_array, char * value);
-Null_Terminated_String_Array  ntsa_join(  Null_Terminated_String_Array a1, Null_Terminated_String_Array a2, bool dup);
+Null_Terminated_String_Array ntsa_join(  Null_Terminated_String_Array a1, Null_Terminated_String_Array a2, bool dup);
+Null_Terminated_String_Array ntsa_copy(Null_Terminated_String_Array a1, bool dup);
+Null_Terminated_String_Array ntsa_prepend(char * value, Null_Terminated_String_Array string_array, bool dup);
+Null_Terminated_String_Array ntsa_create_empty_array();
 
 Null_Terminated_String_Array strsplit(const char * str_to_split, const char* delims);
 Null_Terminated_String_Array strsplit_maxlength(
@@ -109,6 +99,7 @@ bool str_to_float(const char * sval, float * p_fval);
 // Hex value conversion.
 //
 
+char * canonicalize_possible_hex_value(char * string_value);
 bool hhs_to_byte_in_buf(const char * s,  Byte * result);    // converts null terminated string into buffer
 bool any_one_byte_hex_string_to_byte_in_buf(const char * s, Byte * result);
 bool hhc_to_byte_in_buf(const char * hh, Byte * result);    // converts 2 characters at hh into buffer
@@ -124,7 +115,7 @@ char * hexstring2(
           const char *          sepstr,     // separator string between hex digits
           bool                  uppercase,  // use upper case hex characters
           char *                buffer,     // buffer in which to return hex string
-          int                   bufsz);     // buffer size
+          size_t                bufsz);     // buffer size
 char * hexstring3_t(
           const unsigned char * bytes,      // bytes to convert
           int                   len,        // number of bytes
@@ -154,4 +145,9 @@ int vf0printf(FILE * stream, const char * format, va_list ap);
 bool all_bytes_zero(Byte * bytes, int bytect);
 bool apply_filter_terms(const char * text, char ** terms, bool ignore_case);
 
+#ifdef __cplusplus
+}
+#endif
+
+
 #endif /* STRINGUTIL_H_ */
