@@ -1,31 +1,10 @@
-/* udev_util.c
- *
- * <copyright>
- * Copyright (C) 2016-2019 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
-
-// Adapted from source code at http://www.signal11.us/oss/udev/
-
 /** @file udev_util.c
  * UDEV utility functions
  */
+// Copyright (C) 2016-2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
+// Adapted from source code at http://www.signal11.us/oss/udev/
 
 /** \cond */
 #include <assert.h>
@@ -251,17 +230,21 @@ void report_udev_device(struct udev_device * dev, int depth) {
    udev_list_entry_foreach(cur_entry, properties_list_entry) {
       const char * prop_name   = udev_list_entry_get_name(cur_entry);
       const char * prop_value  = udev_list_entry_get_value(cur_entry);
+#ifndef NDEBUG
       const char * prop_value2 = udev_device_get_property_value(dev, prop_name);
       assert(streq(prop_value, prop_value2));
+#endif
       rpt_vstring(d2, "%s -> %s", prop_name, prop_value);
    }
 
    rpt_vstring(d1, "Sysattrs:");
    udev_list_entry_foreach(cur_entry, sysattr_list_entry) {
       const char * attr_name   = udev_list_entry_get_name(cur_entry);
+#ifndef NDEBUG
       const char * attr_value  = udev_list_entry_get_value(cur_entry);
-      const char * attr_value2 = udev_device_get_sysattr_value(dev, attr_name);
       assert(attr_value == NULL);
+#endif
+      const char * attr_value2 = udev_device_get_sysattr_value(dev, attr_name);
       // hex_dump( (Byte*) attr_value2, strlen(attr_value2)+1);
       if (attr_value2 && strchr(attr_value2, '\n')) {
       // if (streq(attr_name, "uevent")) {
