@@ -1,14 +1,14 @@
 /** @file parse_capabilities.h
  */
 
-// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef PARSE_CAPABILITIES_H_
 #define PARSE_CAPABILITIES_H_
 
 /** \cond */
-#include <glib.h>
+#include <glib-2.0/glib.h>
 
 #include "util/data_structures.h"
 /** \endcond */
